import string

from module_info import *

from process_common import *
from process_operations import *

dword      = 0x8000000000000000
dword_mask = 0xffffffffffffffff

density_bits      = 32
fkf_density_mask  = 0xFFFF #16K

#terain condition flags
fkf_plain             = 0x00000004
fkf_steppe            = 0x00000008
fkf_snow              = 0x00000010
fkf_desert            = 0x00000020
fkf_plain_forest      = 0x00000400
fkf_steppe_forest     = 0x00000800
fkf_snow_forest       = 0x00001000
fkf_desert_forest     = 0x00002000
fkf_terrain_mask      = 0x0000ffff

fkf_realtime_ligting  = 0x00010000 #deprecated
fkf_point_up          = 0x00020000 #uses auto-generated point-up(quad) geometry for the flora kind
fkf_align_with_ground = 0x00040000 #align the flora object with the ground normal
fkf_grass             = 0x00080000 #is grass
fkf_on_green_ground   = 0x00100000 #populate this flora on green ground
fkf_rock              = 0x00200000 #is rock 
fkf_tree              = 0x00400000 #is tree -> note that if you set this parameter, you should pass additional alternative tree definitions
fkf_snowy             = 0x00800000
fkf_guarantee         = 0x01000000

fkf_speedtree         = 0x02000000  #NOT FUNCTIONAL: we have removed speedtree support on M&B Warband

fkf_has_colony_props  = 0x04000000  # if fkf_has_colony_props -> then you can define colony_radius and colony_treshold of the flora kind


def density(g):
  if (g > fkf_density_mask):
    g = fkf_density_mask
  return ((dword | g) << density_bits)


fauna_kinds = [
	#############################################################
	#This section is here for backwards-compatibilty only.
  ("grass",density(0),[("001xenoargh_grass_a","0"),("001xenoargh_grass_b","0"),("001xenoargh_grass_c","0"),("001xenoargh_grass_d","0"),("001xenoargh_grass_e","0"),("001xenoargh_grass_a2","0"),("001xenoargh_grass_b2","0"),("001xenoargh_grass_c2","0"),("001xenoargh_grass_d2","0"),("001xenoargh_grass_e2","0")]),
  ("grass_bush",0,[("xenoargh_grass_bush001","0"),("xenoargh_grass_bush002","0"),("xenoargh_grass_bush003","0"),("xenoargh_grass_bush004","0"),("xenoargh_grass_bush005","0"),("xenoargh_grass_bush006","0"),("xenoargh_grass_bush007","0"),("xenoargh_grass_bush008","0"),("xenoargh_grass_bush009","0"),("xenoargh_grass_bush010","0"),("xenoargh_grass_bush011","0"),("xenoargh_grass_bush012","0")]),
  ("grass_saz",0,[("xenoargh_grass_bush001","0"),("xenoargh_grass_bush002","0"),("xenoargh_grass_bush003","0"),("xenoargh_grass_bush004","0"),("xenoargh_grass_bush005","0"),("xenoargh_grass_bush006","0"),("xenoargh_grass_bush007","0"),("xenoargh_grass_bush008","0"),("xenoargh_grass_bush009","0"),("xenoargh_grass_bush010","0"),("xenoargh_grass_bush011","0"),("xenoargh_grass_bush012","0")]),
  ("grass_purple",0,[("xenoargh_grass_bush001","0"),("xenoargh_grass_bush002","0"),("xenoargh_grass_bush003","0"),("xenoargh_grass_bush004","0"),("xenoargh_grass_bush005","0"),("xenoargh_grass_bush006","0"),("xenoargh_grass_bush007","0"),("xenoargh_grass_bush008","0"),("xenoargh_grass_bush009","0"),("xenoargh_grass_bush010","0"),("xenoargh_grass_bush011","0"),("xenoargh_grass_bush012","0")]),
  ("fern",0,[("xenoargh_fern001","0"),("xenoargh_fern002","0"),("xenoargh_fern003","0"),("xenoargh_fern004","0"),("xenoargh_fern005","0"),("xenoargh_fern006","0")]),
  ("grass_steppe",0,[("001xenoargh_grass_yellow_a","0"),("001xenoargh_grass_yellow_b","0"),("001xenoargh_grass_yellow_c","0"),("001xenoargh_grass_yellow_d","0"),("001xenoargh_grass_yellow_e","0"),("001xenoargh_grass_yellow_a2","0"),("001xenoargh_grass_yellow_b2","0"),("001xenoargh_grass_yellow_c2","0"),("001xenoargh_grass_yellow_d2","0"),("001xenoargh_grass_yellow_e2","0")]),

  ("grass_bush_g",0,[("xenoargh_bunchgrass001","0"),("xenoargh_bunchgrass002","0"),("xenoargh_bunchgrass003","0"),("xenoargh_bunchgrass004","0"),("xenoargh_bunchgrass005","0"),("xenoargh_bunchgrass006","0")]),
  ("grass_bush_h",0,[("xenoargh_spikeweed001","0"),("xenoargh_spikeweed002","0"),("xenoargh_spikeweed003","0"),("xenoargh_spikeweed004","0"),("xenoargh_spikeweed005","0"),("xenoargh_spikeweed006","0")]),
  ("grass_bush_i",0,[("xenoargh_randomweed001","0"),("xenoargh_randomweed002","0"),("xenoargh_randomweed003","0"),("xenoargh_randomweed004","0"),("xenoargh_randomweed005","0"),("xenoargh_randomweed006","0")]),
  ("grass_bush_j",0,[("xenoargh_low_weed001","0"),("xenoargh_low_weed002","0"),("xenoargh_low_weed003","0"),("xenoargh_low_weed004","0")]),
  ("grass_bush_k",0,[("xenoargh_whiteflower001","0"),("xenoargh_whiteflower002","0"),("xenoargh_whiteflower003","0"),("xenoargh_whiteflower004","0")]),
  ("grass_bush_l",0,[("grass_bush_l01","0"),("grass_bush_l02","0")]),
  
  ("thorn_a",0,[("thorn_a","0"),("thorn_b","0"),("thorn_c","0"),("thorn_d","0")]),

  ("basak",0,[("basak","0")]),
  ("common_plant",0,[("common_plant","0")]),
  ("small_plant",0,[("small_plant","0"),("small_plant_b","0"),("small_plant_c","0")]),
  ("buddy_plant",0,[("buddy_plant","0"),("buddy_plant_b","0")]),
  ("yellow_flower",0,[("yellow_flower","0"),("yellow_flower_b","0")]),
  ("spiky_plant",fkf_align_with_ground|0,[("spiky_plant","0")]),
  ("seedy_plant",0,[("seedy_plant_a","0")]),
  ("blue_flower",0,[("blue_flower","0")]),
  ("big_bush",0,[("big_bush","0")]),

  ("bushes02_a",0,[("bushes02_a","bo_bushes02_a"),("bushes02_b","bo_bushes02_b"),("bushes02_c","bo_bushes02_c")]),
  ("bushes03_a",0,[("bushes03_a","0"),("bushes03_b","0"),("bushes03_c","0")]),
  ("bushes04_a",0,[("bushes04_a","0"),("bushes04_b","0"),("bushes04_c","0")]),
  ("bushes05_a",0,[("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0")]),
  ("bushes06_a",0,[("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0")]),
  ("bushes07_a",0,[("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0")]),
  ("bushes08_a",0,[("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0")]),
  ("bushes09_a",0,[("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0")]),
  ("bushes10_a",0,[("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0")]),
  ("bushes11_a",0,[("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0")]),
  ("bushes12_a",0,[("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

  ("aspen",0,[("aspen_a","bo_aspen_a",("0","0")),("aspen_b","bo_aspen_b",("0","0")),("aspen_c","bo_aspen_c",("0","0"))]),
  ("pine_1",0,[("pine_1_a","bo_pine_1_a",("0","0")),("pine_1_b","bo_pine_1_b",("0","0"))]),
  ("pine_2",0,[("pine_2_a","bo_pine_2_a",("0","0"))]),
  ("pine_3",0,[("pine_3_a","bo_pine_3_a",("0","0"))]),
  ("pine_4",0,[("pine_4_a","bo_pine_4_a",("0","0"))]),
  ("snowy_pine",0,[("tree_snowy_a","bo_tree_snowy_a",("0","0"))]),
  ("snowy_pine_2",0,[("snowy_pine_2","bo_snowy_pine_2",("0","0"))]),

  ("small_rock",0,[("llew_rock_boulder_1", "bo_llew_rock_boulder_1"),("llew_rock_boulder_2", "bo_llew_rock_boulder_2"),("llew_rock_boulder_3", "bo_llew_rock_boulder_3"),("llew_rock_boulder_4", "bo_llew_rock_boulder_4"),("llew_rock_boulder_5", "bo_llew_rock_boulder_5"),("llew_rock_boulder_6", "bo_llew_rock_boulder_6"),("llew_rock_cliff_1", "bo_llew_rock_cliff_1"),("llew_rock_cliff_2", "bo_llew_rock_cliff_2"),("llew_rock_cliff_3", "bo_llew_rock_cliff_3"),("llew_rock_cliff_4", "bo_llew_rock_cliff_4"),("llew_rock_cliff_5", "bo_llew_rock_cliff_5"),("llew_rock_cliff_6", "bo_llew_rock_cliff_6"),
("llew_rock_flat_1", "bo_llew_rock_flat_1"),("llew_rock_flat_2", "bo_llew_rock_flat_2"),("llew_rock_flat_3", "bo_llew_rock_flat_3"),("llew_rock_flat_4", "bo_llew_rock_flat_4"),("llew_rock_flat_5", "bo_llew_rock_flat_5"),("llew_rock_flat_6", "bo_llew_rock_flat_6")]),
  
  ("rock_snowy",0,[("rock_snowy_a","bo_rock_snowy_a"),("rock_snowy_b","bo_rock_snowy_b"),("rock_snowy_c","bo_rock_snowy_c"),]),
  
  
  ("rock",0,[("llew_rock_boulder_1", "bo_llew_rock_boulder_1"),("llew_rock_boulder_2", "bo_llew_rock_boulder_2"),("llew_rock_boulder_3", "bo_llew_rock_boulder_3"),("llew_rock_boulder_4", "bo_llew_rock_boulder_4"),("llew_rock_boulder_5", "bo_llew_rock_boulder_5"),("llew_rock_boulder_6", "bo_llew_rock_boulder_6"),  
("llew_rock_cliff_1", "bo_llew_rock_cliff_1"),("llew_rock_cliff_2", "bo_llew_rock_cliff_2"),("llew_rock_cliff_3", "bo_llew_rock_cliff_3"),("llew_rock_cliff_4", "bo_llew_rock_cliff_4"),("llew_rock_cliff_5", "bo_llew_rock_cliff_5"),("llew_rock_cliff_6", "bo_llew_rock_cliff_6"),("llew_rock_flat_1", "bo_llew_rock_flat_1"),("llew_rock_flat_2", "bo_llew_rock_flat_2"),("llew_rock_flat_3", "bo_llew_rock_flat_3"),("llew_rock_flat_4", "bo_llew_rock_flat_4"),("llew_rock_flat_5", "bo_llew_rock_flat_5"),("llew_rock_flat_6", "bo_llew_rock_flat_6")]),

  ("rock_snowy2",0,[("rock1_snowy","bo_rock1"),("rock2_snowy","bo_rock2"),("rock4_snowy","bo_rock4"),("rock6_snowy","bo_rock6"),]),

  ("tree_1",0,[("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0"))]),
  ("tree_3",0,[("tree_3_a","bo_tree_3_a",("0","0")),("tree_3_b","bo_tree_3_b",("0","0"))]),
  ("tree_4",0,[("tree_4_a","bo_tree_4_a",("0","0")),("tree_4_b","bo_tree_4_b",("0","0"))]),
  ("tree_5",0,[("tree_5_a","bo_tree_5_a",("0","0")),("tree_5_b","bo_tree_5_b",("0","0")),("tree_5_c","bo_tree_5_c",("0","0")),("tree_5_d","bo_tree_5_d",("0","0"))]),
  ("tree_6",0,[("tree_6_a","bo_tree_6_a",("0","0")),("tree_6_b","bo_tree_6_b",("0","0")),("tree_6_c","bo_tree_6_c",("0","0")),("tree_6_d","bo_tree_6_d",("0","0"))]),
  ("tree_7",0,[("tree_7_a","bo_tree_7_a",("0","0")),("tree_7_b","bo_tree_7_b",("0","0")),("tree_7_c","bo_tree_7_c",("0","0"))]),
  ("tree_8",0,[("tree_8_a","bo_tree_8_a",("0","0")),("tree_8_b","bo_tree_8_b",("0","0")),("tree_8_c","bo_tree_8_c",("0","0"))]),

  ("tree_9",0,[("tree_9_a","bo_tree_9_a",("0","0")),("tree_9_b","bo_tree_9_a",("0","0")),("tree_9_c","bo_tree_9_a",("0","0"))]),
  ("tree_10",0,[("tree_10_a","bo_tree_10_a",("0","0")),("tree_10_b","bo_tree_10_a",("0","0")),("tree_10_c","bo_tree_10_a",("0","0"))]),
  ("tree_11",0,[("tree_11_a","bo_tree_11_a",("0","0")),("tree_11_b","bo_tree_11_a",("0","0")),("tree_11_c","bo_tree_11_a",("0","0"))]),
  ("tree_12",0,[("tree_12_a","bo_tree_12_a",("0","0")),("tree_12_b","bo_tree_12_b",("0","0")),("tree_12_c","bo_tree_12_c",("0","0"))]),
  ("tree_14",0,[("tree_14_a","bo_tree_14_a",("0","0")),("tree_14_b","bo_tree_14_b",("0","0")),("tree_14_c","bo_tree_14_c",("0","0"))]),
  ("tree_15",0,[("tree_15_a","bo_tree_15_a",("mb_test1","tree_a")),("tree_15_b","bo_tree_15_b",("mb_test1","tree_a")),("tree_15_c","bo_tree_15_c",("mb_test1","tree_b"))]),
  ("tree_16",0,[("tree_16_a","bo_tree_16_a",("0","0")),("tree_16_b","bo_tree_16_b",("0","0"))]),

  ("tree_17",0,[("tree_17_a","bo_tree_17_a",("0","0")),("tree_17_b","bo_tree_17_b",("0","0")),("tree_17_c","bo_tree_17_c",("0","0")),("tree_17_d","bo_tree_17_d",("0","0"))]),

  ("palm",0,[("palm_a","bo_palm_a",("0","0")),("xenoargh_palm_tree01","bo_xenoargh_palm_tree01",("0","0")),("xenoargh_palm_tree02","bo_xenoargh_palm_tree02",("0","0")),("xenoargh_palm_tree03","bo_xenoargh_palm_tree03",("0","0"))]),

  ("tree_new_1",0,[("tree_a01","bo_tree_a01",("0","0")),("tree_a02","bo_tree_a01",("0","0"))]),
  ("bush_new_1",0,[("bush_a01","0"),("bush_a02","0")]),
  ("bush_new_2",0,[("bush_new_a","0")]),
  ("bush_new_3",0,[("bush_new_c","0")]),
  ("bush_new_4",0,[("bush_new_c","0")]),

  ("dry_bush",0,[("dry_bush","0")]),
  ("dry_leaves",0,[("dry_leaves","0")]),

  ("tree_new_2",0,[("tree_b01","bo_tree_b01",("0","0")),("tree_b02","bo_tree_b02",("0","0"))]),
  ("tree_new_3",0,[("tree_c01","bo_tree_c01",("0","0")),("tree_c02","bo_tree_c02",("0","0"))]),

  ("tree_plane",0,[("tree_18_a","bo_tree_18_a",("0","0")),("tree_18_b","bo_tree_18_b",("0","0"))]),
  ("tree_19",0,[("tree_19_a","bo_tree_19_a",("0","0"))]),
  ("beech",density(3),[("tree_20_a","bo_tree_20_a",("0","0")),("tree_20_b","bo_tree_20_b",("0","0"))]),

  ("tall_tree",0,[("tall_tree_a","bo_tall_tree_a",("0","0"))]),

  ("tree_e",0,[("tree_e_1","bo_tree_e_1",("0","0")),("tree_e_2","bo_tree_e_2",("0","0")),("tree_e_3","bo_tree_e_3",("0","0"))]),
  ("tree_f",0,[("tree_f_1","bo_tree_f_1",("0","0")),("tree_f_2","bo_tree_f_1",("0","0")),("tree_f_3","bo_tree_f_1",("0","0"))]),
  ("grape_vineyard",density(0),[("grape_vineyard","bo_grape_vineyard")]),
  ("grape_vineyard_stake",density(0),[("grape_vineyard_stake","bo_grape_vineyard_stake")]),  
  
  ("wheat",0,[("wheat_a","0"),("wheat_b","0"),("wheat_c","0"),("wheat_d","0")]),
  
  ("valleyRock_rounded",0,[("valleyRock_rounded_1","bo_valleyRock_rounded_1"),("valleyRock_rounded_2","bo_valleyRock_rounded_2"),("valleyRock_rounded_3","bo_valleyRock_rounded_3"),("valleyRock_rounded_4","bo_valleyRock_rounded_4")]),
  
  ("valleyRock_flat",0,[("valleyRock_flat_1","bo_valleyRock_flat_1"),("valleyRock_flat_2","bo_valleyRock_flat_2"),("valleyRock_flat_3","bo_valleyRock_flat_3"),("valleyRock_flat_4","bo_valleyRock_flat_4"),("valleyRock_flat_5","bo_valleyRock_flat_5"),("valleyRock_flat_6","bo_valleyRock_flat_6")]),
  
  ("valleyRock_flatRounded_small",0,[("valleyRock_flatRounded_small_1","bo_valleyRock_flatRounded_small_1"),("valleyRock_flatRounded_small_2","bo_valleyRock_flatRounded_small_2"),("valleyRock_flatRounded_small_3","bo_valleyRock_flatRounded_small_3")]),
  
  ("valleyRock_flatRounded_big",0,[("valleyRock_flatRounded_big_1","bo_valleyRock_flatRounded_big_1"),("valleyRock_flatRounded_big_2","bo_valleyRock_flatRounded_big_2")]),
  
  ("beanrow",0,[("xenoargh_beanrow001","0"),("xenoargh_beanrow002","0")]),
  ("wheatrow",0,[("xenoargh_wheatrow001","0"),("xenoargh_wheatrow002","0")]),
  
  ("wheat_field",0,[("xenoargh_wheatfield001","0"),("xenoargh_wheatfield002","0")]),
  ("bean_field",0,[("xenoargh_beanfield001","0"),("xenoargh_beanfield002","0")]),
  
  ("snowy_pine_3",0,[("snowy_pine_3","bo_snowy_pine_3",("0","0"))]),
  
  
 	#############################################################
	#PLAINS
	#Some explanation.
	#First off, we have the grass.  It gets auto-generated, and is a reasonable load if kept within tolerances.  
	#Try to keep total density < 3000 if you don't want to kill framerate.
	#Then we have bushes.  Max is probably < 1000, but it's going to vary considerably, keep it conservative.
	#Then trees.  Like bushes, but much lower counts are wise.  Keep it low to save framerate, they're expensive (400 triangles+).
	#Lastly, random junk of various kinds.  The houses add a lot of atmosphere, but need to be kept really low.
	#The other stuff, like the rocks, can be used pretty liberally, maybe 300-500 or so.

	##########Grass
	("grass_plains",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_guarantee|fkf_align_with_ground|fkf_plain|density(1000),[("001xenoargh_grass_a","0"),("001xenoargh_grass_b","0"),("001xenoargh_grass_c","0"),("001xenoargh_grass_d","0"),("001xenoargh_grass_e","0"),("001xenoargh_grass_a2","0"),("001xenoargh_grass_b2","0"),("001xenoargh_grass_c2","0"),("001xenoargh_grass_d2","0"),("001xenoargh_grass_e2","0")]),	
	
	("grass_plains_yellow",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_plain|density(375),[("001xenoargh_grass_yellowgreen_a","0"),("001xenoargh_grass_yellowgreen_b","0"),("001xenoargh_grass_yellowgreen_c","0"),("001xenoargh_grass_yellowgreen_d","0"),("001xenoargh_grass_yellowgreen_e","0"),("001xenoargh_grass_yellowgreen_a2","0"),("001xenoargh_grass_yellowgreen_b2","0"),("001xenoargh_grass_yellowgreen_c2","0"),("001xenoargh_grass_yellowgreen_d2","0"),("001xenoargh_grass_yellowgreen_e2","0")]),
	
	("grass_plains_green",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_plain|density(375),[("001xenoargh_grass_a","0"),("001xenoargh_grass_b","0"),("001xenoargh_grass_c","0"),("001xenoargh_grass_d","0"),("001xenoargh_grass_e","0"),("001xenoargh_grass_a2","0"),("001xenoargh_grass_b2","0"),("001xenoargh_grass_c2","0"),("001xenoargh_grass_d2","0"),("001xenoargh_grass_e2","0")]),	
	
	("grass_plains_greener",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_plain|density(750),[("001xenoargh_greener_grass_a","0"),("001xenoargh_greener_grass_b","0"),("001xenoargh_greener_grass_c","0"),("001xenoargh_greener_grass_d","0"),("001xenoargh_greener_grass_e","0"),("001xenoargh_greener_grass_a2","0"),("001xenoargh_greener_grass_b2","0"),("001xenoargh_greener_grass_c2","0"),("001xenoargh_greener_grass_d2","0"),("001xenoargh_greener_grass_e2","0")]),	

	("grass_plains_weedmix01",fkf_grass|fkf_point_up|fkf_align_with_ground|fkf_plain|density(750),[("xenoargh_whiteflower001","0"),("xenoargh_whiteflower002","0"),("xenoargh_whiteflower003","0"),("xenoargh_whiteflower004","0"),("xenoargh_grass_bush001","0"),("xenoargh_grass_bush002","0"),("xenoargh_grass_bush003","0"),("xenoargh_grass_bush004","0"),("xenoargh_grass_bush005","0"),("xenoargh_grass_bush006","0"),("xenoargh_grass_bush007","0"),("xenoargh_grass_bush008","0"),("xenoargh_grass_bush009","0"),("xenoargh_grass_bush010","0"),("xenoargh_grass_bush011","0"),("xenoargh_grass_bush012","0")]),

	("grass_plains_weedmix02",fkf_point_up|fkf_align_with_ground|fkf_plain|density(750),[("xenoargh_whiteflower001","0"),("xenoargh_whiteflower002","0"),("xenoargh_grass_bush001","0"),("xenoargh_grass_bush002","0"),("xenoargh_grass_bush003","0"),("xenoargh_grass_bush004","0"),("xenoargh_grass_bush005","0"),("xenoargh_grass_bush006","0"),("grass_bush_l01","0"),("grass_bush_l02","0"),("thorn_a","0"),("thorn_b","0"),("thorn_c","0"),("thorn_d","0"),("basak","0"),("common_plant","0"),("small_plant","0"),("small_plant_b","0"),("small_plant_c","0"),("buddy_plant","0"),("buddy_plant_b","0"),("yellow_flower","0"),("yellow_flower_b","0"),("spiky_plant","0"),("seedy_plant_a","0"),("blue_flower","0"),("big_bush","0")]),

	##########Bushes
	("grass_plains_bushmix01",fkf_plain|density(150),[("bushes02_a","bo_bushes02_a"),("bushes02_b","bo_bushes02_b"),("bushes02_c","bo_bushes02_c"),("bushes04_a","0"),("bushes04_b","0"),("bushes04_c","0"),("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0"),("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0"),("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0"),("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0"),("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

#Used on_green_ground for this mix, because of the small trees (bushes02_a).
	("grass_plains_bushmix02",fkf_plain|fkf_on_green_ground|density(250),[("bushes02_a","bo_bushes02_a"),("bushes02_b","bo_bushes02_b"),("bushes02_c","bo_bushes02_c"),("bushes04_a","0"),("bushes04_b","0"),("bushes04_c","0"),("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0"),("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0"),("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0"),("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0"),("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

#Used on_green_ground for this mix, because of the small trees (bushes02_a).
	("grass_plains_bushmix03",fkf_plain|fkf_on_green_ground|density(150),[("bushes02_a","bo_bushes02_a"),("bushes02_b","bo_bushes02_b"),("bushes02_c","bo_bushes02_c"),("bushes04_a","0"),("bushes04_b","0"),("bushes04_c","0"),("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0"),("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0")]),			

	("grass_plains_bushmix04",fkf_plain|density(150),[("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0"),("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0"),("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),	

	##########Trees
	#WARNING:  THESE ARE THE HIGHEST-POLY TREES IN THE GAME, AND I REMOVED THE LODS (too much pop-in), DO NOT USE HIGH DENSITY, IT *WILL* KILL FRAMERATE
	("grass_plains_treemix01",fkf_plain|fkf_tree|fkf_on_green_ground|density(5),[("tree_a01","bo_tree_a01",("0","0")),("tree_a02","bo_tree_a01",("0","0")),("tree_b01","bo_tree_b01",("0","0")),("tree_b02","bo_tree_b01",("0","0")),("tree_c01","bo_tree_c01",("0","0")),("tree_c02","bo_tree_c01",("0","0"))]),

	#Dead bushy things- pretty cheap, makes for a scruffy atmosphere.
	("grass_plains_treemix02",fkf_plain|fkf_tree|fkf_on_green_ground|density(500),[("bush_dead_a","0",("0","0")),("bush_dead_b","0",("0","0")),("bush_dead_c","0",("0","0"))]),	
	
	#Most of the deciduous trees, in one package.  Keep density reasonable, or fillrate may be an issue.
	("grass_plains_treemix03",fkf_plain|fkf_tree|fkf_on_green_ground|density(10),[("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0")),("tree_3_a","bo_tree_3_a",("0","0")),("tree_3_b","bo_tree_3_b",("0","0")),("tree_4_a","bo_tree_4_a",("0","0")),("tree_4_b","bo_tree_4_b",("0","0")),("tree_5_a","bo_tree_5_a",("0","0")),("tree_5_b","bo_tree_5_b",("0","0")),("tree_5_c","bo_tree_5_c",("0","0")),("tree_5_d","bo_tree_5_d",("0","0")),("tree_6_a","bo_tree_6_a",("0","0")),("tree_6_b","bo_tree_6_b",("0","0")),("tree_6_c","bo_tree_6_c",("0","0")),("tree_6_d","bo_tree_6_d",("0","0")),("tree_7_a","bo_tree_7_a",("0","0")),("tree_7_b","bo_tree_7_b",("0","0")),("tree_7_c","bo_tree_7_c",("0","0")),("tree_8_a","bo_tree_8_a",("0","0")),("tree_8_b","bo_tree_8_b",("0","0")),("tree_8_c","bo_tree_8_c",("0","0")),("tree_9_a","bo_tree_9_a",("0","0")),("tree_9_b","bo_tree_9_a",("0","0")),("tree_9_c","bo_tree_9_a",("0","0")),("tree_10_a","bo_tree_10_a",("0","0")),("tree_10_b","bo_tree_10_a",("0","0")),("tree_10_c","bo_tree_10_a",("0","0")),("tree_11_a","bo_tree_11_a",("0","0")),("tree_11_b","bo_tree_11_a",("0","0")),("tree_11_c","bo_tree_11_a",("0","0")),("tree_12_a","bo_tree_12_a",("0","0")),("tree_12_b","bo_tree_12_b",("0","0")),("tree_12_c","bo_tree_12_c",("0","0")),("tree_14_a","bo_tree_14_a",("0","0")),("tree_14_b","bo_tree_14_b",("0","0")),("tree_14_c","bo_tree_14_c",("0","0")),("tree_15_a","bo_tree_15_a",("0","0")),("tree_15_b","bo_tree_15_b",("0","0")),("tree_15_c","bo_tree_15_c",("0","0")),("tree_16_a","bo_tree_16_a",("0","0")),("tree_16_b","bo_tree_16_b",("0","0")),("tree_17_a","bo_tree_17_a",("0","0")),("tree_17_b","bo_tree_17_b",("0","0")),("tree_17_c","bo_tree_17_c",("0","0")),("tree_17_d","bo_tree_17_d",("0","0"))]),
	
	##########Random procedural junk
	#Village huts / houses.  Creates a nice random atmosphere of peopled relics. Numbers have to be low, to avoid pathing problems.
	("grass_plains_houses",fkf_plain|fkf_tree|fkf_on_green_ground|density(2),[("village_house_a","bo_village_house_a",("0","0")),("village_house_b","bo_village_house_b",("0","0")),("village_house_c","bo_village_house_c",("0","0")),("village_house_d","bo_village_house_d",("0","0")),("village_hut_a","bo_village_hut_a",("0","0")),]),

	#Stumps and fallen trees, with a few regular trees sprinkled in.  Creates a nice atmosphere.
	("grass_plains_stumps_and_trees",fkf_plain|fkf_tree|fkf_on_green_ground|fkf_align_with_ground|density(25),[("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0")),("tree_3_a","bo_tree_3_a",("0","0")),("tree_3_b","bo_tree_3_b",("0","0")),("destroy_tree_a","bo_destroy_tree_a",("0","0")),("destroy_tree_b","bo_destroy_tree_b",("0","0")),("tree_stump_a","bo_tree_stump_a",("0","0")),("tree_stump_b","bo_tree_stump_b",("0","0")),("tree_stump_c","bo_tree_stump_c",("0","0")),("tree_stump_a","bo_tree_stump_a",("0","0")),("tree_stump_b","bo_tree_stump_b",("0","0")),("tree_stump_c","bo_tree_stump_c",("0","0")),]),		

	#Stumps and rocks, because lumpy battlefields are occasionally fun.
	("grass_plains_stumps_and_rocks",fkf_plain|fkf_rock|fkf_on_green_ground|fkf_align_with_ground|density(25),[("llew_rock_boulder_1", "bo_llew_rock_boulder_1"),("llew_rock_boulder_2", "bo_llew_rock_boulder_2"),("llew_rock_boulder_3", "bo_llew_rock_boulder_3"),("llew_rock_boulder_4", "bo_llew_rock_boulder_4"),("llew_rock_boulder_5", "bo_llew_rock_boulder_5"),("llew_rock_boulder_6", "bo_llew_rock_boulder_6"),("llew_rock_flat_1", "bo_llew_rock_flat_1"),("llew_rock_flat_2", "bo_llew_rock_flat_2"),("llew_rock_flat_3", "bo_llew_rock_flat_3"),("llew_rock_flat_4", "bo_llew_rock_flat_4"),("llew_rock_flat_5", "bo_llew_rock_flat_5"),("llew_rock_flat_6", "bo_llew_rock_flat_6"),("destroy_tree_a","bo_destroy_tree_a"),("destroy_tree_b","bo_destroy_tree_b"),("tree_stump_a","bo_tree_stump_a"),("tree_stump_b","bo_tree_stump_b"),("tree_stump_c","bo_tree_stump_c"),("tree_stump_a","bo_tree_stump_a"),("tree_stump_b","bo_tree_stump_b"),("tree_stump_c","bo_tree_stump_c")]),	
	
	#end PLAINS
	#############################################################

 	#############################################################
	#PLAINS FORESTS
	#Significantly lower grass counts, slightly higher tree counts, bush counts and junk counts.

	##########Grass
	("grass_plains_forest",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_guarantee|fkf_align_with_ground|fkf_plain_forest|density(300),[("001xenoargh_grass_a","0"),("001xenoargh_grass_b","0"),("001xenoargh_grass_c","0"),("001xenoargh_grass_d","0"),("001xenoargh_grass_e","0"),("001xenoargh_grass_a2","0"),("001xenoargh_grass_b2","0"),("001xenoargh_grass_c2","0"),("001xenoargh_grass_d2","0"),("001xenoargh_grass_e2","0")]),	
	
	("grass_plains_forest_yellow",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_plain_forest|density(150),[("001xenoargh_grass_yellowgreen_a","0"),("001xenoargh_grass_yellowgreen_b","0"),("001xenoargh_grass_yellowgreen_c","0"),("001xenoargh_grass_yellowgreen_d","0"),("001xenoargh_grass_yellowgreen_e","0"),("001xenoargh_grass_yellowgreen_a2","0"),("001xenoargh_grass_yellowgreen_b2","0"),("001xenoargh_grass_yellowgreen_c2","0"),("001xenoargh_grass_yellowgreen_d2","0"),("001xenoargh_grass_yellowgreen_e2","0")]),
	
	("grass_plains_forest_green",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_plain_forest|density(150),[("001xenoargh_grass_a","0"),("001xenoargh_grass_b","0"),("001xenoargh_grass_c","0"),("001xenoargh_grass_d","0"),("001xenoargh_grass_e","0"),("001xenoargh_grass_a2","0"),("001xenoargh_grass_b2","0"),("001xenoargh_grass_c2","0"),("001xenoargh_grass_d2","0"),("001xenoargh_grass_e2","0")]),	
	
	("grass_plains_forest_greener",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_plain_forest|density(300),[("001xenoargh_greener_grass_a","0"),("001xenoargh_greener_grass_b","0"),("001xenoargh_greener_grass_c","0"),("001xenoargh_greener_grass_d","0"),("001xenoargh_greener_grass_e","0"),("001xenoargh_greener_grass_a2","0"),("001xenoargh_greener_grass_b2","0"),("001xenoargh_greener_grass_c2","0"),("001xenoargh_greener_grass_d2","0"),("001xenoargh_greener_grass_e2","0")]),	

	("grass_plains_forest_weedmix01",fkf_grass|fkf_point_up|fkf_align_with_ground|fkf_plain_forest|density(300),[("xenoargh_whiteflower001","0"),("xenoargh_whiteflower002","0"),("xenoargh_whiteflower003","0"),("xenoargh_whiteflower004","0"),("xenoargh_grass_bush001","0"),("xenoargh_grass_bush002","0"),("xenoargh_grass_bush003","0"),("xenoargh_grass_bush004","0"),("xenoargh_grass_bush005","0"),("xenoargh_grass_bush006","0"),("xenoargh_grass_bush007","0"),("xenoargh_grass_bush008","0"),("xenoargh_grass_bush009","0"),("xenoargh_grass_bush010","0"),("xenoargh_grass_bush011","0"),("xenoargh_grass_bush012","0")]),

	("grass_plains_forest_weedmix02",fkf_point_up|fkf_align_with_ground|fkf_plain_forest|density(300),[("xenoargh_whiteflower001","0"),("xenoargh_whiteflower002","0"),("xenoargh_grass_bush001","0"),("xenoargh_grass_bush002","0"),("xenoargh_grass_bush003","0"),("xenoargh_grass_bush004","0"),("xenoargh_grass_bush005","0"),("xenoargh_grass_bush006","0"),("grass_bush_l01","0"),("grass_bush_l02","0"),("thorn_a","0"),("thorn_b","0"),("thorn_c","0"),("thorn_d","0"),("basak","0"),("common_plant","0"),("small_plant","0"),("small_plant_b","0"),("small_plant_c","0"),("buddy_plant","0"),("buddy_plant_b","0"),("yellow_flower","0"),("yellow_flower_b","0"),("spiky_plant","0"),("seedy_plant_a","0"),("blue_flower","0"),("big_bush","0")]),

	##########Bushes
	("grass_plains_forest_bushmix01",fkf_plain_forest|density(200),[("bushes02_a","bo_bushes02_a"),("bushes02_b","bo_bushes02_b"),("bushes02_c","bo_bushes02_c"),("bushes04_a","0"),("bushes04_b","0"),("bushes04_c","0"),("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0"),("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0"),("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0"),("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0"),("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

#Used on_green_ground for this mix, because of the small trees (bushes02_a).
	("grass_plains_forest_bushmix02",fkf_plain_forest|fkf_on_green_ground|density(350),[("bushes02_a","bo_bushes02_a"),("bushes02_b","bo_bushes02_b"),("bushes02_c","bo_bushes02_c"),("bushes04_a","0"),("bushes04_b","0"),("bushes04_c","0"),("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0"),("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0"),("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0"),("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0"),("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

#Used on_green_ground for this mix, because of the small trees (bushes02_a).
	("grass_plains_forest_bushmix03",fkf_plain_forest|fkf_on_green_ground|density(200),[("bushes02_a","bo_bushes02_a"),("bushes02_b","bo_bushes02_b"),("bushes02_c","bo_bushes02_c"),("bushes04_a","0"),("bushes04_b","0"),("bushes04_c","0"),("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0"),("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0")]),			

	("grass_plains_forest_bushmix04",fkf_plain_forest|density(200),[("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0"),("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0"),("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),	

	##########Trees
	#WARNING:  THESE ARE THE HIGHEST-POLY TREES IN THE GAME, AND I REMOVED THE LODS (too much pop-in), DO NOT USE HIGH DENSITY, IT *WILL* KILL FRAMERATE
	("grass_plains_forest_treemix01",fkf_plain_forest|fkf_tree|fkf_on_green_ground|density(6),[("tree_a01","bo_tree_a01",("0","0")),("tree_a02","bo_tree_a01",("0","0")),("tree_b01","bo_tree_b01",("0","0")),("tree_b02","bo_tree_b01",("0","0")),("tree_c01","bo_tree_c01",("0","0")),("tree_c02","bo_tree_c01",("0","0"))]),

	#Dead bushy things- pretty cheap, makes for a scruffy atmosphere.
	#NOTE:  Not using fkf_tree here- I wanted less spacing and higher counts.
	("grass_plains_forest_treemix02",fkf_plain_forest|fkf_on_green_ground|density(1250),[("bush_dead_a","0"),("bush_dead_b","0"),("bush_dead_c","0")]),	
	
	#Most of the deciduous trees, in one package.  Keep density reasonable, or fillrate may be an issue.
	("grass_plains_forest_treemix03",fkf_plain_forest|fkf_tree|fkf_on_green_ground|density(15),[("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0")),("tree_3_a","bo_tree_3_a",("0","0")),("tree_3_b","bo_tree_3_b",("0","0")),("tree_4_a","bo_tree_4_a",("0","0")),("tree_4_b","bo_tree_4_b",("0","0")),("tree_5_a","bo_tree_5_a",("0","0")),("tree_5_b","bo_tree_5_b",("0","0")),("tree_5_c","bo_tree_5_c",("0","0")),("tree_5_d","bo_tree_5_d",("0","0")),("tree_6_a","bo_tree_6_a",("0","0")),("tree_6_b","bo_tree_6_b",("0","0")),("tree_6_c","bo_tree_6_c",("0","0")),("tree_6_d","bo_tree_6_d",("0","0")),("tree_7_a","bo_tree_7_a",("0","0")),("tree_7_b","bo_tree_7_b",("0","0")),("tree_7_c","bo_tree_7_c",("0","0")),("tree_8_a","bo_tree_8_a",("0","0")),("tree_8_b","bo_tree_8_b",("0","0")),("tree_8_c","bo_tree_8_c",("0","0")),("tree_9_a","bo_tree_9_a",("0","0")),("tree_9_b","bo_tree_9_a",("0","0")),("tree_9_c","bo_tree_9_a",("0","0")),("tree_10_a","bo_tree_10_a",("0","0")),("tree_10_b","bo_tree_10_a",("0","0")),("tree_10_c","bo_tree_10_a",("0","0")),("tree_11_a","bo_tree_11_a",("0","0")),("tree_11_b","bo_tree_11_a",("0","0")),("tree_11_c","bo_tree_11_a",("0","0")),("tree_12_a","bo_tree_12_a",("0","0")),("tree_12_b","bo_tree_12_b",("0","0")),("tree_12_c","bo_tree_12_c",("0","0")),("tree_14_a","bo_tree_14_a",("0","0")),("tree_14_b","bo_tree_14_b",("0","0")),("tree_14_c","bo_tree_14_c",("0","0")),("tree_15_a","bo_tree_15_a",("0","0")),("tree_15_b","bo_tree_15_b",("0","0")),("tree_15_c","bo_tree_15_c",("0","0")),("tree_16_a","bo_tree_16_a",("0","0")),("tree_16_b","bo_tree_16_b",("0","0")),("tree_17_a","bo_tree_17_a",("0","0")),("tree_17_b","bo_tree_17_b",("0","0")),("tree_17_c","bo_tree_17_c",("0","0")),("tree_17_d","bo_tree_17_d",("0","0"))]),
	
	##########Random procedural junk
	#Village huts / houses.  Creates a nice random atmosphere of peopled relics. Numbers have to be low, to avoid pathing problems.
	("grass_plains_forest_houses",fkf_plain_forest|fkf_tree|fkf_on_green_ground|density(2),[("village_house_a","bo_village_house_a",("0","0")),("village_house_b","bo_village_house_b",("0","0")),("village_house_c","bo_village_house_c",("0","0")),("village_house_d","bo_village_house_d",("0","0")),("village_hut_a","bo_village_hut_a",("0","0")),]),

	#Stumps and fallen trees, with a few regular trees sprinkled in.  Creates a nice atmosphere.
	("grass_plains_forest_stumps_and_trees",fkf_plain_forest|fkf_tree|fkf_on_green_ground|fkf_align_with_ground|density(45),[("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0")),("tree_3_a","bo_tree_3_a",("0","0")),("tree_3_b","bo_tree_3_b",("0","0")),("destroy_tree_a","bo_destroy_tree_a",("0","0")),("destroy_tree_b","bo_destroy_tree_b",("0","0")),("tree_stump_a","bo_tree_stump_a",("0","0")),("tree_stump_b","bo_tree_stump_b",("0","0")),("tree_stump_c","bo_tree_stump_c",("0","0")),("tree_stump_a","bo_tree_stump_a",("0","0")),("tree_stump_b","bo_tree_stump_b",("0","0")),("tree_stump_c","bo_tree_stump_c",("0","0")),]),		

	#Stumps and rocks, because lumpy battlefields are occasionally fun.
	("grass_plains_forest_stumps_and_rocks",fkf_plain_forest|fkf_rock|fkf_on_green_ground|fkf_align_with_ground|density(45),[("llew_rock_boulder_1", "bo_llew_rock_boulder_1"),("llew_rock_boulder_2", "bo_llew_rock_boulder_2"),("llew_rock_boulder_3", "bo_llew_rock_boulder_2"),("llew_rock_boulder_4", "bo_llew_rock_boulder_4"),("llew_rock_boulder_5", "bo_llew_rock_boulder_5"),("llew_rock_boulder_6", "bo_llew_rock_boulder_6"),("llew_rock_flat_1", "bo_llew_rock_flat_1"),("llew_rock_flat_2", "bo_llew_rock_flat_2"),("llew_rock_flat_3", "bo_llew_rock_flat_3"),("llew_rock_flat_4", "bo_llew_rock_flat_4"),("llew_rock_flat_5", "bo_llew_rock_flat_5"),("llew_rock_flat_6", "bo_llew_rock_flat_6"),("destroy_tree_a","bo_destroy_tree_a"),("destroy_tree_b","bo_destroy_tree_b"),("tree_stump_a","bo_tree_stump_a"),("tree_stump_b","bo_tree_stump_b"),("tree_stump_c","bo_tree_stump_c"),("tree_stump_a","bo_tree_stump_a"),("tree_stump_b","bo_tree_stump_b"),("tree_stump_c","bo_tree_stump_c")]),	
	
	#end PLAINS FOREST
	#############################################################   	

 	#############################################################
	#STEPPE AND STEPPE FOREST
	#Significantly higher grass counts, different weed mix.
	#Lower overall tree mix, more pines, fewer deciduous trees, more rocks.

	##########Grass
	("grass_steppe",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_guarantee|fkf_align_with_ground|fkf_steppe|fkf_steppe_forest|density(1200),[("001xenoargh_grass_yellow_a","0"),("001xenoargh_grass_yellow_b","0"),("001xenoargh_grass_yellow_c","0"),("001xenoargh_grass_yellow_d","0"),("001xenoargh_grass_yellow_e","0"),("001xenoargh_grass_yellow_a2","0"),("001xenoargh_grass_yellow_b2","0"),("001xenoargh_grass_yellow_c2","0"),("001xenoargh_grass_yellow_d2","0"),("001xenoargh_grass_yellow_e2","0")]),	
	
	("grass_steppe_yellow",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_steppe|fkf_steppe_forest|density(1200),[("001xenoargh_grass_yellowgreen_a","0"),("001xenoargh_grass_yellowgreen_b","0"),("001xenoargh_grass_yellowgreen_c","0"),("001xenoargh_grass_yellowgreen_d","0"),("001xenoargh_grass_yellowgreen_e","0"),("001xenoargh_grass_yellowgreen_a2","0"),("001xenoargh_grass_yellowgreen_b2","0"),("001xenoargh_grass_yellowgreen_c2","0"),("001xenoargh_grass_yellowgreen_d2","0"),("001xenoargh_grass_yellowgreen_e2","0")]),
	
	("grass_steppe_green",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_steppe|fkf_steppe_forest|density(600),[("001xenoargh_grass_yellow_a","0"),("001xenoargh_grass_yellow_b","0"),("001xenoargh_grass_yellow_c","0"),("001xenoargh_grass_yellow_d","0"),("001xenoargh_grass_yellow_e","0"),("001xenoargh_grass_yellow_a2","0"),("001xenoargh_grass_yellow_b2","0"),("001xenoargh_grass_yellow_c2","0"),("001xenoargh_grass_yellow_d2","0"),("001xenoargh_grass_yellow_e2","0")]),	
	
	("grass_steppe_greener",fkf_grass|fkf_on_green_ground|fkf_point_up|fkf_align_with_ground|fkf_steppe|fkf_steppe_forest|density(600),[("001xenoargh_grass_a","0"),("001xenoargh_grass_b","0"),("001xenoargh_grass_c","0"),("001xenoargh_grass_d","0"),("001xenoargh_grass_e","0"),("001xenoargh_grass_a2","0"),("001xenoargh_grass_b2","0"),("001xenoargh_grass_c2","0"),("001xenoargh_grass_d2","0"),("001xenoargh_grass_e2","0")]),	

	("grass_steppe_weedmix01",fkf_grass|fkf_point_up|fkf_align_with_ground|fkf_steppe|fkf_steppe_forest|density(800),[("xenoargh_grass_bush001","0"),("xenoargh_grass_bush002","0"),("xenoargh_grass_bush003","0"),("xenoargh_grass_bush004","0"),("xenoargh_grass_bush005","0"),("xenoargh_grass_bush006","0")]),

	("grass_steppe_weedmix02",fkf_point_up|fkf_align_with_ground|fkf_steppe|density(800),[("thorn_a","0"),("thorn_b","0"),("thorn_c","0"),("thorn_d","0"),("basak","0"),("common_plant","0"),("small_plant","0"),("small_plant_b","0"),("small_plant_c","0"),("buddy_plant","0"),("buddy_plant_b","0"),("spiky_plant","0"),("seedy_plant_a","0"),("blue_flower","0"),("big_bush","0")]),

	##########Bushes
	("grass_steppe_bushmix01",fkf_steppe|fkf_steppe_forest|density(200),[("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

#Used on_green_ground for this mix, because of the small trees (bushes02_a).
	("grass_steppe_bushmix02",fkf_steppe|fkf_steppe_forest|fkf_on_green_ground|density(450),[("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

#Used on_green_ground for this mix, because of the small trees (bushes02_a).
	("grass_steppe_bushmix03",fkf_steppe|fkf_steppe_forest|fkf_on_green_ground|density(200),[("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0"),("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0")]),			

	("grass_steppe_bushmix04",fkf_steppe|fkf_steppe_forest|density(200),[("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0"),("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0"),("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),	

	##########Trees
	#Pine-only forest.  Higher density.
	("grass_steppe_treemix01",fkf_steppe|fkf_steppe_forest|fkf_tree|fkf_on_green_ground|density(20),[("pine_4_a","bo_pine_4_a",("0","0")),("pine_3_a","bo_pine_3_a",("0","0")),("pine_3_a","bo_pine_3_a",("0","0")),("pine_2_a","bo_pine_2_a",("0","0")),("pine_2_a","bo_pine_2_a",("0","0")),("pine_1_a","bo_pine_1_a",("0","0")),("pine_1_b","bo_pine_1_b",("0","0")),("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0"))]),

	#Dead bushy things- pretty cheap, makes for a scruffy atmosphere.
	#NOTE:  Not using fkf_tree here- I wanted less spacing and higher counts.
	("grass_steppe_treemix02",fkf_steppe|fkf_steppe_forest|density(1250),[("bush_dead_a","0"),("bush_dead_b","0"),("bush_dead_c","0")]),	
	
	#Pines with some deciduous trees.
	("grass_steppe_treemix03",fkf_steppe|fkf_steppe_forest|fkf_tree|fkf_on_green_ground|density(30),[("pine_4_a","bo_pine_4_a",("0","0")),("pine_3_a","bo_pine_3_a",("0","0")),("pine_3_a","bo_pine_3_a",("0","0")),("pine_2_a","bo_pine_2_a",("0","0")),("pine_2_a","bo_pine_2_a",("0","0")),("pine_1_a","bo_pine_1_a",("0","0")),("pine_1_b","bo_pine_1_b",("0","0")),("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0")),("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0")),("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0")),("tree_1_a","bo_tree_1_a",("0","0")),("tree_1_b","bo_tree_1_b",("0","0")),("tree_14_a","bo_tree_14_a",("0","0")),("tree_14_b","bo_tree_14_b",("0","0")),("tree_14_c","bo_tree_14_c",("0","0")),("tree_15_a","bo_tree_15_a",("0","0")),("tree_15_b","bo_tree_15_b",("0","0")),("tree_15_c","bo_tree_15_c",("0","0")),("tree_16_a","bo_tree_16_a",("0","0")),("tree_16_b","bo_tree_16_b",("0","0")),("tree_17_a","bo_tree_17_a",("0","0")),("tree_17_b","bo_tree_17_b",("0","0")),("tree_17_c","bo_tree_17_c",("0","0")),("tree_17_d","bo_tree_17_d",("0","0"))]),
	
	##########Random procedural junk
	#Village huts / houses.  Creates a nice random atmosphere of peopled relics. Numbers have to be low, to avoid pathing problems.
	("grass_steppe_houses",fkf_steppe|fkf_steppe_forest|fkf_tree|fkf_on_green_ground|density(1),[("village_house_a","bo_village_house_a",("0","0")),("village_house_b","bo_village_house_b",("0","0")),("village_house_c","bo_village_house_c",("0","0")),("village_house_d","bo_village_house_d",("0","0")),("village_hut_a","bo_village_hut_a",("0","0")),]),

	#Stumps and fallen trees, with a few regular trees sprinkled in.  Creates a nice atmosphere.
	("grass_steppe_stumps_and_trees",fkf_steppe|fkf_steppe_forest|fkf_tree|fkf_align_with_ground|fkf_on_green_ground|density(35),[("destroy_tree_a","bo_destroy_tree_a",("0","0")),("destroy_tree_b","bo_destroy_tree_b",("0","0")),("tree_stump_a","bo_tree_stump_a",("0","0")),("tree_stump_b","bo_tree_stump_b",("0","0")),("tree_stump_c","bo_tree_stump_c",("0","0")),("tree_stump_a","bo_tree_stump_a",("0","0")),("tree_stump_b","bo_tree_stump_b",("0","0")),("tree_stump_c","bo_tree_stump_c",("0","0")),]),		

	#Stumps and rocks, because lumpy battlefields are occasionally fun.  Used align_with_ground here; makes a difference.
	("grass_steppe_stumps_and_rocks",fkf_steppe|fkf_steppe_forest|fkf_rock|fkf_align_with_ground|fkf_on_green_ground|density(65),[("llew_rock_boulder_1", "bo_llew_rock_boulder_1"),("llew_rock_boulder_2", "bo_llew_rock_boulder_2"),("llew_rock_boulder_3", "bo_llew_rock_boulder_3"),("llew_rock_boulder_4", "bo_llew_rock_boulder_4"),("llew_rock_boulder_5", "bo_llew_rock_boulder_5"),("llew_rock_boulder_6", "bo_llew_rock_boulder_6"),("llew_rock_flat_1", "bo_llew_rock_flat_1"),("llew_rock_flat_2", "bo_llew_rock_flat_2"),("llew_rock_flat_3", "bo_llew_rock_flat_3"),("llew_rock_flat_4", "bo_llew_rock_flat_4"),("llew_rock_flat_5", "bo_llew_rock_flat_5"),("llew_rock_flat_6", "bo_llew_rock_flat_6"),("destroy_tree_a","bo_destroy_tree_a"),("destroy_tree_b","bo_destroy_tree_b"),("tree_stump_a","bo_tree_stump_a"),("tree_stump_b","bo_tree_stump_b"),("tree_stump_c","bo_tree_stump_c"),("tree_stump_a","bo_tree_stump_a"),("tree_stump_b","bo_tree_stump_b"),("tree_stump_c","bo_tree_stump_c")]),	
	
	#end STEPPE / STEPPE FOREST
	############################################################# 

 	#############################################################
	#SNOW AND SNOW FOREST
	#Well, it's snow- boring as hell snow, by default.
	#This adds some variety, but not much, to snow. Some grass, because there's grass under snow.  The usual snowy trees and snowy rocks.

	##########Grass
	("grass_snow",fkf_grass|fkf_point_up|fkf_guarantee|fkf_align_with_ground|fkf_snow|fkf_snow_forest|density(100),[("001xenoargh_grass_yellow_a","0"),("001xenoargh_grass_yellow_b","0"),("001xenoargh_grass_yellow_c","0"),("001xenoargh_grass_yellow_d","0"),("001xenoargh_grass_yellow_e","0"),("001xenoargh_grass_yellow_a2","0"),("001xenoargh_grass_yellow_b2","0"),("001xenoargh_grass_yellow_c2","0"),("001xenoargh_grass_yellow_d2","0"),("001xenoargh_grass_yellow_e2","0")]),	

	##########Bushes
	("grass_snow_bushmix01",fkf_snow|fkf_snow_forest|density(30),[("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

#Used on_green_ground for this mix, because of the small trees (bushes02_a).
	("grass_snow_bushmix02",fkf_snow|fkf_snow_forest|density(30),[("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),

#Used on_green_ground for this mix, because of the small trees (bushes02_a).
	("grass_snow_bushmix03",fkf_snow|fkf_snow_forest|density(30),[("bushes05_a","0"),("bushes05_b","0"),("bushes05_c","0"),("bushes06_a","0"),("bushes06_b","0"),("bushes06_c","0")]),			

	("grass_snow_bushmix04",fkf_snow|fkf_snow_forest|density(30),[("bushes07_a","0"),("bushes07_b","0"),("bushes07_c","0"),("bushes08_a","0"),("bushes08_b","0"),("bushes08_c","0"),("bushes09_a","0"),("bushes09_b","0"),("bushes09_c","0"),("bushes10_a","0"),("bushes10_b","0"),("bushes10_c","0"),("bushes11_a","0"),("bushes11_b","0"),("bushes11_c","0"),("bushes12_a","0"),("bushes12_b","0"),("bushes12_c","0")]),	

	##########Trees
	#Pine-only forest.  Higher density.
	("grass_snow_treemix01",fkf_snow|fkf_snow_forest|fkf_guarantee|fkf_tree|density(35),[("tree_snowy_a","bo_tree_snowy_a",("0","0")),("tree_snowy_a","bo_tree_snowy_a",("0","0")),("snowy_pine_2","bo_snowy_pine_2",("0","0")),("snowy_pine_3","bo_snowy_pine_3",("0","0"))]),

	#Dead bushy things- pretty cheap, makes for a scruffy atmosphere.
	#NOTE:  Not using fkf_tree here- I wanted less spacing and higher counts.  Uses fkf_rock, because snow uses funny gen code.
	("grass_snow_treemix02",fkf_snow|fkf_snow_forest|fkf_guarantee|density(300),[("bush_dead_a","0"),("bush_dead_b","0"),("bush_dead_c","0")]),	

	
	##########Random procedural junk
	#Village huts / houses.  Creates a nice random atmosphere of peopled relics. Numbers have to be low, to avoid pathing problems.
	("grass_snow_houses",fkf_snow|fkf_snow_forest|fkf_tree|density(1),[("village_house_a","bo_village_house_a",("0","0")),("village_house_b","bo_village_house_b",("0","0")),("village_house_c","bo_village_house_c",("0","0")),("village_house_d","bo_village_house_d",("0","0")),("village_hut_a","bo_village_hut_a",("0","0")),]),


	#Stumps and rocks, because lumpy battlefields are occasionally fun.  Used align_with_ground here; makes a difference.
	("grass_snow_stumps_and_rocks",fkf_snow|fkf_snow_forest|fkf_rock|fkf_align_with_ground|density(65),[("destroy_tree_a","bo_destroy_tree_a"),("destroy_tree_b","bo_destroy_tree_b"),("tree_stump_a","bo_tree_stump_a"),("tree_stump_b","bo_tree_stump_b"),("tree_stump_c","bo_tree_stump_c"),("tree_stump_a","bo_tree_stump_a"),("tree_stump_b","bo_tree_stump_b"),("tree_stump_c","bo_tree_stump_c")]),	
	
	#end SNOW / SNOW FOREST
	############################################################# 

 	#############################################################
	#DESERT AND DESERT FOREST
	#Well, it's the desert.  Even less interesting than snow, frankly.
	#This adds some variety.  Rocks, the occasional hut.  That kind of stuff.

	##########Grass
	#Just kidding.  It's a desert, silly.

	##########Bushes
	#Desert, remember?

	##########Trees
	#Palm trees.  Because it's what we've got.  Higher density.  Note that fkf_tree is off, because we don't want the engine cull it out like it usually does.
	("grass_desert_treemix01",fkf_desert|fkf_desert_forest|density(6),[("palm_a","bo_palm_a"),("xenoargh_palm_tree01","bo_xenoargh_palm_tree01"),("xenoargh_palm_tree02","bo_xenoargh_palm_tree02"),("xenoargh_palm_tree03","bo_xenoargh_palm_tree03")]),
	#...and another one, to be compatible with engine weirdness...
	("grass_desert_treemix02",fkf_desert_forest|fkf_tree|density(4),[("palm_a","bo_palm_a",("0","0")),("xenoargh_palm_tree01","bo_xenoargh_palm_tree01",("0","0")),("xenoargh_palm_tree02","bo_xenoargh_palm_tree02",("0","0")),("xenoargh_palm_tree03","bo_xenoargh_palm_tree03",("0","0"))]),

	##########Random procedural junk
	#Rocks, because lumpy battlefields are occasionally fun.  Used align_with_ground here; makes a difference.
	("grass_desert_stumps_and_rocks",fkf_desert|fkf_desert_forest|fkf_align_with_ground|density(3),[("llew_rock_boulder_1", "bo_llew_rock_boulder_1"),("llew_rock_boulder_2", "bo_llew_rock_boulder_2"),("llew_rock_boulder_3", "bo_llew_rock_boulder_3"),("llew_rock_boulder_4", "bo_llew_rock_boulder_4"),("llew_rock_boulder_5", "bo_llew_rock_boulder_5"),("llew_rock_boulder_6", "bo_llew_rock_boulder_6"),("llew_rock_flat_1", "bo_llew_rock_flat_1"),("llew_rock_flat_2", "bo_llew_rock_flat_2"),("llew_rock_flat_3", "bo_llew_rock_flat_3"),("llew_rock_flat_4", "bo_llew_rock_flat_4"),("llew_rock_flat_5", "bo_llew_rock_flat_5"),("llew_rock_flat_6", "bo_llew_rock_flat_6")]),	
	
	#end DESERT / DESERT FOREST
	#############################################################   	
	
]


def save_fauna_kinds():
  file = open(export_dir + "/data/flora_kinds.txt","w")
  file.write("%d\n"%len(fauna_kinds))
  for fauna_kind in fauna_kinds:
    meshes_list = fauna_kind[2]
    file.write("%s %d %d\n"%(fauna_kind[0], (dword_mask & fauna_kind[1]), len(meshes_list)))
    for m in meshes_list:
      file.write(" %s "%(m[0]))
      if (len(m) > 1):
        file.write(" %s\n"%(m[1]))
      else:
        file.write(" 0\n")
      if ( fauna_kind[1] & (fkf_tree|fkf_speedtree) ):  #if this fails make sure that you have entered the alternative tree definition (NOT FUNCTIONAL in Warband)
        speedtree_alternative = m[2]
        file.write(" %s %s\n"%(speedtree_alternative[0], speedtree_alternative[1]))
    if ( fauna_kind[1] & fkf_has_colony_props ):
      file.write(" %s %s\n"%(fauna_kind[3], fauna_kind[4]))
  file.close()

def two_to_pow(x):
  result = 1
  for i in xrange(x):
    result = result * 2
  return result

fauna_mask = 0x80000000000000000000000000000000
low_fauna_mask =             0x8000000000000000
def save_python_header():
  file = open("./fauna_codes.py","w")
  for i_fauna_kind in xrange(len(fauna_kinds)):
    file.write("%s_1 = 0x"%(fauna_kinds[i_fauna_kind][0]))
    file.write("%x\n"%(fauna_mask | two_to_pow(i_fauna_kind)))
    file.write("%s_2 = 0x"%(fauna_kinds[i_fauna_kind][0]))
    file.write("%x\n"%(fauna_mask | ((low_fauna_mask|two_to_pow(i_fauna_kind)) << 64)))
    file.write("%s_3 = 0x"%(fauna_kinds[i_fauna_kind][0]))
    file.write("%x\n"%(fauna_mask | ((low_fauna_mask|two_to_pow(i_fauna_kind)) << 64) | two_to_pow(i_fauna_kind)))
  file.close()

print "Exporting flora data..."
save_fauna_kinds()
