from header_factions import *

####################################################################################################################
#  Each faction record contains the following fields:
#  1) Faction id: used for referencing factions in other files.
#     The prefix fac_ is automatically added before each faction id.
#  2) Faction name.
#  3) Faction flags. See header_factions.py for a list of available flags
#  4) Faction coherence. Relation between members of this faction.
#  5) Relations. This is a list of relation records.
#     Each relation record is a tuple that contains the following fields:
#    5.1) Faction. Which other faction this relation is referring to
#    5.2) Value: Relation value between the two factions.
#         Values range between -1 and 1.
#  6) Ranks
#  7) Faction color (default is gray)
####################################################################################################################

default_kingdom_relations = [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.05),("mountain_bandits", -0.02),("forest_bandits", -0.02)]
factions = [
  ("no_faction","No Faction",0, 0.9, [], []),
  ("commoners","Commoners",0, 0.1,[("player_faction",0.1)], []),
  ("outlaws","Outlaws", max_player_rating(-30), 0.5,[("commoners",-0.6),("player_faction",-0.15),("mountain_bandits", -0.05),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05)], [], 0x888888),
# Factions before this point are hardwired into the game end their order should not be changed.

  ("neutral","Neutral",0, 0.1,[("player_faction",0.0)], [],0xFFFFFF),
  ("innocents","Innocents", ff_always_hide_label, 0.5,[("outlaws",-0.05)], []),
  ("merchants","Merchants", ff_always_hide_label, 0.5,[("outlaws",-0.5),], []),

  ("dark_knights","{!}Dark Knights", 0, 0.5,[("innocents",-0.9),("player_faction",-0.4)], []),

  ("culture_of_swadians",  "{!}culture_of_swadians", 0, 0.9, [], []),
  ("culture_of_vaegirs",  "{!}culture_of_vaegirs", 0, 0.9, [], []),
  ("culture_of_khergits",  "{!}culture_of_khergits", 0, 0.9, [], []),
  ("culture_of_nords",  "{!}culture_of_nords", 0, 0.9, [], []),
  ("culture_of_rhodoks",  "{!}culture_of_rhodoks", 0, 0.9, [], []),
  ("culture_of_sarranids",  "{!}culture_of_sarranids", 0, 0.9, [], []),
  ("culture_of_death_knights", "{!}culture_of_death_knights", 0, 0.9, [], []),
  ("culture_of_templars", "{!}culture_of_templars", 0, 0.9, [], []),

#  ("swadian_caravans","Swadian Caravans", 0, 0.5,[("outlaws",-0.8), ("dark_knights",-0.2)], []),
#  ("vaegir_caravans","Vaegir Caravans", 0, 0.5,[("outlaws",-0.8), ("dark_knights",-0.2)], []),

  ("player_faction","Player Faction",0, 0.9, [], []),
  ("player_supporters_faction","Player's Supporters",0, 0.9, [("player_faction",1.00),("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05)], [], 0xFF4433), #changed name so that can tell difference if shows up on map
  
  #NEW FACTION ALLIANCES
  #Each of the existing Factions now has multiple relationships with the Bandits...
  #...and Deserters of other Factions.  Some are allied with one another, some hate each other.
  #All Factions now have a liking for the Manhunters, as they are general-purpose defenders of the... 
  #...realms against bandit incursions.
  #In addition, all Factions now have explicit relationships with the Death Knights (i.e., bad)...
  #...and the Holy Army (good).
  
  #The new Factions are as follows:
  #Kingdoms of Swadians, Vaegirs, Khergits, Sarranids, Nords and Rhodoks.
  #Kingdoms of the Holy See, Death Knights.
  #Forest, Desert, Tundra, Sea, Mountain and Steppe Bandits (these are rebels).
  #General Bandit group, not categorized.
  #Manhunters, liked by all of the non-bandits and hated by everybody else.
  #Slavers, hate all of the bandits but are neutral to everybody else.
  #Merchants and other neutrals, generally hated by the bandits and are liked by everybody else.
  
  
  #SWADIANS
  ("kingdom_of_swadians",  "Kingdom of Swadia", 0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05),("death_knights", -1.0),("templars", 1.0)], [], 0xEE7744),
  
  ("kingdom_of_vaegirs",  "Kingdom of Vaegirs",    0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05),("death_knights", -1.0),("templars", 1.0)], [], 0xA52A2A),
  
  ("kingdom_of_khergits",  "Khergit Khanate", 0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05),("death_knights", -1.0),("templars", 1.0)], [], 0xCC99FF),
  
  ("kingdom_of_nords",  "Kingdom of Nords",    0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05),("death_knights", -1.0),("templars", 1.0)], [], 0x33DDDD),

  ("kingdom_of_rhodoks",  "Kingdom of Rhodoks",  0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05),("death_knights", -1.0),("templars", 1.0)], [], 0x33DD33),
  
  ("kingdom_of_sarranids",  "Sarranid Sultanate",  0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05),("death_knights", -1.0),("templars", 1.0)], [], 0xDDDD33),



  ("kingdoms_end","{!}kingdoms_end", 0, 0,[], []),

  ("robber_knights",  "{!}robber_knights", 0, 0.1, [], []),

  ("khergits","{!}Khergits", 0, 0.5,[("player_faction",0.0)], []),
  ("black_khergits","{!}Black Khergits", 0, 0.5,[("player_faction",-0.3),("kingdom_of_swadians",-0.02),("kingdom_of_vaegirs",-0.02)], []),
  
  ("death_knights","The Army of Darkness", max_player_rating(-30), 1.0,[("player_faction",-1),("player_supporters_faction",-1),("templars",-1)], [], 0xDC143C),
  
  ("templars","The Army of Light", max_player_rating(-30), 1.0,[("death_knights",-1)], [], 0x6495ED),

##  ("rebel_peasants","Rebel Peasants", 0, 0.5,[("vaegirs",-0.5),("player_faction",0.0)], []),

  ("manhunters","Manhunters", 0, 0.5,[("outlaws",-0.6),("player_faction",0.1)], []),
  ("deserters","Deserters", 0, 0.5,[("manhunters",-0.6),("merchants",-0.5),("player_faction",-0.0),("outlaws",-0.1),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05)], [], 0x888888),
  
  ("mountain_bandits","Mountain Bandits", 0, 0.5,[("commoners",-0.2),("merchants",-0.5),("manhunters",-0.6),("player_faction",-0.15),("outlaws",-0.1),("forest_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05)], [], 0x888888),
  
  ("forest_bandits","Forest Bandits", 0, 0.5,[("commoners",-0.2),("merchants",-0.5),("manhunters",-0.6),("player_faction",-0.15),("outlaws",-0.1),("mountain_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05)], [], 0x888888),
  
   ("desert_bandits","Desert Bandits", 0, 0.5,[("commoners",-0.2),("merchants",-0.5),("manhunters",-0.6),("player_faction",-0.15),("outlaws",-0.1),("mountain_bandits", -0.05),("forest_bandits", -0.05),("taiga_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05)], [], 0x888888),

  ("taiga_bandits","Taiga Bandits", 0, 0.5,[("commoners",-0.2),("merchants",-0.5),("manhunters",-0.6),("player_faction",-0.15),("outlaws",-0.1),("mountain_bandits", -0.05),("desert_bandits", -0.05),("forest_bandits", -0.05),("steppe_bandits", -0.05),("sea_raiders", -0.05)], [], 0x888888),

  ("steppe_bandits","Steppe Bandits", 0, 0.5,[("commoners",-0.2),("merchants",-0.5),("manhunters",-0.6),("player_faction",-0.15),("outlaws",-0.1),("mountain_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("forest_bandits", -0.05),("sea_raiders", -0.05)], [], 0x888888), 

  ("sea_raiders","Sea Raiders", 0, 0.5,[("commoners",-0.2),("merchants",-0.5),("manhunters",-0.6),("player_faction",-0.15),("outlaws",-0.1),("mountain_bandits", -0.05),("desert_bandits", -0.05),("taiga_bandits", -0.05),("forest_bandits", -0.05),("steppe_bandits", -0.05)], [], 0x888888),   

  ("undeads","{!}Undeads", max_player_rating(-30), 0.5,[("commoners",-0.7),("player_faction",-0.5)], []),
  ("slavers","{!}Slavers", 0, 0.1, [], []),
  ("peasant_rebels","{!}Peasant Rebels", 0, 1.0,[("noble_refugees",-1.0),("player_faction",-0.4)], []),
  ("noble_refugees","{!}Noble Refugees", 0, 0.5,[], []),
  
  
]
