from header_common import *
from header_skills import *

####################################################################################################################
#  Each skill contains the following fields:
#  1) Skill id (string): used for referencing skills in other files. The prefix skl_ is automatically added before each skill-id .
#  2) Skill name (string).
#  3) Skill flags (int). See header_skills.py for a list of available flags
#  4) Maximum level of the skill (int).
#  5) Skill description (string): used in character window for explaining the skills.
# 
####################################################################################################################

#Hardcoded skills are {names (indexes, beginning with 0)}:
# Trade (1)
# Leadership (2)
# Prisoner Management (3)
# First Aid (9)
# Surgery (10)
# Wound Treatment (11)
# Inventory Management (12)
# Spotting (13)
# Pathfinding (14)
# Tactics (15)
# Tracking (16)
# Trainer (17)
# Engineer (18)
# Horse Archery (24)
# Riding (25)
# Athletics (26)
# Shield (27)
# Weapon Master (28)
# Power Draw (34)
# Power Throw (35)
# Power Strike (36)
# Ironflesh (37)
#
# The effects of these skills can only be removed if the skill is disabled with sf_inactive flag.
# If you want to add a new skill, use the reserved skills or use non-hardcoded skills.

skills = [
  ("trade","Trade",sf_base_att_cha|sf_effects_party,10,"Every level of this skill reduces your trade penalty by 5%%, and has other benefits throughout the game, such as better prices for prisoners.  A very valuable skill for high-level players and trader characters. (Party skill)"),
  ("leadership","Leadership",sf_base_att_int,10,"Every point increases maximum number of troops you can command by 50, increases your party morale and reduces troop wages by 8%%. This is a vital skill if you want to maintain large armies.  (Leader skill)"),
  ("prisoner_management", "Prisoner Management",sf_base_att_cha|sf_effects_party,10,"Every level of this skill increases maximum number of prisoners this party can hold by 15. (Party skill)"), 
  ("combat_training","Combat Training",sf_base_att_int,10,"This skill represents your training (as opposed to your natural talent or lessons learned from using your weapons).  Each level gives you a 3%% chance to get a critical hit, which deals 3X normal damage. (Personal skill)"), 
  ("reserved_2","Reserved Skill 2",sf_base_att_cha|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_3","Reserved Skill 3",sf_base_att_cha|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_4","Reserved Skill 4",sf_base_att_cha|sf_inactive,10,"This is a reserved skill."), 
  ("persuasion","Persuasion", sf_base_att_cha|sf_effects_party,10, "This skill helps you make other people accept your point of view, and helps you get better prices, both buying and selling. It's potentially a very useful skill for a long game.  (Party skill)"),
  ("engineer","Engineer",sf_base_att_int|sf_effects_party,10,"This skill allows you to construct siege equipment and fief improvements more efficiently. (Party skill)"),
  ("first_aid", "First Aid",sf_base_att_int|sf_effects_party,10,"Heroes regain 5%% per skill level of hit-points lost during mission. (Party skill)"), 
  ("surgery","Surgery",sf_base_att_int|sf_effects_party,10,"Each point to this skill gives a 4%% chance that a mortally struck party member will be wounded rather than killed. (Party skill)"), 
  ("wound_treatment","Wound Treatment",sf_base_att_int|sf_effects_party,10,"Party healing speed is increased by 20%% per level of this skill. (Party skill)"), 
  #Inventory management isn't allowed to be a Party skill.  Since it's a fairly useless drain on points otherwise... it's automatically set to 10.
  ("inventory_management","Inventory Management",sf_base_att_int,10,"Increases inventory capacity by +6 per skill level. (Personal skill)"), 
  ("spotting","Spotting",sf_base_att_int|sf_effects_party,10,"Party seeing range is increased by 10%% per skill level. (Party skill)"),
  ("pathfinding","Path-finding",sf_base_att_int|sf_effects_party,10,"Party map speed is increased by 3%% per skill level. Vital for moving large armies around.  (Party skill)"), 
  ("heroism","Heroism",sf_base_att_cha|sf_effects_party,15,"Each point increases all damage dealt by attacks by 1 for non-heroes, 2 for heroes.  It also increases your Battle Advantage a little bit. (Party skill)"),
  ("tracking","Tracking",sf_base_att_int|sf_effects_party,10,"Tracks become more informative. (Party skill)"),
  ("trainer","Trainer",sf_base_att_int|sf_effects_party,10,"Every day, each hero with this skill adds some experience to every other member of the party whose level is lower than his/hers. Experience gained at each level is: {0,4,10,16,23,30,38,46,55,65,80}. (Party skill)"),
  ("reserved_5","Reserved Skill 5",sf_base_att_int|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_6","Reserved Skill 6",sf_base_att_int|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_7","Reserved Skill 7",sf_base_att_int|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_8","Reserved Skill 8",sf_base_att_int|sf_inactive,10,"This is a reserved skill."), 
  ("looting","Looting",sf_base_att_agi|sf_effects_party,10,"This skill increases the amount of loot obtained by 10%% per skill level. (Party skill)"), 
  ("horse_archery","Horse Archery",sf_base_att_agi,10,"Reduces damage and accuracy penalties for archery and throwing from horseback. (Personal skill)"),
  ("riding","Riding",sf_base_att_agi,10,"Enables you to ride horses of higher difficulty levels and increases your riding speed and manuever. (Personal skill)"),
  ("reflexes","Reflexes",sf_base_att_agi,10,"Improves your running speed. Gives you a 5%% chance to dodge a blow and take little or no damage (3%% for non-heroes).  Very useful skill for all classes.  (Personal skill)"),
  ("shield"," ",sf_base_att_agi|sf_inactive,0," "),
  ("weapon_master","Weapon Master",sf_base_att_agi,10,"Makes it easier to learn weapon proficiencies and increases the proficiency limits. Limits go as: 60, 100, 140, 180, 220, 260, 300, 340, 380, 420. (Personal skill)"),
  ("reserved_9","Reserved Skill 9",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_10","Reserved Skill 10",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_11","Reserved Skill 11",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_12","Reserved Skill 12",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_13","Reserved Skill 13",sf_base_att_agi|sf_inactive,10,"This is a reserved skill."), 
  ("power_draw"," ",sf_base_att_str|sf_inactive,0," "),
  ("power_throw"," ",sf_base_att_str|sf_inactive,0," "),
  ("power_strike"," ",sf_base_att_str|sf_inactive,0," "),
  ("toughness","Toughness",sf_base_att_str,10,"Each point to this skill increases hit points by +2. Increases your overall toughness- you take 7%% less damage per point (4%% for non-heroes).  Very important skill for HTH fighters.  (Personal skill)"), 
  ("reserved_14","Reserved Skill 14",sf_base_att_str|sf_inactive,10,"This is a reserved skill."), 
  ("reserved_15","Reserved Skill 15",sf_base_att_str|sf_inactive,10,"This is a reserved skill."), 
  ("defensive_tactics","Defensive Tactics",sf_base_att_int|sf_effects_party,15,"Your ability to instruct your troops on the proper use of terrain, weapons and fighting techniques to defend themselves.  Each level lowers damage taken by 1 for non-heroes, 2 for heroes. (party skill)"), 
  ("reserved_17"," ",sf_base_att_str,0," "), 
  ("reserved_18"," ",sf_base_att_str,0," "), 
]
