from module_constants import *
from ID_factions import *
from header_items import  *
from header_operations import *
from header_triggers import *

####################################################################################################################
#  Each item record contains the following fields:
#  1) Item id: used for referencing items in other files.
#     The prefix itm_ is automatically added before each item id.
#  2) Item name. Name of item as it'll appear in inventory window
#  3) List of meshes.  Each mesh record is a tuple containing the following fields:
#    3.1) Mesh name.
#    3.2) Modifier bits that this mesh matches.
#     Note that the first mesh record is the default.
#  4) Item flags. See header_items.py for a list of available flags.
#  5) Item capabilities. Used for which animations this item is used with. See header_items.py for a list of available flags.
#  6) Item value.
#  7) Item stats: Bitwise-or of various stats about the item such as:
#      weight,abundance,difficulty,head_armor,body_armor,leg_armor,etc...
#  8) Modifier bits: Modifiers that can be applied to this item.
#  9) [Optional] Triggers: List of simple triggers to be associated with the item.
#  10) [Optional] Factions: List of factions that item can be found as merchandise.
####################################################################################################################

# Some constants for ease of use.
#Please note that NONE of these categories are used in this mod, in favor of flat balance, until such time as the data directory overrides can be included (and perhaps not then, either).
imodbits_none = 0
imodbits_horse_basic = imodbit_swaybacked|imodbit_lame|imodbit_spirited|imodbit_heavy|imodbit_stubborn
imodbits_cloth  = imodbit_tattered|imodbit_ragged|imodbit_sturdy|imodbit_thick|imodbit_hardened
imodbits_armor  = imodbit_rusty|imodbit_battered|imodbit_crude|imodbit_thick|imodbit_reinforced|imodbit_lordly
imodbits_plate  = imodbit_cracked|imodbit_rusty|imodbit_battered|imodbit_crude|imodbit_thick|imodbit_reinforced|imodbit_lordly
imodbits_polearm = imodbit_cracked|imodbit_bent|imodbit_balanced
imodbits_shield  = imodbit_cracked|imodbit_battered|imodbit_thick|imodbit_reinforced
imodbits_sword   = imodbit_rusty|imodbit_chipped|imodbit_balanced|imodbit_tempered
imodbits_sword_fine   = imodbit_rusty|imodbit_chipped|imodbit_balanced|imodbit_tempered|imodbit_masterwork
imodbits_axe   = imodbit_rusty|imodbit_chipped|imodbit_heavy
imodbits_mace   = imodbit_rusty|imodbit_chipped|imodbit_heavy
imodbits_pick   = imodbit_rusty|imodbit_chipped|imodbit_balanced|imodbit_heavy
imodbits_bow = imodbit_cracked|imodbit_bent|imodbit_strong|imodbit_masterwork
imodbits_crossbow = imodbit_cracked|imodbit_bent|imodbit_strong|imodbit_masterwork
imodbits_firearm = imodbit_cracked|imodbit_bent|imodbit_strong|imodbit_masterwork
imodbits_missile   = imodbit_bent|imodbit_large_bag
imodbits_thrown   = imodbit_bent|imodbit_heavy|imodbit_balanced|imodbit_large_bag
imodbits_thrown_minus_heavy = imodbit_bent|imodbit_balanced|imodbit_large_bag

imodbits_horse_good = imodbit_spirited|imodbit_heavy
imodbits_good   = imodbit_sturdy|imodbit_thick|imodbit_hardened|imodbit_reinforced
imodbits_bad    = imodbit_rusty|imodbit_chipped|imodbit_tattered|imodbit_ragged|imodbit_cracked|imodbit_bent


# Replace winged mace/spiked mace with: Flanged mace / Knobbed mace?
# Fauchard (majowski glaive) 
items = [
# item_name,mesh_name,item_properties,item_capabilities,slot_no,cost,bonus_flags,weapon_flags,scale,view_dir,pos_offset
 ["no_item","INVALID ITEM",[("practice_sword",0)],itp_type_one_handed_wpn|itp_primary|itp_secondary,itc_longsword,3,weight(1.5)|spd_rtng(103)|weapon_length(90)|swing_damage(16,blunt)|thrust_damage(10,blunt),imodbits_none],

 ["tutorial_spear","Spear",[("spear",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_spear,1,weight(4.5)|difficulty(0)|spd_rtng(80)|weapon_length(158)|swing_damage(0,cut)|thrust_damage(19,pierce),imodbits_none],
 ["tutorial_club","Club",[("club",0)],itp_type_one_handed_wpn|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,1,weight(2.5)|difficulty(0)|spd_rtng(95)|weapon_length(95)|swing_damage(11,blunt)|thrust_damage(0,pierce),imodbits_none],
 ["tutorial_battle_axe","Battle Axe",[("battle_ax",0)],itp_type_two_handed_wpn|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry,itc_nodachi|itcf_carry_axe_back,1,weight(5)|difficulty(0)|spd_rtng(88)|weapon_length(108)|swing_damage(27,cut)|thrust_damage(0,pierce),imodbits_none],
 ["tutorial_arrows","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows,itcf_carry_quiver_back,1,weight(3)|abundance(160)|weapon_length(95)|thrust_damage(0,pierce)|max_ammo(20),imodbits_none],
 ["tutorial_bolts","Bolts",[("bolt",0),("flying_missile",ixmesh_flying_ammo),("bolt_bag",ixmesh_carry),("bolt_bag_b",ixmesh_carry|imodbit_large_bag)],itp_type_bolts,itcf_carry_quiver_right_vertical,1,weight(2.25)|abundance(90)|weapon_length(55)|thrust_damage(0,pierce)|max_ammo(18),imodbits_none],
 ["tutorial_short_bow","Short Bow",[("short_bow",0),("short_bow_carry",ixmesh_carry)],itp_type_bow|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,1,weight(1)|difficulty(0)|spd_rtng(98)|shoot_speed(49)|thrust_damage(12,pierce  ),imodbits_none],
 ["tutorial_crossbow","Crossbow",[("crossbow",0)],itp_type_crossbow|itp_primary|itp_two_handed|itp_cant_reload_on_horseback,itcf_shoot_crossbow|itcf_carry_crossbow_back,1,weight(3)|difficulty(0)|spd_rtng(42)|shoot_speed(68)|thrust_damage(32,pierce)|max_ammo(1),imodbits_none],
 ["tutorial_throwing_daggers","Throwing Daggers",[("throwing_dagger",0)],itp_type_thrown|itp_primary,itcf_throw_knife,1,weight(3.5)|difficulty(0)|spd_rtng(102)|shoot_speed(25)|thrust_damage(16,cut)|max_ammo(14)|weapon_length(0),imodbits_none],
 ["tutorial_saddle_horse","Saddle Horse",[("saddle_horse",0)],itp_type_horse,0,0,abundance(90)|body_armor(3)|difficulty(0)|horse_speed(40)|horse_maneuver(38)|horse_charge(8),imodbits_none],
 ["tutorial_shield","Kite Shield",[("shield_kite_a",0)],itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,100,weight(2.5)|hit_points(480)|body_armor(1)|spd_rtng(82)|weapon_length(150),imodbits_none],
 ["tutorial_staff_no_attack","Staff",[("wooden_staff",0)],itp_type_polearm|itp_offset_lance|itp_primary|itp_penalty_with_shield|itp_wooden_parry|itp_wooden_attack,itc_parry_polearm|itcf_carry_sword_back,9,weight(3.5)|spd_rtng(120)|weapon_length(115)|swing_damage(0,blunt)|thrust_damage(0,blunt),imodbits_none],
 ["tutorial_staff","Staff",[("wooden_staff",0)],itp_type_polearm|itp_offset_lance|itp_primary|itp_penalty_with_shield|itp_wooden_parry|itp_wooden_attack,itc_staff|itcf_carry_sword_back,9,weight(3.5)|spd_rtng(120)|weapon_length(115)|swing_damage(16,blunt)|thrust_damage(16,blunt),imodbits_none],
 ["tutorial_sword","Sword",[("long_sword",0),("scab_longsw_a",ixmesh_carry)],itp_type_one_handed_wpn|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,1,weight(1.5)|difficulty(0)|spd_rtng(100)|weapon_length(102)|swing_damage(18,cut)|thrust_damage(15,pierce),imodbits_none],
 ["tutorial_axe","Axe",[("iron_ax",0)],itp_type_two_handed_wpn|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry,itc_nodachi|itcf_carry_axe_back,1,weight(4)|difficulty(0)|spd_rtng(91)|weapon_length(108)|swing_damage(19,cut)|thrust_damage(0,pierce),imodbits_none],

 ["tutorial_dagger","Dagger",[("practice_dagger",0)],itp_type_one_handed_wpn|itp_primary|itp_secondary,itc_longsword,3,weight(1.5)|spd_rtng(103)|weapon_length(40)|swing_damage(16,blunt)|thrust_damage(10,blunt),imodbits_none],


 ["horse_meat","Horse Meat",[("raw_meat",0)],itp_type_goods|itp_consumable|itp_food,0,100,weight(40)|food_quality(30)|max_ammo(40),imodbits_none],
# Items before this point are hardwired and their order should not be changed!
 ["practice_sword","Practice Sword",[("practice_sword",0)],itp_type_one_handed_wpn|itp_primary|itp_secondary|itp_wooden_parry|itp_wooden_attack,itc_longsword,3,weight(1.5)|spd_rtng(135)|weapon_length(90)|swing_damage(35,blunt)|thrust_damage(25,blunt),imodbits_none],
 ["heavy_practice_sword","Heavy Practice Sword",[("heavy_practicesword",0)],itp_type_two_handed_wpn|itp_two_handed|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_greatsword,
    00,weight(6.25)|spd_rtng(90)|weapon_length(128)|swing_damage(55,blunt)|thrust_damage(25,blunt),imodbits_none],
 ["practice_dagger","Practice Dagger",[("practice_dagger",0)],itp_type_one_handed_wpn|itp_primary|itp_secondary|itp_no_parry|itp_wooden_attack,itc_dagger|itcf_carry_dagger_front_left,2,weight(0.5)|spd_rtng(170)|weapon_length(47)|swing_damage(20,blunt)|thrust_damage(15,blunt),imodbits_none],
 ["practice_axe","Practice Axe",[("hatchet",0)],itp_type_one_handed_wpn|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry,itc_scimitar|itcf_carry_axe_left_hip,24,weight(2)|spd_rtng(100)|weapon_length(75)|swing_damage(40,blunt)|thrust_damage(0,pierce),imodbits_none],
 ["arena_axe","Axe",[("arena_axe",0)],itp_type_one_handed_wpn|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry,itc_scimitar|itcf_carry_axe_left_hip,
 137,weight(1.5)|spd_rtng(100)|weapon_length(69)|swing_damage(40,blunt)|thrust_damage(0,pierce),imodbits_none],
 ["arena_sword","Sword",[("arena_sword_one_handed",0),("sword_medieval_b_scabbard",ixmesh_carry),],itp_type_one_handed_wpn|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn, 243,weight(1.5)|spd_rtng(135)|weapon_length(95)|swing_damage(35,blunt)|thrust_damage(25,blunt),imodbits_none],
 ["arena_sword_two_handed","Two Handed Sword",[("arena_sword_two_handed",0)],itp_type_two_handed_wpn|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back,
 670,weight(2.75)|spd_rtng(90)|weapon_length(110)|swing_damage(55,blunt)|thrust_damage(25,blunt),imodbits_none],
 ["arena_lance","Lance",[("arena_lance",0)],itp_type_polearm|itp_primary|itp_wooden_parry,itc_staff|itcf_carry_spear,
 90,weight(2.5)|spd_rtng(70)|weapon_length(150)|swing_damage(20,blunt)|thrust_damage(65,blunt),imodbits_none],
 ["practice_staff","Practice Staff",[("wooden_staff",0)],itp_type_polearm|itp_offset_lance|itp_primary|itp_penalty_with_shield|itp_wooden_parry|itp_wooden_attack,itc_staff|itcf_carry_sword_back,9,weight(2.5)|spd_rtng(115)|weapon_length(118)|swing_damage(20,blunt)|thrust_damage(18,blunt),imodbits_none],
 ["practice_lance","Practice Lance",[("joust_of_peace",0)],itp_couchable|itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry|itp_wooden_attack,itc_greatlance,100,weight(4.25)|spd_rtng(70)|weapon_length(240)|swing_damage(0,blunt)|thrust_damage(65,blunt),imodbits_none],
 ["practice_shield","Practice Shield",[("shield_round_a",0)],itp_type_shield|itp_wooden_parry,itcf_carry_round_shield,100,weight(3.5)|body_armor(1)|hit_points(200)|spd_rtng(100)|shield_width(50),imodbits_none],
 ["practice_bow","Practice Bow",[("hunting_bow",0),("hunting_bow_carry",ixmesh_carry)],itp_type_bow|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,1,weight(1.5)|spd_rtng(110)|shoot_speed(60)|thrust_damage(30,blunt),imodbits_none],
##                                                     ("hunting_bow",0)],itp_type_bow|itp_two_handed|itp_primary|itp_attach_left_hand,itcf_shoot_bow,4,weight(1.5)|spd_rtng(90)|shoot_speed(40)|thrust_damage(19,blunt),imodbits_none],
 ["practice_crossbow","Practice Crossbow",[("crossbow_a",0)],itp_type_crossbow|itp_primary|itp_two_handed,itcf_shoot_crossbow|itcf_carry_crossbow_back,1,weight(3)|spd_rtng(25)|shoot_speed(110)|thrust_damage(65,blunt)|max_ammo(1),imodbits_none],
 ["practice_javelin","Practice Javelins",[("javelin",0),("javelins_quiver_new",ixmesh_carry)],itp_type_thrown|itp_primary|itp_next_item_as_melee,itcf_throw_javelin|itcf_carry_quiver_back|itcf_show_holster_when_drawn,1,weight(5)|spd_rtng(125)|shoot_speed(30)|thrust_damage(35,blunt)|max_ammo(50)|weapon_length(75),imodbits_none],
 ["practice_javelin_melee","practice_javelin_melee",[("javelin",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_staff,1,weight(1)|difficulty(0)|spd_rtng(91)|swing_damage(12,blunt)|thrust_damage(14,blunt)|weapon_length(75),imodbits_none],
 ["practice_throwing_daggers","Throwing Daggers",[("throwing_dagger",0)],itp_type_thrown|itp_primary,itcf_throw_knife,1,weight(3.5)|spd_rtng(102)|shoot_speed(25)|thrust_damage(16,blunt)|max_ammo(10)|weapon_length(0),imodbits_none],
 ["practice_throwing_daggers_100_amount","Throwing Daggers",[("throwing_dagger",0)],itp_type_thrown|itp_primary,itcf_throw_knife,1,weight(3.5)|spd_rtng(102)|shoot_speed(25)|thrust_damage(16,blunt)|max_ammo(100)|weapon_length(0),imodbits_none],
# ["cheap_shirt","Cheap Shirt",[("shirt",0)],itp_type_body_armor|itp_covers_legs,0,4,weight(1.25)|body_armor(3),imodbits_none],
 ["practice_horse","Practice Horse",[("saddle_horse",0)],itp_type_horse,0,37,body_armor(10)|horse_speed(40)|horse_maneuver(37)|horse_charge(14),imodbits_none],
 ["practice_arrows","Practice Arrows",[("arena_arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows,itcf_carry_quiver_back,1,weight(1.5)|weapon_length(95)|max_ammo(80),imodbits_none],
## ["practice_arrows","Practice Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo)],itp_type_arrows,0,31,weight(1.5)|weapon_length(95)|max_ammo(80),imodbits_none],
 ["practice_bolts","Practice Bolts",[("bolt",0),("flying_missile",ixmesh_flying_ammo),("bolt_bag",ixmesh_carry),("bolt_bag_b",ixmesh_carry|imodbit_large_bag)],itp_type_bolts,itcf_carry_quiver_right_vertical,1,weight(2.25)|weapon_length(55)|max_ammo(49),imodbits_none],
 ["practice_arrows_10_amount","Practice Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows,itcf_carry_quiver_back,1,weight(1.5)|weapon_length(95)|max_ammo(10),imodbits_none],
 ["practice_arrows_100_amount","Practice Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows,itcf_carry_quiver_back,1,weight(1.5)|weapon_length(95)|max_ammo(100),imodbits_none],
 ["practice_bolts_9_amount","Practice Bolts",[("bolt",0),("flying_missile",ixmesh_flying_ammo),("bolt_bag",ixmesh_carry),("bolt_bag_b",ixmesh_carry|imodbit_large_bag)],itp_type_bolts,itcf_carry_quiver_right_vertical,1,weight(2.25)|weapon_length(55)|max_ammo(9),imodbits_none],
 ["practice_boots","Practice Boots",[("boot_nomad_a",0)],itp_type_foot_armor|itp_civilian|itp_attach_armature,0,100,weight(1)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(10),imodbits_none],
 ["red_tourney_armor","Red Tourney Armor",[("tourn_armor_a",0)],itp_type_body_armor|itp_covers_legs,0,152,weight(15.0)|body_armor(20)|leg_armor(6),imodbits_none],
 ["blue_tourney_armor","Blue Tourney Armor",[("mail_shirt",0)],itp_type_body_armor|itp_covers_legs,0,152,weight(15.0)|body_armor(20)|leg_armor(6),imodbits_none],
 ["green_tourney_armor","Green Tourney Armor",[("leather_vest",0)],itp_type_body_armor|itp_covers_legs,0,152,weight(15.0)|body_armor(20)|leg_armor(6),imodbits_none],
 ["gold_tourney_armor","Gold Tourney Armor",[("padded_armor",0)],itp_type_body_armor|itp_covers_legs,0,152,weight(15.0)|body_armor(20)|leg_armor(6),imodbits_none],
 ["red_tourney_helmet","Red Tourney Helmet",[("flattop_helmet_new",0),("flattop_helmet_new_store",ixmesh_inventory)],itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,126,weight(2)|head_armor(16),imodbits_none],
 ["blue_tourney_helmet","Blue Tourney Helmet",[("segmented_helm_new",0),("segmented_helm_new_store",ixmesh_inventory)],itp_type_head_armor|itp_attach_armature,0,126,weight(2)|head_armor(16),imodbits_none],
 ["green_tourney_helmet","Green Tourney Helmet",[("hood_c",0),("hood_c_store",ixmesh_inventory)],itp_type_head_armor|itp_attach_armature,0,126,weight(2)|head_armor(16),imodbits_none],
 ["gold_tourney_helmet","Gold Tourney Helmet",[("hood_a",0),("hood_a_store",ixmesh_inventory)],itp_type_head_armor|itp_attach_armature,0,126,weight(2)|head_armor(16),imodbits_none],

["arena_shield_red","Shield",[("arena_shield_red",0)],itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,42,weight(2)|hit_points(360)|body_armor(1)|spd_rtng(100)|weapon_length(60),imodbits_none],
["arena_shield_blue","Shield",[("arena_shield_blue",0)],itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,42,weight(2)|hit_points(360)|body_armor(1)|spd_rtng(100)|weapon_length(60),imodbits_none],
["arena_shield_green","Shield",[("arena_shield_green",0)],itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,42,weight(2)|hit_points(360)|body_armor(1)|spd_rtng(100)|weapon_length(60),imodbits_none],
["arena_shield_yellow","Shield",[("arena_shield_yellow",0)],itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,42,weight(2)|hit_points(360)|body_armor(1)|spd_rtng(100)|weapon_length(60),imodbits_none],

["arena_armor_white","Arena Armor White",[("arena_armorW_new",0)],itp_type_body_armor|itp_covers_legs,0,600,weight(16)|abundance(100)|head_armor(0)|body_armor(29)|leg_armor(13),imodbits_none],
["arena_armor_red","Arena Armor Red",[("arena_armorR_new",0)],itp_type_body_armor|itp_covers_legs,0,600,weight(16)|abundance(100)|head_armor(0)|body_armor(29)|leg_armor(13),imodbits_none],
["arena_armor_blue","Arena Armor Blue",[("arena_armorB_new",0)],itp_type_body_armor|itp_covers_legs,0,600,weight(16)|abundance(100)|head_armor(0)|body_armor(29)|leg_armor(13),imodbits_none],
["arena_armor_green","Arena Armor Green",[("arena_armorG_new",0)],itp_type_body_armor|itp_covers_legs,0,600,weight(16)|abundance(100)|head_armor(0)|body_armor(29)|leg_armor(13),imodbits_none],
["arena_armor_yellow","Arena Armor Yellow",[("arena_armorY_new",0)],itp_type_body_armor|itp_covers_legs,0,600,weight(16)|abundance(100)|head_armor(0)|body_armor(29)|leg_armor(13),imodbits_none],
["arena_tunic_white","Arena Tunic White ",[("arena_tunicW_new",0)],itp_type_body_armor|itp_covers_legs,0,47,weight(2)|abundance(100)|head_armor(0)|body_armor(16)|leg_armor(6),imodbits_none],
["arena_tunic_red","Arena Tunic Red",[("arena_tunicR_new",0)],itp_type_body_armor|itp_covers_legs,0,27,weight(2)|abundance(100)|head_armor(0)|body_armor(16)|leg_armor(6),imodbits_none],
["arena_tunic_blue","Arena Tunic Blue",[("arena_tunicB_new",0)],itp_type_body_armor|itp_covers_legs,0,27,weight(2)|abundance(100)|head_armor(0)|body_armor(16)|leg_armor(6),imodbits_none],
["arena_tunic_green","Arena Tunic Green",[("arena_tunicG_new",0)],itp_type_body_armor|itp_covers_legs,0,27,weight(2)|abundance(100)|head_armor(0)|body_armor(16)|leg_armor(6),imodbits_none],
["arena_tunic_yellow","Arena Tunic Yellow",[("arena_tunicY_new",0)],itp_type_body_armor|itp_covers_legs,0,27,weight(2)|abundance(100)|head_armor(0)|body_armor(16)|leg_armor(6),imodbits_none],
#headwear
["arena_helmet_red","Arena Helmet Red",[("arena_helmetR",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(26)|body_armor(0)|leg_armor(0),imodbits_none],
["arena_helmet_blue","Arena Helmet Blue",[("arena_helmetB",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(26)|body_armor(0)|leg_armor(0),imodbits_none],
["arena_helmet_green","Arena Helmet Green",[("arena_helmetG",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(26)|body_armor(0)|leg_armor(0),imodbits_none],
["arena_helmet_yellow","Arena Helmet Yellow",[("arena_helmetY",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(26)|body_armor(0)|leg_armor(0),imodbits_none],
["steppe_helmet_white","Steppe Helmet White",[("steppe_helmetW",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(20)|body_armor(0)|leg_armor(0),imodbits_none],
["steppe_helmet_red","Steppe Helmet Red",[("steppe_helmetR",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(20)|body_armor(0)|leg_armor(0),imodbits_none],
["steppe_helmet_blue","Steppe Helmet Blue",[("steppe_helmetB",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(20)|body_armor(0)|leg_armor(0),imodbits_none],
["steppe_helmet_green","Steppe Helmet Green",[("steppe_helmetG",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(20)|body_armor(0)|leg_armor(0),imodbits_none],
["steppe_helmet_yellow","Steppe Helmet Yellow",[("steppe_helmetY",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(20)|body_armor(0)|leg_armor(0),imodbits_none],
["tourney_helm_white","Tourney Helm White",[("tourney_helmR",0)],itp_type_head_armor|itp_covers_head,0,760,weight(2.75)|abundance(100)|head_armor(30)|body_armor(0)|leg_armor(0),imodbits_none],
["tourney_helm_red","Tourney Helm Red",[("tourney_helmR",0)],itp_type_head_armor|itp_covers_head,0,760,weight(2.75)|abundance(100)|head_armor(30)|body_armor(0)|leg_armor(0),imodbits_none],
["tourney_helm_blue","Tourney Helm Blue",[("tourney_helmB",0)],itp_type_head_armor|itp_covers_head,0,760,weight(2.75)|abundance(100)|head_armor(30)|body_armor(0)|leg_armor(0),imodbits_none],
["tourney_helm_green","Tourney Helm Green",[("tourney_helmG",0)],itp_type_head_armor|itp_covers_head,0,760,weight(2.75)|abundance(100)|head_armor(30)|body_armor(0)|leg_armor(0),imodbits_none],
["tourney_helm_yellow","Tourney Helm Yellow",[("tourney_helmY",0)],itp_type_head_armor|itp_covers_head,0,760,weight(2.75)|abundance(100)|head_armor(30)|body_armor(0)|leg_armor(0),imodbits_none],
["arena_turban_red","Arena Turban",[("tuareg_open",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(26)|body_armor(0)|leg_armor(0),imodbits_none],
["arena_turban_blue","Arena Turban",[("tuareg_open",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(26)|body_armor(0)|leg_armor(0),imodbits_none],
["arena_turban_green","Arena Turban",[("tuareg_open",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(26)|body_armor(0)|leg_armor(0),imodbits_none],
["arena_turban_yellow","Arena Turban",[("tuareg_open",0)],itp_type_head_armor|itp_fit_to_head,0,187,weight(1.25)|abundance(100)|head_armor(26)|body_armor(0)|leg_armor(0),imodbits_none],

# A treatise on The Method of Mechanical Theorems Archimedes
 
#This book must be at the beginning of readable books
 ["book_tactics","De Re Militari",[("book_a",0)],itp_type_book,0,4000,weight(2)|abundance(100),imodbits_none],
 ["book_persuasion","Rhetorica ad Herennium",[("book_b",0)],itp_type_book,0,5000,weight(2)|abundance(100),imodbits_none],
 ["book_leadership","The Life of Alixenus the Great",[("book_d",0)],itp_type_book,0,4200,weight(2)|abundance(100),imodbits_none],
 ["book_intelligence","Essays on Logic",[("book_e",0)],itp_type_book,0,2900,weight(2)|abundance(100),imodbits_none],
 ["book_trade","A Treatise on the Value of Things",[("book_f",0)],itp_type_book,0,3100,weight(2)|abundance(100),imodbits_none],
 ["book_weapon_mastery","On the Art of Fighting with Swords",[("book_d",0)],itp_type_book,0,4200,weight(2)|abundance(100),imodbits_none],
 ["book_engineering","Method of Mechanical Theorems",[("book_open",0)],itp_type_book,0,4000,weight(2)|abundance(100),imodbits_none],

#Reference books
#This book must be at the beginning of reference books
 ["book_wound_treatment_reference","The Book of Healing",[("book_c",0)],itp_type_book,0,3500,weight(2)|abundance(100),imodbits_none],
 ["book_training_reference","Manual of Arms",[("book_open",0)],itp_type_book,0,3500,weight(2)|abundance(100),imodbits_none],
 ["book_surgery_reference","The Great Book of Surgery",[("book_c",0)],itp_type_book,0,3500,weight(2)|abundance(100),imodbits_none],

 #TRADE GOODS
 #These are trade goods referenced in new_trade_system (see module_scripts).
 #They have an explicit relationship with their "home locations".  The comments below tell you the default settings.
 
 #Spice comes from Sarranid town of Shariz
 ["spice","Spice",[("spice_sack",0)],itp_merchandise|itp_type_goods|itp_consumable,0,5000,weight(40)|abundance(25)|max_ammo(50),imodbits_none,[],[fac_kingdom_of_sarranids]],

 #Salt comes from Nord town of Sargoth
 ["salt","Salt",[("salt_sack",0)],itp_merchandise|itp_type_goods,0,1300,weight(50)|abundance(120),imodbits_none,[],[fac_kingdom_of_nords]],

 #Oil comes from Rhodok town of Jelkala
 ["oil","Oil",[("oil",0)],itp_merchandise|itp_type_goods|itp_consumable,0,1400,weight(50)|abundance(60)|max_ammo(50),imodbits_none,[],[fac_kingdom_of_rhodoks]],

 #Pottery comes from Khergit town of Ichamur
 ["pottery","Pottery",[("jug",0)],itp_merchandise|itp_type_goods,0,1200,weight(50)|abundance(90),imodbits_none,[],[fac_kingdom_of_khergits]],

 #Flax comes from Nord town of Wercheg
 ["raw_flax","Flax Bundle",[("raw_flax",0)],itp_merchandise|itp_type_goods,0,400,weight(40)|abundance(90),imodbits_none,[],[fac_kingdom_of_nords]],
 
 #Linen comes from Vaegir town of Halmar
 ["linen","Linen",[("linen",0)],itp_merchandise|itp_type_goods,0,2500,weight(40)|abundance(90),imodbits_none,[],[fac_kingdom_of_vaegirs]],

 #Raw wool comes from Swadian town of Suno
 ["wool","Wool",[("wool_sack",0)],itp_merchandise|itp_type_goods,0,600,weight(40)|abundance(90),imodbits_none,[],[fac_kingdom_of_swadians]],
 
 #Wool cloth comes from Sarranid town of Bariyye
 ["wool_cloth","Wool Cloth",[("linen",0)],itp_merchandise|itp_type_goods,0,2000,weight(40)|abundance(90),imodbits_none,[],[fac_kingdom_of_sarranids]],

 #Raw silk comes from Swadian town of Dhirim
 ["raw_silk","Raw Silk",[("linen",0)],itp_merchandise|itp_type_goods,0,1200,weight(30)|abundance(90),imodbits_none,[],[fac_kingdom_of_swadians]],
 
 #Raw dyes come from Sarranid town of Durquba
 ["raw_dyes","Dyes",[("dyes",0)],itp_merchandise|itp_type_goods,0,2000,weight(10)|abundance(90),imodbits_none,[],[fac_kingdom_of_sarranids]],
 
 #Velvet comes from Swadian town of Praven
 ["velvet","Velvet",[("velvet",0)],itp_merchandise|itp_type_goods,0,5500,weight(40)|abundance(30),imodbits_none,[],[fac_kingdom_of_swadians]],

 #Iron comes from Nord town of Reyvadin
 ["iron","Iron",[("iron",0)],itp_merchandise|itp_type_goods,0,2600,weight(60)|abundance(60),imodbits_none,[],[fac_kingdom_of_nords]],
 
 #Tools come from Rhodok town of Veluca
 ["tools","Tools",[("iron_hammer",0)],itp_merchandise|itp_type_goods,0,4000,weight(50)|abundance(90),imodbits_none,[],[fac_kingdom_of_rhodoks]],

 #Raw leather comes from Khergit town of Narra
 ["raw_leather","Hides",[("leatherwork_inventory",0)],itp_merchandise|itp_type_goods,0,1200,weight(40)|abundance(90),imodbits_none,[],[fac_kingdom_of_khergits]],
 
 #Leatherwork comes from Sarranid town of Ahmerrad
 ["leatherwork","Leatherwork ",[("leatherwork_frame",0)],itp_merchandise|itp_type_goods,0,2500,weight(40)|abundance(90),imodbits_none,[],[fac_kingdom_of_sarranids]],
 
 #Raw dates come from Sarranid town of Bariyye
 ["raw_date_fruit","Date Fruit",[("date_inventory",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,300,weight(5)|food_quality(10)|max_ammo(10),imodbits_none,[],[fac_kingdom_of_sarranids]],
 
 #Furs come from Vaegir town of Curaw
 ["furs","Furs",[("fur_pack",0)],itp_merchandise|itp_type_goods,0,1500,weight(40)|abundance(90),imodbits_none,[],[fac_kingdom_of_vaegirs]],

 #Wine comes from Vaegir town of Khudan
 ["wine","Wine",[("amphora_slim",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,5000,weight(5)|abundance(110)|max_ammo(10)|food_quality(600),imodbits_none,[],[fac_kingdom_of_vaegirs]],

 #Ale comes from Swadian town of Uxkhal
 ["ale","Ale",[("ale_barrel",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,4000,weight(5)|abundance(110)|max_ammo(10)|food_quality(500),imodbits_none,[],[fac_kingdom_of_swadians]],

# ["dry_bread","wheat_sack",itp_type_goods|itp_consumable,0,slt_none,view_goods,95,weight(2),max_ammo(50),imodbits_none],
#foods (first one is smoked_fish)
 ["smoked_fish","Smoked Fish",[("smoked_fish",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,300,weight(15)|abundance(110)|food_quality(50)|max_ammo(50),imodbits_none],
 ["cheese","Cheese",[("cheese_b",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,700,weight(1)|abundance(110)|food_quality(80)|max_ammo(10),imodbits_none],
 ["honey","Honey",[("honey_pot",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,200,weight(1)|abundance(110)|food_quality(90)|max_ammo(5),imodbits_none],
 ["sausages","Sausages",[("sausages",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,800,weight(3)|abundance(110)|food_quality(85)|max_ammo(10),imodbits_none],
 ["cabbages","Cabbages",[("cabbage",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,300,weight(3)|abundance(110)|food_quality(60)|max_ammo(10),imodbits_none],
 ["dried_meat","Dried Meat",[("smoked_meat",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,85,weight(5)|abundance(100)|food_quality(65)|max_ammo(10),imodbits_none],
 ["apples","Fruit",[("apple_basket",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,440,weight(3)|abundance(110)|food_quality(100)|max_ammo(5),imodbits_none],
 ["raw_grapes","Grapes",[("grapes_inventory",0)],itp_merchandise|itp_consumable|itp_type_goods,0,700,weight(3)|abundance(90)|food_quality(120)|max_ammo(5),imodbits_none],#x2 for wine
 ["raw_olives","Olives",[("olive_inventory",0)],itp_merchandise|itp_consumable|itp_type_goods,0,1000,weight(3)|abundance(90)|food_quality(30)|max_ammo(40),imodbits_none],#x3 for oil
 ["grain","Grain",[("wheat_sack",0)],itp_merchandise|itp_type_goods|itp_consumable,0,300,weight(15)|abundance(110)|food_quality(20)|max_ammo(50),imodbits_none],

 ["cattle_meat","Beef",[("raw_meat",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,800,weight(3)|abundance(100)|food_quality(150)|max_ammo(5),imodbits_none],
 ["bread","Bread",[("bread_a",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,500,weight(10)|abundance(110)|food_quality(40)|max_ammo(50),imodbits_none],
 ["chicken","Chicken",[("chicken",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,900,weight(2)|abundance(110)|food_quality(140)|max_ammo(5),imodbits_none],
 ["pork","Pork",[("pork",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,1100,weight(2)|abundance(100)|food_quality(160)|max_ammo(5),imodbits_none],
 ["butter","Butter",[("butter_pot",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,900,weight(1)|abundance(110)|food_quality(120)|max_ammo(10),imodbits_none],
  ["hardtack","Hard-Tack",[("food_crate",0)],itp_merchandise|itp_type_goods|itp_consumable|itp_food,0,2000,weight(15)|abundance(200)|food_quality(5)|max_ammo(200),imodbits_none],
 

#************************************************************************************************
# ITEMS before this point are hardcoded into item_codes.h and their order should not be changed!
#************************************************************************************************
#I have no idea whether the above warning is actually true anymore but dared not bring order to this chaos lest it Break Stuff- xeno

# Quest Items
 ["siege_supply","Supplies",[("ale_barrel",0)],itp_type_goods,0,96,weight(40)|abundance(70),imodbits_none],
 ["quest_wine","Wine",[("amphora_slim",0)],itp_type_goods,0,46,weight(40)|abundance(60)|max_ammo(50),imodbits_none],
 ["quest_ale","Ale",[("ale_barrel",0)],itp_type_goods,0,31,weight(40)|abundance(70)|max_ammo(50),imodbits_none],
 


 
# Tutorial Items

 ["tutorial_sword","Sword",[("long_sword",0),("scab_longsw_a",ixmesh_carry)],itp_type_one_handed_wpn|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,1,weight(1.5)|difficulty(0)|spd_rtng(100)|weapon_length(102)|swing_damage(18,cut)|thrust_damage(15,pierce),imodbits_none],
 ["tutorial_axe","Axe",[("iron_ax",0)],itp_type_two_handed_wpn|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry,itc_nodachi|itcf_carry_axe_back,1,weight(4)|difficulty(0)|spd_rtng(91)|weapon_length(108)|swing_damage(19,cut)|thrust_damage(0,pierce),imodbits_none],
 ["tutorial_spear","Spear",[("spear",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_spear,1,weight(4.5)|difficulty(0)|spd_rtng(80)|weapon_length(158)|swing_damage(0,cut)|thrust_damage(19,pierce),imodbits_none],
 ["tutorial_club","Club",[("club",0)],itp_type_one_handed_wpn|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,1,weight(2.5)|difficulty(0)|spd_rtng(95)|weapon_length(95)|swing_damage(11,blunt)|thrust_damage(0,pierce),imodbits_none],
 ["tutorial_battle_axe","Battle Axe",[("battle_ax",0)],itp_type_two_handed_wpn|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry,itc_nodachi|itcf_carry_axe_back,1,weight(5)|difficulty(0)|spd_rtng(88)|weapon_length(108)|swing_damage(27,cut)|thrust_damage(0,pierce),imodbits_none],
 ["tutorial_arrows","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows,itcf_carry_quiver_back,1,weight(3)|abundance(160)|weapon_length(95)|thrust_damage(0,pierce)|max_ammo(20),imodbits_none],
 ["tutorial_bolts","Bolts",[("bolt",0),("flying_missile",ixmesh_flying_ammo),("bolt_bag",ixmesh_carry),("bolt_bag_b",ixmesh_carry|imodbit_large_bag)],itp_type_bolts,itcf_carry_quiver_right_vertical,1,weight(2.25)|abundance(90)|weapon_length(63)|thrust_damage(0,pierce)|max_ammo(18),imodbits_none],
 ["tutorial_short_bow","Short Bow",[("short_bow",0),("short_bow_carry",ixmesh_carry)],itp_type_bow|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,1,weight(1)|difficulty(0)|spd_rtng(98)|shoot_speed(49)|thrust_damage(12,pierce  ),imodbits_none],
 ["tutorial_crossbow","Crossbow",[("crossbow_a",0)],itp_type_crossbow|itp_primary|itp_two_handed|itp_cant_reload_on_horseback,itcf_shoot_crossbow|itcf_carry_crossbow_back,1,weight(3)|difficulty(0)|spd_rtng(42)|shoot_speed(68)|thrust_damage(32,pierce)|max_ammo(1),imodbits_none],
 ["tutorial_throwing_daggers","Throwing Daggers",[("throwing_dagger",0)],itp_type_thrown|itp_primary,itcf_throw_knife,1,weight(3.5)|difficulty(0)|spd_rtng(102)|shoot_speed(25)|thrust_damage(16,cut)|max_ammo(14)|weapon_length(0),imodbits_none],
 ["tutorial_saddle_horse","Saddle Horse",[("saddle_horse",0)],itp_type_horse,0,0,abundance(90)|body_armor(3)|difficulty(0)|horse_speed(40)|horse_maneuver(38)|horse_charge(8),imodbits_none],
 ["tutorial_shield","Kite Shield",[("shield_kite_a",0)],itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,100,weight(2.5)|hit_points(480)|body_armor(1)|spd_rtng(82)|weapon_length(150),imodbits_none],
 ["tutorial_staff_no_attack","Staff",[("wooden_staff",0)],itp_type_polearm|itp_offset_lance|itp_primary|itp_penalty_with_shield|itp_wooden_parry|itp_wooden_attack,itc_parry_polearm|itcf_carry_sword_back,9,weight(3.5)|spd_rtng(120)|weapon_length(115)|swing_damage(0,blunt)|thrust_damage(0,blunt),imodbits_none],
 ["tutorial_staff","Staff",[("wooden_staff",0)],itp_type_polearm|itp_offset_lance|itp_primary|itp_penalty_with_shield|itp_wooden_parry|itp_wooden_attack,itc_staff|itcf_carry_sword_back,9,weight(3.5)|spd_rtng(120)|weapon_length(115)|swing_damage(16,blunt)|thrust_damage(16,blunt),imodbits_none],

# Horses: sumpter horse/ pack horse,saddle horse,steppe horse,warm blood,geldling,stallion,war mount,charger,
# Carthorse,hunter,heavy hunter,hackney,palfrey,courser,destrier.

#Racing horses- high speed,OK manuever,lousy charge,armor and hitpoints
 ["light_horse_start","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["arabian_horse_a","Nomad Horse",[("arabian_horse_a",0)],itp_merchandise|itp_type_horse,0,2000,abundance(100)|hit_points(80)|body_armor(25)|difficulty(2)|horse_speed(60)|horse_maneuver(40)|horse_charge(15)|horse_scale(100),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["arabian_horse_b","Sarranid Horse",[("arabian_horse_b",0)],itp_merchandise|itp_type_horse,0,3000,abundance(80)|hit_points(90)|body_armor(25)|difficulty(4)|horse_speed(65)|horse_maneuver(40)|horse_charge(15)|horse_scale(100),imodbits_none,[],[fac_kingdom_of_sarranids]],
 
 #Barf's awesome skeletal horse!
 ["barf_skeletal_horse","Skeleton Horse",[("barf_skeletal_horse",0)],itp_type_horse,0,1000,abundance(90)|hit_points(100)|body_armor(45)|difficulty(1)|horse_speed(45)|horse_maneuver(45)|horse_charge(25)|horse_scale(100),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],
 
 #Typical European horses- medium in all respects
["sumpter_horse","Sumpter Horse",[("sumpter_horse",0)],itp_merchandise|itp_type_horse,0,1000,abundance(90)|hit_points(100)|body_armor(45)|difficulty(1)|horse_speed(45)|horse_maneuver(45)|horse_charge(25)|horse_scale(100),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],
 
["xenoargh_appaloosa01","Appaloosa",[("xenoargh_appaloosa01",0)],itp_merchandise|itp_type_horse,0,1500,abundance(90)|hit_points(125)|body_armor(45)|difficulty(1)|horse_speed(45)|horse_maneuver(40)|horse_charge(25)|horse_scale(100),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],
 
["saddle_horse","Saddle Horse",[("saddle_horse",0)],itp_merchandise|itp_type_horse,0,2500,abundance(90)|hit_points(120)|body_armor(55)|difficulty(2)|horse_speed(45)|horse_maneuver(45)|horse_charge(40)|horse_scale(104),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],
 
["xenoargh_bayappy","Bay",[("xenoargh_bayappy",0)],itp_merchandise|itp_type_horse,0,3000,abundance(90)|hit_points(140)|body_armor(55)|difficulty(2)|horse_speed(45)|horse_maneuver(45)|horse_charge(40)|horse_scale(104),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],
  
["xenoargh_baydark","Bay",[("xenoargh_baydark",0)],itp_merchandise|itp_type_horse,0,3000,abundance(90)|hit_points(120)|body_armor(55)|difficulty(2)|horse_speed(50)|horse_maneuver(45)|horse_charge(40)|horse_scale(104),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],  
 
["courser","Courser",[("courser",0)],itp_merchandise|itp_type_horse,0,3500,abundance(70)|body_armor(55)|hit_points(150)|difficulty(2)|horse_speed(50)|horse_maneuver(45)|horse_charge(35)|horse_scale(106),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],
  
["xenoargh_blackleapord","Spotted Courser",[("xenoargh_blackleapord",0)],itp_merchandise|itp_type_horse,0,4000,abundance(70)|body_armor(60)|hit_points(150)|difficulty(2)|horse_speed(50)|horse_maneuver(45)|horse_charge(35)|horse_scale(106),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]], 
  
["hunter","Hunter",[("hunting_horse",0)],itp_merchandise|itp_type_horse,0,4500,abundance(60)|hit_points(200)|body_armor(55)|difficulty(3)|horse_speed(45)|horse_maneuver(45)|horse_charge(45)|horse_scale(108),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_buckskin","Buckskin Hunter",[("xenoargh_buckskin",0)],itp_merchandise|itp_type_horse,0,6000,abundance(60)|hit_points(200)|body_armor(65)|difficulty(3)|horse_speed(50)|horse_maneuver(45)|horse_charge(40)|horse_scale(108),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],
 
 #Khergit horse- high manueverability,low hitpoints,medium speed,low charge
["steppe_horse","Steppe Horse",[("steppe_horse",0)],itp_merchandise|itp_type_horse,0,2500,abundance(100)|hit_points(80)|body_armor(35)|difficulty(2)|horse_speed(40)|horse_maneuver(55)|horse_charge(20)|horse_scale(98),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],
 
["light_horse_end","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["medium_horse_start","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
 
#Heavy horses with barding.  Slow,really good charge,great armor and hitpoints,lousy manuever.  Very expensive.
["warhorse","War Horse",[("warhorse",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(125)|difficulty(4)|horse_speed(40)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#Better speed, lower armor. 
["warhorse_chain","War Horse",[("warhorse_chain",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding01","War Horse",[("warhorse_chain_barding01",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding02","War Horse",[("warhorse_chain_barding02",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding03","War Horse",[("warhorse_chain_barding03",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding04","War Horse",[("warhorse_chain_barding04",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding05","War Horse",[("warhorse_chain_barding05",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding06","War Horse",[("warhorse_chain_barding06",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding07","War Horse",[("warhorse_chain_barding07",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding08","War Horse",[("warhorse_chain_barding08",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(45)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["warhorse_chain_barding_templar","Templar's Steed",[("warhorse_chain_barding_templar",0)],itp_merchandise|itp_type_horse,0,7000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(50)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#Khergit warhorse- lower armor than warhorse, better manuever.
["warhorse_steppe","Steppe Charger",[("warhorse_steppe",0)],itp_merchandise|itp_type_horse,0,6000,abundance(50)|hit_points(250)|body_armor(100)|difficulty(4)|horse_speed(40)|horse_maneuver(40)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_khergits]],
 
["medium_horse_end","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
 
 ["heavy_horse_start","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
 
#Camels.
["camel","Camel",[("camel",0)],itp_merchandise|itp_type_horse,0,5000,abundance(100)|hit_points(200)|body_armor(100)|difficulty(3)|horse_speed(40)|horse_maneuver(30)|horse_charge(55)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_sarranids]],
 
["camel2","War Camel",[("camel2",0)],itp_merchandise|itp_type_horse,0,10000,abundance(40)|hit_points(300)|body_armor(130)|difficulty(3)|horse_speed(40)|horse_maneuver(30)|horse_charge(70)|horse_scale(120),imodbits_none,[],[fac_kingdom_of_sarranids]],

["camel3","Trader's Camel",[("camel3",0)],itp_merchandise|itp_type_horse,0,7000,abundance(40)|hit_points(300)|body_armor(100)|difficulty(3)|horse_speed(45)|horse_maneuver(35)|horse_charge(50)|horse_scale(110),imodbits_none,[],[fac_kingdom_of_sarranids]],
 
#Chargers have superior charge, armor and hitpoints.  Best horses in the game for running people down.
["charger","Charger",[("charger",0)],itp_merchandise|itp_type_horse,0,10000,abundance(40)|hit_points(300)|body_armor(130)|difficulty(5)|horse_speed(40)|horse_maneuver(30)|horse_charge(65)|horse_scale(112),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians]],

["spak_warhorse_imperial","Imperial Charger",[("spak_warhorse_imperial",0)],itp_merchandise|itp_type_horse,0,10000,abundance(40)|hit_points(300)|body_armor(130)|difficulty(5)|horse_speed(40)|horse_maneuver(30)|horse_charge(65)|horse_scale(112),imodbits_none,[],[fac_kingdom_of_rhodoks]],

#Sarranid charger.  Lower speed, better armor.
["warhorse_sarranid","Mamluke Charger",[("warhorse_sarranid",0)],itp_merchandise|itp_type_horse,0,15000,abundance(40)|hit_points(300)|body_armor(150)|difficulty(5)|horse_speed(37)|horse_maneuver(30)|horse_charge(65)|horse_scale(112),imodbits_none,[],[fac_kingdom_of_sarranids]],

["spak_warhorse_holy","Holy Charger",[("spak_warhorse_holy",0)],itp_merchandise|itp_type_horse,0,15000,abundance(40)|hit_points(300)|body_armor(150)|difficulty(5)|horse_speed(37)|horse_maneuver(30)|horse_charge(65)|horse_scale(112),imodbits_none,[],[fac_kingdom_of_swadians]],

#Heaviest horse in game.  High-end charger.  Only Rhodok towns.
["charger_plate","Destrier",[("taleworlds_charger_plate",0)],itp_merchandise|itp_type_horse,0,25000,abundance(40)|hit_points(350)|body_armor(170)|difficulty(5)|horse_speed(37)|horse_maneuver(30)|horse_charge(65)|horse_scale(112),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["spak_horny_charger_plate","Horned Destrier",[("spak_horny_charger_plate",0)],itp_merchandise|itp_type_horse,0,30000,abundance(40)|hit_points(350)|body_armor(170)|difficulty(5)|horse_speed(37)|horse_maneuver(30)|horse_charge(70)|horse_scale(112),imodbits_none,[],[fac_kingdom_of_rhodoks]],

 ["heavy_horse_end","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["pilgrim_disguise","Pilgrim Disguise",[("pilgrim_outfit",0)],0|itp_type_body_armor|itp_covers_legs|itp_civilian,0,25,weight(2)|abundance(100)|head_armor(0)|body_armor(19)|leg_armor(8)|difficulty(0),imodbits_none],
["pilgrim_hood","Pilgrim Hood",[("pilgrim_hood",0)],0|itp_type_head_armor|itp_civilian,0,35,weight(1.25)|abundance(100)|head_armor(14)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

#----------ARMOR-----------#
#GLOVES
#Considerably better protection from combat gloves,vast increase in price and difficulty ratings.
["gloves_start","Gloves Start (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["leather_gloves","Leather Gloves",[("leather_gloves_L",0)],itp_merchandise|itp_type_hand_armor,0,300,weight(0.25)|abundance(120)|body_armor(5)|difficulty(0),imodbits_none],

["mail_mittens","Mail Mittens",[("mail_mittens_L",0)],itp_merchandise|itp_type_hand_armor,0,1000,weight(1.5)|abundance(100)|body_armor(10)|difficulty(12),imodbits_none],

["narf_mail_gauntlets","Mail Gauntlets",[("narf_mail_gauntlets_L",0)],itp_merchandise|itp_type_hand_armor,0,1500,weight(1.5)|abundance(100)|body_armor(15)|difficulty(12),imodbits_none],

["scale_gauntlets","Scale Gauntlets",[("scale_gauntlets_b_L",0)],itp_merchandise|itp_type_hand_armor,0,2500,weight(2)|abundance(100)|body_armor(20)|difficulty(13),imodbits_none],

["lamellar_gauntlets","Lamellar Gauntlets",[("scale_gauntlets_a_L",0)],itp_merchandise|itp_type_hand_armor,0,3000,weight(2.5)|abundance(100)|body_armor(25)|difficulty(14),imodbits_none],

["narf_wisby_gauntlets_black","Splinted (Wisby) Gauntlets",[("narf_wisby_gauntlets_black_L",0)],itp_merchandise|itp_type_hand_armor,0,3000,weight(2.5)|abundance(100)|body_armor(25)|difficulty(14),imodbits_none],

["narf_wisby_gauntlets_red","Splinted (Wisby) Gauntlets",[("narf_wisby_gauntlets_red_L",0)],itp_merchandise|itp_type_hand_armor,0,3000,weight(2.5)|abundance(100)|body_armor(25)|difficulty(14),imodbits_none],

#SacredStoneHead's Kote (Japanese heavy gloves)
["ssh_kote","Kote",[("ssh_kote_L",0)],itp_merchandise|itp_type_hand_armor,0,3000,weight(2.5)|abundance(100)|body_armor(25)|difficulty(14),imodbits_none,[],[fac_kingdom_of_khergits]],

#Heavy gauntlets
["narf_milanese_gauntlets","Milanese Gauntlets",[("narf_milanese_gauntlet_L",0)],itp_merchandise|itp_type_hand_armor,0,7000,weight(3.0)|abundance(100)|body_armor(35)|difficulty(16),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["narf_milanese_gauntlets_black","Black Gauntlets",[("narf_milanese_gauntlet_black_L",0)],itp_merchandise|itp_type_hand_armor,0,8000,weight(3.0)|abundance(100)|body_armor(40)|difficulty(16),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_gauntlets","Striped Gauntlets",[("narf_gauntlet_L",0)],itp_merchandise|itp_type_hand_armor,0,5000,weight(3.0)|abundance(100)|body_armor(30)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["narf_plate_mittens","Plate Mittens",[("narf_plate_mittens_L",0)],itp_merchandise|itp_type_hand_armor,0,6000,weight(3.0)|abundance(100)|body_armor(35)|difficulty(16),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["spak_dark_gauntlets","Dark Gauntlets",[("spak_dark_gauntletsL",0)],itp_merchandise|itp_type_hand_armor,0,6000,weight(3.0)|abundance(100)|body_armor(35)|difficulty(16),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["narf_plate_mittens_dark","Dark Plate Mittens",[("narf_plate_mittens_dark_L",0)],itp_merchandise|itp_type_hand_armor,0,6000,weight(3.0)|abundance(100)|body_armor(35)|difficulty(16),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["amade_bronze_gauntlets","Bronze Gauntlets",[("amade_bronze_gauntlet_L",0)],itp_merchandise|itp_type_hand_armor,0,5000,weight(3.0)|abundance(100)|body_armor(25)|difficulty(15),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_steel_gauntlets","Steel Gauntlets",[("amade_steel_gauntlet_L",0)],itp_merchandise|itp_type_hand_armor,0,5000,weight(3.0)|abundance(100)|body_armor(25)|difficulty(15),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["gauntlets","Gauntlets",[("gauntlets_L",0),("gauntlets_L",imodbit_reinforced)],itp_merchandise|itp_type_hand_armor,0,5000,weight(3.0)|abundance(100)|body_armor(25)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["gloves_end","Gloves End (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#FOOTWEAR
#Considerably more protection from heavy footwear,with vast changes in weight / requirements / price.  Sorted by weight.
#Sarranid gear exceptionalism is gone now.

["boots_start","Boots Start (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#Civilian footwear
["ankle_boots","Ankle Boots",[("ankle_boots_a_new",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,25,weight(1)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wrapping_boots","Wrapping Boots",[("wrapping_boots_a",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,100,weight(1)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits]],

["woolen_hose","Woolen Hose",[("woolen_hose_a",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,100,weight(1)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["blue_hose","Blue Hose",[("blue_hose_a",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,100,weight(1)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["sarranid_boots_a","Sarranid Shoes",[("sarranid_shoes",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,200,weight(1)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

 #Light footwear
 ["light_leather_boots","Light Leather Boots",[("light_leather_boots",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(30)|difficulty(4),imodbits_none],
 
  ["narf_rus_shoes","Ruski Shoes",[("narf_rus_shoes",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(30)|difficulty(4),imodbits_none,[],[fac_kingdom_of_vaegirs]],
 
["hunter_boots","Hunter Boots",[("hunter_boots_a",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(30)|difficulty(4),imodbits_none],

["hide_boots","Hide Boots",[("hide_boots_a",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,300,weight(2)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(30)|difficulty(5),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],
 
["sarranid_boots_b","Sarranid Leather Boots",[("sarranid_boots",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,300,weight(3)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(32)|difficulty(6),imodbits_none,[],[fac_kingdom_of_sarranids]],

["nomad_boots","Nomad Boots",[("nomad_boots_a",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,400,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(34)|difficulty(6),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["leather_boots","Leather Boots",[("leather_boots_a",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(36)|difficulty(7),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],
 
["khergit_leather_boots","Khergit Leather Boots",[("khergit_leather_boots",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(36)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["narf_rus_cav_boots","Ruski Cavalry Boots",[("narf_rus_cav_boots",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(37)|difficulty(7),imodbits_none,[],[fac_kingdom_of_vaegirs]],
 
["narf_bear_paw_shoes","Bear Paw Shoes",[("narf_bear_paw_shoes",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(36)|difficulty(7),imodbits_none,[],[fac_kingdom_of_rhodoks]],
  
 #Heavy footwear
["khergit_guard_boots","Khergit Guard Boots",[("lamellar_boots_a",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,1350,weight(4.25)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(45)|difficulty(8),imodbits_none,[],[fac_kingdom_of_khergits]],

["splinted_leather_greaves","Splinted Leather Greaves",[("leather_greaves_a",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
1500,weight(4.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(50)|difficulty(8),imodbits_none],

["mail_chausses","Mail Chausses",[("mail_chausses_a",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
2000,weight(5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(60)|difficulty(8),imodbits_none],

["sarranid_boots_d","Sarranid Mail Boots",[("sarranid_mail_chausses",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,
2200,weight(5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(64)|difficulty(8),imodbits_none,[],[fac_kingdom_of_sarranids]],

["splinted_greaves","Splinted Greaves",[("splinted_greaves_a",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
2500,weight(7)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(70)|difficulty(10),imodbits_none],

#DtheHun's boots
["xena_boots","Heavy Boots",[("xena_boots",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
2500,weight(7)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(70)|difficulty(10),imodbits_none],

["rogue_csizma","Thief's Boots",[("rogue_csizma",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
2500,weight(7)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(70)|difficulty(10),imodbits_none],

["mail_boots","Mail Boots",[("mail_boots_a",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
3000,weight(8)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(80)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["sarranid_boots_c","Plated Boots",[("sarranid_camel_boots",0)],itp_merchandise|itp_type_foot_armor|itp_civilian|itp_attach_armature,0,
3000,weight(4.25)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(80)|difficulty(12),imodbits_none,[],[fac_kingdom_of_sarranids]],

["iron_greaves","Iron Greaves",[("iron_greaves_a",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
3500,weight(9)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(90)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["narf_splinted_greaves_spurs","Heavy Splinted Greaves",[("narf_splinted_greaves_spurs",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
3500,weight(9)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(90)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#SacredStoneHead's Awesome Boots!
["ssh_suneate","Sunate",[("ssh_suneate",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,3500,weight(4.25)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(90)|difficulty(14),imodbits_none,[],[fac_kingdom_of_khergits]],
 
["amade_bronze_greaves","Bronze Greaves",[("amade_bronze_greaves",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_steel_greaves","Steel Greaves",[("amade_steel_greaves",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_steel_greaves","Steel Greaves",[("narf_steel_greaves",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_steel_greaves_dark","Dark Steel Greaves",[("narf_steel_greaves_dark",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_shynbaulds","Shynbaulds",[("narf_shynbaulds",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_shynbaulds_black","Black Shynbaulds",[("narf_shynbaulds_black",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
5000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(110)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_shynbaulds_dark","Dark Shynbaulds",[("narf_shynbaulds_dark",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["plate_boots","Plate Boots",[("plate_boots",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["sarranid_plate_boots","Bronze Plate Boots",[("bronze_plate_boots",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_sarranids]],

["rhodok_plate_boots","Dark Plate Boots",[("dark_plate_boots",0)],itp_merchandise|itp_type_foot_armor|itp_attach_armature,0,
4000,weight(3.5)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(100)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["black_greaves","Black Greaves",[("black_greaves",0)],itp_type_foot_armor|itp_attach_armature,0,
4500,weight(9)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(110)|difficulty(16),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["boots_end","Boots End (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#BODY ARMOR
#Divided up into 5 main sections:  clothes,padded,leather,mail,plate.  Lamellar / banded / scale is all considered mail.
#Each has considerable differences in weight,cost,performance.

#Clothes.  These are ordinary clothes,and offer practically no protection.  For the most part,their armor stats have been left at Native levels,because the new balance means they're practically worthless and most of them aren't ever seen in combat anyhow.  If I had time,I'd make a Fallout 3 style system,where some of them give you a serious boost to sales,persuasion,courting,etc. (especially for female characters,this would be a big deal) but for now they're just priced on whether they're "noble" clothing or typical civilian wear.

["armor_start","Armor Start (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["male_civ_clothes_start","Male Civilian Clothes Start (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#Civilian Clothes
["tunic_with_green_cape","Tunic with Green Cape",[("peasant_man_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,6,weight(1)|abundance(100)|head_armor(0)|body_armor(6)|leg_armor(2)|difficulty(0),imodbits_none],

["cloaked_tunic","Cloaked Tunic",[("nobleman_outf",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,348,weight(4)|abundance(100)|head_armor(0)|body_armor(14)|leg_armor(10)|difficulty(0),imodbits_none],

["sleeveless_tunic","Sleeveless Tunic",[("nobleman_outf",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,348,weight(4)|abundance(100)|head_armor(0)|body_armor(14)|leg_armor(10)|difficulty(0),imodbits_none],

["sleeveless_leather_tunic","Sleeveless Leather Tunic",[("nobleman_outf",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,348,weight(4)|abundance(100)|head_armor(0)|body_armor(14)|leg_armor(10)|difficulty(0),imodbits_none],

["linen_shirt","Linen Shirt",[("linen_tunic",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,348,weight(4)|abundance(100)|head_armor(0)|body_armor(14)|leg_armor(10)|difficulty(0),imodbits_none],

["shirt","Shirt",[("shirt",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,3,weight(1)|abundance(100)|head_armor(0)|body_armor(5)|leg_armor(0)|difficulty(0),imodbits_none],
 
["linen_tunic","Linen Tunic",[("shirt_a",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,6,weight(1)|abundance(100)|head_armor(0)|body_armor(6)|leg_armor(1)|difficulty(0),imodbits_none],
 
["short_tunic","Green Tunic",[("rich_tunic_a",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1)|abundance(100)|head_armor(0)|body_armor(7)|leg_armor(1)|difficulty(0),imodbits_none],

["red_shirt","Red Shirt",[("rich_tunic_a",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1)|abundance(100)|head_armor(0)|body_armor(7)|leg_armor(1)|difficulty(0),imodbits_none],

["red_tunic","Red Tunic",[("arena_tunicR_new",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1)|abundance(100)|head_armor(0)|body_armor(7)|leg_armor(1)|difficulty(0),imodbits_none],
 
["green_tunic","Green Tunic",[("arena_tunicG_new",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1)|abundance(100)|head_armor(0)|body_armor(7)|leg_armor(1)|difficulty(0),imodbits_none],

["blue_tunic","Blue Tunic",[("arena_tunicB_new",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1)|abundance(100)|head_armor(0)|body_armor(7)|leg_armor(1)|difficulty(0),imodbits_none],

["robe","Robe",[("robe",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,31,weight(1.5)|abundance(100)|head_armor(0)|body_armor(8)|leg_armor(6)|difficulty(0),imodbits_none],

["coarse_tunic","Tunic with vest",[("coarse_tunic_a",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,47,weight(2)|abundance(100)|head_armor(0)|body_armor(11)|leg_armor(6)|difficulty(0),imodbits_none],

["leather_apron","Leather Apron",[("leather_apron",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,61,weight(3)|abundance(100)|head_armor(0)|body_armor(12)|leg_armor(7)|difficulty(0),imodbits_none],

["tabard","Tabard",[("tabard_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,100,weight(3)|abundance(100)|head_armor(0)|body_armor(14)|leg_armor(6)|difficulty(0),imodbits_none],

["sarranid_cloth_robe","Worn Robe",[("sar_robe",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,33,weight(1)|abundance(100)|head_armor(0)|body_armor(9)|leg_armor(9)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_cloth_robe_b","Worn Robe",[("sar_robe_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,33,weight(1)|abundance(100)|head_armor(0)|body_armor(9)|leg_armor(9)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["wool_coat","Wool Coat",[("nobleman_outf",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,348,weight(4)|abundance(100)|head_armor(0)|body_armor(14)|leg_armor(10)|difficulty(0),imodbits_none],

["female_civ_clothes_start","Female Civilian Clothes Start (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#FEMALE CIVILIAN GEAR
["red_dress","Red Dress",[("red_dress",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,200,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none],

["brown_dress","Brown Dress",[("brown_dress",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,200,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none],

["green_dress","Green Dress",[("green_dress",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,200,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none],

["sarranid_common_dress","Sarranid Dress",[("sarranid_common_dress",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,200,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_common_dress_b","Sarranid Dress",[("sarranid_common_dress_b",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,200,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["dress","Dress",[("blue_dress_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,6,weight(1)|abundance(100)|head_armor(0)|body_armor(6)|leg_armor(2)|difficulty(0),imodbits_none],

["blue_dress","Blue Dress",[("blue_dress_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,6,weight(1)|abundance(100)|head_armor(0)|body_armor(6)|leg_armor(2)|difficulty(0),imodbits_none],

["peasant_dress","Peasant Dress",[("peasant_dress_b_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,6,weight(1)|abundance(100)|head_armor(0)|body_armor(6)|leg_armor(2)|difficulty(0),imodbits_none],

["woolen_dress01","Woolen Dress",[("woolen_dress01",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(8)|leg_armor(2)|difficulty(0),imodbits_none],

["woolen_dress02","Woolen Dress",[("woolen_dress02",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(8)|leg_armor(2)|difficulty(0),imodbits_none],

["woolen_dress03","Woolen Dress",[("woolen_dress03",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(8)|leg_armor(2)|difficulty(0),imodbits_none],

["woolen_dress04","Woolen Dress",[("woolen_dress04",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(8)|leg_armor(2)|difficulty(0),imodbits_none],

["woolen_dress05","Woolen Dress",[("woolen_dress05",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(8)|leg_armor(2)|difficulty(0),imodbits_none],

["woolen_dress06","Woolen Dress",[("woolen_dress06",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,200,weight(1.75)|abundance(100)|head_armor(0)|body_armor(8)|leg_armor(2)|difficulty(0),imodbits_none],
 
["sarranid_dress_a","Dress",[("woolen_dress",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,33,weight(1)|abundance(100)|head_armor(0)|body_armor(9)|leg_armor(9)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_dress_b","Dress",[("woolen_dress",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,33,weight(1)|abundance(100)|head_armor(0)|body_armor(9)|leg_armor(9)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["female_civ_clothes_end","Female Civilian Clothes End (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#Quest-specific - perhaps can be used for prisoners,
["burlap_tunic","Burlap Tunic",[("shirt",0)],itp_type_body_armor|itp_covers_legs,0,5,weight(1)|abundance(100)|head_armor(0)|body_armor(3)|leg_armor(1)|difficulty(0),imodbits_none],

#Noble clothing- nothing special except the cost,and the fact it's available as merchandise.  I'd like to add a bonus to Persuasion for them later- dressing appropriately (for the culture) would be nice.
#Special Note: There was a *ton* of junk here,due to items that had been removed from the game,duplicate entries in all but name,etc.  These problems have been fixed throughout the code.
["nobleman_outfit","Nobleman Outfit",[("nobleman_outfit_b_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(15)|leg_armor(12)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_rich_tunic01","Nice Tunic",[("wei_xiadi_rich_tunic01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(15)|leg_armor(12)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_rich_tunic02","Nice Tunic",[("wei_xiadi_rich_tunic02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(15)|leg_armor(12)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["merchant_outfit","Merchant Outfit",[("nobleman_outf",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(15)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["fur_coat","Fur Coat",[("fur_coat",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,600,weight(6)|abundance(100)|head_armor(0)|body_armor(13)|leg_armor(6)|difficulty(0),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["wei_xiadi_kher_kaftan","Caftan",[("wei_xiadi_kher_kaftan",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,600,weight(6)|abundance(100)|head_armor(0)|body_armor(13)|leg_armor(6)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_kher_robe","Robe",[("wei_xiadi_kher_robe",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,600,weight(6)|abundance(100)|head_armor(0)|body_armor(13)|leg_armor(6)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["lady_dress_ruby","Lady Dress",[("lady_dress_r",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["lady_dress_green","Lady Dress",[("lady_dress_g",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["lady_dress_blue","Lady Dress",[("lady_dress_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["khergit_lady_dress","Khergit Lady Dress",[("khergit_lady_dress",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["khergit_lady_dress_b","Khergit Leather Lady Dress",[("khergit_lady_dress_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["sarranid_lady_dress","Sarranid Lady Dress",[("sarranid_lady_dress",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_lady_dress_b","Sarranid Lady Dress",[("sarranid_lady_dress_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["court_dress","Court Dress",[("court_dress",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(14)|leg_armor(4)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rich_outfit","Rich Outfit",[("merchant_outf",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(4)|abundance(100)|head_armor(0)|body_armor(16)|leg_armor(4)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#Padded Armor
#This is armor that's light and cheap and has no requirement,but pretty darn weak otherwise.  Still quite a lot more expensive than ordinary clothing.
#Arranged by cost / protection.

#DtheHun's Anar dresses- treated as very light armors
["anar2_dress","Anar Dress",[("anar2_dress",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_attach_armature,0,750,weight(10)|abundance(100)|head_armor(0)|body_armor(30)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["anar_dress","Anar Dress",[("anar_dress",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_attach_armature,0,750,weight(10)|abundance(100)|head_armor(0)|body_armor(30)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["archers_vest","Archer's Padded Vest",[("archers_vest",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,700,weight(10)|abundance(100)|head_armor(0)|body_armor(30)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["pino_peasant_archer","Archer's Padded Tunic",[("pino_peasant_archer",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,1000,weight(10)|abundance(100)|head_armor(0)|body_armor(40)|leg_armor(20)|difficulty(0),imodbits_none,[],[fac_kingdom_of_swadians]],

["pelt_coat","Pelt Coat",[("thick_coat_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,1000,weight(11)|abundance(100)|head_armor(0)|body_armor(40)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["nomad_robe","Nomad Robe",[("nomad_robe_a",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs|itp_civilian,0,1200,weight(12)|abundance(100)|head_armor(0)|body_armor(45)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["padded_cloth","Aketon",[("padded_cloth_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1200,weight(12)|abundance(100)|head_armor(0)|body_armor(45)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["padded_armor","Padded Armor",[("padded_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,2000,weight(12)|abundance(100)|head_armor(0)|body_armor(55)|leg_armor(25)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["wei_xiadi_sar_jellaba_blue","Blue Jellaba",[("wei_xiadi_sar_jellaba_blue",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1200,weight(12)|abundance(100)|head_armor(0)|body_armor(45)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["wei_xiadi_sar_jellaba_white","White Jellaba",[("wei_xiadi_sar_jellaba_white",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1200,weight(12)|abundance(100)|head_armor(0)|body_armor(45)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["wei_xiadi_sar_robe01","Robe",[("wei_xiadi_sar_robe01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1200,weight(12)|abundance(100)|head_armor(0)|body_armor(45)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["wei_xiadi_sar_robe02","Robe",[("wei_xiadi_sar_robe02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1200,weight(12)|abundance(100)|head_armor(0)|body_armor(45)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["gambeson","Gambeson",[("white_gambeson",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1500,weight(13)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["blue_gambeson","Blue Gambeson",[("blue_gambeson",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1500,weight(13)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["red_gambeson","Red Gambeson",[("red_gambeson_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1500,weight(13)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_rod_gambeson","Green Gambeson",[("wei_xiadi_rod_gambeson",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1500,weight(13)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["narf_gambeson","White Gambeson",[("narf_gambeson",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1500,weight(13)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(10)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["aketon_green","Padded Cloth",[("padded_cloth_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1700,weight(14)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_rod_padded_cloth","Padded Cloth",[("wei_xiadi_rod_padded_cloth",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1700,weight(14)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["sarranid_cavalry_robe","Cavalry Robe",[("arabian_armor_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,1700,weight(14)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(15)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["skirmisher_armor","Skirmisher Armor",[("skirmisher_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2000,weight(15)|abundance(100)|head_armor(0)|body_armor(50)|leg_armor(20)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

 #Leather Armor
 #This is the first class where Strength requirements come into play.
 #They are heavier than all the cloth armors,but are still fairly cheap and light.  These are the first "serious" armors,in terms of practical protection.
#Exception for the jacket,as it doesn't provide leg protection.
["leather_jacket","Leather Jacket",[("leather_jacket_new",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,2200,weight(11)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(0)|difficulty(5),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits]],
 
#Vests and jerkins
["leather_vest","Leather Vest",[("leather_vest_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,2200,weight(16)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(25)|difficulty(5),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["nomad_vest","Nomad Vest",[("nomad_vest_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,2200,weight(16)|abundance(50)|head_armor(0)|body_armor(60)|leg_armor(25)|difficulty(5),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["leather_jerkin","Leather Jerkin",[("ragged_leather_jerkin",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,2200,weight(16)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(25)|difficulty(5),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],
 #Coats
["rawhide_coat","Rawhide Coat",[("tribal_warrior_outfit_a_new",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,2500,weight(17)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(30)|difficulty(6),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["wei_xiadi_rod_thick_coat","Thick Coat",[("wei_xiadi_rod_thick_coat",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,2500,weight(17)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(30)|difficulty(6),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["ragged_outfit","Ragged Outfit",[("ragged_outfit_a_new",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,2500,weight(17)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(30)|difficulty(6),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["wei_xiadi_kher_ragged_outfit","Ragged Outfit",[("wei_xiadi_kher_ragged_outfit",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,2500,weight(17)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(30)|difficulty(6),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["pino_peasant_man_leather","Archer's Leathers",[("pino_peasant_man_leather",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,2700,weight(17)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(45)|difficulty(6),imodbits_none,[],[fac_kingdom_of_swadians]],
 
#Hide / Fur / Light cured Leather
["nomad_armor","Nomad Armor",[("nomad_armor_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["steppe_armor","Lamellar Leather Armor",[("khergit_armor_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(70)|leg_armor(20)|difficulty(7),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["xenoargh_lamellar_leather_variant001","Lamellar Leather Armor",[("xenoargh_lamellar_leather_variant001",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_leather_variant002","Lamellar Leather Armor",[("xenoargh_lamellar_leather_variant002",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(70)|leg_armor(25)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_leather_variant003","Lamellar Leather Armor",[("xenoargh_lamellar_leather_variant003",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(60)|leg_armor(35)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_leather_variant004","Lamellar Leather Armor",[("xenoargh_lamellar_leather_variant004",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_leather_variant005","Lamellar Leather Armor",[("xenoargh_lamellar_leather_variant005",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_leather_variant006","Lamellar Leather Armor",[("xenoargh_lamellar_leather_variant006",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_kher_lamellar_vest01","Lamellar Leather Armor",[("wei_xiadi_kher_lamellar_vest01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_kher_lamellar_vest02","Lamellar Leather Armor",[("wei_xiadi_kher_lamellar_vest02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["tribal_warrior_outfit","Tribal Warrior Outfit",[("tribal_warrior_outfit_a_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits,fac_kingdom_of_sarranids]],

["khergit_armor","Khergit Armor",[("coat_of_plates_b",0)],itp_merchandise|itp_covers_legs|itp_type_body_armor,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(65)|leg_armor(30)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_kher_skirmisher_armor","Khergit Skirmisher's Armor",[("wei_xiadi_kher_skirmisher_armor",0)],itp_merchandise|itp_covers_legs|itp_type_body_armor,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(70)|leg_armor(20)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["pino_peasant_leather_scale","Leather Scale Armor",[("pino_peasant_leather_scale",0)],itp_merchandise|itp_covers_legs|itp_type_body_armor,0,2700,weight(18)|abundance(100)|head_armor(0)|body_armor(75)|leg_armor(25)|difficulty(7),imodbits_none,[],[fac_kingdom_of_rhodoks]],

#Cured Leather
["light_leather","Light Leather",[("light_leather",0)],itp_type_body_armor|itp_covers_legs|itp_merchandise,0,2700,weight(5)|abundance(100)|head_armor(0)|body_armor(75)|leg_armor(40)|difficulty(7),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["leather_armor","Leather Armor",[("tattered_leather_armor_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3000,weight(19)|abundance(100)|head_armor(0)|body_armor(80)|leg_armor(40)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["sarranid_leather_armor","Sarranid Leather Armor",[("sarranid_leather_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3300,weight(19)|abundance(100)|head_armor(0)|body_armor(85)|leg_armor(40)|difficulty(8),imodbits_none,[],[fac_kingdom_of_sarranids]],

["wei_xiadi_sar_leather_armor","Nomad's Leather Armor",[("wei_xiadi_sar_leather_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3300,weight(19)|abundance(100)|head_armor(0)|body_armor(80)|leg_armor(45)|difficulty(8),imodbits_none,[],[fac_kingdom_of_sarranids]],

["padded_leather","Padded Leather",[("leather_armor_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,3200,weight(20)|abundance(100)|head_armor(0)|body_armor(80)|leg_armor(40)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_green_leather_armor","Green Padded Leather",[("xenoargh_green_leather_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,4000,weight(20)|abundance(15)|head_armor(0)|body_armor(90)|leg_armor(50)|difficulty(8),imodbits_none,[],[fac_kingdom_of_swadians]],

["studded_leather_coat","Studded Leather Coat",[("leather_armor_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3500,weight(21)|abundance(100)|head_armor(0)|body_armor(80)|leg_armor(40)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["pino_peasant_man_spiked_mail","Spiked Mail",[("pino_peasant_man_spiked_mail",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3500,weight(21)|abundance(100)|head_armor(0)|body_armor(90)|leg_armor(20)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["pino_peasant_man_leather_mail","Leather with Mail",[("pino_peasant_man_leather_mail",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3500,weight(21)|abundance(100)|head_armor(0)|body_armor(85)|leg_armor(25)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["pino_peasant_archer_mail","Archer's Mail",[("pino_peasant_archer_mail",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3500,weight(21)|abundance(100)|head_armor(0)|body_armor(80)|leg_armor(35)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["pino_peasant_leather_ring","Light Ringmail",[("pino_peasant_leather_ring",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3500,weight(21)|abundance(100)|head_armor(0)|body_armor(80)|leg_armor(30)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["brigandine_red","Brigandine",[("brigandine_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3500,weight(21)|abundance(100)|head_armor(0)|body_armor(90)|leg_armor(45)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_rod_brigandine","Brigandine",[("wei_xiadi_rod_brigandine",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3500,weight(21)|abundance(100)|head_armor(0)|body_armor(90)|leg_armor(45)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#DtheHun's brilliant Rogue Armor and Blademaster armor, treated as leather with light plates (utterly unrealistic, but hey, it's a game)
["blademaster_armor","Female Barbarian Armor",[("blademaster_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3550,weight(21)|abundance(100)|head_armor(0)|body_armor(85)|leg_armor(40)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rogue_armor1","Female Rogue's Armor",[("rogue_armor1",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3550,weight(21)|abundance(100)|head_armor(0)|body_armor(85)|leg_armor(40)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]], 

["rogue_armor2","Female Rogue's Armor",[("rogue_armor2",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,3550,weight(21)|abundance(100)|head_armor(0)|body_armor(85)|leg_armor(45)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]], 

#Chain Mail
#This is serious (and expensive) protection.  Included in this category are things like byrnies and coats of plates,which had a similar level of protection,but not banded armor,which is actually plate armor.  Chain mails are still fairly lightweight,but are past the "trigger point" in terms of having to level up Companions and give them enough Strength to actually wear them.  There are a lot of sub-categories,so I've broken it up a bit.
#Light mails
["mail_shirt","Mail Shirt",[("mail_shirt_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,5000,weight(22)|abundance(100)|head_armor(0)|body_armor(90)|leg_armor(45)|difficulty(10),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["sarranid_mail_shirt","Sarranid Mail Shirt",[("sarranian_mail_shirt",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,5000,weight(22)|abundance(100)|head_armor(0)|body_armor(90)|leg_armor(45)|difficulty(10),imodbits_none,[],[fac_kingdom_of_sarranids]],

["wei_xiadi_sar_byrnie","Sarranid Mail Byrnie",[("wei_xiadi_sar_byrnie",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,5000,weight(22)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(35)|difficulty(10),imodbits_none,[],[fac_kingdom_of_sarranids]],

["pino_peasant_archer_scale","Light Scale Armor",[("pino_peasant_archer_scale",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,6000,weight(22)|abundance(100)|head_armor(0)|body_armor(95)|leg_armor(45)|difficulty(10),imodbits_none,[],[fac_kingdom_of_khergits,fac_kingdom_of_vaegirs]],

["mail_hauberk","Mail Hauberk",[("hauberk_a_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,7000,weight(26)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_swa_hauberk","Mail Hauberk",[("wei_xiadi_swa_hauberk",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,7000,weight(26)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_swadians]],

["wei_xiadi_sar_hauberk","Sarranid Hauberk",[("wei_xiadi_sar_hauberk",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8000,weight(26)|abundance(100)|head_armor(0)|body_armor(110)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_sarranids]],

#Byrnies / Haubergeons
["byrnie","Byrnie",[("byrnie_a_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8000,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords]],

["rathos_mail_byrnie","Heavy Byrnie",[("rathos_mail_byrnie",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9000,weight(27)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(50)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords]],

["haubergeon","Haubergeon",[("haubergeon_c",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8000,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_mail_shirt01","Heavy Mail Shirt",[("wei_xiadi_mail_shirt01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9000,weight(27)|abundance(100)|head_armor(0)|body_armor(115)|leg_armor(55)|difficulty(11),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_mail_shirt02","Heavy Mail Shirt",[("wei_xiadi_mail_shirt02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9000,weight(27)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_nord_leather_chain","Mail Shirt With Leather",[("wei_xiadi_nord_leather_chain",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9000,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(65)|difficulty(11),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["pino_haubergeon_jco","Heavy Haubergeon",[("pino_haubergeon_jco",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9000,weight(27)|abundance(100)|head_armor(0)|body_armor(115)|leg_armor(55)|difficulty(11),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#DtheHun's Aribeth / Warrior Princess armors, treated like heavy chainmail.  Again, not realistic, but it's a game.
["aribeth_armor","Female Light Plate",[("aribeth_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8050,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords]],

["aribeth_armor_dark","Female Light Plate",[("aribeth_armor_dark",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8050,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_new_female_armor01","Female Light Plate",[("xenoargh_new_female_armor01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8050,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords,fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["xenoargh_female_armor_sisters_of_mercy1","Sister's Light Plate",[("xenoargh_female_armor_sisters_of_mercy1",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8050,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords,fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["xenoargh_new_female_armor_blue01","Female Blue Light Plate",[("xenoargh_new_female_armor_blue01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8050,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords,fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["xenoargh_new_female_armor_green01","Female Green Light Plate",[("xenoargh_new_female_armor_green01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8050,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords,fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["xenoargh_new_female_armor_orange01","Female Orange Light Plate",[("xenoargh_new_female_armor_orange01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8050,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords,fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["xena_armor","Female Reinforced Leather",[("xena_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8050,weight(27)|abundance(100)|head_armor(0)|body_armor(105)|leg_armor(60)|difficulty(11),imodbits_none,[],[fac_kingdom_of_nords]],
 
#Lamellar armor
#Mediocre armor, not quite protective enough to fully protect vs. bows, and not nearly enough vs. 2-handers.  And it's not even cheap.
#My heavy leather lamellar with breastplates
["xenoargh_lamellar_breastplate_variant001","Lamellar Leather w/ Breastplate",[("xenoargh_lamellar_breastplate_variant001",0),("xenoargh_lamellar_breastplate_variant001",ixmesh_inventory)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8500,weight(24)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(30)|difficulty(11),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_breastplate_variant002","Lamellar Leather w/ Breastplate",[("xenoargh_lamellar_breastplate_variant002",0),("xenoargh_lamellar_breastplate_variant002",ixmesh_inventory)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8500,weight(24)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(30)|difficulty(11),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_breastplate_variant003","Lamellar Leather w/ Breastplate",[("xenoargh_lamellar_breastplate_variant003",0),("xenoargh_lamellar_breastplate_variant003",ixmesh_inventory)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8500,weight(24)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(30)|difficulty(11),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_breastplate_variant004","Lamellar Leather w/ Breastplate",[("xenoargh_lamellar_breastplate_variant004",0),("xenoargh_lamellar_breastplate_variant004",ixmesh_inventory)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8500,weight(24)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(30)|difficulty(11),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_breastplate_variant005","Lamellar Leather w/ Breastplate",[("xenoargh_lamellar_breastplate_variant005",0),("xenoargh_lamellar_breastplate_variant005",ixmesh_inventory)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8500,weight(24)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(30)|difficulty(11),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_lamellar_breastplate_variant006","Lamellar Leather w/ Breastplate",[("xenoargh_lamellar_breastplate_variant006",0),("xenoargh_lamellar_breastplate_variant006",ixmesh_inventory)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8500,weight(24)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(30)|difficulty(11),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_nord_lamellar_plain","Lamellar Leather w/ Breastplate",[("wei_xiadi_nord_lamellar_plain",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8500,weight(24)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(30)|difficulty(11),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_nord_lamellar_purple","Lamellar Leather w/ Breastplate",[("wei_xiadi_nord_lamellar_purple",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,8500,weight(24)|abundance(100)|head_armor(0)|body_armor(100)|leg_armor(30)|difficulty(11),imodbits_none,[],[fac_kingdom_of_khergits]],
 
["khergit_guard_armor","Khergit Guard Armor",[("lamellar_armor_a",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,9000,weight(28)|abundance(100)|head_armor(0)|body_armor(115)|leg_armor(60)|difficulty(12),imodbits_none,[],[fac_kingdom_of_khergits]],
 
["lamellar_vest","Lamellar Vest",[("lamellar_vest_a",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,9000,weight(28)|abundance(100)|head_armor(0)|body_armor(110)|leg_armor(65)|difficulty(12),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits]],

["lamellar_vest_khergit","Khergit Lamellar Vest",[("lamellar_vest_b",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,9000,weight(28)|abundance(100)|head_armor(0)|body_armor(115)|leg_armor(60)|difficulty(12),imodbits_none,[],[fac_kingdom_of_khergits]],

["pinky_lamellar_vest","Scaled Lamellar Vest",[("pinky_lamellar_vest",0)],itp_merchandise|itp_type_body_armor|itp_civilian|itp_covers_legs,0,9500,weight(28)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(40)|difficulty(12),imodbits_none,[],[fac_kingdom_of_khergits]],

["lamellar_armor","Lamellar Armor",[("lamellar_armor_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9000,weight(28)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(50)|difficulty(13),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits]],

["arabian_armor_b","Sarranid Guard Armor",[("arabian_armor_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9000,weight(28)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(50)|difficulty(13),imodbits_none,[],[fac_kingdom_of_sarranids]],

["khergit_elite_armor","Khergit Elite Armor",[("lamellar_armor_c",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(28)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(60)|difficulty(13),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_lamellar_armor03","Khergit Lamellar With Leather",[("wei_xiadi_lamellar_armor03",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9500,weight(28)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(50)|difficulty(13),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_lamellar_armor01","Khergit Heavy Lamellar",[("wei_xiadi_lamellar_armor01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(28)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(60)|difficulty(13),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_lamellar_chainmail","Lamellar With Chainmail",[("wei_xiadi_lamellar_chainmail",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(28)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(80)|difficulty(13),imodbits_none,[],[fac_kingdom_of_khergits]],

["vaegir_elite_armor","Vaegir Elite Armor",[("lamellar_armor_d",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(28)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(60)|difficulty(13),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["sarranid_elite_armor","Sarranid Elite Armor",[("tunic_armor_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,10000,weight(28)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(60)|difficulty(13),imodbits_none,[],[fac_kingdom_of_khergits]],

["mamluke_mail","Mamluke Mail",[("sarranid_elite_cavalry_fixed",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,11000,weight(28)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(60)|difficulty(13),imodbits_none,[],[fac_kingdom_of_sarranids]],

["narf_rus_lamellar_a","Ruski Lamellar",[("narf_rus_lamellar_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9200,weight(28)|abundance(100)|head_armor(0)|body_armor(110)|leg_armor(70)|difficulty(13),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_rus_lamellar_b","Ruski Lamellar",[("narf_rus_lamellar_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9200,weight(28)|abundance(100)|head_armor(0)|body_armor(110)|leg_armor(70)|difficulty(13),imodbits_none,[],[fac_kingdom_of_vaegirs]],

#Scale armor and coats of plates.  Starting from here, armor starts being highly resistant to arrows.
["narf_kuyak_a","Ruski Kuyak",[("narf_kuyak_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(30)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_kuyak_b","Ruski Kuyak",[("narf_kuyak_b",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(50)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["pino_peasant_leather_plate","Jack of Plates",[("pino_peasant_leather_plate",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10500,weight(30)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_nords]],

["wei_xiadi_sar_leather_chain","Sarranid Chain With Leather",[("wei_xiadi_sar_leather_chain",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(30)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_sarranids]],

["cuir_bouilli","Cuir Bouilli",[("cuir_bouilli_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(30)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_nord_cuir_bouilli","Cuir Bouilli",[("wei_xiadi_nord_cuir_bouilli",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10500,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_hard_leather01","Cuir Bouilli",[("xenoargh_hard_leather01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(55)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["xenoargh_hard_leather02","Cuir Bouilli",[("xenoargh_hard_leather02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(55)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["wei_xiadi_scale_armor02","Light Scale Armor",[("wei_xiadi_scale_armor02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,9500,weight(30)|abundance(100)|head_armor(0)|body_armor(115)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["scale_armor","Scale Armor",[("lamellar_armor_e",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10000,weight(30)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_scale_armor01","Heavy Scale Armor",[("wei_xiadi_scale_armor01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(140)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["coat_of_plates","Coat of Plates",[("coat_of_plates_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_rod_coat_of_plates","Coat of Plates",[("wei_xiadi_rod_coat_of_plates",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11500,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["coat_of_plates_red","Coat of Plates",[("coat_of_plates_red",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_samurai_armor01","O-Yoroi Armor",[("wei_xiadi_samurai_armor01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(28)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_samurai_armor02","O-Yoroi Armor",[("wei_xiadi_samurai_armor02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(28)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(75)|difficulty(14),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_nord_spiked_mail","Heavy Spiked Mail",[("wei_xiadi_nord_spiked_mail",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["narf_rus_scale","Ruski Scale",[("narf_rus_scale",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11500,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["wei_xiadi_lamellar_armor02","Khergit Elite Lamellar",[("wei_xiadi_lamellar_armor02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs|itp_civilian,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(140)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_khergits]],

#DtheHun's female plate armor
["blademaster_armor_chainmail","Female Barbarian Chainmail",[("blademaster_armor_chainmail",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rogue_armor_blue","Female Rogue's Chain",[("rogue_armor_blue",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_sarranids,fac_kingdom_of_khergits]],

["rogue_armor_blue2","Female Rogue's Chain",[("rogue_armor_blue2",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(120)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_sarranids,fac_kingdom_of_khergits]],

["aribeth_armor_dark_chainmail","Female Dark Armor",[("aribeth_armor_dark_chainmail",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor02","Female Medium Plate",[("xenoargh_new_female_armor02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor03","Female Medium Plate",[("xenoargh_new_female_armor03",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11050,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_female_armor_sisters_of_mercy2","Sister's Medium Plate",[("xenoargh_female_armor_sisters_of_mercy2",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_female_armor_sisters_of_mercy3","Sister's Medium Plate",[("xenoargh_female_armor_sisters_of_mercy3",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11050,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_blue02","Female Blue Medium Plate",[("xenoargh_new_female_armor_blue02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_blue03","Female Blue Medium Plate",[("xenoargh_new_female_armor_blue03",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11050,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_green02","Female Green Medium Plate",[("xenoargh_new_female_armor_green02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_green03","Female Green Medium Plate",[("xenoargh_new_female_armor_green03",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11050,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_orange02","Female Orange Medium Plate",[("xenoargh_new_female_armor_orange02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10050,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_orange03","Female Orange Medium Plate",[("xenoargh_new_female_armor_orange03",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11050,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#Maille with surcoats.  Better protection for both arms and legs here from maille and heavy undergarments.
["mail_with_surcoat","Mail with Surcoat",[("mail_long_surcoat_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["surcoat_over_mail","Surcoat over Mail",[("surcoat_over_mail_new",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_surcoat_english","Heraldic Mail with Surcoat",[("rathos_surcoat_english",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_surcoat_redyellow","Heraldic Mail",[("rathos_surcoat_redyellow",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(140)|leg_armor(80)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_surcoat_halfblue","Heraldic Mail",[("rathos_surcoat_halfblue",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(150)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_surcoat_halfred","Heraldic Mail",[("rathos_surcoat_halfred",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11500,weight(30)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(75)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_surcoat_white","Heraldic Mail with Tabard",[("rathos_surcoat_white",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_sar_plain_surcoat","Sarranid Mail With Surcoat",[("wei_xiadi_sar_plain_surcoat",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12500,weight(30)|abundance(100)|head_armor(0)|body_armor(150)|leg_armor(80)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_sarranid_mamluk_robes","Mamluk Robes",[("wei_xiadi_sarranid_mamluk_robes",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,13000,weight(30)|abundance(100)|head_armor(0)|body_armor(150)|leg_armor(80)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_surcoat_rhodok01","Rhodok Surcoat With Mail",[("wei_xiadi_surcoat_rhodok01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_surcoat_rhodok02","Rhodok Surcoat With Mail",[("wei_xiadi_surcoat_rhodok02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(145)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_swa_surcoat_fancy01","Swadian Surcoat With Mail",[("wei_xiadi_swa_surcoat_fancy01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(65)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_swa_surcoat_fancy02","Swadian Surcoat With Mail",[("wei_xiadi_swa_surcoat_fancy02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(130)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_swa_surcoat_fancy03","Swadian Surcoat With Mail",[("wei_xiadi_swa_surcoat_fancy03",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12500,weight(30)|abundance(100)|head_armor(0)|body_armor(150)|leg_armor(80)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_swa_surcoat_green","Swadian Surcoat With Mail",[("wei_xiadi_swa_surcoat_green",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(80)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_swa_surcoat_purple","Swadian Surcoat With Mail",[("wei_xiadi_swa_surcoat_purple",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,10500,weight(30)|abundance(100)|head_armor(0)|body_armor(125)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_swa_surcoat_v","Swadian Surcoat With Mail",[("wei_xiadi_swa_surcoat_v",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11500,weight(30)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(75)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_swa_tabard","Swadian Tabard With Mail",[("wei_xiadi_swa_tabard",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(140)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_templar_surcoat_red","Templar Surcoat With Mail",[("wei_xiadi_templar_surcoat_red",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11500,weight(30)|abundance(100)|head_armor(0)|body_armor(135)|leg_armor(75)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["wei_xiadi_templar_surcoat_white","Templar Surcoat With Mail",[("wei_xiadi_templar_surcoat_white",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,11000,weight(30)|abundance(100)|head_armor(0)|body_armor(140)|leg_armor(60)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#Wei Xiadi's Cataphract armor- really, really heavy scale, basically:
["wei_xiadi_cataphract_armor","Cataphract's Scale",[("wei_xiadi_cataphract_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(140)|leg_armor(75)|difficulty(14),imodbits_none,[],[fac_kingdom_of_sarranids]],

["wei_xiadi_sarranid_mamluk_armor","Mamluke's Scale",[("wei_xiadi_sarranid_mamluk_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,12000,weight(30)|abundance(100)|head_armor(0)|body_armor(145)|leg_armor(90)|difficulty(14),imodbits_none,[],[fac_kingdom_of_sarranids]], 

 #Plate Mail
 #This is the penultimate armor for the time period.  Cost is huge.
 #Provides excellent protection,but at a serious cost in terms of weight and difficulty,as well as money.
 
 #Dejawolf's Byrnie occupies an unrealistic position in this hierarchy, simply because it's awesome.
["dejawolf_vikingbyrnie_blue","Carls's Byrnie (elite)",[("dejawolf_vikingbyrnie_blue",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,14000,weight(37)|abundance(100)|head_armor(0)|body_armor(140)|leg_armor(70)|difficulty(14),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_vikingbyrnie","Thegn's Byrnie (elite)",[("dejawolf_vikingbyrnie",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,16000,weight(37)|abundance(100)|head_armor(0)|body_armor(150)|leg_armor(90)|difficulty(15),imodbits_none,[],[fac_kingdom_of_nords]],
 
["light_mail_and_plate","Light Mail and Plate",[("light_mail_and_plate",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,14000,weight(37)|abundance(100)|head_armor(0)|body_armor(140)|leg_armor(80)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_sarranids,fac_kingdom_of_swadians,fac_kingdom_of_nords]],
 
["banded_armor","Banded Armor",[("banded_armor_a",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,15000,weight(40)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(85)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs,fac_kingdom_of_khergits]],

#Youhou's awesome Assassin armors
["youhou_assassin_armor","Assassin's Armor",[("youhou_assassin_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,15500,weight(40)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(90)|difficulty(14),imodbits_none,[],[fac_kingdom_of_sarranids]],

["youhou_assassin_armor_red","Red Assassin's Armor",[("youhou_assassin_armor_red",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,15500,weight(40)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(90)|difficulty(14),imodbits_none,[],[fac_kingdom_of_sarranids]],

#Narf's Transitional Armors- mixed mail-and-plate, not quite full plate mail
["narf_corrazina_green","Corrazina",[("narf_corrazina_green",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,16000,weight(42)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(80)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["narf_corrazina_grey","Corrazina",[("narf_corrazina_grey",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,16000,weight(42)|abundance(100)|head_armor(0)|body_armor(150)|leg_armor(90)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["narf_corrazina_red","Corrazina",[("narf_corrazina_red",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,16000,weight(42)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(70)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["wei_xiadi_swa_mail_and_plate","Mail and Plate",[("wei_xiadi_swa_mail_and_plate",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(90)|difficulty(15),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["wei_xiadi_swa_mail_and_plate_black","Mail and Plate",[("wei_xiadi_swa_mail_and_plate_black",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(165)|leg_armor(80)|difficulty(15),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["ssh_yoroi","O-Yoroi Elite Armor",[("ssh_yoroi",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,22000,weight(35)|abundance(100)|head_armor(0)|body_armor(165)|leg_armor(80)|difficulty(15),imodbits_none,[],[fac_kingdom_of_khergits]],

["wei_xiadi_swa_mail_and_plate_blue","Mail and Plate",[("wei_xiadi_swa_mail_and_plate_blue",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(90)|difficulty(15),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["narf_early_transitional_blue","Mail and Plate",[("narf_early_transitional_blue",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(90)|difficulty(15),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["narf_early_transitional_orange","Mail and Plate",[("narf_early_transitional_orange",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(150)|leg_armor(95)|difficulty(15),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["narf_early_transitional_white","Mail and Plate",[("narf_early_transitional_white",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(85)|difficulty(15),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["narf_early_transitional_templar","Templar's Plate",[("narf_early_transitional_templar",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,21000,weight(45)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(95)|difficulty(15),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["mail_and_plate","Mail and Plate",[("mail_and_plate",0)],itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(85)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_metal_leather01","Mail and Plate",[("xenoargh_metal_leather01",0)],itp_type_body_armor|itp_covers_legs,0,21000,weight(45)|abundance(100)|head_armor(0)|body_armor(155)|leg_armor(90)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["xenoargh_metal_leather02","Mail and Plate",[("xenoargh_metal_leather02",0)],itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(150)|leg_armor(85)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["narf_armour_slashed","Mail and Plate",[("narf_armour_slashed",0)],itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(75)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_armour_stripes","Mail and Plate",[("narf_armour_stripes",0)],itp_type_body_armor|itp_covers_legs,0,20000,weight(45)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(75)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["xenoargh_new_female_armor04","Female Heavy Plate",[("xenoargh_new_female_armor04",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,21050,weight(30)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(95)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_female_armor_sisters_of_mercy4","Sister's Heavy Plate",[("xenoargh_female_armor_sisters_of_mercy4",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,21050,weight(30)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(95)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_blue04","Female Blue Heavy Plate",[("xenoargh_new_female_armor_blue04",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,21050,weight(30)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(95)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_green04","Female Green Heavy Plate",[("xenoargh_new_female_armor_green04",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,21050,weight(30)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(95)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_new_female_armor_orange04","Female Orange Heavy Plate",[("xenoargh_new_female_armor_orange04",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,21050,weight(30)|abundance(100)|head_armor(0)|body_armor(160)|leg_armor(95)|difficulty(15),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#DtheHun's Female Armors, top level female plate.  If you want better protection than this, you need to wear non-girly stuff.
["dthehun_new_battle_armor01","Female Plate Mail",[("dthehun_new_battle_armor01",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,22050,weight(47)|abundance(100)|head_armor(0)|body_armor(170)|leg_armor(80)|difficulty(16),imodbits_none,[],[fac_kingdom_of_swadians]],

["dthehun_new_battle_armor02","Female Plate Mail",[("dthehun_new_battle_armor02",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,23050,weight(47)|abundance(100)|head_armor(0)|body_armor(175)|leg_armor(80)|difficulty(16),imodbits_none,[],[fac_kingdom_of_swadians]],

["dthehun_new_battle_armor03","Female Plate Mail",[("dthehun_new_battle_armor03",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,23050,weight(47)|abundance(100)|head_armor(0)|body_armor(180)|leg_armor(70)|difficulty(16),imodbits_none,[],[fac_kingdom_of_swadians]],

["dthehun_new_battle_armor04","Female Plate Mail",[("dthehun_new_battle_armor04",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,23550,weight(47)|abundance(100)|head_armor(0)|body_armor(175)|leg_armor(85)|difficulty(16),imodbits_none,[],[fac_kingdom_of_swadians]],

["dthehun_new_battle_armor05","Female Plate Mail",[("dthehun_new_battle_armor05",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,25050,weight(47)|abundance(100)|head_armor(0)|body_armor(185)|leg_armor(90)|difficulty(16),imodbits_none,[],[fac_kingdom_of_swadians]],

#Heavy plate armors.
["plate_armor","Plate Armor",[("full_plate_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,25000,weight(50)|abundance(100)|head_armor(0)|body_armor(180)|leg_armor(90)|difficulty(17),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#Special plate for Sarranid Champions
["sarranid_plate_armor","Bronze Plate Armor",[("bronze_full_plate_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,25000,weight(50)|abundance(100)|head_armor(0)|body_armor(185)|leg_armor(85)|difficulty(17),imodbits_none,[],[fac_kingdom_of_sarranids]],

#Special plate for Rhodok Champions
["rhodok_plate_armor","Dark Plate Armor",[("dark_full_plate_armor",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,25000,weight(50)|abundance(100)|head_armor(0)|body_armor(190)|leg_armor(80)|difficulty(17),imodbits_none,[],[fac_kingdom_of_rhodoks]],

#Amade's great fantasy plate mails
["amade_bronze_plate","Bronze Plate Armor",[("amade_bronze_plate",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,25000,weight(50)|abundance(100)|head_armor(0)|body_armor(180)|leg_armor(90)|difficulty(17),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_bronze_plate_champion","Champion's Plate Armor",[("amade_bronze_plate_champion",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,25000,weight(50)|abundance(100)|head_armor(0)|body_armor(180)|leg_armor(90)|difficulty(17),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_steel_plate","Steel Plate Armor",[("amade_steel_plate",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,25000,weight(50)|abundance(100)|head_armor(0)|body_armor(180)|leg_armor(90)|difficulty(17),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_steel_plate_sergeant","Steel Plate Armor",[("amade_steel_plate_sergeant",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,25000,weight(50)|abundance(100)|head_armor(0)|body_armor(180)|leg_armor(90)|difficulty(17),imodbits_none,[],[fac_kingdom_of_vaegirs]],

#SacredStoneHead's awesome Valgarde armor
["ssh_valgarde_armor","Valgarde's Armor",[("ssh_huscarl_armour",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,26000,weight(50)|abundance(100)|head_armor(0)|body_armor(185)|leg_armor(90)|difficulty(17),imodbits_none,[],[fac_kingdom_of_nords]],

["narf_gothic_plate","Gothic Plate",[("narf_gothic_plate",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,30000,weight(52)|abundance(100)|head_armor(0)|body_armor(190)|leg_armor(95)|difficulty(18),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_gothic_plate_black","Black Plate",[("narf_gothic_plate_black",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,35000,weight(52)|abundance(100)|head_armor(0)|body_armor(200)|leg_armor(105)|difficulty(18),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_milanese_plate","Milanese Plate",[("narf_milanese_plate",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,32000,weight(52)|abundance(100)|head_armor(0)|body_armor(195)|leg_armor(100)|difficulty(18),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_milanese_plate_dark","Dark Milanese Plate",[("narf_milanese_plate_dark",0)],itp_merchandise|itp_type_body_armor|itp_covers_legs,0,32000,weight(52)|abundance(100)|head_armor(0)|body_armor(200)|leg_armor(95)|difficulty(18),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["black_armor","Black Armor",[("black_armor",0)],itp_type_body_armor|itp_covers_legs,0,30000,weight(52)|abundance(100)|head_armor(0)|body_armor(190)|leg_armor(95)|difficulty(18),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["xenoargh_dark_armor02","Dark Armor",[("xenoargh_dark_armor02",0)],itp_type_body_armor|itp_covers_legs,0,20000,weight(40)|abundance(100)|head_armor(0)|body_armor(180)|leg_armor(90)|difficulty(14),imodbits_none],

["xenoargh_dark_armor","Spiked Armor",[("xenoargh_dark_armor",0)],itp_type_body_armor|itp_covers_legs,0,25000,weight(40)|abundance(100)|head_armor(0)|body_armor(185)|leg_armor(85)|difficulty(15),imodbits_none],

["xenoargh_dark_armor_red","Red Spiked Armor",[("xenoargh_dark_armor_red",0)],itp_type_body_armor|itp_covers_legs,0,25000,weight(40)|abundance(100)|head_armor(0)|body_armor(185)|leg_armor(85)|difficulty(15),imodbits_none],

["xenoargh_dark_armor_yellow","Yellow Spiked Armor",[("xenoargh_dark_armor_yellow",0)],itp_type_body_armor|itp_covers_legs,0,25000,weight(40)|abundance(100)|head_armor(0)|body_armor(180)|leg_armor(90)|difficulty(15),imodbits_none],

#Skeleton Armors; not for players!
["skeleton_armor01","Skeleton Armor",[("barf_skeleton_armor",0)],itp_type_body_armor|itp_unique|itp_covers_legs,0,300,weight(15)|abundance(0)|head_armor(0)|body_armor(125)|leg_armor(15)|difficulty(1),imodbits_none],

["skeleton_armor02","Skeleton Armor",[("barf_skeleton_greaves",0)],itp_type_body_armor|itp_unique|itp_covers_legs,0,300,weight(10)|abundance(0)|head_armor(0)|body_armor(0)|leg_armor(75)|difficulty(1),imodbits_none],

["skeleton_armor03","Skeleton Armor",[("barf_skeleton_armor_full",0)],itp_type_body_armor|itp_unique|itp_covers_legs,0,300,weight(25)|abundance(0)|head_armor(0)|body_armor(125)|leg_armor(75)|difficulty(1),imodbits_none],

["armor_end","Armor End (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

 
#HATS AND HELMETS
#Hats are civilian gear.  Protection is pretty much zero,and for the most part stats have been left at Native levels.
["helms_start","Helms Start (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["sarranid_head_cloth","Lady Head Cloth",[("tulbent",0),("tulbent_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,1,weight(0.5)|abundance(25)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_head_cloth_b","Lady Head Cloth",[("tulbent_b",0),("tulbent_b_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,1,weight(0.5)|abundance(25)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_felt_head_cloth","Head Cloth",[("common_tulbent",0),("common_tulbent_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,1,weight(0.5)|abundance(25)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_felt_head_cloth_b","Head Cloth",[("common_tulbent_b",0),("common_tulbent_b_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,1,weight(0.5)|abundance(25)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["turret_hat_ruby","Turret Hat",[("turret_hat_r",0)],itp_type_head_armor|itp_civilian|itp_fit_to_head,0,70,weight(0.5)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["turret_hat_blue","Turret Hat",[("turret_hat_b",0)],itp_type_head_armor|itp_civilian|itp_fit_to_head,0,80,weight(0.5)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["turret_hat_green","Barbette",[("barbette_new",0),("barbette_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_fit_to_head|itp_attach_armature,0,70,weight(0.5)|abundance(30)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["head_wrappings","Head Wrapping",[("head_wrapping",0)],itp_type_head_armor|itp_fit_to_head,0,80,weight(0.25)|head_armor(10),imodbit_tattered|imodbit_ragged|imodbit_sturdy|imodbit_thick],

["court_hat","Turret Hat",[("court_hat",0),("court_hat_store",ixmesh_inventory)],itp_type_head_armor|itp_civilian|itp_fit_to_head|itp_attach_armature,0,80,weight(0.5)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["wimple_a","Wimple",[("wimple_a_new",0),("wimple_a_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,80,weight(0.5)|abundance(30)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["wimple_with_veil","Wimple with Veil",[("wimple_b_new",0),("wimple_b_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,80,weight(0.5)|abundance(30)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["straw_hat","Straw Hat",[("straw_hat_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,9,weight(1)|abundance(100)|head_armor(2)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["common_hood","Hood",[("hood_new",0),("hood_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,9,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["hood_b","Hood",[("hood_b",0),("hood_b_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,9,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["hood_c","Hood",[("hood_c",0),("hood_c_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,9,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["hood_d","Hood",[("hood_d",0),("hood_d_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,9,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["headcloth","Headcloth",[("headcloth_a_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,1,weight(0.5)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["woolen_hood","Woolen Hood",[("woolen_hood",0),("woolen_hood_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,4,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["fur_hat","Fur Hat",[("fur_hat_a_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,4,weight(0.5)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["woolen_cap","Woolen Cap",[("woolen_cap_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,2,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["felt_hat","Felt Hat",[("felt_hat_a_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,4,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["felt_hat_b","Felt Hat",[("felt_hat_b_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,4,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["female_hood","Lady's Hood",[("ladys_hood_new",0),("ladys_hood_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,9,weight(1)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],

["khergit_lady_hat","Khergit Lady Hat",[("khergit_lady_hat",0)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_doesnt_cover_hair|itp_fit_to_head,0,1,weight(0.5)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["khergit_lady_hat_b","Khergit Lady Leather Hat",[("khergit_lady_hat_b",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head|itp_civilian,0,1,weight(0.5)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],


#Cloth Helms- better than nothing,but only just.  They don't cost much,though.
["arming_cap","Arming Cap",[("arming_cap_a_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,200,weight(1)|abundance(100)|head_armor(55)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids,fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["padded_coif","Padded Coif",[("padded_coif_a_new",0),("padded_coif_a_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature|itp_covers_beard,0,100,weight(1)|abundance(100)|head_armor(50)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids,fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["turban","Turban",[("tuareg_open",0),("tuareg_open_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,200,weight(1)|abundance(100)|head_armor(55)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["desert_turban","Desert Turban",[("tuareg",0),("tuareg_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_civilian|itp_attach_armature,0,100,weight(1)|abundance(100)|head_armor(50)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

#Leather Hats.  Real protection at last,but a bit pricier.
["nomad_cap","Nomad Cap",[("nomad_cap_a_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,300,weight(1)|abundance(100)|head_armor(70)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["nomad_cap_b","Nomad Cap",[("nomad_cap_b_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,500,weight(1)|abundance(100)|head_armor(80)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["steppe_cap","Steppe Cap",[("steppe_cap_a_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,400,weight(1)|abundance(100)|head_armor(70)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["leather_cap","Leather Cap",[("leather_cap_a_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,500,weight(1)|abundance(100)|head_armor(80)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["leather_steppe_cap_a","Steppe Cap",[("leather_steppe_cap_a_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,800,weight(1)|abundance(100)|head_armor(90)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["leather_steppe_cap_b","Steppe Cap",[("tattered_steppe_cap_b_new",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,500,weight(1)|abundance(100)|head_armor(80)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["leather_warrior_cap","Leather Warrior Cap",[("skull_cap_new_b",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,400,weight(1)|abundance(100)|head_armor(75)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_khergits]],

["sarranid_felt_hat","Covered Helm",[("sar_helmet3",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,700,weight(1)|abundance(100)|head_armor(90)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_sarranids]],

["xenoargh_musketeer_hat_black01","Musketeer's Hat",[("xenoargh_musketeer_hat_black01",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,500,weight(1)|abundance(100)|head_armor(80)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["xenoargh_musketeer_hat_black02","Musketeer's Hat",[("xenoargh_musketeer_hat_black02",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,500,weight(1)|abundance(100)|head_armor(80)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["xenoargh_musketeer_hat_red01","Musketeer's Hat",[("xenoargh_musketeer_hat_red01",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,500,weight(1)|abundance(100)|head_armor(80)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["xenoargh_musketeer_hat_red02","Musketeer's Hat",[("xenoargh_musketeer_hat_red02",0)],itp_merchandise|itp_type_head_armor|itp_civilian,0,500,weight(1)|abundance(100)|head_armor(80)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none,[],[fac_kingdom_of_rhodoks]],


#Mail-and-pads coifs.  Better than leather,but still not terrific.  Availability reduced (less loot).
["mail_coif","Mail Coif",[("mail_coif_new",0),("mail_coif_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,2500,weight(5)|abundance(100)|head_armor(130)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords,fac_kingdom_of_sarranids]],

#Open Helms.  Superior protection vs. most attack,prices are steep at this end,availability after battle reduced (less loot).
#Base stata: cost 4000, protection 160, diff 9
["skullcap","Skullcap",[("skull_cap_new_a",0)],itp_merchandise|itp_type_head_armor,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["footman_helmet","Footman's Helmet",[("skull_cap_new",0),("skull_cap_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,2500,weight(7.0)|abundance(25)|head_armor(120)|body_armor(0)|leg_armor(0)|difficulty(7),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["spak_demon_hood","Demon's Mask",[("spak_demon_hood",0)],itp_type_head_armor,0,2500,weight(7.0)|abundance(25)|head_armor(120)|body_armor(0)|leg_armor(0)|difficulty(7),imodbits_none,[],[fac_kingdom_of_nords]],

["nasal_helmet","Nasal Helmet",[("nasal_helmet_b",0)],itp_merchandise|itp_type_head_armor,0,4500,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["lamellar_helmet_a","Horse Archer's Helm",[("lamellar_helmet_a",0)],itp_merchandise|itp_type_head_armor,0,5000,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_khergits]],

["lamellar_helmet_b","Warrior's Helmet",[("lamellar_helmet_b",0)],itp_merchandise|itp_type_head_armor,0,4500,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_khergits]],

["norman_helmet","Helmet with Cap",[("norman_helmet_a",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["tattered_steppe_cap_a_new","Khergit Soldier's Cap",[("tattered_steppe_cap_a_new",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_khergits]],

["segmented_helmet","Segmented Helmet",[("segmented_helm_new",0),("segmented_helm_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4500,weight(7.0)|abundance(25)|head_armor(165)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["helmet_with_neckguard","Helmet with Neckguard",[("neckguard_helm_new",0),("neckguard_helm_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,3000,weight(7.0)|abundance(25)|head_armor(130)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#Rathos's neat fluted helmet, very distinctive mesh :-)
["rathos_fluted_helmet","Fluted Helmet",[("rathos_fluted_helmet",0)],itp_merchandise|itp_type_head_armor,0,3000,weight(7.0)|abundance(25)|head_armor(130)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks]],

#Rathos's awesome 11th Cent. Norman / Frankish conic helms :-)
["rathos_11thcent_conicalhelm","Conic Helm with Noseguard",[("rathos_11thcent_conicalhelm",0),("rathos_11thcent_conicalhelm_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_11thcent_conicalhelm_black","Conic Helm with Noseguard",[("rathos_11thcent_conicalhelm_black",0),("rathos_11thcent_conicalhelm_black_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_11thcent_conicalhelm_blue","Conic Helm with Noseguard",[("rathos_11thcent_conicalhelm_blue",0),("rathos_11thcent_conicalhelm_blue_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_11thcent_conicalhelm_bs","Conic Helm with Noseguard",[("rathos_11thcent_conicalhelm_bs",0),("rathos_11thcent_conicalhelm_bs_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_11thcent_conicalhelm_bw","Conic Helm with Noseguard",[("rathos_11thcent_conicalhelm_bw",0),("rathos_11thcent_conicalhelm_bw_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_11thcent_conicalhelm_rs","Conic Helm with Noseguard",[("rathos_11thcent_conicalhelm_rs",0),("rathos_11thcent_conicalhelm_rs_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_11thcent_conicalhelm_rw","Conic Helm with Noseguard",[("rathos_11thcent_conicalhelm_rw",0),("rathos_11thcent_conicalhelm_rw_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["rathos_11thcent_conicalhelm_ry","Conic Helm with Noseguard",[("rathos_11thcent_conicalhelm_ry",0),("rathos_11thcent_conicalhelm_ry_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["flat_topped_helmet","Flat Topped Helmet",[("flattop_helmet_new",0),("flattop_helmet_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["kettle_hat","Kettle Hat",[("kettle_hat_new",0),("kettle_hat_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["spiked_helmet","Spiked Helmet",[("spiked_helmet_new",0)],itp_merchandise|itp_type_head_armor,0,6000,weight(7.0)|abundance(25)|head_armor(175)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["nordic_helmet","Nordic Helmet",[("helmet_w_eyeguard_new",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["sarranid_warrior_cap","Sarranid Warrior Cap",[("tuareg_helmet",0),("tuareg_helmet_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_horseman_helmet","Horseman Helmet",[("sar_helmet2",0)],itp_merchandise|itp_type_head_armor,0,5000,weight(7.0)|abundance(25)|head_armor(175)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_helmet1","Sarranid Keffiyeh Helmet",[("sar_helmet1",0),("sar_helmet1_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_mail_coif","Sarranid Mail Coif",[("tuareg_helmet2",0),("tuareg_helmet2_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,2500,weight(7.0)|abundance(25)|head_armor(120)|body_armor(0)|leg_armor(0)|difficulty(7),imodbits_none,[],[fac_kingdom_of_sarranids]],

["sarranid_veiled_helmet","Sarranid Veiled Helmet",[("sar_helmet4",0),("sar_helmet4_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_sarranids]],

["nordic_archer_helmet","Leather-Faced Helmet",[("Helmet_A_vs2",0)],itp_merchandise|itp_type_head_armor,0,1700,weight(7.0)|abundance(25)|head_armor(110)|body_armor(0)|leg_armor(0)|difficulty(7),imodbits_none,[],[fac_kingdom_of_nords]],

["nordic_veteran_archer_helmet","Leather-Faced Helmet",[("Helmet_A",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["nordic_footman_helmet","Nordic Footman Helmet",[("Helmet_B_vs2",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["nordic_fighter_helmet","Nordic Fighter Helmet",[("Helmet_B",0)],itp_merchandise|itp_type_head_armor,0,5000,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["nordic_huscarl_helmet","Nordic Huscarl's Helmet",[("Helmet_C_vs2",0)],itp_merchandise|itp_type_head_armor,0,7000,weight(7.0)|abundance(25)|head_armor(190)|body_armor(0)|leg_armor(0)|difficulty(10),imodbits_none,[],[fac_kingdom_of_nords]],

["nordic_warlord_helmet","Nordic Warlord Helmet",[("Helmet_C",0)],itp_merchandise|itp_type_head_armor,0,6000,weight(7.0)|abundance(25)|head_armor(175)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["vaegir_fur_cap","Cap with Fur",[("vaeg_helmet3",0)],itp_merchandise|itp_type_head_armor,0,3000,weight(7.0)|abundance(25)|head_armor(130)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_khergits]],

["vaegir_fur_helmet","Vaegir Helmet",[("vaeg_helmet2",0)],itp_merchandise|itp_type_head_armor,0,2000,weight(7.0)|abundance(25)|head_armor(110)|body_armor(0)|leg_armor(0)|difficulty(7),imodbits_none,[],[fac_kingdom_of_khergits]],

["vaegir_spiked_helmet","Spiked Cap",[("vaeg_helmet1",0)],itp_merchandise|itp_type_head_armor,0,4500,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_khergits]],

["vaeg_helmet5","Captain's Helm",[("vaeg_helmet5",0)],itp_merchandise|itp_type_head_armor,0,6000,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_khergits]],

["vaegir_lamellar_helmet","Helmet with Lamellar Guard",[("vaeg_helmet4",0)],itp_merchandise|itp_type_head_armor,0,3000,weight(7.0)|abundance(25)|head_armor(125)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["vaegir_noble_helmet","Vaegir Nobleman Helmet",[("vaeg_helmet7",0),("vaeg_helmet7_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,5500,weight(7.0)|abundance(25)|head_armor(175)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["vaeg_helmet8","Vaegir Face Helm",[("vaeg_helmet8",0)],itp_merchandise|itp_type_head_armor,0,5500,weight(7.0)|abundance(25)|head_armor(175)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["bascinet","Bascinet",[("bascinet_avt_new",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["bascinet_2","Bascinet with Aventail",[("bascinet_new_a",0),("bascinet_new_a_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none],

["bascinet_3","Bascinet with Nose Guard",[("bascinet_new_b",0),("bascinet_new_b_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4500,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

#The Immortal Helm with face mask.
["sar_helmet_immortal","Immortal Helm",[("sar_helmet_immortal",0),("sar_helmet_immortal_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4500,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_sarranids]],

["guard_helmet","Guard Helmet",[("reinf_helmet_new",0),("reinf_helmet_new_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_nords]],

["narf_rus_helm","Ruski Soldier's Helm",[("narf_rus_helm",0),("narf_inv_rus_helm",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,5000,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_tagancha_helm_a","Ruski Leader's Helm",[("narf_tagancha_helm_a",0),("narf_inv_tagancha_helm_a",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,6000,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_tagancha_helm_b","Ruski Elite Helm",[("narf_tagancha_helm_b",0),("narf_inv_tagancha_helm_b",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,6500,weight(7.0)|abundance(25)|head_armor(190)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_gnezdovo_helm_a","Ruski Archer's Helm",[("narf_gnezdovo_helm_a",0),("narf_inv_gnezdovo_helm_a",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,5500,weight(7.0)|abundance(25)|head_armor(175)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_gnezdovo_helm_b","Ruski Archer's Helm",[("narf_gnezdovo_helm_b",0),("narf_inv_gnezdovo_helm_b",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,5500,weight(7.0)|abundance(25)|head_armor(175)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_nikolskoe_helm","Ruski Assault Helm",[("narf_nikolskoe_helm",0),("narf_inv_nikolskoe_helm",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,7500,weight(7.0)|abundance(25)|head_armor(185)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["narf_novogrod_helm","Ruski Elite Assault Helm",[("narf_novogrod_helm",0),("narf_inv_novogrod_helm",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(205)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

#Amade's awesome bronze
["amade_bronze_winged_helm","Bronze Winged Helm",[("amade_bronze_winged_helm",0),("amade_bronze_winged_helm_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,6000,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_bronze_winged_helm_small","Bronze Winged Helm",[("amade_bronze_winged_helm_plain",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_steel_winged_helm","Steel Winged Helm",[("amade_steel_winged_helm",0),("amade_steel_winged_helm_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,6000,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_steel_winged_helm_small","Steel Winged Helm",[("amade_steel_winged_helm_plain",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["amade_vaegir_champion_helm","Vaegir Champion's Helm",[("amade_vaegir_champion_helm",0),("amade_vaegir_champion_helm_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4500,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_vaegirs]],

#Dejawolf's poly-heavy helms (use with care,even with LODs they're expensive to render)
["dejawolf_byzantion","Byzantion",[("dejawolf_byzantion",0),("dejawolf_byzantion_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_byzantion2","Byzantion",[("dejawolf_byzantion_painted",0),("dejawolf_byzantion_painted_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_kettlehat1","Kettlehat",[("dejawolf_kettlehat1",0),("dejawolf_kettlehat1_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,5000,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_kettlehat2","Kettlehat",[("dejawolf_kettlehat1_painted",0),("dejawolf_kettlehat1_painted_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_kettlehat3","Kettlehat",[("dejawolf_kettlehat2",0),("dejawolf_kettlehat2_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,3500,weight(7.0)|abundance(25)|head_armor(150)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_kettlehat4","Kettlehat",[("dejawolf_kettlehat2_painted",0),("dejawolf_kettlehat2_painted_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,4500,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

#Spak Evil Helms:
["spak_horned_helm","Horned Helm",[("spak_horned_helm",0)],itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["spak_skull_helm","Skull Helm",[("spak_skull_helm",0)],itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

#Narf / BNW's nice open helms
["narf_combed_morion","Combed Morion",[("narf_combed_morion",0)],itp_merchandise|itp_type_head_armor,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["narf_combed_morion2","Combed Morion",[("narf_combed_morion2",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["narf_kettlehat","Morion",[("narf_kettlehat",0)],itp_merchandise|itp_type_head_armor,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_open_sallet","Open Sallet",[("narf_open_sallet",0)],itp_merchandise|itp_type_head_armor,0,5000,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_sarranids]],

["narf_open_sallet_coif","Open Sallet",[("narf_open_sallet_coif",0),("narf_open_sallet_coif_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,5500,weight(7.0)|abundance(25)|head_armor(175)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians,fac_kingdom_of_sarranids]],

#My awesome Romance Viking helms.
["xenoargh_viking_horned_helm01","Viking Helm",[("xenoargh_viking_helm",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,5000,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_horned_helm02","Viking Helm",[("xenoargh_viking_helm_eyepiece",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,6000,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_horned_helm03","Viking Helm",[("xenoargh_viking_helm_eyepiece_noflap",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,4500,weight(7.0)|abundance(25)|head_armor(165)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_horned_helm04","Viking Helm",[("xenoargh_viking_helm_plain",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_steel_winged_helm01","Viking Helm",[("xenoargh_winged_viking_helm",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,5000,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_steel_winged_helm02","Viking Helm",[("xenoargh_winged_viking_helm_eyepiece",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,6000,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_steel_winged_helm03","Viking Helm",[("xenoargh_winged_viking_helm_nocheek",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_steel_winged_helm04","Viking Helm",[("xenoargh_winged_viking_helm_plain",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_gold_winged_helm01","Viking Helm",[("xenoargh_gold_winged_viking_helm",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,5000,weight(7.0)|abundance(25)|head_armor(170)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_gold_winged_helm02","Viking Helm",[("xenoargh_gold_winged_viking_helm_eyepiece",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,6000,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_gold_winged_helm03","Viking Helm",[("xenoargh_gold_winged_viking_helm_nocheek",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,4000,weight(7.0)|abundance(25)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_viking_gold_winged_helm04","Viking Helm",[("xenoargh_gold_winged_viking_helm_plain",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_female_crown01","Battle Crown",[("xenoargh_female_crown01",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3050,weight(7.0)|abundance(75)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_mask_gold","Gold Mask",[("xenoargh_mask_gold",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3050,weight(7.0)|abundance(50)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["xenoargh_mask_black","Black Mask",[("xenoargh_mask_black",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3050,weight(7.0)|abundance(50)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_sarranids]],

["xenoargh_mask_domino","Domino Mask",[("xenoargh_mask_domino",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3050,weight(7.0)|abundance(50)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_swadians]],

["xenoargh_hornmask_black","Sister's Mask",[("xenoargh_hornmask_black",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3550,weight(7.0)|abundance(50)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_swadians]],

["xenoargh_hornmask_blue","Blue Mask",[("xenoargh_hornmask_blue",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3550,weight(7.0)|abundance(50)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_khergits]],

["xenoargh_hornmask_green","Green Mask",[("xenoargh_hornmask_green",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3550,weight(7.0)|abundance(50)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["xenoargh_hornmask_red","Red Mask",[("xenoargh_hornmask_red",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3550,weight(7.0)|abundance(50)|head_armor(160)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_sarranids]],

["xenoargh_half_mask","Half-Mask",[("xenoargh_half_mask",0)],itp_merchandise|itp_type_head_armor|itp_doesnt_cover_hair|itp_fit_to_head,0,3050,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_swadians]],

#Youhou's Assassin helms
["youhou_assassin_hood","Assassin's Hood",[("youhou_assassin_hood",0)],itp_merchandise|itp_type_head_armor,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_sarranids]],

["youhou_assassin_hood_red","Red Assassin's Hood",[("youhou_assassin_hood_red",0)],itp_merchandise|itp_type_head_armor,0,3000,weight(7.0)|abundance(25)|head_armor(140)|body_armor(0)|leg_armor(0)|difficulty(8),imodbits_none,[],[fac_kingdom_of_sarranids]],

["xenoargh_leaf_helm","Leaf Helm",[("xenoargh_leaf_helm",0),("xenoargh_leaf_helm_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_doesnt_cover_hair,0,6050,weight(7.0)|abundance(75)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(10),imodbits_none,[],[fac_kingdom_of_nords]],

["xenoargh_shell_helm","Shell Helm",[("xenoargh_shell_helm",0),("xenoargh_shell_helm_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_doesnt_cover_hair,0,7050,weight(7.0)|abundance(75)|head_armor(190)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_nords]],

#Jan Tuma's wonderful 16th-century Sturmhaube.  Good protection at a reasonable price.
["jtm_sturmhaube_1","Sturmhaube",[("jtm_sturmhaube_1",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_2","Sturmhaube",[("jtm_sturmhaube_2",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_3","Sturmhaube",[("jtm_sturmhaube_3",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_4","Sturmhaube",[("jtm_sturmhaube_4",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_5","Sturmhaube",[("jtm_sturmhaube_5",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_6","Sturmhaube",[("jtm_sturmhaube_6",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_7","Sturmhaube",[("jtm_sturmhaube_7",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

#Blackened
["jtm_sturmhaube_1b","Sturmhaube",[("jtm_sturmhaube_1B",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_2b","Sturmhaube",[("jtm_sturmhaube_2B",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_3b","Sturmhaube",[("jtm_sturmhaube_3B",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_4b","Sturmhaube",[("jtm_sturmhaube_4B",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_5b","Sturmhaube",[("jtm_sturmhaube_5B",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_6b","Sturmhaube",[("jtm_sturmhaube_6B",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_7b","Sturmhaube",[("jtm_sturmhaube_7B",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

#Black and White
["jtm_sturmhaube_1bw","Sturmhaube",[("jtm_sturmhaube_1BW",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_2bw","Sturmhaube",[("jtm_sturmhaube_2BW",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_3bw","Sturmhaube",[("jtm_sturmhaube_3BW",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_4bw","Sturmhaube",[("jtm_sturmhaube_4BW",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_5bw","Sturmhaube",[("jtm_sturmhaube_5BW",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_6bw","Sturmhaube",[("jtm_sturmhaube_6BW",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["jtm_sturmhaube_7bw","Sturmhaube",[("jtm_sturmhaube_7BW",0)],itp_merchandise|itp_type_head_armor|itp_covers_beard,0,8500,weight(7.0)|abundance(25)|head_armor(180)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],


#################################################################
#Full Helms.  Penultimate protection at a price.  Very rare drop.
#Base balance: cost 8000, protection 220, diff 12

#More of Rathos's helms!  12th Century Faceplate design.
["rathos_12thcent_faceplate","Helm with Faceguard",[("rathos_12thcent_faceplate",0),("rathos_12thcent_faceplate_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,8500,weight(7.0)|abundance(25)|head_armor(205)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12thcent_faceplate_bw","Helm with Faceguard",[("rathos_12thcent_faceplate_bw",0),("rathos_12thcent_faceplate_bw_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,8500,weight(7.0)|abundance(25)|head_armor(205)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12thcent_faceplate_bwq","Helm with Faceguard",[("rathos_12thcent_faceplate_bwq",0),("rathos_12thcent_faceplate_bwq_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,8500,weight(7.0)|abundance(25)|head_armor(205)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12thcent_faceplate_gy","Helm with Faceguard",[("rathos_12thcent_faceplate_gy",0),("rathos_12thcent_faceplate_gy_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,8500,weight(7.0)|abundance(25)|head_armor(205)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12thcent_faceplate_rw","Helm with Faceguard",[("rathos_12thcent_faceplate_rw",0),("rathos_12thcent_faceplate_rw_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,8500,weight(7.0)|abundance(25)|head_armor(205)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12thcent_faceplate_wbs","Helm with Faceguard",[("rathos_12thcent_faceplate_wbs",0),("rathos_12thcent_faceplate_wbs_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,8500,weight(7.0)|abundance(25)|head_armor(205)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

#Rathos 12 Century Great Helms.
["rathos_12th_cent_greathelm","Great Helm",[("rathos_12th_cent_greathelm",0),("rathos_12th_cent_greathelm_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,9500,weight(7.0)|abundance(25)|head_armor(215)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12th_cent_greathelm_brass","Great Helm",[("rathos_12th_cent_greathelm_brass",0),("rathos_12th_cent_greathelm_brass_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,9500,weight(7.0)|abundance(25)|head_armor(215)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12th_cent_greathelm_bw","Great Helm",[("rathos_12th_cent_greathelm_bw",0),("rathos_12th_cent_greathelm_bw_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,9500,weight(7.0)|abundance(25)|head_armor(215)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12th_cent_greathelm_rw","Great Helm",[("rathos_12th_cent_greathelm_rw",0),("rathos_12th_cent_greathelm_rw_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,9500,weight(7.0)|abundance(25)|head_armor(215)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_12th_cent_greathelm_ry","Great Helm",[("rathos_12th_cent_greathelm_ry",0),("rathos_12th_cent_greathelm_ry_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_head,0,9500,weight(7.0)|abundance(25)|head_armor(215)|body_armor(0)|leg_armor(0)|difficulty(9),imodbits_none,[],[fac_kingdom_of_swadians]],

#Taleworlds helms
["vaegir_war_helmet","Vaegir War Helmet",[("vaeg_helmet6",0),("vaeg_helmet6_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,10000,weight(7.0)|abundance(5)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["vaegir_mask","Vaegir War Mask",[("vaeg_helmet9",0),("vaeg_helmet9_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,9000,weight(7.0)|abundance(5)|head_armor(230)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["black_helmet","Black Helmet",[("black_helm",0)],itp_type_head_armor,0,7000,weight(7.0)|abundance(5)|head_armor(200)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none],

["ssh_kabuto","Kabuto",[("ssh_kabuto",0),("ssh_kabuto_inv",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature,0,7500,weight(2)|abundance(100)|head_armor(205)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_khergits]],

#Dejawolf's barbutas
["dejawolf_barbuta1","Barbuta",[("dejawolf_barbuta1",0)],itp_merchandise|itp_type_head_armor,0,8000,weight(7.0)|abundance(5)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["dejawolf_barbuta2","Barbuta",[("dejawolf_barbuta2",0)],itp_merchandise|itp_type_head_armor,0,6500,weight(7.0)|abundance(5)|head_armor(200)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["dejawolf_barbuta3","Barbuta",[("dejawolf_barbuta3",0)],itp_merchandise|itp_type_head_armor,0,9000,weight(7.0)|abundance(5)|head_armor(230)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["dejawolf_barbuta4","Barbuta",[("dejawolf_barbuta4",0)],itp_merchandise|itp_type_head_armor,0,8000,weight(7.0)|abundance(5)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_vaegirs]],

#Niek's Rhulg helmet.
["rhulg_helmet","Rhulg Helmet",[("niek_rhulg_helmet",0)],itp_merchandise|itp_type_head_armor|itp_fit_to_head,0,10000,weight(7.0)|abundance(5)|head_armor(240)|body_armor(0)|leg_armor(0)|difficulty(16),imodbits_none,[],[fac_kingdom_of_vaegirs]],

#Past here covers head.
#My Evil Helms:
["xenoargh_evil_helm","Evil Helm",[("xenoargh_evil_helm",0),("xenoargh_evil_helm_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8000,weight(7.0)|abundance(25)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["xenoargh_evil_helm_yellow","Yellow Evil Helm",[("xenoargh_evil_helm_yellow",0),("xenoargh_evil_helm_yellow_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8000,weight(7.0)|abundance(25)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["xenoargh_evil_helm_white","White Evil Helm",[("xenoargh_evil_helm_white",0),("xenoargh_evil_helm_white_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8000,weight(7.0)|abundance(25)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["full_helm","Full Helm",[("great_helmet_new_b",0),("great_helmet_new_b_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head|itp_attach_armature,0,8000,weight(7.0)|abundance(5)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["great_helmet","Great Helmet",[("great_helmet_new",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,9500,weight(7.0)|abundance(5)|head_armor(235)|body_armor(0)|leg_armor(0)|difficulty(13),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["winged_great_helmet","Winged Great Helmet",[("maciejowski_helmet_new",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,7000,weight(7.0)|abundance(5)|head_armor(215)|body_armor(0)|leg_armor(0)|difficulty(11),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

#Dejawolf's full helms. Use with care; even with LODs,these are performance-intensive.
["dejawolf_valgarde_helm","Valgarde's Helm",[("dejawolf_valgarde",0),("dejawolf_valgarde_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head|itp_attach_armature,0,9000,weight(7.0)|abundance(5)|head_armor(230)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_flemish_armet","Flemish Armet",[("dejawolf_flemish_armet",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,10000,weight(7.0)|abundance(5)|head_armor(240)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_greatbascinet1","Bascinet",[("dejawolf_greatbascinet1",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8000,weight(7.0)|abundance(5)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_greathelm1","Great Helm",[("dejawolf_greathelm1",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,9000,weight(7.0)|abundance(5)|head_armor(230)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_greathelm1_dark","Dark Great Helm",[("dejawolf_greathelm1_dark",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,9000,weight(7.0)|abundance(5)|head_armor(230)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["dejawolf_greathelm2","Great Helm",[("dejawolf_greathelm2",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none],

["dejawolf_klappvisier","Klappvisier",[("dejawolf_klappvisier",0),("dejawolf_klappvisier_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head|itp_attach_armature,0,9000,weight(7.0)|abundance(5)|head_armor(230)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["dejawolf_milanese_sallet","Milanese Sallet",[("dejawolf_milanese_sallet",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,10500,weight(7.0)|abundance(5)|head_armor(245)|body_armor(0)|leg_armor(0)|difficulty(14),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

#Ratho's classic Great Helms.
["rathos_greathelm","Great Helm",[("rathos_greathelm",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_greathelm_black","Great Helm",[("rathos_greathelm_black",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_greathelm_blue","Great Helm",[("rathos_greathelm_blue",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_greathelm_brass","Great Helm",[("rathos_greathelm_brass",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_swadians]],

["rathos_greathelm_red","Great Helm",[("rathos_greathelm_red",0)],itp_merchandise|itp_type_head_armor|itp_covers_head,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_swadians]],

#Narf's full helms.
["narf_visored_sallet","Visored Sallet",[("narf_visored_sallet",0)],itp_merchandise|itp_type_head_armor,0,7000,weight(7.0)|abundance(5)|head_armor(190)|body_armor(0)|leg_armor(0)|difficulty(10),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_visored_sallet_coif","Visored Sallet",[("narf_visored_sallet_coif",0),("narf_visored_sallet_coif_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_visored_sallet_coif_black","Black Sallet",[("narf_visored_sallet_coif_black",0),("narf_visored_sallet_coif_black_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_attach_armature|itp_covers_beard,0,12000,weight(7.0)|abundance(5)|head_armor(235)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_hounskull","Hounskull",[("narf_hounskull",0),("narf_hounskull_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head|itp_attach_armature,0,9000,weight(7.0)|abundance(5)|head_armor(235)|body_armor(0)|leg_armor(0)|difficulty(13),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_sugarloaf","Sugarloaf Helm",[("narf_sugarloaf",0)],itp_merchandise|itp_type_head_armor,0,8000,weight(7.0)|abundance(5)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_sugarloaf_templar","Templar's Helm",[("narf_sugarloaf_templar",0)],itp_merchandise|itp_type_head_armor,0,8000,weight(7.0)|abundance(5)|head_armor(220)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_sugarloaf_coif","Sugarloaf Helm",[("narf_sugarloaf_coif",0),("narf_sugarloaf_coif_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head|itp_attach_armature,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["narf_sugarloaf_coif_templar","Templar's Helm",[("narf_sugarloaf_coif_templar",0),("narf_sugarloaf_coif_templar_store",ixmesh_inventory)],itp_merchandise|itp_type_head_armor|itp_covers_head|itp_attach_armature,0,8500,weight(7.0)|abundance(5)|head_armor(225)|body_armor(0)|leg_armor(0)|difficulty(12),imodbits_none,[],[fac_kingdom_of_rhodoks,fac_kingdom_of_swadians]],

["helms_end","Helms End (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],


#ALEPH "ARMOR"  
#JUST HERE FOR ALEPH GOODNESS
["aleph_suit","Aleph Armor (SHOULD NOT BE SOLD OR SEEN IN INVENTORIES)",[("xeno_bullet",0),("xenoargh_CannonWalker_inv",ixmesh_inventory)],itp_type_head_armor,0,100000,weight(0.0)|abundance(0)|head_armor(220)|body_armor(220)|leg_armor(220),imodbits_none],


# SHIELDS
#Shields have been reorganized into distinct classes:
#Wooden shields- typical birch / limewood with canvas and glue,these are cheap,light,and fairly weak, but are very fast.
#Board shields- these are heavy, with mediocre resistance but excellent hitpoints.  They require a *lot* of chopping with hand strokes to destroy,and will take considerable hits from missiles.  They are very sloooooow by comparision.
#Nordic style shields- these had a heavy boss, so they come with more weight and slower speed ratings then wooden shields, and higher resistance, but similar hitpoints.
#Hide shields- well-made hide shields of cured leather with glue and a thin wood backing, these are slower than wooden shields but have more hitpoints.
#Metal-faced shields- typically fairly expensive and smallish for their weight, provides good resistance and excellent hitpoints.  They are slower than wooden shields and Nordic shields but are very hard to destroy.

#Shield Start Placeholder
["shields_start","Shields Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#WOODEN SHIELDS
["wooden_shield","Wooden Shield",[("shield_round_a",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_round_shield,100,weight(4)|hit_points(150)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["shield_heater_c","Heater Shield",[("shield_heater_c",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["norman_shield_1","Kite Shield",[("norman_shield_1",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["norman_shield_2","Kite Shield",[("norman_shield_2",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["norman_shield_3","Kite Shield",[("norman_shield_3",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["norman_shield_4","Kite Shield",[("norman_shield_4",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["norman_shield_5","Kite Shield",[("norman_shield_5",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["norman_shield_6","Kite Shield",[("norman_shield_6",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["norman_shield_7","Kite Shield",[("norman_shield_7",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["norman_shield_8","Kite Shield",[("norman_shield_8",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["old_round_shield","Old Round Shield",[("xenoargh_round_shield05_plain",0),("xenoargh_round_shield05_plain_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["skeleton_round_shield","Skeleton Round Shield",[("xenoargh_round_shield05",0),("xenoargh_round_shield05_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["old_green_round_shield","Old Green Round Shield",[("xenoargh_round_shield05_paint01",0),("xenoargh_round_shield05_paint01_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["old_blue_round_shield","Old Blue Round Shield",[("xenoargh_round_shield05_paint02",0),("xenoargh_round_shield05_paint02_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_round_shield03_01","Faded Round Shield",[("xenoargh_round_shield03_01",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["xenoargh_round_shield03_02","Soldier's Round Shield",[("xenoargh_round_shield03_02",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_round_shield03_03","Patterned Round Shield",[("xenoargh_round_shield03_03",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_round_shield03_04","Poor Knight's Round Shield",[("xenoargh_round_shield03_04",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_round_shield03_05","Castellan's Round Shield",[("xenoargh_round_shield03_05",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_round_shield03_06","Plain Round Shield",[("xenoargh_round_shield03_06",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["round_shield01_plain","Round Shield",[("xenoargh_round_shield01",0),("xenoargh_round_shield01_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["round_shield01_octopus","Octopus Shield",[("xenoargh_round_shield01_octopus",0),("xenoargh_round_shield01_octopus_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["round_shield01_viking01","Round Shield",[("xenoargh_round_shield01_viking01",0),("xenoargh_round_shield01_viking01_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["round_shield01_viking02","Nord Round Shield",[("xenoargh_round_shield01_viking02",0),("xenoargh_round_shield01_viking02_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["round_shield01_viking03","Nord Round Shield",[("xenoargh_round_shield01_viking03",0),("xenoargh_round_shield01_viking03_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["round_shield01_viking04","Nord Round Shield",[("xenoargh_round_shield01_viking04",0),("xenoargh_round_shield01_viking04_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["xenoargh_round_shield02","Heavy Round Shield",[("xenoargh_round_shield02",0),("xenoargh_round_shield02_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_round_shield02_paint01","Raider's Round Shield",[("xenoargh_round_shield02_paint01",0),("xenoargh_round_shield02_paint01_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_round_shield02_paint02","Reaver's Round Shield",[("xenoargh_round_shield02_paint02",0),("xenoargh_round_shield02_paint02_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_round_shield02_paint03","Viking's Round Shield",[("xenoargh_round_shield02_paint03",0),("xenoargh_round_shield02_paint03_carry",ixmesh_carry)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["xenoargh_shield_small04","Huscarl's Round Shield",[("xenoargh_shield_small04",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_kite01_01","Kite Shield",[("xenoargh_shield_kite01_01",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["xenoargh_shield_kite01_02","Heavy Kite Shield",[("xenoargh_shield_kite01_02",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["xenoargh_shield_kite01_03","Horseman's Kite Shield",[("xenoargh_shield_kite01_03",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["xenoargh_shield_kite01_04","Knightly Kite Shield",[("xenoargh_shield_kite01_04",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_kite01_05","Plain Kite Shield",[("xenoargh_shield_kite01_05",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_kite01_06","Old Kite Shield",[("xenoargh_shield_kite01_06",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,
33,weight(2)|hit_points(165)|body_armor(35)|spd_rtng(200)|shield_width(36)|shield_height(70),imodbits_none],

["heater_shield_old","Old Heater Shield",[("xenoargh_tableau_shield_heater_1",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,0,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["heater_shield_swadian","Swadian Heater Shield",[("xenoargh_tableau_shield_heater_1_swadian",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,0,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
  
["heater_shield_generic","Heater Shield",[("xenoargh_tableau_shield_heater_2",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,0,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["heater_shield_heavy","Heavy Heater Shield",[("xenoargh_tableau_shield_heater_3",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,0,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["xenoargh_shield_heater01","Horseman's Heater Shield",[("xenoargh_shield_heater01",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],
 
["xenoargh_shield_heater02","Knightly Heater Shield",[("xenoargh_shield_heater02",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_heater03","Heater Shield",[("xenoargh_shield_heater03",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_heater04","Heater Shield",[("xenoargh_shield_heater04",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_heater05","Heater Shield",[("xenoargh_shield_heater05",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_heater06","Heater Shield",[("xenoargh_shield_heater06",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_heater07","Heater Shield",[("xenoargh_shield_heater07",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_heater08","Heater Shield",[("xenoargh_shield_heater08",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(5)|hit_points(175)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["xenoargh_shield_heater09","Grim Shield",[("xenoargh_shield_heater09",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,1000,weight(5)|hit_points(250)|body_armor(40)|spd_rtng(200)|shield_width(50),imodbits_none],

["spak_knight_shield02","Crusader's Shield",[("spak_knight_shield02",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,1000,weight(5)|hit_points(300)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none],

["spak_knight_shield_templar","Templar's Shield",[("spak_knight_shield_templar",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,1000,weight(5)|hit_points(300)|body_armor(35)|spd_rtng(200)|shield_width(50),imodbits_none,[],[fac_kingdom_of_swadians,fac_kingdom_of_rhodoks]],

["spak_knight_shield","Imperial Shield",[("spak_knight_shield",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,1000,weight(5)|hit_points(250)|body_armor(35)|spd_rtng(215)|shield_width(50),imodbits_none],

#BOARD SHIELDS
#Wooden, 1 cm. thick or sometimes thicker, often heavier woods (like oak).  Much more durable (some of them laugh at bullets) reasonably priced, but sloooooow.
["old_board_shield01","Old Board Shield",[("xenoargh_shield_pavise_2",0)],itp_merchandise|itp_type_shield|itp_cant_use_on_horseback|itp_wooden_parry,0,500,weight(25)|hit_points(1000)|body_armor(35)|spd_rtng(50)|shield_width(43)|shield_height(100),imodbits_none],
 
["old_board_shield_rhodok","Plain Board Shield",[("xenoargh_shield_pavise_2_rhodok",0)],itp_merchandise|itp_type_shield|itp_cant_use_on_horseback|itp_wooden_parry,0,500,weight(25)|hit_points(1000)|body_armor(35)|spd_rtng(50)|shield_width(43)|shield_height(100),imodbits_none],
 
["pavise_shield_plain","Pavise Shield",[("xenoargh_shield_pavise_1",0)],itp_merchandise|itp_type_shield|itp_cant_use_on_horseback|itp_wooden_parry,0,500,weight(25)|hit_points(1100)|body_armor(40)|spd_rtng(50)|shield_width(43)|shield_height(100),imodbits_none],
 
["pavise_shield_fancy_cross","Heavy Pavise Shield",[("xenoargh_shield_pavise_1_fancy_cross",0)],itp_merchandise|itp_type_shield|itp_cant_use_on_horseback|itp_wooden_parry,0,1000,weight(25)|hit_points(1200)|body_armor(45)|spd_rtng(50)|shield_width(43)|shield_height(100),imodbits_none],

["pavise_shield_painted01","Painted Pavise Shield",[("xenoargh_shield_pavise_1_paint01",0)],itp_merchandise|itp_type_shield|itp_cant_use_on_horseback|itp_wooden_parry,0,1000,weight(25)|hit_points(1200)|body_armor(45)|spd_rtng(50)|shield_width(43)|shield_height(100),imodbits_none],

["pavise_shield_painted02","Painted Pavise Shield",[("xenoargh_shield_pavise_1_paint02",0)],itp_merchandise|itp_type_shield|itp_cant_use_on_horseback|itp_wooden_parry,0,1000,weight(25)|hit_points(1200)|body_armor(45)|spd_rtng(50)|shield_width(43)|shield_height(100),imodbits_none],

#NORDIC SHIELDS
["nordic_shield","Nordic Shield",[("shield_round_b",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_round_shield,400,weight(8)|hit_points(200)|body_armor(55)|spd_rtng(150)|shield_width(50),imodbits_none],

["xenoargh_shield_small01","Round Cavalry Shield",[("xenoargh_shield_small01",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_round_shield,500,weight(8)|hit_points(175)|body_armor(55)|spd_rtng(175)|shield_width(40),imodbits_none],
 
["xenoargh_shield_small02","Elite Cavalry Shield",[("xenoargh_shield_small02",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_round_shield,500,weight(8)|hit_points(175)|body_armor(55)|spd_rtng(175)|shield_width(40),imodbits_none],

["xenoargh_shield_small03","Plain Cavalry Shield",[("xenoargh_shield_small03",0)],itp_merchandise|itp_type_shield|itp_wooden_parry,itcf_carry_round_shield,500,weight(8)|hit_points(175)|body_armor(55)|spd_rtng(175)|shield_width(40),imodbits_none],

#HIDE SHIELDS
#Tougher than board shields, but slower.  It's all tradeoffs.
["fur_covered_shield","Fur Covered Shield",[("shield_kite_m",0)],itp_type_shield|itp_wooden_parry,itcf_carry_kite_shield,300,weight(15)|hit_points(300)|body_armor(40)|spd_rtng(145)|shield_width(81),imodbits_none],

["leather_covered_round_shield","Leather Covered Round Shield",[("shield_round_d",0)],itp_type_shield|itp_wooden_parry,itcf_carry_round_shield,400,weight(5)|hit_points(150)|body_armor(40)|spd_rtng(145)|shield_width(40),imodbits_none],

["hide_covered_round_shield","Hide Covered Round Shield",[("shield_round_f",0)],itp_type_shield|itp_wooden_parry,itcf_carry_round_shield,400,weight(6)|hit_points(175)|body_armor(40)|spd_rtng(145)|shield_width(40),imodbits_none],

#METAL-FACED SHIELDS
#Weird category- huge hitpoints, great resistance, sloooow speed (which is why, IRL, they weren't seen much in the Middle Ages- phalanx was out, speedy shields were popular).
#Expensive, to boot.
["plate_covered_round_shield","Plate Covered Round Shield",[("shield_round_e",0)],itp_type_shield,itcf_carry_round_shield,3000,weight(13)|hit_points(700)|body_armor(80)|spd_rtng(80)|shield_width(40),imodbits_none],

["plate_shield","Plate Shield",[("shield_dragon",0)],itp_type_shield,itcf_carry_round_shield,3000,weight(13)|hit_points(700)|body_armor(80)|spd_rtng(80)|shield_width(40),imodbits_none],

["steel_shield","Steel Shield",[("xenoargh_steel_shield_new",0)],itp_merchandise|itp_type_shield,itcf_carry_round_shield,2500,weight(13)|hit_points(700)|body_armor(75)|spd_rtng(80)|shield_width(40),imodbits_none],

["steel_shield_herald01","Steel Shield",[("xenoargh_steel_shield_new_herald01",0)],itp_merchandise|itp_type_shield,itcf_carry_round_shield,2500,weight(13)|hit_points(700)|body_armor(75)|spd_rtng(80)|shield_width(40),imodbits_none],

["steel_shield_herald02","Steel Shield",[("xenoargh_steel_shield_new_herald02",0)],itp_merchandise|itp_type_shield,itcf_carry_round_shield,2500,weight(13)|hit_points(700)|body_armor(75)|spd_rtng(80)|shield_width(40),imodbits_none],

["steel_shield_herald03","Steel Shield",[("xenoargh_steel_shield_new_herald03",0)],itp_merchandise|itp_type_shield,itcf_carry_round_shield,2500,weight(13)|hit_points(700)|body_armor(75)|spd_rtng(80)|shield_width(40),imodbits_none],

["spak_hermitage_shield_1","Bronze Shield",[("spak_hermitage_shield_1",0)],itp_merchandise|itp_type_shield,itcf_carry_round_shield,2500,weight(13)|hit_points(700)|body_armor(75)|spd_rtng(80)|shield_width(40),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["spak_hermitage_shield_2","Bronze Shield",[("spak_hermitage_shield_2",0)],itp_merchandise|itp_type_shield,itcf_carry_round_shield,2500,weight(15)|hit_points(900)|body_armor(75)|spd_rtng(70)|shield_width(40),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["spak_hermitage_shield_3","Bronze Shield",[("spak_hermitage_shield_3",0)],itp_merchandise|itp_type_shield,itcf_carry_round_shield,2500,weight(15)|hit_points(900)|body_armor(75)|spd_rtng(70)|shield_width(40),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["spak_bronze_shield","Bronze Shield",[("spak_bronze_shield",0)],itp_merchandise|itp_type_shield,itcf_carry_round_shield,2500,weight(15)|hit_points(900)|body_armor(75)|spd_rtng(70)|shield_width(40),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["spak_asmodeus_shield","Evil Shield",[("spak_asmodeus_shield",0)],itp_type_shield,itcf_carry_round_shield,3500,weight(14)|hit_points(450)|body_armor(75)|spd_rtng(150)|shield_width(40),imodbits_none],

#Special Flesh Golem item
["spak_golem_shield","Evil Shield",[("spak_golem_shield",0)],itp_type_shield|itp_unique,itcf_carry_round_shield,3500,weight(14)|hit_points(450)|body_armor(150)|spd_rtng(50)|shield_width(40),imodbits_none],

#Shield End Placeholder
["shields_end","Shields End (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],


#HAND WEAPONS
#Hand Weapons have been reorganized and divided into several distinct categories.  Because this area may be of interest to many modders wanting to rebalance stats,I've taken a couple of hours to clean up the entries.

#Genrally speaking,there is a larger difference in speeds vs. Native.  A knife is a very fast weapon,but short and weak.  A two-handed axe is slow, but long and very deadly.  This re-emphasis differentiates the weapons more than Native did,and makes all the weapons have more of a point, including the broadswords that everybody loves their knights to carry(medium at everything,so it's a good counter for everything else).

#Prices are a lot higher, especially for military arms.  Metal was scarce; swords and fine axes took a lot of specialist work to make, and were very expensive, and this applies to all the military arms.  Spears and simple clubs will seem OP for price;  this is because they *are* OP for price IRL, they're one of the most effective weapons for cost developed by humans, as well as being one of the most ancient.  It's just how it is,please don't write me email to complain ;-)

#There are higher differences in quality,mainly differentiated by higher speeds (rather than damage) for cost.  A crappy longsword and a master-crafted blade will both cut an unarmored person's arm clean off; but there's a really appreciable difference in their practical speed of use,in the hands of an expert.  Sadly,there's no broken-weapon code in the game engine,otherwise I'd include that factor also.

#Daggers and Knives:  typically the speed is a lot higher than Native, damage is increased slightly,and most cannot be used to block.  They're still not great for melee vs. armored and armed opponents.  Costs are generally low, but higher than Native.

#Clubs(one hand):  Includes everything from wooden sticks to one-armed picks and maces.  
#One-handed,damage varies from crappy to very heavy, damage may be blunt or cutting or piercing,depending on historical usage.  
#Speeds are typically fast,costs vary widely according to quality.

#Swords(one hand): these are in three categories:
#Cutting swords,which are the "medium" weapon in all respects- decent damage on cut,poor on pierce,speed is medium.  Typically a two-hit kill on an unarmored target.
#Specialist cutting swords (i.e. falchions and other such weapons).  No piercing attack,deadly on cut,a bit slower speed than a cutting sword of similar length.
#Piercing swords.  These typically have a very good speed,OK pierce,mediocre cut.  Good for dueling,not terrific for melee.
#Final note on swords:  yes,they're expensive.  Yes,this is realistic,real swords weren't cheap.  Capturing them on the battlefield is nice though :-)

#Axes(one hand): these are all pretty similar- good cut,mediocre speed,retains damage vs. shield.  Typically a two-hit kill on an unarmored man at worst,typically better.  Damage and speeds have been changed,along with cost (which generally went up a lot).

#Clubs(two hand):  two-handed clubs are generally very heavy weapons.  They are typically one-hit kill+ on unarmored people,sometimes significantly moreso.  There aren't a lot of "war clubs" in Native,the OS ones I added give us a wider range.  The peasant's sledgehammer,which looks like it's all wood,is still short of total death,but the warhammers will generally just kill most targets outright,even with armor on.  The heavier weapons crush blocks.

#Swords(two hand):  these were,both because of expense and the vulnerability of the users to archers,pretty rare on real medieval battlefields.  That said,if you want to field an army of two-handed greatswords,claymores,zweihanders,etc.,there are now troops in the mod for you!  These are typically faster than the two-handed clubs,but significantly slower than one-handed weapons.

#Axes(two hand):  these fit into a medium area between the swords and clubs.  They are probably the most lethal hand weapons,when speed,range,and damage is considered,other than warhammers and halberds,and can side-swing.  Cut damage only,no poke,but a hit is generaly a kill.

#Spears(either):  spears faced all sorts of nerfs in Native.  They're gone.  Spears were the most ubiquitous weapon of any period up to the advent of firearms for good reasons- cost,shield walls,cavalry-stopping hedges,and penetration.  Generally speaking,they are mediocre when used two-handed,except on an overhead strike,but poke attacks are fast and do some damage.  They no longer get side-swings,if they did before,simply because the damage simulation doesn't handle that correctly (it should not do overhand strike damage on a wild side-swing,but it does).

#Halberds and the bazillion variants(either):  a spear combined with an axe.  One of the most ubiquitous weapons of infantry during this period,an extremely deadly weapon when used without a shield,and a very effective weapon when using one.  Generally a one-hit kill on an overhand strike,slightly less poke than spears.  They no longer get side-swings,if they did before.

#Staves:  these weren't seen a lot on battlefields,largely because a spear is more effective,but they do blunt damage and they're generally cheap and moderately effective.

#Two-handed Blunt Objects.  Can you say "blunt force trauma"?  Actually most of them do Pierce (ignore armor) and all of 'em are crush-through.  
#These are nasty, nasty weapons, in theory, but very sloooooow.

#Weird Stuff:  There are a few weird things that just don't fit anywhere else,or were atypical of the period,but might have shown up.  For example,I put a punch-dagger in,for assassin types.  A catch-all for stuff that just didn't fit elsewhere.

["weapons_start","Weapons Start (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#DAGGERS(one hand)
#Non-combat Knives and Small Cleavers(one hand,cut/pierce)
["knife","Knife",[("peasant_knife_new",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry,itc_dagger|itcf_carry_dagger_front_left,100,weight(0.5)|difficulty(0)|spd_rtng(200)|weapon_length(40)|swing_damage(20,cut)|thrust_damage(40,pierce),imodbits_none],

["butchering_knife","Butchering Knife",[("khyber_knife_new",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry,itc_dagger|itcf_carry_dagger_front_right,75,weight(0.75)|difficulty(0)|spd_rtng(200)|weapon_length(60)|swing_damage(25,cut)|thrust_damage(15,pierce),imodbits_none],

["cleaver","Cleaver",[("cleaver_new",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry|itp_wooden_parry,itc_cleaver,200,weight(1.5)|difficulty(0)|spd_rtng(175)|weapon_length(30)|swing_damage(45,cut)|thrust_damage(0,pierce),imodbits_none],

["dagger","Dagger",[("dagger_b",0),("dagger_b_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry,itc_dagger|itcf_carry_dagger_front_left|itcf_show_holster_when_drawn,
200,weight(0.75)|difficulty(0)|spd_rtng(200)|weapon_length(47)|swing_damage(15,cut)|thrust_damage(40,pierce),imodbits_none],

#Mackie's daggers and knives.
["mackie_billhook","Billhook",[("mackie_billhook",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry,itc_dagger,100,weight(0.5)|difficulty(0)|spd_rtng(200)|weapon_length(42)|swing_damage(30,cut)|thrust_damage(10,pierce),imodbits_none],

["mackie_dagger","Bhuddist Dagger",[("mackie_dagger",0),("mackie_dagger_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry,itc_dagger|itcf_carry_dagger_front_left,500,weight(0.5)|difficulty(0)|spd_rtng(200)|weapon_length(42)|swing_damage(20,cut)|thrust_damage(45,pierce),imodbits_none],

#These are basically just big cleavers; I've made them fairly clumsy, but dangerous.
["mackie_beefsplitter01","Beef Splitter",[("mackie_beefsplitter01",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry,itc_cleaver,1500,weight(0.5)|difficulty(0)|spd_rtng(100)|weapon_length(80)|swing_damage(75,cut)|thrust_damage(1,pierce),imodbits_none],

["mackie_beefsplitter02","Beef Splitter",[("mackie_beefsplitter02",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry,itc_cleaver,1600,weight(0.5)|difficulty(0)|spd_rtng(95)|weapon_length(83)|swing_damage(80,cut)|thrust_damage(1,pierce),imodbits_none],

#Luigi's awesome weapons.
["lui_clamshelldagger","Clamshell Dagger",[("lui_clamshelldagger",0),("lui_clamshelldagger_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary,itc_rapier|itcf_carry_dagger_front_left, 400,weight(0.75)|difficulty(0)|spd_rtng(200)|weapon_length(50)|swing_damage(25,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_clamshelldagger","Clamshell Dagger",[("lui_clamshelldagger",0),("lui_clamshelldagger_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary,itc_rapier|itcf_carry_dagger_front_left, 400,weight(0.75)|difficulty(0)|spd_rtng(200)|weapon_length(50)|swing_damage(25,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_duelistdagger","Duelist's Dagger",[("lui_duelistdagger",0),("lui_duelistdagger_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary,itc_rapier|itcf_carry_dagger_front_left, 500,weight(0.75)|difficulty(0)|spd_rtng(200)|weapon_length(60)|swing_damage(20,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_estocdagger","Piercing Dagger",[("lui_estocdagger",0),("lui_estocdagger_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary,itc_rapier|itcf_carry_dagger_front_left, 500,weight(0.75)|difficulty(0)|spd_rtng(200)|weapon_length(60)|swing_damage(15,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_killerdagger","Assasin's Dagger",[("lui_killerdagger",0),("lui_killerdagger_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary,itc_rapier|itcf_carry_dagger_front_left, 2000,weight(0.75)|abundance(5)|difficulty(0)|spd_rtng(220)|weapon_length(50)|swing_damage(15,cut)|thrust_damage(65,pierce),imodbits_none],

#CLUBS(one hand)
#Wooden Clubs(blunt)
["wooden_stick","Wooden Stick",[("wooden_stick",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,1,weight(2.5)|difficulty(0)|spd_rtng(150)|weapon_length(90)|swing_damage(25,blunt)|thrust_damage(0,pierce),imodbits_none],

["club","Club",[("club",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,
200,weight(2.5)|difficulty(0)|spd_rtng(150)|weapon_length(95)|swing_damage(30,blunt)|thrust_damage(0,pierce),imodbits_none],

["cudgel","Cudgel",[("rrr_club1",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,
500,weight(2.5)|difficulty(0)|spd_rtng(130)|weapon_length(90)|swing_damage(40,blunt)|thrust_damage(0,pierce),imodbits_none],

["capped_cudgel","Iron-capped Cudgel",[("rrr_club2",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,2500,weight(2.5)|difficulty(0)|spd_rtng(130)|weapon_length(90)|swing_damage(45,blunt)|thrust_damage(0,pierce),imodbits_none],

#Mackie's Dog Cudgels and wooden mace.
["mackie_dog_cudgel","Dog Cudgel",[("mackie_dog_cudgel",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,2700,weight(2.5)|difficulty(0)|spd_rtng(140)|weapon_length(87)|swing_damage(45,blunt)|thrust_damage(0,pierce),imodbits_none],

["mackie_dog_cudgel_alt01","Elite Dog Cudgel",[("mackie_dog_cudgel_alt01",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,5500,weight(2.5)|difficulty(0)|spd_rtng(145)|weapon_length(87)|swing_damage(60,blunt)|thrust_damage(0,pierce),imodbits_none],

["mackie_great_lakes_mace","Tribal Mace",[("mackie_great_lakes_mace",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,450,weight(1.0)|difficulty(0)|spd_rtng(145)|weapon_length(56)|swing_damage(40,blunt)|thrust_damage(0,pierce),imodbits_none],

["mackie_bat","Long Club",[("mackie_bat",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry|itp_wooden_attack,itc_scimitar,450,weight(1.0)|difficulty(0)|spd_rtng(110)|weapon_length(117)|swing_damage(50,blunt)|thrust_damage(0,pierce),imodbits_none],

#Maces(pierce)
["archer_mace","Archer's Mace",[("rrr_mace4",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,1300,weight(3.5)|difficulty(0)|spd_rtng(140)|weapon_length(49)|swing_damage(40,pierce)|thrust_damage(0,pierce),imodbits_none],

["winged_mace","Flanged Mace",[("flanged_mace",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2000,weight(3.5)|difficulty(0)|spd_rtng(130)|weapon_length(80)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],

["balanced_winged_mace","Balanced Winged Mace",[("rrr_mace1",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2500,weight(3.5)|difficulty(0)|spd_rtng(135)|weapon_length(56)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],

["sarranid_mace_1","Iron Mace",[("mace_small_d",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2500,weight(3.5)|difficulty(0)|spd_rtng(135)|weapon_length(56)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],
 
["mace_2","Knobbed Mace",[("mace_a",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2500,weight(3.5)|difficulty(0)|spd_rtng(135)|weapon_length(56)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],
 
["mace_3","Spiked Mace",[("mace_c",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2700,weight(3.5)|difficulty(0)|spd_rtng(120)|weapon_length(56)|swing_damage(50,pierce)|thrust_damage(0,pierce),imodbits_none],

["mackie_bat_nailed","Long Spiked Club",[("mackie_bat_nailed",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2700,weight(3.5)|difficulty(0)|spd_rtng(100)|weapon_length(117)|swing_damage(55,pierce)|thrust_damage(0,pierce),imodbits_none],
 
["mace_4","Winged Mace",[("mace_b",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2700,weight(3.5)|difficulty(0)|spd_rtng(135)|weapon_length(56)|swing_damage(50,pierce)|thrust_damage(0,pierce),imodbits_none],

#Mackie's awesome morning star.
["mackie_morning_star","Short Morning Star",[("mackie_morning_star",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,3000,weight(3.5)|difficulty(0)|spd_rtng(135)|weapon_length(58)|swing_damage(50,pierce)|thrust_damage(0,pierce),imodbits_none],

#Luigi's awesome set.
["lui_mace","Winged Mace",[("lui_mace",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2500,weight(3.5)|difficulty(0)|spd_rtng(135)|weapon_length(65)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],

["lui_manhuntermace","Manhunter's Mace",[("lui_manhuntermace",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2700,weight(3.5)|difficulty(0)|spd_rtng(125)|weapon_length(65)|swing_damage(50,pierce)|thrust_damage(0,pierce),imodbits_none],

#This is one of the elite Pierce class weapons, fairly expensive.
["lui_manhunter_polehammer","Manhunter's Short Polehammer",[("lui_manhunter_polehammer",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,8500,weight(3.5)|difficulty(0)|spd_rtng(115)|weapon_length(110)|swing_damage(55,pierce)|thrust_damage(0,pierce),imodbits_none],

#Military Hammers(pierce)
["pickaxe","Pickaxe",[("fighting_pick_new",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry,itc_scimitar,500,weight(4)|difficulty(0)|spd_rtng(120)|weapon_length(80)|swing_damage(30,pierce)|thrust_damage(0,pierce),imodbits_none],

["hammer","Military Hammer",[("military_hammer",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,
1000,weight(4)|difficulty(0)|spd_rtng(130)|weapon_length(55)|swing_damage(40,pierce)|thrust_damage(0,pierce),imodbits_none],

["military_pick","Military Pick",[("steel_pick_new",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2000,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(62)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],

#My short War Pick; elite class weapon.
["xeno_war_pick01","Fine War Pick",[("xeno_war_pick01",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,
10000,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(77)|swing_damage(55,pierce)|thrust_damage(0,pierce),imodbits_none],

#Mackie's Mangler, short version; nasty.
["mackie_mangler_short","Short Mangler",[("mackie_mangler_short",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,
10000,weight(4)|difficulty(0)|spd_rtng(125)|weapon_length(98)|swing_damage(55,pierce)|thrust_damage(0,pierce),imodbits_none],

["fine_military_hammer","Fine Military Hammer",[("rrr_hammer1",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2500,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(62)|swing_damage(50,pierce)|thrust_damage(0,pierce),imodbits_none],

["fine_military_hammer2","Balanced Military Hammer",[("rrr_hammer3",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,3000,weight(4)|difficulty(0)|spd_rtng(145)|weapon_length(63)|swing_damage(50,pierce)|thrust_damage(0,pierce),imodbits_none],

["masterwork_military_hammer2","Masterwork Military Hammer",[("rrr_hammer3",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,5000,weight(4)|difficulty(0)|spd_rtng(150)|weapon_length(63)|swing_damage(50,pierce)|thrust_damage(0,pierce),imodbits_none,],

#Spiked hammers,with thrust attack(pierce)
["spiked_hammer","Hammer with Spike",[("rrr_hammer2",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
2500,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(70)|swing_damage(45,pierce)|thrust_damage(35,pierce),imodbits_none],

["spiked_hammer2","Hammer with Spike",[("rrr_hammer4",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
3000,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(70)|swing_damage(50,pierce)|thrust_damage(35,pierce),imodbits_none],

["bec_de_corbin_a","War Hammer",[("bec_de_corbin_a",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
3000,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(70)|swing_damage(50,pierce)|thrust_damage(35,pierce),imodbits_none],

#Luigi's awesome set.
["lui_empirehammer","War Hammer",[("lui_empirehammer",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
2500,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(70)|swing_damage(45,pierce)|thrust_damage(35,pierce),imodbits_none],

["lui_empirewarhammer","Fine War Hammer",[("lui_empirewarhammer",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
3000,weight(4)|difficulty(0)|spd_rtng(150)|weapon_length(75)|swing_damage(50,pierce)|thrust_damage(35,pierce),imodbits_none],

["lui_empirewarhammerb","Masterwork War Hammer",[("lui_empirewarhammerb",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
8500,weight(4)|difficulty(0)|spd_rtng(160)|weapon_length(75)|swing_damage(60,pierce)|thrust_damage(35,pierce),imodbits_none],

["lui_empirewarhammerc","War Hammer",[("lui_empirewarhammerc",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
2500,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(70)|swing_damage(45,pierce)|thrust_damage(35,pierce),imodbits_none],

["lui_inquisitionhammer","Fine War Hammer",[("lui_inquisitionhammer",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
3000,weight(4)|difficulty(0)|spd_rtng(150)|weapon_length(65)|swing_damage(50,pierce)|thrust_damage(35,pierce),imodbits_none],

["lui_knighthammer","Masterwork War Hammer",[("lui_knighthammer",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
6500,weight(4)|difficulty(0)|spd_rtng(120)|weapon_length(75)|swing_damage(65,pierce)|thrust_damage(35,pierce),imodbits_none],

["lui_knighthammerbastard","War Hammer",[("lui_knighthammerbastard",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
2500,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(85)|swing_damage(45,pierce)|thrust_damage(35,pierce),imodbits_none],

["lui_knighthammerempire","Fine War Hammer",[("lui_knighthammerempire",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_spiked_hammer,
6500,weight(4)|difficulty(0)|spd_rtng(150)|weapon_length(85)|swing_damage(55,pierce)|thrust_damage(35,pierce),imodbits_none],



#Spiked maces(pierce)
["spiked_club","Spiked Club",[("spiked_club",0)],itp_type_one_handed_wpn|itp_primary|itp_wooden_parry,itc_scimitar,
500,weight(6)|difficulty(0)|spd_rtng(120)|weapon_length(97)|swing_damage(40,pierce)|thrust_damage(0,pierce),imodbits_none],

["mace_1","Spiked Club",[("mace_d",0)],itp_type_one_handed_wpn|itp_primary|itp_wooden_parry,itc_scimitar,
500,weight(6)|difficulty(0)|spd_rtng(120)|weapon_length(97)|swing_damage(40,pierce)|thrust_damage(0,pierce),imodbits_none],

["morningstar","Morningstar",[("mace_morningstar_new",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2000,weight(5)|difficulty(0)|spd_rtng(140)|weapon_length(53)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],

["morningstar2","Morningstar",[("rrr_mace3",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,2000,weight(5)|difficulty(0)|spd_rtng(140)|weapon_length(53)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],

["spiked_mace","Spiked Mace",[("spiked_mace_new",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,3000,weight(5)|difficulty(0)|spd_rtng(120)|weapon_length(90)|swing_damage(50,pierce)|thrust_damage(0,pierce),imodbits_none],

#SWORDS (one hand)
#Falchions,scimitars,military cleavers.  Falchions and scimitars are fast,medium cutters,a good spammer weapon. Cleavers are very heavy cutters with long reach and slower swing rates and are Unbalanced,but are amongst the only one-handers that can crush through blocks.
#Taleworlds
["falchion","Falchion",[("falchion_new",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,
1500,weight(2.5)|difficulty(8)|spd_rtng(120)|weapon_length(73)|swing_damage(70,cut)|thrust_damage(0,pierce),imodbits_none],

["scimitar","Scimitar",[("scimitar_a",0),("scab_scimeter_a",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(97)|swing_damage(70,cut)|thrust_damage(0,pierce),imodbits_none],

["scimitar_b","Horseman's Scimitar",[("scimitar_b",0),("scab_scimeter_b",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2500,weight(1.5)|difficulty(0)|spd_rtng(135)|weapon_length(103)|swing_damage(75,cut)|thrust_damage(0,pierce),imodbits_none],

#Mackie's falchions, cutlass, "basehuitls"
["mackie_basehuitl_plain","Tribal Sword",[("mackie_basehuitl_plain",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,1700,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(93)|swing_damage(70,cut)|thrust_damage(0,pierce),imodbits_none],

["mackie_basehuitl","Tribal Elder's Sword",[("mackie_basehuitl",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,4000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(93)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none],

["mackie_cutlass","Infighter's Cutlass",[("mackie_cutlass",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,3000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(53)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none],

["mackie_falchion01","Soldier's Falchion",[("mackie_falchion01",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,3000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(76)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none],

["mackie_falchion02","Guard's Falchion",[("mackie_falchion02",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,5000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(92)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none],

#Youhou's Assassin blade.
["youhou_assassin_sword","Assassin's Sword",[("youhou_assassin_sword",0),("youhou_assassin_sword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip,10000,weight(1.5)|abundance(10)|difficulty(0)|spd_rtng(150)|weapon_length(115)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_sarranids]],

#Luigi's fantastic scimitar models
["lui_cimeterre","Scimitar",[("lui_cimeterre",0),("lui_cimeterre_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(97)|swing_damage(70,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_cimeterreb","Scimitar",[("lui_cimeterreb",0),("lui_cimeterreb_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(97)|swing_damage(70,cut)|thrust_damage(0,pierce),imodbits_none],

#Elite scimitar, longer length, higher damage
["lui_cimeterrec","Elite Scimitar",[("lui_cimeterrec",0),("lui_cimeterrec_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip,4000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(120)|swing_damage(75,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_khergitexecutioneronehand","Executioner's Sword",[("lui_khergitexecutioneronehand",0),("lui_khergitexecutioneronehand_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,3000,weight(1.5)|difficulty(0)|spd_rtng(100)|weapon_length(97)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Faster swing, lower length
["lui_dussack","Dussack",[("lui_dussack",0),("lui_dussack_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(82)|swing_damage(70,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["lui_longdussack","Long Dussack",[("lui_longdussack",0),("lui_longdussack_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(135)|weapon_length(100)|swing_damage(75,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["spak_butcher","Butcher",[("spak_butcher",0),("spak_butcher_scab",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_back,8000,weight(4)|difficulty(15)|spd_rtng(135)|weapon_length(90)|swing_damage(125,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

#Rare drop / shop, a prestige item with the fastest strike in the sub-class.
["lui_sultancimitar","Sultan's Scimitar",[("lui_sultancimitar",0),("lui_sultancimitar_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar|itcf_carry_sword_left_hip,10000,weight(1.5)|abundance(10)|difficulty(0)|spd_rtng(145)|weapon_length(103)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_sarranids]],

#Military cleavers(unbalanced,crush-through, not something you want to face)
#Taleworlds
["military_cleaver_b","Soldier's Cleaver",[("military_cleaver_b",0)],itp_crush_through|itp_unbalanced|itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,3000,weight(5)|difficulty(0)|spd_rtng(110)|weapon_length(90)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none],

["military_cleaver_c","Military Cleaver",[("military_cleaver_c",0)],itp_crush_through|itp_unbalanced|itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_scimitar,4500,weight(1.5)|difficulty(0)|spd_rtng(115)|weapon_length(90)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none],

#Broadswords
#This is a very broad (no pun intended) category,that covers European styles of broadsword and arabian/mongol sabre styles.
#These are cutting swords,with a weak piercing thrust (almost useless vs. heavy armor).
#They are very fast weapons with a lower damage than falchions and cleavers,so they make up in striking speed what they lack in total power.
#They generally have a very tight range of stats,other than fine examples with even greater speed (and a much higher cost).
#Taleworlds
["arabian_sword_a","Sarranid Sword",[("arabian_sword_a",0),("scab_arabian_sword_a",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["arabian_sword_b","Sarranid Arming Sword",[("arabian_sword_b",0),("scab_arabian_sword_b",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["sarranid_cavalry_sword","Sarranid Cavalry Sword",[("arabian_sword_c",0),("scab_arabian_sword_c",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2500,weight(1.5)|difficulty(0)|spd_rtng(145)|weapon_length(105)|swing_damage(65,cut)|thrust_damage(50,pierce),imodbits_none],

["arabian_sword_d","Sarranid Guard Sword",[("arabian_sword_d",0),("scab_arabian_sword_d",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,4000,weight(1.5)|difficulty(0)|spd_rtng(145)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["sword_medieval_a","Sword",[("sword_medieval_a",0),("sword_medieval_a_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],
 
["sword_medieval_b","Sword",[("sword_medieval_b",0),("sword_medieval_b_scabbard",ixmesh_carry),("sword_rusty_a",imodbit_rusty),("sword_rusty_a_scabbard",ixmesh_carry|imodbit_rusty)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["sword_medieval_c","Arming Sword",[("sword_medieval_c",0),("sword_medieval_c_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,3000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(95)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["sword_medieval_c_long","Arming Sword",[("sword_medieval_c_long",0),("sword_medieval_c_long_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2000,weight(1.5)|difficulty(0)|weapon_length(100)|spd_rtng(140)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["sword_medieval_d_long","Long Arming Sword",[("sword_medieval_d_long",0),("sword_medieval_d_long_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(65,cut)|thrust_damage(50,pierce),imodbits_none],

["sword_viking_1","Nordic Sword",[("sword_viking_c",0),("sword_viking_c_scabbard ",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,3000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(94)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["sword_viking_2","Nordic Sword",[("sword_viking_b",0),("sword_viking_b_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["sword_viking_3","Nordic War Sword",[("sword_viking_a",0),("sword_viking_a_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["sword_khergit_1","Nomad Sabre",[("khergit_sword_b",0),("khergit_sword_b_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(97)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["sword_khergit_2","Sabre",[("khergit_sword_c",0),("khergit_sword_c_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(97)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["sword_khergit_3","Sabre",[("khergit_sword_a",0),("khergit_sword_a_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(97)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["sword_khergit_4","Heavy Sabre",[("khergit_sword_d",0),("khergit_sword_d_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,4000,weight(1.5)|difficulty(0)|spd_rtng(125)|weapon_length(97)|swing_damage(75,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Mackie's falcata
["mackie_falcata01","Nordic Falcata",[("mackie_falcata01",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword,7000,weight(1.5)|difficulty(0)|spd_rtng(135)|weapon_length(88)|swing_damage(75,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

#RRR's awesome sword collection.
["rrr_agincourt_brhil","Agincourt",[("rrr_agincourt_brhil",0),("rrr_agincourt_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(110)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_agincourt_steel","Agincourt",[("rrr_agincourt_steel",0),("rrr_agincourt_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(110)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_baron_brhil","Baron's Sword",[("rrr_baron_brhil",0),("rrr_baron_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(110)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_baron_steel","Baron's Sword",[("rrr_baron_steel",0),("rrr_baron_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(110)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_bayeux_brhil","Bayeux",[("rrr_bayeux_brhil",0),("rrr_bayeux_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_bayeux_steel","Bayeux",[("rrr_bayeux_steel",0),("rrr_bayeux_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(95)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_berserkr_brhil","Beserkir",[("rrr_berserkr_brhil",0),("rrr_berserkr_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(145)|weapon_length(90)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_berserkr_steel","Beserkir",[("rrr_berserkr_steel",0),("rrr_berserkr_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(155)|weapon_length(90)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_caithness_brhil","Caithness",[("rrr_caithness_brhil",0),("rrr_caithness_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_caithness_steel","Caithness",[("rrr_caithness_steel",0),("rrr_caithness_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_castellan_brhil","Castellan",[("rrr_castellan_brhil",0),("rrr_castellan_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_castellan_steel","Castellan",[("rrr_castellan_steel",0),("rrr_castellan_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_clontarf_brhil","Clontarf",[("rrr_clontarf_brhil",0),("rrr_clontarf_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(110)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_clontarf_steel","Clontarf",[("rrr_clontarf_steel",0),("rrr_clontarf_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(110)|weapon_length(110)|swing_damage(80,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_constable_brhil","Constable's Blade",[("rrr_constable_brhil",0),("rrr_constable_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_constable_steel","Constable's Blade",[("rrr_constable_steel",0),("rrr_constable_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_count_brhil","Count's Sword",[("rrr_count_brhil",0),("rrr_count_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_count_steel","Count's Sword",[("rrr_count_steel",0),("rrr_count_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_crecy_brhil","Crecy",[("rrr_crecy_brhil",0),("rrr_crecy_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_crecy_steel","Crecy",[("rrr_crecy_steel",0),("rrr_crecy_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(105)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_duke_brhil","Duke's Sword",[("rrr_duke_brhil",0),("rrr_duke_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_duke_steel","Duke's Sword",[("rrr_duke_steel",0),("rrr_duke_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_gaddhjalt_brhil","Gaddhjalt",[("rrr_gaddhjalt_brhil",0),("rrr_gaddhjalt_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(105)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_gaddhjalt_steel","Gaddhjalt",[("rrr_gaddhjalt_steel",0),("rrr_gaddhjalt_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_gotland_brhil","Gotland",[("rrr_gotland_brhil",0),("rrr_gotland_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_gotland_steel","Gotland",[("rrr_gotland_steel",0),("rrr_gotland_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(105)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_hersir_brhil","Hersir",[("rrr_hersir_brhil",0),("rrr_hersir_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(145)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_hersir_steel","Hersir",[("rrr_hersir_steel",0),("rrr_hersir_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(155)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_hospitaller_brhil","Hospitaller's Blade",[("rrr_hospitaller_brhil",0),("rrr_hospitaller_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_hospitaller_steel","Hospitaller's Blade",[("rrr_hospitaller_steel",0),("rrr_hospitaller_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_huskarl_brhil","Huskarl's Sword",[("rrr_huskarl_brhil",0),("rrr_huskarl_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(155)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_huskarl_steel","Huskarl's Sword",[("rrr_huskarl_steel",0),("rrr_huskarl_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(125)|weapon_length(95)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_jarl_brhil","Jarl's Sword",[("rrr_jarl_brhil",0),("rrr_jarl_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(145)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_jarl_steel","Jarl's Sword",[("rrr_jarl_steel",0),("rrr_jarl_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3500,weight(1.5)|difficulty(0)|spd_rtng(155)|weapon_length(95)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_knight_brhil","Knight's Blade",[("rrr_knight_brhil",0),("rrr_knight_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(125)|weapon_length(95)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_knight_steel","Knight's Blade",[("rrr_knight_steel",0),("rrr_knight_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(145)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_laird_brhil","Laird",[("rrr_laird_brhil",0),("rrr_laird_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(90)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_laird_steel","Laird",[("rrr_laird_steel",0),("rrr_laird_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(90)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_landgraf_brhil","Landgraf",[("rrr_landgraf_brhil",0),("rrr_landgraf_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(110)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_landgraf_steel","Landgraf",[("rrr_landgraf_steel",0),("rrr_landgraf_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(110)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_mercenary_brhil","Mercenary's Blade",[("rrr_mercenary_brhil",0),("rrr_mercenary_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_mercenary_steel","Mercenary's Blade",[("rrr_mercenary_steel",0),("rrr_mercenary_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_norman_brhil","Norman Sword",[("rrr_norman_brhil",0),("rrr_norman_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(155)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_norman_steel","Norman Sword",[("rrr_norman_steel",0),("rrr_norman_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(125)|weapon_length(95)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_poitiers_brhil","Poitiers",[("rrr_poitiers_brhil",0),("rrr_poitiers_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(155)|weapon_length(90)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_poitiers_steel","Poitiers",[("rrr_poitiers_steel",0),("rrr_poitiers_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(165)|weapon_length(90)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_prince_brhil","Prince's Sword",[("rrr_prince_brhil",0),("rrr_prince_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,4000,weight(1.5)|difficulty(0)|spd_rtng(125)|weapon_length(95)|swing_damage(75,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_prince_steel","Prince's Sword",[("rrr_prince_steel",0),("rrr_prince_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(145)|weapon_length(95)|swing_damage(65,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_reeve_brhil","Reeve's Blade",[("rrr_reeve_brhil",0),("rrr_reeve_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(90)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_reeve_steel","Reeve's Blade",[("rrr_reeve_steel",0),("rrr_reeve_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(90)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_regent_brhil","Regent's Sword",[("rrr_regent_brhil",0),("rrr_regent_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_regent_steel","Regent's Sword",[("rrr_regent_steel",0),("rrr_regent_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_ritter_brhil","Ritter",[("rrr_ritter_brhil",0),("rrr_ritter_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_ritter_steel","Ritter",[("rrr_ritter_steel",0),("rrr_ritter_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_sempach_brhil","Sempach",[("rrr_sempach_brhil",0),("rrr_sempach_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_sempach_steel","Sempach",[("rrr_sempach_steel",0),("rrr_sempach_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(105)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_senlac_brhil","Senlac",[("rrr_senlac_brhil",0),("rrr_senlac_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_senlac_steel","Senlac",[("rrr_senlac_steel",0),("rrr_senlac_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_sovereign_brhil","Sovereign's Sword",[("rrr_sovereign_brhil",0),("rrr_sovereign_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(85)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_sovereign_steel","Sovereign's Sword",[("rrr_sovereign_steel",0),("rrr_sovereign_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(85)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_squire_brhil","Squire's Blade",[("rrr_squire_brhil",0),("rrr_squire_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_squire_steel","Squire's Blade",[("rrr_squire_steel",0),("rrr_squire_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_steward_brhil","Steward's Blade",[("rrr_steward_brhil",0),("rrr_steward_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_steward_steel","Steward's Blade",[("rrr_steward_steel",0),("rrr_steward_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_talhoffer_brhil","Talhoffer",[("rrr_talhoffer_brhil",0),("rrr_talhoffer_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(115)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_talhoffer_steel","Talhoffer",[("rrr_talhoffer_steel",0),("rrr_talhoffer_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(115)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_templar_brhil","Templar's Sword",[("rrr_templar_brhil",0),("rrr_templar_brhil_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_templar_steel","Templar's Sword",[("rrr_templar_steel",0),("rrr_templar_steel_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

#Mackie's Bastard and kriegsmesser
["mackie_bastard","Bastard Sword",[("mackie_bastard",0)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword,7000,weight(1.5)|difficulty(0)|spd_rtng(125)|weapon_length(145)|swing_damage(115,cut)|thrust_damage(50,pierce),imodbits_none],

["mackie_kriegsmesser","Kriegsmesser",[("mackie_kriegsmesser",0)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,7000,weight(1.5)|difficulty(0)|spd_rtng(135)|weapon_length(119)|swing_damage(115,cut)|thrust_damage(50,pierce),imodbits_none],

#Luigi's awesome sword collection.
["lui_armingfour","Arming Sword",[("lui_armingfour",0),("lui_armingfour_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(130)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_armingthree","Arming Sword",[("lui_armingthree",0),("lui_armingthree_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(125)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_armingtwo","Arming Sword",[("lui_armingtwo",0),("lui_armingtwo_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(115)|weapon_length(120)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_bastardsword","Bastard Sword",[("lui_bastardsword",0),("lui_bastardsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4000,weight(1.5)|difficulty(0)|spd_rtng(135)|weapon_length(115)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_bastardswordb","Bastard Sword",[("lui_bastardswordb",0),("lui_bastardswordb_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword,4500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(90,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_bastardswordc","Bastard Sword",[("lui_bastardswordc",0),("lui_bastardswordc_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4500,weight(1.5)|difficulty(0)|spd_rtng(110)|weapon_length(115)|swing_damage(120,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_bastardswordd","Bastard Sword",[("lui_bastardswordd",0),("lui_bastardswordd_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(105)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_bastardsworde","Bastard Sword",[("lui_bastardsworde",0),("lui_bastardsworde_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(100)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

#Special Note:  this is an extremely high-poly sword, and should not be used for regular troops.
["lui_bastardswordempire","Calradian Bastard Sword",[("lui_bastardswordempire",0),("lui_bastardswordempire_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,5000,weight(1.5)|difficulty(0)|spd_rtng(110)|weapon_length(115)|swing_damage(120,cut)|thrust_damage(55,pierce),imodbits_none],

["lui_bastardswordknight","Knight's Bastard Sword",[("lui_bastardswordknight",0),("lui_bastardswordknight_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4000,weight(1.5)|difficulty(0)|spd_rtng(125)|weapon_length(120)|swing_damage(110,cut)|thrust_damage(45,pierce),imodbits_none],

["lui_bastardswordnew","Bastard Sword",[("lui_bastardswordnew",0),("lui_bastardswordnew_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_clamshelloneh","Clamshell Sword",[("lui_clamshelloneh",0),("lui_clamshelloneh_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,6000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(95)|swing_damage(120,cut)|thrust_damage(55,pierce),imodbits_none],

["lui_claymorebastard","Bastard Claymore",[("lui_claymorebastard",0),("lui_claymorebastard_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4500,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(105)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_claymorebroad","Broad Claymore",[("lui_claymorebroad",0),("lui_claymorebroad_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,6000,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(95)|swing_damage(125,cut)|thrust_damage(45,pierce),imodbits_none],

["lui_crosswordonehand","Arming Sword",[("lui_crosswordonehand",0),("lui_crosswordonehand_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(110)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_darkswordoneh","Dark Sword",[("lui_darkswordoneh",0),("lui_darkswordoneh_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(100)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_dukesword","Duke's Sword",[("lui_dukesword",0),("lui_dukesword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4500,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(125)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_generalsword","General's Sword",[("lui_generalsword",0),("lui_generalsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(110)|weapon_length(120)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_gothiconehander","Gothic Sword",[("lui_gothiconehander",0),("lui_gothiconehander_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_highelflongsword","Relic Sword",[("lui_highelflongsword",0),("lui_highelflongsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,7000,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(102)|swing_damage(90,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_nordicbastard","Nordic Bastard",[("lui_nordicbastard",0),("lui_nordicbastard_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2000,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(110)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_onehander","Arming Sword",[("lui_onehander",0),("lui_onehander_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,2500,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(105)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_onehands","Onehander",[("lui_onehands",0),("lui_onehands_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,3000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(100)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_paladinsword","Paladin's Sword",[("lui_paladinsword",0),("lui_paladinsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4500,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(115)|swing_damage(110,cut)|thrust_damage(55,pierce),imodbits_none],

["lui_perfectsword","Perfect Sword",[("lui_perfectsword",0),("lui_perfectsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,5500,weight(1.5)|difficulty(0)|spd_rtng(140)|weapon_length(120)|swing_damage(110,cut)|thrust_damage(55,pierce),imodbits_none],

["lui_realbastard","Real Bastard Sword",[("lui_realbastard",0),("lui_realbastard_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,6000,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(110)|swing_damage(120,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_swadianlordsword","Swadian Lord's Sword",[("lui_swadianlordsword",0),("lui_swadianlordsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,4000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(115)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_vaegirlordsword","Vaegir Lord's Sword",[("lui_vaegirlordsword",0),("lui_vaegirlordsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,4500,weight(1.5)|difficulty(0)|spd_rtng(135)|weapon_length(115)|swing_damage(70,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_vaegirlordswordb","Vaegir Lord's Sword",[("lui_vaegirlordswordb",0),("lui_vaegirlordswordb_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,7000,weight(1.5)|difficulty(0)|spd_rtng(100)|weapon_length(125)|swing_damage(130,cut)|thrust_damage(55,pierce),imodbits_none],

["lui_vaegirsword","Vaegir Sword",[("lui_vaegirsword",0),("lui_vaegirsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,4000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(95)|swing_damage(60,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_windswordoneh","Wind Sword",[("lui_windswordoneh",0),("lui_windswordoneh_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,5000,weight(1.5)|difficulty(0)|spd_rtng(165)|weapon_length(90)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

#My Flamberge; long for a bastard sword, light, fast, weaker cut but very handy.
["xenoargh_flamberge","Flamberge",[("xenoargh_flamberge",0),("xenoargh_flamberge_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_bastardsword|itcf_carry_sword_left_hip,8000,weight(1.5)|difficulty(0)|spd_rtng(165)|weapon_length(138)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

["katana","Katana",[("katana",0),("katana_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,4000,weight(1.5)|difficulty(0)|spd_rtng(150)|weapon_length(95)|swing_damage(75,cut)|thrust_damage(50,pierce),imodbits_none],

["spak_evil_katana","Devil's Katana",[("spak_evil_katana",0),("spak_evil_katana_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_primary,itc_longsword|itcf_carry_sword_left_hip,9000,weight(1.5)|difficulty(0)|spd_rtng(175)|weapon_length(95)|swing_damage(95,cut)|thrust_damage(50,pierce),imodbits_none],

 #Short Swords
 #For the most part, these are Roman-style weapons descended from the classical stabbing swords.
 #This includes Nordic swords, small slashing weapons like the wazakashi, and the sword-cane, which was an oddity.
 #European-style short swords get extra penetration, as their typical use would include a lot of...
 #...attempts to get into chinks in armor, joint attacks, etc.
 
["sword_medieval_b_small","Short Sword",[("sword_medieval_b_small",0),("sword_medieval_b_small_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_extra_penetration,itc_smallsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2500,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(85)|swing_damage(40,pierce)|thrust_damage(50,pierce),imodbits_none],
 
["sword_medieval_c_small","Short Arming Sword",[("sword_medieval_c_small",0),("sword_medieval_c_small_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_extra_penetration,itc_smallsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,3500,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(86)|swing_damage(45,pierce)|thrust_damage(50,pierce),imodbits_none],
 
["sword_viking_3_small","Nordic Short War Sword",[("sword_viking_a_small",0),("sword_viking_a_small_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_extra_penetration,itc_smallsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,1700,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(86)|swing_damage(40,pierce)|thrust_damage(50,pierce),imodbits_none],
 
["sword_viking_2_small","Nordic Short Sword",[("sword_viking_b_small",0),("sword_viking_b_small_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_extra_penetration,itc_smallsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,2500,weight(1.5)|difficulty(0)|spd_rtng(160)|weapon_length(86)|swing_damage(40,pierce)|thrust_damage(50,pierce),imodbits_none],

#Luigi's awesome sword collection
["lui_shortsword","Calradian Short Sword",[("lui_shortsword",0),("lui_shortsword_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_extra_penetration,itc_smallsword|itcf_carry_sword_left_hip,5000,weight(1.5)|difficulty(0)|spd_rtng(165)|weapon_length(80)|swing_damage(45,pierce)|thrust_damage(50,pierce),imodbits_none],

["lui_mercenaryshort","Mercenary's Short Sword",[("lui_mercenaryshort",0),("lui_mercenaryshort_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_extra_penetration,itc_smallsword|itcf_carry_sword_left_hip,3500,weight(1.5)|difficulty(0)|spd_rtng(165)|weapon_length(80)|swing_damage(45,pierce)|thrust_damage(50,pierce),imodbits_none],

#Mackie's awesome swordcane; a true oddity in the mod but I loved it too much to leave it out.
["mackie_swordcane_blade","Gentleman's Sword-Cane",[("mackie_swordcane_blade",0),("mackie_swordcane_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_back,5000,weight(1.5)|difficulty(0)|spd_rtng(165)|weapon_length(79)|swing_damage(55,cut)|thrust_damage(60,pierce),imodbits_none],

["mackie_swordcane_blade","Lordly Sword-Cane",[("mackie_swordcane_blade",0),("mackie_swordcane_scabbard_alt",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_back,20000,weight(1.5)|difficulty(0)|spd_rtng(165)|weapon_length(79)|swing_damage(55,cut)|thrust_damage(80,pierce),imodbits_none],

#This is the only weapon in this sub-class that is ~not~ a stabbing weapon.
["strange_short_sword","Wakizashi",[("wakizashi",0),("wakizashi_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip|itcf_show_holster_when_drawn,1700,weight(1.5)|difficulty(0)|spd_rtng(165)|weapon_length(65)|swing_damage(55,cut)|thrust_damage(50,pierce),imodbits_none],

#Estocs, Rapiers, and other Stabbing Swords
#May have higher speed.  Better stabbing damage; generally has much poorer slashing damage.  
#These are swords for people who prefer speedy strikes and a quick retreat over hacking and slashing in the middle of a crowd.
#Luigi's Fantastic Swords (pretty much everything in this category is from this revised set, but I like noting the origin anyhow)

#Rapiers
["lui_commanderapier","Commander's Rapier",[("lui_commanderapier",0),("lui_commanderapier_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,3000,weight(2)|difficulty(0)|spd_rtng(140)|weapon_length(124)|swing_damage(50,cut)|thrust_damage(65,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["lui_empirerapier","Imperial Rapier",[("lui_empirerapier",0),("lui_empirerapier_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,3000,weight(2)|difficulty(0)|spd_rtng(140)|weapon_length(124)|swing_damage(50,cut)|thrust_damage(65,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["lui_empirerapierb","Imperial Rapier",[("lui_empirerapierb",0),("lui_empirerapierb_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,3000,weight(2)|difficulty(0)|spd_rtng(140)|weapon_length(124)|swing_damage(50,cut)|thrust_damage(65,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],
 
["lui_rapier","Light Rapier",[("lui_rapier",0),("lui_rapier_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,3000,weight(2)|difficulty(0)|spd_rtng(150)|weapon_length(116)|swing_damage(45,cut)|thrust_damage(60,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["lui_rapierb","Light Rapier",[("lui_rapierb",0),("lui_rapierb_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,3000,weight(2)|difficulty(0)|spd_rtng(150)|weapon_length(120)|swing_damage(50,cut)|thrust_damage(60,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["lui_rapierc","Short Rapier",[("lui_rapierc",0),("lui_rapierc_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,3000,weight(2)|difficulty(0)|spd_rtng(155)|weapon_length(98)|swing_damage(50,cut)|thrust_damage(60,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["lui_rapierd","Heavy Rapier",[("lui_rapierd",0),("lui_rapierd_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,5000,weight(2)|difficulty(0)|spd_rtng(145)|weapon_length(118)|swing_damage(70,cut)|thrust_damage(65,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["lui_rapiere","Guarded Rapier",[("lui_rapiere",0),("lui_rapiere_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,3500,weight(2)|difficulty(0)|spd_rtng(150)|weapon_length(120)|swing_damage(60,cut)|thrust_damage(60,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

["lui_rapierg","Long Rapier",[("lui_rapierg",0),("lui_rapierg_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_rapier|itcf_carry_sword_left_hip,4000,weight(2)|difficulty(0)|spd_rtng(150)|weapon_length(124)|swing_damage(65,cut)|thrust_damage(60,pierce),imodbits_none,[],[fac_kingdom_of_rhodoks]],

#Estocs 
["lui_estoc","Estoc",[("lui_estoc",0),("lui_estoc_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,4000,weight(2)|difficulty(0)|spd_rtng(140)|weapon_length(124)|swing_damage(40,pierce)|thrust_damage(75,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_estocc","Strong Estoc",[("lui_estocc",0),("lui_estocc_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,5000,weight(2)|difficulty(0)|spd_rtng(140)|weapon_length(124)|swing_damage(45,pierce)|thrust_damage(75,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_estocd","Swift Estoc",[("lui_estocd",0),("lui_estocd_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,5000,weight(2)|difficulty(0)|spd_rtng(150)|weapon_length(124)|swing_damage(40,pierce)|thrust_damage(75,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_newestoc","Masterwork Estoc",[("lui_newestoc",0),("lui_newestoc_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,8000,weight(2)|difficulty(0)|spd_rtng(145)|weapon_length(126)|swing_damage(50,pierce)|thrust_damage(75,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_empireestoc","Imperial Estoc",[("lui_empireestoc",0),("lui_empireestoc_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,4500,weight(2)|difficulty(0)|spd_rtng(135)|weapon_length(126)|swing_damage(45,pierce)|thrust_damage(75,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],
 
["lui_empireestocb","Knight's Estoc",[("lui_empireestocb",0),("lui_empireestocb_scabbard",ixmesh_carry)],itp_type_one_handed_wpn|itp_merchandise|itp_primary,itc_longsword|itcf_carry_sword_left_hip,5500,weight(2)|difficulty(0)|spd_rtng(145)|weapon_length(126)|swing_damage(45,pierce)|thrust_damage(75,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],
 
#AXES(one hand)
#Generally have power / speed much like scimtars- greater power for size, slower speed, extra damage vs. shields remains, larger axes crush-through.
#Hatchets and small axes
["hatchet","Hatchet",[("hatchet",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry,itc_scimitar|itcf_carry_axe_left_hip,1000,weight(2)|difficulty(0)|spd_rtng(130)|weapon_length(60)|swing_damage(60,cut)|thrust_damage(0,pierce),imodbits_none],

["hand_axe","Hand Axe",[("hatchet",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry,itc_scimitar|itcf_carry_axe_left_hip,1200,weight(2)|difficulty(7)|spd_rtng(135)|weapon_length(75)|swing_damage(60,cut)|thrust_damage(0,pierce),imodbits_none],

#War axes- one-handed long axes- slower than swords, but crush-through and extra penetration
["fighting_axe","Fighting Axe",[("fighting_ax",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(90)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["sarranid_axe_a","Iron Battle Axe",[("one_handed_battle_axe_g",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4500,weight(2.5)|difficulty(9)|spd_rtng(110)|weapon_length(90)|swing_damage(110,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_sarranids]],

["one_handed_war_axe_a","One Handed Axe",[("one_handed_war_axe_a",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(120)|weapon_length(71)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none],

["one_handed_battle_axe_a","One Handed Battle Axe",[("one_handed_battle_axe_a",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4000,weight(2.5)|difficulty(9)|spd_rtng(120)|weapon_length(73)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none],

["one_handed_war_axe_b","One Handed War Axe",[("one_handed_war_axe_b",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(110)|weapon_length(76)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none],

["one_handed_battle_axe_b","One Handed Battle Axe",[("one_handed_battle_axe_b",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(72)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none],

["one_handed_battle_axe_c","One Handed Battle Axe",[("one_handed_battle_axe_c",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,3000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(76)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none],

["sarranid_axe_b","Iron War Axe",[("one_handed_battle_axe_h",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(110)|weapon_length(71)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_sarranids]],

#Luigi's OSP Axes
["lui_baronaxeoneh","Baron's Axe",[("lui_baronaxeoneh",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,5000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(70)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_battleaxea","Battle Axe",[("lui_battleaxea",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4000,weight(2.5)|difficulty(9)|spd_rtng(125)|weapon_length(70)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_battleaxeb","Battle Axe",[("lui_battleaxeb",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(110)|weapon_length(75)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_battleaxenew","Battle Axe",[("lui_battleaxenew",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,3000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(70)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_battleaxenewb","Battle Axe",[("lui_battleaxenewb",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2500,weight(2.5)|difficulty(9)|spd_rtng(135)|weapon_length(70)|swing_damage(75,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_battleaxetwoh","Battle Axe",[("lui_battleaxetwoh",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4000,weight(2.5)|difficulty(9)|spd_rtng(130)|weapon_length(75)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_knightaxetwoha","Battle Axe",[("lui_knightaxetwoha",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,3000,weight(2.5)|difficulty(9)|spd_rtng(120)|weapon_length(70)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_kjergitaxe","Khergit Battle Axe",[("lui_kjergitaxe",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,3500,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(70)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["lui_knightaxeoneh","Battle Axe",[("lui_knightaxeoneh",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(130)|weapon_length(60)|swing_damage(70,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_knightaxeonehb","Battle Axe",[("lui_knightaxeonehb",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,3000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(65)|swing_damage(95,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_knightaxeonehc","Battle Axe",[("lui_knightaxeonehc",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(120)|weapon_length(65)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_knightaxeonehd","Battle Axe",[("lui_knightaxeonehd",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4000,weight(2.5)|difficulty(9)|spd_rtng(125)|weapon_length(65)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_knightaxeonehe","Battle Axe",[("lui_knightaxeonehe",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(70)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_swadianonehandedaxe","Battle Axe",[("lui_swadianonehandedaxe",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,3000,weight(2.5)|difficulty(9)|spd_rtng(125)|weapon_length(65)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_vaegiraxe","Battle Axe",[("lui_vaegiraxe",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,3000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(70)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_swadians]],

["lui_vikingaxebastard","Viking Axe",[("lui_vikingaxebastard",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(70)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["lui_vikingaxeoneh","Viking Axe",[("lui_vikingaxeoneh",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,3000,weight(2.5)|difficulty(9)|spd_rtng(115)|weapon_length(70)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["lui_vikingaxeonehb","Viking Axe",[("lui_vikingaxeonehb",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(135)|weapon_length(60)|swing_damage(75,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["lui_vikingaxeonehc","Viking Axe",[("lui_vikingaxeonehc",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4000,weight(2.5)|difficulty(9)|spd_rtng(125)|weapon_length(65)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["lui_warbastardaxea","Sarranid Axe",[("lui_warbastardaxea",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(110)|weapon_length(75)|swing_damage(90,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_sarranids]],

["lui_waronehandedaxe","Viking Axe",[("lui_waronehandedaxe",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(125)|weapon_length(65)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["lui_waronehandedaxeb","War Axe",[("lui_waronehandedaxeb",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,4000,weight(2.5)|difficulty(9)|spd_rtng(120)|weapon_length(70)|swing_damage(85,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_waronehandedaxec","Viking Axe",[("lui_waronehandedaxec",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_left_hip,2000,weight(2.5)|difficulty(9)|spd_rtng(130)|weapon_length(55)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

#Mackie's axes.
#Mackie's double axe.  Probably only good for chopping wood IRL, but it's cool looking.
["mackie_double_axe","Rhulg Short Axe",[("mackie_double_axe",0),("mackie_double_axe_carry",ixmesh_carry)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_back,5000,weight(2.5)|difficulty(10)|spd_rtng(125)|weapon_length(53)|swing_damage(100,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_vaegirs]],

["mackie_bearded_axe","Valgarde's Axe",[("mackie_bearded_axe",0),("mackie_bearded_axe_carry",ixmesh_carry)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_back,10000,weight(2.5)|difficulty(10)|spd_rtng(105)|weapon_length(100)|swing_damage(120,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["mackie_celtic_axe","Celtic Bard's Axe",[("mackie_celtic_axe",0),("mackie_celtic_axe_carry",ixmesh_carry)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_back,8000,weight(2.5)|difficulty(10)|spd_rtng(125)|weapon_length(98)|swing_damage(100,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["mackie_varangian_axe","Varangian Axe",[("mackie_varangian_axe",0),("mackie_varangian_axe_carry",ixmesh_carry)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_back,4500,weight(2.5)|difficulty(10)|spd_rtng(125)|weapon_length(75)|swing_damage(80,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

#This is a really ridiculous kitbashed axe.  Damage is extreme, speed is terrible.
["mackie_pendulum_axe","Guan Yu's Axe",[("mackie_pendulum_axe",0),("mackie_pendulum_axe_carry",ixmesh_carry)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_back,10000,weight(2.5)|difficulty(15)|spd_rtng(105)|weapon_length(80)|swing_damage(150,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Grom!
["xeno_grom","Grom",[("xeno_grom",0),("xeno_grom_carry",ixmesh_carry)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_back,15000,weight(2.5)|difficulty(15)|spd_rtng(135)|weapon_length(46)|swing_damage(140,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_vaegirs]],

#Flesh Golem special item!
["xeno_golem_axe","Reaper Axe",[("xeno_golem_axe",0)],itp_crush_through|itp_type_one_handed_wpn|itp_primary|itp_bonus_against_shield|itp_extra_penetration|itp_unique,itc_scimitar|itcf_carry_axe_back,15000,weight(2.5)|difficulty(15)|spd_rtng(75)|weapon_length(80)|swing_damage(200,cut)|thrust_damage(0,pierce),imodbits_none],

#The Taleworlds "Viking" axe, heh.  A classic.
["taleworlds_doubleaxe","Rhulg Axe",[("taleworlds_doubleaxe",0)],itp_crush_through|itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_scimitar|itcf_carry_axe_back,3500,weight(2.5)|difficulty(11)|spd_rtng(90)|weapon_length(100)|swing_damage(115,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_vaegirs]],

#Two-Handed Swords
#Great attacks, but far slower than one-handed swords.  The largest swords can crush through blocks, and most can damage even knights in full plate.
#Needless to say, if you're not wearing full plate, then getting hit with these will ruin your day ;-)
#Like one-handed swords, there are a lot of sub-categories.

#Scimitars and Military Cleavers
#Very high slash damage, no thrust attack.  Cleavers can crush through blocks, scimitars may not, with cooresponding speed penalties for weight.

#Not at the high end, damage-wise, but fast and reasonably cheap.
#Taleworlds
["khergit_sword_two_handed_a","Two Handed Sabre",[("khergit_sword_two_handed_a",0)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi|itcf_carry_sword_back, 5000,weight(5)|difficulty(12)|spd_rtng(115)|weapon_length(122)|swing_damage(100,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],
 
["khergit_sword_two_handed_b","Two Handed Sabre",[("khergit_sword_two_handed_b",0)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi|itcf_carry_sword_back, 6000,weight(5)|difficulty(12)|spd_rtng(110)|weapon_length(122)|swing_damage(115,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Luigi's awesome collection
["lui_bastardcimitar","Bastard Scimitar",[("lui_bastardcimitar",0),("lui_bastardcimitar_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi|itcf_carry_sword_back, 6000,weight(5)|difficulty(12)|spd_rtng(90)|weapon_length(117)|swing_damage(140,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_sarranids]],

["lui_khergitexecutioner","Executioner's Sword",[("lui_khergitexecutioner",0),("lui_khergitexecutioner_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi|itcf_carry_sword_back, 6000,weight(5)|difficulty(12)|spd_rtng(100)|weapon_length(122)|swing_damage(130,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["lui_khergitlordexecutioner","Lordly Executioner's Sword",[("lui_khergitlordexecutioner",0),("lui_khergitlordexecutioner_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi|itcf_carry_sword_back, 15000,weight(5)|difficulty(12)|spd_rtng(110)|weapon_length(122)|swing_damage(150,cut)|thrust_damage(0,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Two-handed war cleaver:  deadly dangerous, thank goodness it's so sloooooow
["two_handed_cleaver","War Cleaver",[("military_cleaver_a",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi|itcf_carry_sword_back, 7000,weight(10)|difficulty(15)|spd_rtng(75)|weapon_length(120)|swing_damage(165,cut)|thrust_damage(0,cut),imodbits_none],

["mackie_godenak","Godenak",[("mackie_godenak",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi|itcf_carry_sword_back, 12000,weight(10)|difficulty(15)|spd_rtng(105)|weapon_length(111)|swing_damage(155,cut)|thrust_damage(0,cut),imodbits_none],

#Two-handed swords (claymores, greatswords, big bastard swords, etc.)
#Even higher slash damage, thrust attack is OK, expensive and slower than the big scimitars
#Taleworlds
["sword_two_handed_b","Two Handed Sword",[("sword_two_handed_b",0),("scab_bastardsw_b",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 4000,weight(6)|difficulty(11)|spd_rtng(100)|weapon_length(110)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],
 
["sword_two_handed_a","Great Sword",[("sword_two_handed_a",0),("scab_bastardsw_b",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(120)|swing_damage(120,cut)|thrust_damage(65,pierce),imodbits_none],

["sword_of_war","Sword of War",[("b_bastard_sword",0),("scab_bastardsw_b",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back,6000,weight(6)|difficulty(11)|spd_rtng(80)|weapon_length(130)|swing_damage(125,cut)|thrust_damage(65,pierce),imodbits_none],

["bastard_sword_a","Heavy Bastard Sword",[("bastard_sword_a",0),("bastard_sword_a_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_greatsword|itcf_carry_sword_left_hip,6000,weight(6)|difficulty(11)|spd_rtng(80)|weapon_length(130)|swing_damage(130,cut)|thrust_damage(65,pierce),imodbits_none],

["bastard_sword_b","Long Bastard Sword",[("bastard_sword_b",0),("bastard_sword_b_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_primary,itc_greatsword|itcf_carry_sword_left_hip,5000,weight(6)|difficulty(11)|spd_rtng(85)|weapon_length(130)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

#Luigi's models

["lui_clamshellclaymore","Clamshell Claymore",[("lui_clamshellclaymore",0),("lui_clamshellclaymore_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(120)|swing_damage(130,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_clamshellclaymoreb","Clamshell Claymore",[("lui_clamshellclaymoreb",0),("lui_clamshellclaymoreb_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 8000,weight(6)|difficulty(11)|spd_rtng(85)|weapon_length(150)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_clamshellcross","Clamshell Cross",[("lui_clamshellcross",0),("lui_clamshellcross_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 7000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(125)|swing_damage(140,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_claymoretwoh","Great Claymore",[("lui_claymoretwoh",0),("lui_claymoretwoh_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 6500,weight(6)|difficulty(11)|spd_rtng(75)|weapon_length(140)|swing_damage(140,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_crosstwohander","Two-Handed Cross",[("lui_crosstwohander",0),("lui_crosstwohander_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 6000,weight(6)|difficulty(11)|spd_rtng(85)|weapon_length(125)|swing_damage(140,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_darkswordtwoh","Two-Handed DarkSword",[("lui_darkswordtwoh",0),("lui_darkswordtwoh_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 12500,weight(6)|difficulty(11)|spd_rtng(100)|weapon_length(120)|swing_damage(160,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_darkswordtwohb","Great DarkSword",[("lui_darkswordtwohb",0),("lui_darkswordtwohb_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 15000,weight(6)|difficulty(11)|spd_rtng(95)|weapon_length(139)|swing_damage(170,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_empiretwohander","Imperial Greatsword",[("lui_empiretwohander",0),("lui_empiretwohander_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(125)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_greatsword","Greatsword",[("lui_greatsword",0),("lui_greatsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(125)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_greatswordb","Greatsword",[("lui_greatswordb",0),("lui_greatswordb_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(80)|weapon_length(125)|swing_damage(135,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_greatswordc","Greatsword",[("lui_greatswordc",0),("lui_greatswordc_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(125)|swing_damage(130,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_greatswordd","Greatsword",[("lui_greatswordd",0),("lui_greatswordd_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(125)|swing_damage(120,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_greatsworde","Greatsword",[("lui_greatsworde",0),("lui_greatsworde_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 6000,weight(6)|difficulty(11)|spd_rtng(85)|weapon_length(130)|swing_damage(140,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_highelftwohander","Relic Greatsword",[("lui_highelftwohander",0),("lui_highelftwohander_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 7000,weight(6)|difficulty(11)|spd_rtng(105)|weapon_length(125)|swing_damage(130,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_highelftwohanderb","Relic Greatsword",[("lui_highelftwohanderb",0),("lui_highelftwohanderb_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 7000,weight(6)|difficulty(11)|spd_rtng(125)|weapon_length(125)|swing_damage(120,cut)|thrust_damage(55,pierce),imodbits_none],

["lui_highelftwohanderc","Fine Relic Greatsword",[("lui_highelftwohanderc",0),("lui_highelftwohanderc_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 10000,weight(6)|difficulty(11)|spd_rtng(120)|weapon_length(125)|swing_damage(140,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_inquisitiongreatsword","Inquisitor's Greatsword",[("lui_inquisitiongreatsword",0),("lui_inquisitiongreatsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 6500,weight(6)|difficulty(11)|spd_rtng(80)|weapon_length(130)|swing_damage(140,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_inquisitorredemptionsword","Inquisitor's Redemptor",[("lui_inquisitorredemptionsword",0),("lui_inquisitorredemptionsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 11000,weight(6)|difficulty(11)|spd_rtng(115)|weapon_length(110)|swing_damage(140,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_inquisitorsword","Inquisitor's Sword",[("lui_inquisitorsword",0),("lui_inquisitorsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 9000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(125)|swing_damage(145,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_kingswordtwoh","Two-Handed King's Sword",[("lui_kingswordtwoh",0),("lui_kingswordtwoh_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(95)|weapon_length(115)|swing_damage(120,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_kingswordtwohb","Two-Handed King's Sword",[("lui_kingswordtwohb",0),("lui_kingswordtwohb_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 6500,weight(6)|difficulty(11)|spd_rtng(110)|weapon_length(110)|swing_damage(130,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_kingswordtwohdark","Dark King's Sword",[("lui_kingswordtwohdark",0),("lui_kingswordtwohdark_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 15000,weight(6)|difficulty(13)|spd_rtng(115)|weapon_length(130)|swing_damage(150,cut)|thrust_damage(75,pierce),imodbits_none],

["lui_light","Sword of Light",[("lui_light",0),("lui_light_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 15000,weight(6)|difficulty(11)|spd_rtng(120)|weapon_length(125)|swing_damage(140,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_mercenaryclaymore","Two-Handed Merc's Sword",[("lui_mercenaryclaymore",0),("lui_mercenaryclaymore_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(125)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_midnight","Midnight",[("lui_midnight",0),("lui_midnight_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 11000,weight(6)|difficulty(11)|spd_rtng(75)|weapon_length(145)|swing_damage(150,cut)|thrust_damage(75,pierce),imodbits_none],

["lui_nordictwoh","Nordic Two-Hander",[("lui_nordictwoh",0),("lui_nordictwoh_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(120)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_perfecttwohandedsword","Perfect Two-Handed Sword",[("lui_perfecttwohandedsword",0),("lui_perfecttwohandedsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 8500,weight(6)|difficulty(11)|spd_rtng(100)|weapon_length(135)|swing_damage(135,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_raiderlordsword","Raider Lord's Sword",[("lui_raiderlordsword",0),("lui_raiderlordsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 6000,weight(6)|difficulty(11)|spd_rtng(85)|weapon_length(135)|swing_damage(130,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_roughdaksword","Roughdak Sword",[("lui_roughdaksword",0),("lui_roughdaksword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(100)|weapon_length(110)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_twohander","Inferior Zweihander",[("lui_twohander",0),("lui_twohander_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 4000,weight(6)|difficulty(11)|spd_rtng(100)|weapon_length(110)|swing_damage(110,cut)|thrust_damage(55,pierce),imodbits_none],

["lui_vaegirltwohander","Vaegir Twohander",[("lui_vaegirltwohander",0),("lui_vaegirltwohander_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 6000,weight(6)|difficulty(11)|spd_rtng(80)|weapon_length(140)|swing_damage(130,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_vaegirtwohsword","Inferior Vaegir Twohander",[("lui_vaegirtwohsword",0),("lui_vaegirtwohsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 4000,weight(6)|difficulty(11)|spd_rtng(105)|weapon_length(110)|swing_damage(110,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_windsword","Wind Sword",[("lui_windsword",0),("lui_windsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 4500,weight(6)|difficulty(11)|spd_rtng(100)|weapon_length(120)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_witchsword","Witch's Sword",[("lui_witchsword",0),("lui_witchsword_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 4500,weight(6)|difficulty(11)|spd_rtng(95)|weapon_length(120)|swing_damage(115,cut)|thrust_damage(55,pierce),imodbits_none],

["lui_zweilander","Zweihander",[("lui_zweilander",0),("lui_zweilander_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5500,weight(6)|difficulty(11)|spd_rtng(70)|weapon_length(140)|swing_damage(140,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_zweilanderb","Zweihander",[("lui_zweilanderb",0),("lui_zweilanderb_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(130)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_zweilanderc","Zweihander",[("lui_zweilanderc",0),("lui_zweilanderc_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5500,weight(6)|difficulty(11)|spd_rtng(70)|weapon_length(140)|swing_damage(140,cut)|thrust_damage(65,pierce),imodbits_none],

["lui_zweilanderd","Zweihander",[("lui_zweilanderd",0),("lui_zweilanderd_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 5000,weight(6)|difficulty(11)|spd_rtng(95)|weapon_length(120)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

#My Flambard; light, long, weak cut, good Thrust, great speed.
["xenoargh_flambard","Flambard",[("xenoargh_flambard",0),("xenoargh_flambard_scabbard",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_greatsword|itcf_carry_sword_back, 9000,weight(6)|difficulty(11)|spd_rtng(125)|weapon_length(183)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["nodachi","No Dachi",[("no_dachi",0)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi, 5000,weight(6)|difficulty(11)|spd_rtng(90)|weapon_length(125)|swing_damage(120,cut)|thrust_damage(45,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Mackie's awesome Japanese stuff
["mackie_nagamaki","Nagamaki",[("mackie_nagamaki",0)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi, 5000,weight(6)|difficulty(11)|spd_rtng(80)|weapon_length(122)|swing_damage(130,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["mackie_nodachi","Lord's No Dachi",[("mackie_nodachi",0)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi, 12000,weight(6)|difficulty(11)|spd_rtng(100)|weapon_length(118)|swing_damage(140,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["mackie_strange_sword","Great Nagamaki",[("mackie_strange_sword",0)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi, 7000,weight(6)|difficulty(11)|spd_rtng(75)|weapon_length(136)|swing_damage(140,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Mackie's Panabas.  Totally badass slicing stick-thing :-)
["mackie_panabas","Panabas",[("mackie_panabas",0),("mackie_panabas_carry",ixmesh_carry)],itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_nodachi|itcf_carry_sword_back,5000,weight(6)|difficulty(9)|spd_rtng(110)|weapon_length(111)|swing_damage(125,cut)|thrust_damage(40,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Two-handed Axes
#Great power, slower than swords but even better damage; splits shields and is a crush-through attack.  Very dangerous, even vs. plate armors.

["axe","Axe",[("iron_ax",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5500,weight(4)|difficulty(8)|spd_rtng(85)|weapon_length(108)|swing_damage(140,cut)|thrust_damage(0,pierce),imodbits_none],

["voulge","Voulge",[("voulge",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,6000,weight(4)|difficulty(8)|spd_rtng(80)|weapon_length(119)|swing_damage(150,cut)|thrust_damage(0,pierce),imodbits_none],

["battle_axe","Battle Axe",[("battle_ax",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,6000,weight(4)|difficulty(8)|spd_rtng(75)|weapon_length(108)|swing_damage(145,cut)|thrust_damage(0,pierce),imodbits_none],

["war_axe","War Axe",[("war_ax",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5500,weight(4)|difficulty(8)|spd_rtng(80)|weapon_length(110)|swing_damage(140,cut)|thrust_damage(0,pierce),imodbits_none],

["sarranid_two_handed_axe_a","Sarranid Battle Axe",[("two_handed_battle_axe_g",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5500,weight(4)|difficulty(8)|spd_rtng(95)|weapon_length(95)|swing_damage(120,cut)|thrust_damage(0,pierce),imodbits_none],

["sarranid_two_handed_axe_b","Sarranid War Axe",[("two_handed_battle_axe_h",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(80)|weapon_length(90)|swing_damage(140,cut)|thrust_damage(0,pierce),imodbits_none],

["two_handed_axe","Two Handed Axe",[("two_handed_battle_axe_a",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5500,weight(4.5)|difficulty(10)|spd_rtng(110)|weapon_length(90)|swing_damage(125,cut)|thrust_damage(0,pierce),imodbits_none],

["two_handed_battle_axe_2","Two Handed War Axe",[("two_handed_battle_axe_b",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(85)|weapon_length(96)|swing_damage(130,cut)|thrust_damage(0,pierce),imodbits_none],

["shortened_voulge","Shortened Voulge",[("two_handed_battle_axe_c",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry,itc_nodachi|itcf_carry_axe_back,5500,weight(4)|difficulty(8)|spd_rtng(65)|weapon_length(100)|swing_damage(160,cut)|thrust_damage(0,pierce),imodbits_none],

#Mackie's shortened voulge
["mackie_voulge_short","Shortened Voulge",[("mackie_voulge_short",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5500,weight(4)|difficulty(8)|spd_rtng(90)|weapon_length(80)|swing_damage(140,cut)|thrust_damage(0,pierce),imodbits_none],

["great_axe","Great Axe",[("two_handed_battle_axe_e",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,6500,weight(4)|difficulty(8)|spd_rtng(105)|weapon_length(96)|swing_damage(130,cut)|thrust_damage(0,pierce),imodbits_none],

["long_axe","Long Axe",[("long_axe_a",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(80)|weapon_length(120)|swing_damage(140,cut)|thrust_damage(0,pierce),imodbits_none],

["long_axe_alt","Long Axe",[("long_axe_a",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(75)|weapon_length(120)|swing_damage(145,cut)|thrust_damage(0,pierce),imodbits_none],

["long_axe_b","Long War Axe",[("long_axe_b",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5500,weight(4)|difficulty(8)|spd_rtng(70)|weapon_length(125)|swing_damage(155,cut)|thrust_damage(0,pierce),imodbits_none],

["long_axe_b_alt","Long War Axe",[("long_axe_b",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,6000,weight(4)|difficulty(8)|spd_rtng(95)|weapon_length(125)|swing_damage(145,cut)|thrust_damage(0,pierce),imodbits_none],

["long_axe_c","Great Long Axe",[("long_axe_c",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(80)|weapon_length(127)|swing_damage(135,cut)|thrust_damage(0,pierce),imodbits_none],

["long_axe_c_alt","Great Long Axe",[("long_axe_c",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(105)|weapon_length(127)|swing_damage(125,cut)|thrust_damage(0,pierce),imodbits_none],

["poleaxe","Poleaxe",[("pole_ax",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(65)|weapon_length(180)|swing_damage(160,cut)|thrust_damage(0,pierce),imodbits_none],

#Luigi's OSP Axes

["lui_battleaxetwohb","Battle Axe",[("lui_battleaxetwohb",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,7000,weight(4)|difficulty(8)|spd_rtng(115)|weapon_length(85)|swing_damage(155,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_battleaxetwoh","Battle Axe",[("lui_battleaxetwoh",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,6000,weight(4)|difficulty(8)|spd_rtng(120)|weapon_length(80)|swing_damage(150,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_knightbastardaxea","Battle Axe",[("lui_knightbastardaxea",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(115)|weapon_length(80)|swing_damage(135,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_knightaxetwohb","Battle Axe",[("lui_knightaxetwohb",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,5000,weight(4)|difficulty(8)|spd_rtng(115)|weapon_length(80)|swing_damage(140,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_knightbastardaxea","Battle Axe",[("lui_knightbastardaxea",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,6500,weight(4)|difficulty(8)|spd_rtng(125)|weapon_length(90)|swing_damage(155,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_knightbattleaxenew","Battle Axe",[("lui_knightbattleaxenew",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,4000,weight(4)|difficulty(8)|spd_rtng(120)|weapon_length(80)|swing_damage(130,cut)|thrust_damage(0,pierce),imodbits_none],

["lui_rangeraxe","Battle Axe",[("lui_rangeraxe",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_extra_penetration,itc_nodachi|itcf_carry_axe_back,6000,weight(4)|difficulty(8)|spd_rtng(115)|weapon_length(90)|swing_damage(150,cut)|thrust_damage(0,pierce),imodbits_none],

#Special spear placeholder
["spears_start","Spears Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#Spears (switch-hitters)
#Spears deserve some special mention, in that they now have a dual-attack mode, with an overhand strike and underhand stab w/shield.
#They are also less of a joke than they were in Native vs. armored troops, due to piercing changes, and have a special bonus vs. cavalry.
#They also no longer have the "penalty with shield" tag, which nerfed their damage considerably.
#Forks- good stab, lousy slash.  Pitchforks still suck.
["pitch_fork","Pitch Fork",[("pitch_fork",0)],itp_type_polearm|itp_offset_lance|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear,300,weight(3.5)|difficulty(0)|spd_rtng(110)|weapon_length(154)|swing_damage(35,blunt)|thrust_damage(40,pierce),imodbits_none],

["military_fork","Military Fork",[("military_fork",0)],itp_type_polearm|itp_offset_lance|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear,1500,weight(4.5)|difficulty(0)|spd_rtng(110)|weapon_length(135)|swing_damage(35,blunt)|thrust_damage(45,pierce),imodbits_none],

["battle_fork","Battle Fork",[("battle_fork",0)],itp_type_polearm|itp_offset_lance|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear,3500,weight(4.5)|difficulty(0)|spd_rtng(110)|weapon_length(142)|swing_damage(35,blunt)|thrust_damage(50,pierce),imodbits_none],

#European spear styles- Roman / Greek concept, stabbing / slashing polearm.
#Base stats:  damage 70c, 55p, speed 110, cost 1750
["boar_spear","Boar Spear",[("spear",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
1600,weight(4)|difficulty(0)|spd_rtng(100)|weapon_length(157)|swing_damage(85,cut)|thrust_damage(50,pierce),imodbits_none],

["shortened_spear","Shortened Spear",[("spear_g_1-9m",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
1500,weight(4)|difficulty(0)|spd_rtng(125)|weapon_length(120)|swing_damage(60,cut)|thrust_damage(55,pierce),imodbits_none],

["spear","Spear",[("spear_h_2-15m",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
1700,weight(4)|difficulty(0)|spd_rtng(110)|weapon_length(135)|swing_damage(70,cut)|thrust_damage(55,pierce),imodbits_none],

["bamboo_spear","Bamboo Spear",[("arabian_spear_a_3m",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
1500,weight(4)|difficulty(0)|spd_rtng(125)|weapon_length(160)|swing_damage(60,cut)|thrust_damage(55,pierce),imodbits_none],

["war_spear","War Spear",[("spear_i_2-3m",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
200,weight(4)|difficulty(0)|spd_rtng(120)|weapon_length(150)|swing_damage(70,cut)|thrust_damage(60,pierce),imodbits_none],

["awlpike","Awlpike",[("awl_pike_b",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
2200,weight(4)|difficulty(0)|spd_rtng(120)|weapon_length(165)|swing_damage(50,cut)|thrust_damage(65,pierce),imodbits_none],

["awlpike_long","Long Awlpike",[("awl_pike_a",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
2000,weight(4)|difficulty(0)|spd_rtng(100)|weapon_length(183)|swing_damage(50,cut)|thrust_damage(65,pierce),imodbits_none],

#James's nice spear
["jam_boarspear","Boar Spear",[("jam_boarspear",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
1700,weight(4)|difficulty(0)|spd_rtng(115)|weapon_length(165)|swing_damage(85,cut)|thrust_damage(50,pierce),imodbits_none],

#Luigi's Spears
["lui_vaegirspear","Spear",[("lui_vaegirspear",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
2300,weight(4)|difficulty(0)|spd_rtng(90)|weapon_length(195)|swing_damage(115,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_footmanlance","Spear",[("lui_footmanlance",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
2100,weight(4)|difficulty(0)|spd_rtng(85)|weapon_length(210)|swing_damage(50,cut)|thrust_damage(65,pierce),imodbits_none],

#RRR's Spears
["rrr_spear1","Spear",[("rrr_spear1",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
2100,weight(4)|difficulty(0)|spd_rtng(85)|weapon_length(215)|swing_damage(50,cut)|thrust_damage(65,pierce),imodbits_none],

["rrr_spear2","Spear",[("rrr_spear2",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
2000,weight(4)|difficulty(0)|spd_rtng(90)|weapon_length(215)|swing_damage(60,cut)|thrust_damage(60,pierce),imodbits_none],

["rrr_spear3","Spear",[("rrr_spear3",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
2400,weight(4)|difficulty(0)|spd_rtng(125)|weapon_length(200)|swing_damage(60,cut)|thrust_damage(65,pierce),imodbits_none],

["rrr_spear4","Spear",[("rrr_spear4",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear,
2700,weight(4)|difficulty(0)|spd_rtng(130)|weapon_length(200)|swing_damage(70,cut)|thrust_damage(60,pierce),imodbits_none],

["rrr_spear5","Spear",[("rrr_spear5",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
1700,weight(4)|difficulty(0)|spd_rtng(110)|weapon_length(220)|swing_damage(70,cut)|thrust_damage(55,pierce),imodbits_none],

["rrr_spear6","Spear",[("rrr_spear6",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,
2000,weight(4)|difficulty(0)|spd_rtng(130)|weapon_length(160)|swing_damage(95,cut)|thrust_damage(50,pierce),imodbits_none],


#Dejawolf's Norse spears.
["dejawolf_fjadraspjot","Fjadraspjot (norse spear)",[("dejawolf_fjadraspjot",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(125)|swing_damage(85,cut)|thrust_damage(45,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_hoggkesja","Hoggkesja (norse spear)",[("dejawolf_hoggkesja",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(120)|swing_damage(95,cut)|thrust_damage(40,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_kastspjottmidtaggir","Kastspjottmidtaggir (norse spear)",[("dejawolf_kastspjottmidtaggir",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(115)|swing_damage(75,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_langr_bryntvari","Langr Bryntvari (norse spear)",[("dejawolf_langr_bryntvari",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(120)|weapon_length(190)|swing_damage(95,cut)|thrust_damage(45,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_langr_hoggspjott1","Langr Hoggspjott (norse spear)",[("dejawolf_langr_hoggspjott1",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(120)|weapon_length(185)|swing_damage(100,cut)|thrust_damage(40,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_langr_svia","Langr Sviar (norse spear)",[("dejawolf_langr_svia",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(120)|weapon_length(195)|swing_damage(80,cut)|thrust_damage(60,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_spjotkesja","Spjotkesja (norse spear)",[("dejawolf_spjotkesja",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(120)|weapon_length(195)|swing_damage(80,cut)|thrust_damage(60,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_svia2","Sviar (norse spear)",[("dejawolf_svia2",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(140)|weapon_length(120)|swing_damage(80,cut)|thrust_damage(45,pierce),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_svia","Sviar (norse spear)",[("dejawolf_svia",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_spear|itcf_carry_spear,2000,weight(4)|difficulty(0)|spd_rtng(145)|weapon_length(105)|swing_damage(80,cut)|thrust_damage(45,pierce),imodbits_none,[],[fac_kingdom_of_nords]],
 
["spears_end","Spears End (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
 
#Pikes- no overhead attack, no shield, greater reach.  Ancient tradeoff. Nerfed vs. heavy armor, uses special rules vs. horses
#Base stats:  damage 50b, 40p, speed 90, cost 2000
["pikes_start","Pikes Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["pike","Pike",[("spear_a_3m",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,2000,weight(3.0)|difficulty(0)|spd_rtng(90)|weapon_length(250)|swing_damage(50,blunt)|thrust_damage(40,pierce),imodbits_none],

["ashwood_pike","Ashwood Pike",[("pike",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,1700,weight(3.0)|difficulty(0)|spd_rtng(85)|weapon_length(300)|swing_damage(50,blunt)|thrust_damage(45,pierce),imodbits_none],

#Luigi's Pikes
["lui_largespear","Pike",[("lui_largespear",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,2200,weight(3.0)|difficulty(0)|spd_rtng(80)|weapon_length(305)|swing_damage(100,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_manhunterspear","Manhunter's Pike",[("lui_manhunterspear",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,2200,weight(3.0)|difficulty(0)|spd_rtng(80)|weapon_length(305)|swing_damage(50,blunt)|thrust_damage(45,pierce),imodbits_none],

["lui_onehandedphalanxpike","Phalanxman's Pike",[("lui_onehandedphalanxpike",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,1800,weight(3.0)|difficulty(0)|spd_rtng(100)|weapon_length(295)|swing_damage(50,blunt)|thrust_damage(45,pierce),imodbits_none],

["lui_phalanxpike","Phalanxman's Pike",[("lui_phalanxpike",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,1700,weight(3.0)|difficulty(0)|spd_rtng(95)|weapon_length(330)|swing_damage(50,blunt)|thrust_damage(40,pierce),imodbits_none],

["lui_phalanxpikenew","Long Phalanxman's Pike",[("lui_phalanxpikenew",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,1700,weight(3.0)|difficulty(0)|spd_rtng(90)|weapon_length(370)|swing_damage(50,blunt)|thrust_damage(40,pierce),imodbits_none],

["lui_raiderlordspear","Raider's Pike",[("lui_raiderlordspear",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,2500,weight(3.0)|difficulty(0)|spd_rtng(70)|weapon_length(370)|swing_damage(40,blunt)|thrust_damage(45,pierce),imodbits_none],

["lui_vaegirpike","Pike",[("lui_vaegirpike",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,2000,weight(3.0)|difficulty(0)|spd_rtng(85)|weapon_length(310)|swing_damage(40,blunt)|thrust_damage(40,pierce),imodbits_none],

["lui_khergitpike","Khergit Pike",[("lui_khergitpike",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,2000,weight(3.0)|difficulty(0)|spd_rtng(85)|weapon_length(310)|swing_damage(40,blunt)|thrust_damage(40,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["lui_khergitpikestrong","Khergit Pike",[("lui_khergitpikestrong",0)],itp_type_polearm|itp_merchandise|itp_cant_use_on_horseback|itp_primary|itp_wooden_parry,itc_pike,2000,weight(3.0)|difficulty(0)|spd_rtng(110)|weapon_length(310)|swing_damage(40,blunt)|thrust_damage(45,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Mackie's long Mangler.  Really weird weapon; gets Nodachi attacks, big length, good Pierce everywhere, not slow, you can even use it on horseback.  Ouch.
["mackie_mangler","Mangler Pike",[("mackie_mangler",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry,itc_nodachi,8500,weight(3.0)|difficulty(10)|spd_rtng(115)|weapon_length(200)|swing_damage(50,pierce)|thrust_damage(50,pierce),imodbits_none],

["pikes_end","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#Chinese spear styles- slashing polearm, cutting thrusts, good swing damage.  
#Really should be with other slashing polearms, but it's a "spear", typologically, due to narrow blades so :-P
["hafted_blade_b","Hafted Blade",[("khergit_pike_b",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry|itp_extra_penetration,itc_guandao,1500,weight(2.75)|difficulty(0)|spd_rtng(135)|weapon_length(135)|swing_damage(105,cut)|thrust_damage(75,cut),imodbits_none,[],[fac_kingdom_of_khergits]],

["hafted_blade_a","Hafted Blade",[("khergit_pike_a",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry|itp_extra_penetration,itc_guandao,1600,weight(3.25)|difficulty(0)|spd_rtng(125)|weapon_length(153)|swing_damage(105,cut)|thrust_damage(75,cut),imodbits_none,[],[fac_kingdom_of_khergits]],

#Mackie's Bill
["mackie_bill","Bill",[("mackie_bill",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry|itp_extra_penetration,itc_guandao,2500,weight(3.25)|difficulty(0)|spd_rtng(115)|weapon_length(172)|swing_damage(115,cut)|thrust_damage(65,cut),imodbits_none,[],[fac_kingdom_of_swadians]],

#Luigi's Khergit conversions
["lui_khegitnagita","Hafted Blade",[("lui_khegitnagita",0)],itp_type_polearm|itp_merchandise|itp_primary|itp_wooden_parry|itp_extra_penetration,itc_guandao,2500,weight(3.25)|difficulty(0)|spd_rtng(115)|weapon_length(190)|swing_damage(125,cut)|thrust_damage(75,cut),imodbits_none,[],[fac_kingdom_of_khergits]],

# Goedendag, which deserves a category all of its own, a classic weapon
["club_with_spike_head","Spiked Staff",[("mace_e",0)],itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_merchandise|itp_primary|itp_wooden_parry,itc_bastardsword|itcf_carry_axe_back,1500,weight(2.5)|difficulty(9)|spd_rtng(130)|weapon_length(117)|swing_damage(80,blunt)|thrust_damage(55,pierce),imodbits_none],

#Lances
#They're spears that aren't balanced for use off of a horse.  They're a slow attack, because they can be couched.  They generally pierce.
#This is the non-lethal jousting lance.  No longer merchandise.
["jousting_lance","Jousting Lance",[("joust_of_peace",0)],itp_couchable|itp_type_polearm|itp_primary|itp_two_handed|itp_wooden_parry,itc_greatlance,158,weight(5)|difficulty(0)|spd_rtng(60)|weapon_length(240)|swing_damage(0,cut)|thrust_damage(17,blunt),imodbits_none],

#Lances that actually kill people.
#Double-headed lance, in its own weird sub-category.  Wish it could have appropriate animations, grr.
["double_sided_lance","Double Sided Lance",[("lance_dblhead",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_staff,2000,weight(5.5)|difficulty(0)|spd_rtng(60)|weapon_length(130)|swing_damage(85,cut)|thrust_damage(60,pierce),imodbits_none], 

#Regular lances.
["light_lance","Light Lance",[("spear_b_2-75m",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 2200,weight(2.5)|difficulty(0)|spd_rtng(75)|weapon_length(175)|swing_damage(16,blunt)|thrust_damage(70,pierce),imodbits_none],

["lance","Lance",[("spear_d_2-8m",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 2100,weight(2.5)|difficulty(0)|spd_rtng(70)|weapon_length(180)|swing_damage(16,blunt)|thrust_damage(65,pierce),imodbits_none],

["lui_mercenarylance","Mercenary's Lance",[("lui_mercenarylance",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 3000,weight(2.5)|difficulty(0)|spd_rtng(65)|weapon_length(255)|swing_damage(16,blunt)|thrust_damage(75,pierce),imodbits_none],
 
["heavy_lance","Heavy Lance",[("spear_f_2-9m",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 2700,weight(2.5)|difficulty(0)|spd_rtng(55)|weapon_length(190)|swing_damage(16,blunt)|thrust_damage(75,pierce),imodbits_none],

["great_lance","Great Lance",[("heavy_lance",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 3000,weight(2.5)|difficulty(0)|spd_rtng(50)|weapon_length(240)|swing_damage(16,blunt)|thrust_damage(80,pierce),imodbits_none],

#Luigi's lances
["lui_joustingpike_blue","Long Lance",[("lui_joustingpike_blue",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 3200,weight(2.5)|difficulty(0)|spd_rtng(60)|weapon_length(340)|swing_damage(16,blunt)|thrust_damage(80,pierce),imodbits_none],

["lui_joustingpike_green","Long Lance",[("lui_joustingpike_green",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 3200,weight(2.5)|difficulty(0)|spd_rtng(60)|weapon_length(340)|swing_damage(16,blunt)|thrust_damage(80,pierce),imodbits_none],

["lui_joustingpike_red","Long Lance",[("lui_joustingpike_red",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 3200,weight(2.5)|difficulty(0)|spd_rtng(60)|weapon_length(340)|swing_damage(16,blunt)|thrust_damage(80,pierce),imodbits_none],  

["lui_joustingpike_yellow","Long Lance",[("lui_joustingpike_yellow",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 3200,weight(2.5)|difficulty(0)|spd_rtng(60)|weapon_length(340)|swing_damage(16,blunt)|thrust_damage(80,pierce),imodbits_none],

#POP's lances
["pop_gothic_lance_blue","Great Lance",[("pop_gothic_lance_blue",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 2700,weight(2.5)|difficulty(0)|spd_rtng(60)|weapon_length(270)|swing_damage(16,blunt)|thrust_damage(75,pierce),imodbits_none],

["pop_gothic_lance_green","Great Lance",[("pop_gothic_lance_green",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 2700,weight(2.5)|difficulty(0)|spd_rtng(60)|weapon_length(270)|swing_damage(16,blunt)|thrust_damage(75,pierce),imodbits_none],

["pop_gothic_lance_red","Great Lance",[("pop_gothic_lance_red",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 2700,weight(2.5)|difficulty(0)|spd_rtng(60)|weapon_length(270)|swing_damage(16,blunt)|thrust_damage(75,pierce),imodbits_none],

["pop_gothic_lance_yellow","Great Lance",[("pop_gothic_lance_yellow",0)],itp_couchable|itp_type_polearm|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_greatlance, 2700,weight(2.5)|difficulty(0)|spd_rtng(60)|weapon_length(270)|swing_damage(16,blunt)|thrust_damage(75,pierce),imodbits_none],

#Halberds and the bazillion variants(either):  a spear combined with an axe.  One of the most ubiquitous weapons of infantry during this period,an extremely deadly weapon when used without a shield,and a very effective weapon when using one.  Generally a one-hit kill on an overhand strike,slightly less poke than spears.  
#They no longer get side-swings,if they did before, as IRL that would be very dangerous in close quarters, and ordinary soldiers weren't generally taught to do that.

#Halberds/Scorpions/Poleaxes/Polearms/Polehammers.  The classic medieval slashing / thrusting weapons.  Deadly, by all accounts.  
#Thrust is piercing.  Slower than Glaives, etc. due to heavier head.
#Strangely enough, this class is not really represented in Native, which is very odd, since they were being mentioned in European poetry by 1287...
#... which personally I take as a sign that the weapon had been around for awhile.
#And of course, since I've moved the timeline up a century, I don't have any qualms, so no complaints please ;-)

#These weapons get extra penetration, like axes do.

#Base stats:  damage 110c, 60p, speed 70, cost 3500
#James's Poleaxe / Scorpion
["jam_poleaxe","Poleaxe",[("jam_poleaxe",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(185)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["jam_scorpion","Scorpion",[("jam_scorpion",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(170)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

#Lui's Halberds
["lui_elitehallberd","Elite Halberd",[("lui_elitehallberd",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4500,weight(4.5)|difficulty(10)|spd_rtng(60)|weapon_length(230)|swing_damage(130,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_halberda","Footman's Halberd",[("lui_halberda",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4000,weight(4.5)|difficulty(10)|spd_rtng(75)|weapon_length(205)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_halberdb","Sergeant's Halberd",[("lui_halberdb",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(240)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_halberdc","Knight's Halberd",[("lui_halberdc",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,5500,weight(4.5)|difficulty(10)|spd_rtng(90)|weapon_length(230)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_halberdd","Spear-Axe",[("lui_halberdd",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,5500,weight(4.5)|difficulty(10)|spd_rtng(55)|weapon_length(275)|swing_damage(120,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_halberde","Strong Halberd",[("lui_halberde",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(200)|swing_damage(125,cut)|thrust_damage(50,pierce),imodbits_none],

["lui_hallberd_blue","Liveried Halberd",[("lui_hallberd_blue",0)],itp_type_polearm|itp_crush_through|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4500,weight(4.5)|difficulty(10)|spd_rtng(60)|weapon_length(275)|swing_damage(115,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_hallberd_green","Liveried Halberd",[("lui_hallberd_green",0)],itp_type_polearm|itp_crush_through|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4500,weight(4.5)|difficulty(10)|spd_rtng(60)|weapon_length(275)|swing_damage(115,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_hallberd_red","Liveried Halberd",[("lui_hallberd_red",0)],itp_type_polearm|itp_crush_through|itp_merchandise|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4500,weight(4.5)|difficulty(10)|spd_rtng(60)|weapon_length(275)|swing_damage(115,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_hallberd_yellow","Liveried Halberd",[("lui_hallberd_yellow",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4500,weight(4.5)|difficulty(10)|spd_rtng(60)|weapon_length(275)|swing_damage(115,cut)|thrust_damage(70,pierce),imodbits_none],

["lui_kjergithallberd","Khergit Halberd",[("lui_kjergithallberd",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3100,weight(4.5)|difficulty(10)|spd_rtng(90)|weapon_length(160)|swing_damage(120,cut)|thrust_damage(50,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

["lui_kjergithallberdstrong","Khergit Halberd",[("lui_kjergithallberdstrong",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(230)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none,[],[fac_kingdom_of_khergits]],

#Polehammers really deserve their own slot, but are stuck here for now.
["lui_knightpolehammer","Knight's Polehammer",[("lui_knightpolehammer",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,5800,weight(4.5)|difficulty(10)|spd_rtng(55)|weapon_length(275)|swing_damage(90,pierce)|thrust_damage(65,pierce),imodbits_none],

["lui_knightpolehammerb","Knight's Polehammer",[("lui_knightpolehammerb",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,6500,weight(4.5)|difficulty(10)|spd_rtng(75)|weapon_length(220)|swing_damage(80,pierce)|thrust_damage(60,pierce),imodbits_none],

["lui_knightpolehammerc","Knight's Elite Polehammer",[("lui_knightpolehammerc",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,7500,weight(4.5)|difficulty(10)|spd_rtng(75)|weapon_length(238)|swing_damage(80,pierce)|thrust_damage(60,pierce),imodbits_none],

["lui_manhunterpolehammer","Manhunter's Polehammer",[("lui_manhunterpolehammer",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(65)|weapon_length(220)|swing_damage(80,pierce)|thrust_damage(60,pierce),imodbits_none],

["lui_vaegirhallberd","Broad Halberd",[("lui_vaegirhallberd",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(215)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_vaegirhallberdb","Axe Halberd",[("lui_vaegirhallberdb",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(65)|weapon_length(240)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_vaegirhallberdc","Vaegir Halberd",[("lui_vaegirhallberdc",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4500,weight(4.5)|difficulty(10)|spd_rtng(85)|weapon_length(220)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

#POP's halberds
["pop_halberd","Heavy Halberd",[("pop_halberd",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,5500,weight(4.5)|difficulty(10)|spd_rtng(60)|weapon_length(190)|swing_damage(135,cut)|thrust_damage(50,pierce),imodbits_none],

["pop_ornate_halberd","Ornate Halberd",[("pop_ornate_halberd",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(190)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

#RRR's halberds
#RRR's bill- I've given it knockdown, better speed, less damage, to make it special.
["rrr_bill1","Footman's Bill",[("rrr_bill1",0)],itp_type_polearm|itp_merchandise|itp_two_handed|itp_primary|itp_crush_through|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_can_knock_down|itp_extra_penetration,itc_poleaxe ,4000,weight(4.5)|difficulty(10)|spd_rtng(85)|weapon_length(235)|swing_damage(90,cut)|thrust_damage(50,pierce),imodbits_none],

["rrr_halberd1","Guard's Halberd",[("rrr_halberd1",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4000,weight(4.5)|difficulty(10)|spd_rtng(100)|weapon_length(190)|swing_damage(100,cut)|thrust_damage(60,pierce),imodbits_none],

["rrr_halberd2","Spear Halberd",[("rrr_halberd2",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(95)|weapon_length(190)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["rrr_halberd3","Scalloped Halberd",[("rrr_halberd3",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(210)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["rrr_halberd4","Chopping Halberd",[("rrr_halberd4",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3800,weight(4.5)|difficulty(10)|spd_rtng(60)|weapon_length(240)|swing_damage(130,cut)|thrust_damage(50,pierce),imodbits_none],

#Technically, this is long, skinny voulge, to my eye, but whatever.
["rrr_halberd5","Narrow Voulge",[("rrr_halberd5",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4200,weight(4.5)|difficulty(10)|spd_rtng(80)|weapon_length(210)|swing_damage(110,cut)|thrust_damage(80,cut),imodbits_none],

["rrr_halberd6","Axe Halberd",[("rrr_halberd6",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(190)|swing_damage(130,cut)|thrust_damage(40,pierce),imodbits_none],

["rrr_halberd7","Spiked Halberd",[("rrr_halberd7",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3500,weight(4.5)|difficulty(10)|spd_rtng(85)|weapon_length(170)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["rrr_polearm1","Polearm",[("rrr_polearm1",0)],itp_type_polearm|itp_merchandise|itp_two_handed|itp_crush_through|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,3800,weight(4.5)|difficulty(10)|spd_rtng(85)|weapon_length(170)|swing_damage(70,pierce)|thrust_damage(70,pierce),imodbits_none],

#Mackie's long voulge
["mackie_voulge","Long Voulge",[("mackie_voulge",0)],itp_type_polearm|itp_merchandise|itp_two_handed|itp_crush_through|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe ,4500,weight(4.5)|difficulty(10)|spd_rtng(85)|weapon_length(189)|swing_damage(95,pierce)|thrust_damage(45,pierce),imodbits_none],

#Short Halberds/Polearms- better speed, can be used on horseback.
["lui_smallelitehallberd","Short Halberd",[("lui_smallelitehallberd",0)],itp_merchandise|itp_crush_through|itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_bastardsword|itcf_carry_axe_back,3500,weight(4.5)|difficulty(10)|spd_rtng(100)|weapon_length(120)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_smallhallberda","Short Spiked Halberd",[("lui_smallhallberda",0)],itp_merchandise|itp_crush_through|itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_bastardsword|itcf_carry_axe_back,3500,weight(4.5)|difficulty(10)|spd_rtng(100)|weapon_length(120)|swing_damage(110,cut)|thrust_damage(60,pierce),imodbits_none],

["lui_smallhallberdb","Elite Short Halberd",[("lui_smallhallberdb",0)],itp_merchandise|itp_crush_through|itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_bastardsword|itcf_carry_axe_back,5500,weight(4.5)|difficulty(10)|spd_rtng(120)|weapon_length(120)|swing_damage(120,cut)|thrust_damage(60,pierce),imodbits_none],

#These are short polehammers- neither fish nor fowl.  They have nasty piercing attacks.
["rrr_polehammer1","Knight's Polehammer",[("rrr_polehammer1",0)],itp_merchandise|itp_crush_through|itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_bastardsword|itcf_carry_axe_back,6000,weight(4.5)|difficulty(10)|spd_rtng(120)|weapon_length(120)|swing_damage(60,pierce)|thrust_damage(60,pierce),imodbits_none],

["rrr_polehammer2","Short Polehammer",[("rrr_polehammer2",0)],itp_merchandise|itp_crush_through|itp_type_one_handed_wpn|itp_type_two_handed_wpn|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_bastardsword|itcf_carry_axe_back,6500,weight(4.5)|difficulty(10)|spd_rtng(130)|weapon_length(90)|swing_damage(65,pierce)|thrust_damage(60,pierce),imodbits_none],

["rrr_polehammer3","Inquisitor's Polehammer",[("rrr_polehammer3",0)],itp_merchandise|itp_type_one_handed_wpn|itp_crush_through|itp_type_two_handed_wpn|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_bastardsword|itcf_carry_axe_back,7500,weight(4.5)|difficulty(10)|spd_rtng(115)|weapon_length(160)|swing_damage(70,pierce)|thrust_damage(60,pierce),imodbits_none],

#Bardiches, Voulges, Glaives, Fauchards, etc., etc.- cutting polearms and giant unwieldy axes. Thrust is weak cut.  
#Damage / speed is all over the place- this is a mixed bag indeed.  Big ones are knock-down and very nasty, small ones are basically just two-handed axes, nothing special.
#All get extra penetration, just like axes.
#Base stats:  damage 100c, speed 90, cost 3500

["bardiche","Bardiche",[("two_handed_battle_axe_d",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_nodachi,3500,weight(4.5)|difficulty(10)|spd_rtng(110)|weapon_length(102)|swing_damage(100,cut)|thrust_damage(65,cut),imodbits_none],

["great_bardiche","Great Bardiche",[("two_handed_battle_axe_f",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_nodachi,3900,weight(4.5)|difficulty(10)|spd_rtng(80)|weapon_length(116)|swing_damage(120,cut)|thrust_damage(65,cut),imodbits_none],

["voulge","Voulge",[("two_handed_battle_long_axe_a",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_can_knock_down|itp_extra_penetration,itc_nodachi,4500,weight(4.5)|difficulty(10)|spd_rtng(60)|weapon_length(175)|swing_damage(135,cut)|thrust_damage(65,cut),imodbits_none],

["long_bardiche","Long Bardiche",[("two_handed_battle_long_axe_b",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3800,weight(4.5)|difficulty(10)|spd_rtng(90)|weapon_length(140)|swing_damage(120,cut)|thrust_damage(65,cut),imodbits_none],

["great_long_bardiche","Great Long Bardiche",[("two_handed_battle_long_axe_c",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3700,weight(4.5)|difficulty(10)|spd_rtng(80)|weapon_length(155)|swing_damage(115,cut)|thrust_damage(65,cut),imodbits_none],

["scythe","Scythe",[("scythe",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_nodachi,500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(182)|swing_damage(90,cut)|thrust_damage(65,cut),imodbits_none],

["glaive","Glaive",[("glaive_b",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(90)|weapon_length(157)|swing_damage(100,cut)|thrust_damage(65,cut),imodbits_none],
 
["military_scythe","Military Scythe",[("spear_e_2-5m",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_nodachi,3500,weight(4.5)|difficulty(10)|spd_rtng(90)|weapon_length(155)|swing_damage(100,cut)|thrust_damage(65,cut),imodbits_none],

#James's Fauchard and Voulge
["jam_fauchard","Crescent Fauchard",[("jam_fauchard",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3100,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(180)|swing_damage(120,cut)|thrust_damage(50,cut),imodbits_none],

["jam_voulge_b","Heavy Voulge",[("jam_voulge_b",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_can_knock_down|itp_extra_penetration,itc_poleaxe,5500,weight(4.5)|difficulty(10)|spd_rtng(90)|weapon_length(115)|swing_damage(140,cut)|thrust_damage(65,cut),imodbits_none],

#Luigi's Glaives.
["lui_glaiveb","Glaive",[("lui_glaiveb",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(275)|swing_damage(100,cut)|thrust_damage(65,cut),imodbits_none],

["lui_glaivec","Glaive",[("lui_glaivec",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(70)|weapon_length(275)|swing_damage(100,cut)|thrust_damage(65,cut),imodbits_none],

["lui_glaivenew","Glaive",[("lui_glaivenew",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(75)|weapon_length(255)|swing_damage(100,cut)|thrust_damage(65,cut),imodbits_none],

#RRR's Glaives.
["rrr_glaive1","Glaive",[("rrr_glaive1",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(100)|weapon_length(205)|swing_damage(100,cut)|thrust_damage(65,cut),imodbits_none],

["rrr_glaive2","Glaive",[("rrr_glaive2",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_extra_penetration,itc_poleaxe,3500,weight(4.5)|difficulty(10)|spd_rtng(80)|weapon_length(240)|swing_damage(110,cut)|thrust_damage(65,cut),imodbits_none],

["rrr_glaive3","Hooked Glaive",[("rrr_glaive3",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_cant_use_on_horseback|itp_can_knock_down|itp_extra_penetration,itc_poleaxe,4500,weight(4.5)|difficulty(10)|spd_rtng(80)|weapon_length(250)|swing_damage(110,cut)|thrust_damage(65,cut),imodbits_none],

#Mackie's Tepoztopilli
["mackie_tepoztopilli","Tepoztopilli",[("mackie_tepoztopilli",0)],itp_type_polearm|itp_merchandise|itp_crush_through|itp_two_handed|itp_primary|itp_bonus_against_shield|itp_wooden_parry|itp_unbalanced|itp_extra_penetration,itc_poleaxe,4500,weight(4.5)|difficulty(10)|spd_rtng(110)|weapon_length(155)|swing_damage(110,cut)|thrust_damage(65,cut),imodbits_none],



#Staves:  these weren't seen a lot on battlefields,largely because a spear is more effective, but they do blunt damage and they're generally cheap and moderately effective.
["staff","Staff",[("wooden_staff",0)],itp_type_polearm|itp_offset_lance|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry|itp_wooden_attack|itp_can_knock_down,itc_staff|itcf_carry_sword_back,200,weight(1.5)|difficulty(0)|spd_rtng(120)|weapon_length(130)|swing_damage(40,blunt)|thrust_damage(70,blunt),imodbits_none],

["quarter_staff","Quarter Staff",[("quarter_staff",0)],itp_type_polearm|itp_offset_lance|itp_merchandise|itp_primary|itp_penalty_with_shield|itp_wooden_parry|itp_wooden_attack|itp_can_knock_down,itc_staff|itcf_carry_sword_back,1000,weight(1.5)|difficulty(0)|spd_rtng(130)|weapon_length(140)|swing_damage(100,blunt)|thrust_damage(30,blunt),imodbits_none],

["iron_staff","Iron Staff",[("iron_staff",0)],itp_type_polearm|itp_offset_lance|itp_merchandise|itp_crush_through|itp_primary|itp_penalty_with_shield|itp_wooden_attack|itp_can_knock_down,itc_staff|itcf_carry_sword_back,2000,weight(1.5)|difficulty(0)|spd_rtng(110)|weapon_length(130)|swing_damage(65,blunt)|thrust_damage(140,blunt),imodbits_none],

#Two-handed Blunt Objects.  Can you say "blunt force trauma"?  Actually most of them do Pierce (ignore armor) and all of 'em are crush-through.  
#These are nasty, nasty weapons, in theory, but very sloooooow.
#And thrusting attacks do Blunt.  How about that?  They're 'versatile', which is military-speak for "does nothing well".

#Two-Handed Warhammers / Maces.  Kind've like nastier two-handed swords.  Rather expensive, slower than swords, but very dangerous.
#Base stats:  damage 85p 40b, speed 70, cost 5000
["long_spiked_club","Long Spiked Club",[("mace_long_c",0)],itp_type_two_handed_wpn|itp_two_handed|itp_crush_through|itp_merchandise|itp_primary|itp_wooden_parry|itp_can_knock_down,itc_nodachi|itcf_carry_axe_back,5000,weight(2.5)|difficulty(0)|spd_rtng(70)|weapon_length(126)|swing_damage(85,pierce)|thrust_damage(40,blunt),imodbits_none],

["long_hafted_knobbed_mace","Long Hafted Knobbed Mace",[("mace_long_a",0)],itp_type_two_handed_wpn|itp_two_handed|itp_crush_through|itp_merchandise|itp_primary|itp_wooden_parry|itp_can_knock_down,itc_nodachi|itcf_carry_axe_back,5000,weight(2.5)|difficulty(0)|spd_rtng(75)|weapon_length(133)|swing_damage(85,pierce)|thrust_damage(40,blunt),imodbits_none],

["long_hafted_spiked_mace","Long Hafted Spiked Mace",[("mace_long_b",0)],itp_type_two_handed_wpn|itp_two_handed|itp_crush_through|itp_merchandise|itp_primary|itp_wooden_parry|itp_can_knock_down,itc_nodachi|itcf_carry_axe_back,6000,weight(2.5)|difficulty(0)|spd_rtng(95)|weapon_length(140)|swing_damage(75,pierce)|thrust_damage(40,blunt),imodbits_none],

["mackie_morning_star_long","Long Morning Star",[("mackie_morning_star_long",0)],itp_type_two_handed_wpn|itp_two_handed|itp_crush_through|itp_merchandise|itp_primary|itp_wooden_parry|itp_can_knock_down,itc_nodachi|itcf_carry_axe_back,7000,weight(2.5)|difficulty(0)|spd_rtng(110)|weapon_length(136)|swing_damage(70,pierce)|thrust_damage(40,blunt),imodbits_none],

["sarranid_two_handed_mace_1","Long Iron Mace",[("mace_long_d",0)],itp_type_two_handed_wpn|itp_two_handed|itp_crush_through|itp_merchandise|itp_primary|itp_can_knock_down,itc_nodachi|itcf_carry_axe_back,6500,weight(2.5)|difficulty(0)|spd_rtng(80)|weapon_length(95)|swing_damage(85,pierce)|thrust_damage(40,blunt),imodbits_none],

#Luigi's set
["lui_twohandedmace","Long Flanged Mace",[("lui_twohandedmace",0)],itp_type_two_handed_wpn|itp_two_handed|itp_crush_through|itp_merchandise|itp_primary|itp_can_knock_down,itc_nodachi|itcf_carry_axe_back,6000,weight(2.5)|difficulty(0)|spd_rtng(90)|weapon_length(105)|swing_damage(75,pierce)|thrust_damage(40,blunt),imodbits_none],

#My long war pick
["xeno_war_pick02","Long War Pick",[("xeno_war_pick02",0)],itp_type_two_handed_wpn|itp_two_handed|itp_crush_through|itp_merchandise|itp_primary|itp_can_knock_down|itp_extra_penetration,itc_nodachi,11000,weight(2.5)|difficulty(10)|spd_rtng(100)|weapon_length(153)|swing_damage(75,pierce)|thrust_damage(40,blunt),imodbits_none],

#Giant Hammers.  Pretty much insta-death to anything or anybody in the game on an overhead swing.  No longer allowed side-swings (too dangerous to fellow troops)
#They have serious speed penalties, but you can't complain about the damage.
["warhammer","Great Hammer",[("maul_d",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_primary|itp_two_handed|itp_wooden_parry|itp_wooden_attack|itp_unbalanced|itp_extra_penetration,itc_poleaxe|itcf_carry_spear,5500,weight(9)|difficulty(14)|spd_rtng(65)|weapon_length(70)|swing_damage(95,pierce)|thrust_damage(65,blunt),imodbits_none],

["maul","Maul",[("maul_b",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_primary|itp_two_handed|itp_wooden_parry|itp_wooden_attack|itp_unbalanced|itp_can_knock_down|itp_extra_penetration,itc_poleaxe|itcf_carry_spear,5000,weight(9)|difficulty(14)|spd_rtng(65)|weapon_length(70)|swing_damage(90,pierce)|thrust_damage(65,blunt),imodbits_none],

["sledgehammer","Sledgehammer",[("maul_c",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_primary|itp_two_handed|itp_wooden_parry|itp_wooden_attack|itp_unbalanced|itp_can_knock_down|itp_extra_penetration,itc_poleaxe|itcf_carry_spear,4000,weight(9)|difficulty(14)|spd_rtng(45)|weapon_length(70)|swing_damage(100,pierce)|thrust_damage(65,blunt),imodbits_none],

["rrr_mallet1","Giant Maul",[("rrr_mallet1",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_primary|itp_two_handed|itp_wooden_parry|itp_wooden_attack|itp_unbalanced|itp_can_knock_down|itp_extra_penetration,itc_poleaxe|itcf_carry_spear,5000,weight(9)|difficulty(14)|spd_rtng(50)|weapon_length(105)|swing_damage(115,pierce)|thrust_damage(65,blunt),imodbits_none],

["polehammer","Giant Hammer",[("pole_hammer",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_primary|itp_two_handed|itp_wooden_parry|itp_wooden_attack|itp_unbalanced|itp_can_knock_down|itp_extra_penetration,itc_poleaxe|itcf_carry_spear,5000,weight(9)|difficulty(14)|spd_rtng(40)|weapon_length(130)|swing_damage(130,pierce)|thrust_damage(65,blunt),imodbits_none],

#Weird Stuff:  There are a few weird things that just don't fit anywhere else,or were atypical of the period, but might have shown up.  For example, I put a punch-dagger in,for assassin types.  A catch-all for stuff that just didn't fit elsewhere.

#Sickles- kind've like swords to plowshares in reverse.  An awkward weapon, to say the least, but no longer *totally* harmless.
["sickle","Sickle",[("sickle",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_no_parry|itp_wooden_parry,itc_cleaver,300,weight(1.5)|difficulty(0)|spd_rtng(110)|weapon_length(40)|swing_damage(45,cut)|thrust_damage(0,pierce),imodbits_none],

#Military Sickle- more like a spiked hammer.  Decent weapon.
["military_sickle_a","Military Sickle",[("military_sickle_a",0)],itp_type_one_handed_wpn|itp_merchandise|itp_primary|itp_secondary|itp_wooden_parry,itc_scimitar|itcf_carry_axe_left_hip,1500,weight(1.0)|difficulty(9)|spd_rtng(120)|weapon_length(80)|swing_damage(45,pierce)|thrust_damage(0,pierce),imodbits_none],

#Shortened Miltary scythes- a bunch of bad ideas.  Not heavy enough to be a cleaver, awkwardly balanced for a sword.  Weird weapon, pretty useless, but it looks kewl.
["shortened_military_scythe","Shortened Military Scythe",[("two_handed_battle_scythe_a",0)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary,itc_guandao|itcf_carry_sword_back,4500,weight(3.0)|difficulty(10)|spd_rtng(90)|weapon_length(112)|swing_damage(135,cut)|thrust_damage(0,pierce),imodbits_none],

#Mackie's Pendulum; fantasy giant axe-spear-thing.
["mackie_pendulum","Pendulum",[("mackie_pendulum",0),("mackie_pendulum_carry",ixmesh_carry)],itp_crush_through|itp_type_two_handed_wpn|itp_merchandise|itp_two_handed|itp_primary|itp_extra_penetration,itc_greatsword|itcf_carry_axe_back,7500,weight(6.0)|difficulty(15)|spd_rtng(90)|weapon_length(128)|swing_damage(150,pierce)|thrust_damage(110,cut),imodbits_none,[],[fac_kingdom_of_khergits]],




 


 







["missile_weapons_start","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

######RANGED
#Arrows and Bolts
 ["arrows","Clothyard Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_merchandise|itp_no_pick_up_from_ground|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(160)|weapon_length(95)|thrust_damage(15,pierce)|max_ammo(40),imodbits_none],
 
  ["eastern_arrows","Eastern Arrows",[("amade_bronze_arrow",0),("flying_missile",ixmesh_flying_ammo),("amade_quiver_bronze",ixmesh_carry)],itp_type_arrows|itp_no_pick_up_from_ground|itp_merchandise|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(160)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(75),imodbits_none],
 
#Bolts
 ["bolts","Bolts/Bullets",[("xeno_bullet",0),("flying_missile",ixmesh_flying_ammo),("bolt_bag_c",ixmesh_carry),("xeno_bolts_and_bullets",ixmesh_inventory)],itp_type_bolts|itp_merchandise|itp_can_penetrate_shield|itp_no_pick_up_from_ground|itp_default_ammo,itcf_carry_quiver_right_vertical,2000,weight(2.25)|abundance(90)|weapon_length(150)|thrust_damage(1,pierce)|max_ammo(30),imodbits_none,
 [
 (ti_on_missile_hit,
      [
         #pos1 - Missile hit position
         #param_1 - Shooter agent
         #pos1 - Missile hit position
         #param_1 - Shooter agent
		(copy_position, pos63, pos1),     
		(position_get_z, ":z_pos", pos63),
		(set_fixed_point_multiplier, 100),
		(try_begin),
			(gt, ":z_pos", -50),
			(particle_system_burst,"psys_bullet_dust",pos63,1),
		(else_try),
			(gt, ":z_pos", -300),
			(position_set_z, pos63, -50),
			(particle_system_burst, "psys_water_hit_a", pos63, 8),
			(particle_system_burst, "psys_water_hit_b", pos63, 4),
		(try_end),
]),
]],

#Grenade Fragments
 ["grenade_fragments","Grenade Fragments",[("xeno_bullet",0),("xenoargh_grenade_fragment",ixmesh_flying_ammo),("bolt_bag_c",ixmesh_carry),("xeno_bolts_and_bullets",ixmesh_inventory)],itp_type_bolts|itp_can_penetrate_shield|itp_no_pick_up_from_ground|itp_bonus_against_shield,itcf_carry_quiver_right_vertical,2000,weight(2.25)|abundance(90)|weapon_length(150)|thrust_damage(1,pierce)|max_ammo(30),imodbits_none,
 [
 (ti_on_missile_hit,
      [
         #pos1 - Missile hit position
         #param_1 - Shooter agent
         #pos1 - Missile hit position
         #param_1 - Shooter agent
		(copy_position, pos63, pos1),     
		(position_get_z, ":z_pos", pos63),
		(set_fixed_point_multiplier, 100),
		(try_begin),
			(gt, ":z_pos", -50),
			(particle_system_burst,"psys_bullet_dust",pos63,1),
		(else_try),
			(gt, ":z_pos", -300),
			(position_set_z, pos63, -50),
			(particle_system_burst, "psys_water_hit_a", pos63, 8),
			(particle_system_burst, "psys_water_hit_b", pos63, 4),
		(try_end),
]),
]],

#Shotgun pellets
 ["shotgun_pellets","Shotgun Pellets",[("xeno_bullet",0),("tw_flying_missile_small",ixmesh_flying_ammo),("bolt_bag_c",ixmesh_carry),("xeno_bolts_and_bullets",ixmesh_inventory)],itp_type_bolts|itp_can_penetrate_shield|itp_no_pick_up_from_ground|itp_bonus_against_shield,itcf_carry_quiver_right_vertical,2000,weight(2.25)|abundance(90)|weapon_length(150)|thrust_damage(1,pierce)|max_ammo(30),imodbits_none,
 [
 (ti_on_missile_hit,
      [
         #pos1 - Missile hit position
         #param_1 - Shooter agent
         #pos1 - Missile hit position
         #param_1 - Shooter agent
		(copy_position, pos63, pos1),     
		(position_get_z, ":z_pos", pos63),
		(set_fixed_point_multiplier, 100),
		(try_begin),
			(gt, ":z_pos", -50),
			(particle_system_burst,"psys_bullet_dust",pos63,1),
		(else_try),
			(gt, ":z_pos", -300),
			(position_set_z, pos63, -50),
			(particle_system_burst, "psys_water_hit_a", pos63, 8),
			(particle_system_burst, "psys_water_hit_b", pos63, 4),
		(try_end),
]),
]],

["cannon_ammo_start","Cannon Ammunition Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

 #Ammo for Cannons.
##UNIVERSAL AMMO FOR CANNONS
["universal_cannon_ammo","Cannon Ammunition",[("xeno_bullet",0),("xeno_cannonball_solid",ixmesh_inventory)],itp_type_bolts|itp_merchandise|itp_can_penetrate_shield|itp_no_pick_up_from_ground,0,3500,weight(3)|abundance(200)|weapon_length(150)|thrust_damage(500,pierce)|max_ammo(1000),imodbits_none],
 
["solid_cannon_rounds","Solid Cannon Round",[("xeno_bullet",0),("xeno_cannonball_solid",ixmesh_flying_ammo),("bolt_bag_c",ixmesh_carry),("xeno_cannonball_solid",ixmesh_inventory)],itp_type_bolts|itp_can_penetrate_shield|itp_no_pick_up_from_ground|itp_default_ammo,itcf_carry_quiver_right_vertical,200,weight(3)|abundance(0)|weapon_length(150)|thrust_damage(150,pierce)|max_ammo(30),imodbits_none,
[
(ti_on_missile_hit,
      [

	  	(copy_position, pos63, pos1),     
		(position_get_z, ":z_pos", pos63),
		(set_fixed_point_multiplier, 100),
		(try_begin),
			(gt, ":z_pos", -50),
			(particle_system_burst,"psys_bullet_dust",pos63,2),
		(else_try),
			(gt, ":z_pos", -300),
			(position_set_z, pos63, -50),
			(particle_system_burst, "psys_water_hit_a", pos63, 16),
			(particle_system_burst, "psys_water_hit_b", pos63, 8),
		(try_end),
	  
	  # (try_begin),
		# (init_position,pos63),
		##Solid Round Script
        ##pos1 - Missile hit position
       ## param_1 - Shooter agent
		# (this_or_next|multiplayer_is_dedicated_server),		
		# (this_or_next|multiplayer_is_server),
		# (neg|game_in_multiplayer_mode),		
		# (store_trigger_param_1,":shooter"),
		# (agent_get_team, ":shooter_team", ":shooter"),
		# (init_position,pos63),
		# (copy_position, pos63, pos1),
		# (particle_system_burst,"psys_bullet_dust",pos1,1),
		# (position_set_y, pos63, 0),
		# (try_for_agents,":agent"),	
			# (agent_get_team, ":agent_team", ":agent"),
			# (teams_are_enemies, ":agent_team", ":shooter_team"),
			# (agent_is_active,":agent"),
			# (agent_is_alive,":agent"),	
			# (init_position,pos62),
			# (agent_get_position,pos62,":agent"),
			# (position_set_y, pos62, 0),
			# (get_distance_between_positions,":dist",pos63,pos62),
			# (try_begin),
				# (lt,":dist",200),#2-meter radius,otherwise doesn't register enough			   
				# (agent_set_hit_points,":agent",0,1),#insta-death
				# (agent_deliver_damage_to_agent,":shooter",":agent",400),#really, you're dead now, lol.
			# (try_end),
		# (try_end),
		# (try_end),
]),
]],


["explosive_cannon_rounds","Explosive Cannon Round",[("xeno_bullet",0),("xenoargh_explosive_cannon_flying",ixmesh_flying_ammo),("xeno_cannonball_explosive",ixmesh_inventory)],itp_type_bolts|itp_can_penetrate_shield|itp_no_pick_up_from_ground,itcf_carry_quiver_right_vertical,3000,weight(3)|abundance(0)|weapon_length(150)|thrust_damage(100,pierce)|max_ammo(200),imodbits_none,
   [   
 (ti_on_missile_hit,
      [

  	(store_trigger_param_1,reg1),
	(copy_position, pos62, pos1),#Need to store it in a safe-ish place here.
	(position_get_distance_to_ground_level, ":distance", pos62),
	(try_begin),
		(lt, ":distance", 0),
		(position_set_z_to_ground_level, pos62),
		(position_move_z, pos62, 200),
	(try_end),		
	(play_sound_at_position, "snd_explosive_ammo_explode", pos62),
	
	(try_for_range, ":unused", 1, 34),
		(copy_position, pos3, pos62),
		(store_random_in_range, ":x_move", -500, 500),
		(position_move_x, pos3, ":x_move", 0),
		(store_random_in_range, ":y_move", -500, 500),
		(position_move_y, pos3, ":y_move",0),
		(store_random_in_range, ":z_move", -500, 500),
		(position_move_z, pos3, ":z_move",0),
		
		(position_get_distance_to_ground_level, ":distance", pos3),
		(try_begin),
			(lt, ":distance", 0),
			(position_set_z_to_ground_level, pos3),
			(position_move_z, pos3, 200),
		(try_end),	

		(try_for_range, ":unused", 1, 20),
			(store_random_in_range, ":x_offset", 1, 361),#Random Rotation of X
			(position_rotate_x, pos3, ":x_offset"),	
			(store_random_in_range, ":y_offset", 1, 361),#Random Rotation of Y
			(position_rotate_y, pos3, ":y_offset"),	
			(store_random_in_range, ":z_offset", 1, 361),#Random Rotation of Z
			(position_rotate_z, pos3, ":z_offset"),		
			(store_random_in_range, ":speed", 1500, 2500),
			(add_missile, reg1, pos3, ":speed", "itm_llew2_matchlock_1", 0, "itm_grenade_fragments", 0),
		(try_end),	
		(particle_system_burst,"psys_explosion_dirt",pos3,200),			
		(particle_system_burst,"psys_explosive_explosion_sparks_a",pos3,35),
		(particle_system_burst,"psys_explosive_explosion_sparks_b",pos3,35),
	(try_end),	

			
]),
]],

["explosive_rocket_arrows","Explosive Rocket Arrow",[("xeno_bullet",0),("xenoargh_explosive_rocket_arrow",ixmesh_flying_ammo),("bolt_bag_c",ixmesh_carry),("xenoargh_explosive_rocket_arrow",ixmesh_inventory)],itp_type_bolts|itp_can_penetrate_shield|itp_no_pick_up_from_ground|itp_default_ammo,itcf_carry_quiver_right_vertical,4000,weight(3)|abundance(0)|weapon_length(150)|thrust_damage(30,pierce)|max_ammo(25),imodbits_none,
   [   
 (ti_on_missile_hit,
      [
			(copy_position, pos63, pos1),#Need to store it in a safe-ish place here.
			(position_get_distance_to_ground_level, ":distance", pos63),
			(try_begin),
				(lt, ":distance", 0),
				(position_set_z_to_ground_level, pos63),
				(position_move_z, pos63, 200),
			(try_end),				
			(this_or_next|multiplayer_is_dedicated_server),		  
			(this_or_next|multiplayer_is_server),
			(neg|game_in_multiplayer_mode),	
			(store_trigger_param_1,reg1),
			(try_for_range, ":unused", 1, 4),
				(copy_position, pos2, pos63),
				(store_random_in_range, ":x_offset", 1, 361),#Random Rotation of X
				(position_rotate_x, pos2, ":x_offset"),	
				(store_random_in_range, ":y_offset", 1, 361),#Random Rotation of Y
				(position_rotate_y, pos2, ":y_offset"),	
				(store_random_in_range, ":z_offset", 1, 361),#Random Rotation of Z
				(position_rotate_z, pos2, ":z_offset"),					
				(add_missile, reg1, pos2, 1500, "itm_llew2_matchlock_1", 0, "itm_grenade_fragments", 0),
			(try_end),	
			(particle_system_burst,"psys_explosion_dirt",pos63,200),			
			(particle_system_burst,"psys_explosive_explosion_sparks_a",pos63,35),
			(particle_system_burst,"psys_explosive_explosion_sparks_b",pos63,35),
			(play_sound_at_position, "snd_explosive_ammo_explode", pos63),		  
		  
]),
]],

["mortar_grenade","Mortar Grenade",[("xeno_cannonball_explosive",0)],itp_type_bolts|itp_can_penetrate_shield|itp_no_pick_up_from_ground|itp_default_ammo,itcf_carry_quiver_right_vertical,4000,weight(3)|abundance(0)|weapon_length(150)|thrust_damage(30,pierce)|max_ammo(10),imodbits_none,
[
(ti_on_missile_hit,
      [
			(copy_position, pos63, pos1),#Need to store it in a safe-ish place here.
			(position_get_distance_to_ground_level, ":distance", pos63),
			(try_begin),
				(lt, ":distance", 0),
				(position_set_z_to_ground_level, pos63),
				(position_move_z, pos63, 200),
			(try_end),				
			(this_or_next|multiplayer_is_dedicated_server),		  
			(this_or_next|multiplayer_is_server),
			(neg|game_in_multiplayer_mode),	
			(store_trigger_param_1,reg1),
			(try_for_range, ":unused", 1, 17),
				(copy_position, pos2, pos63),
				(store_random_in_range, ":x_offset", 1, 361),#Random Rotation of X
				(position_rotate_x, pos2, ":x_offset"),	
				(store_random_in_range, ":y_offset", 1, 361),#Random Rotation of Y
				(position_rotate_y, pos2, ":y_offset"),	
				(store_random_in_range, ":z_offset", 1, 361),#Random Rotation of Z
				(position_rotate_z, pos2, ":z_offset"),					
				(add_missile, reg1, pos2, 1500, "itm_sniper_crossbow", 0, "itm_grenade_fragments", 0),
			(try_end),	
			(particle_system_burst,"psys_explosion_dirt",pos63,200),			
			(particle_system_burst,"psys_explosive_explosion_sparks_a",pos63,35),
			(particle_system_burst,"psys_explosive_explosion_sparks_b",pos63,35),
			(play_sound_at_position, "snd_explosive_ammo_explode", pos63),			  
]),
]],


#DEPRECATED; LEFT HERE AS REFERENCE ONLY FOR OLD-STYLE AOE CODE.
#This code works, but the newer style, while computationally more expensive, is cooler.   
["shotshell_cannon_rounds","Shot-Shell Cannon Round",[("xeno_bullet",0),("xeno_cannonball_shotshell",ixmesh_flying_ammo),("bolt_bag_c",ixmesh_carry),("xeno_cannonball_shotshell",ixmesh_inventory)],itp_type_bullets|itp_can_penetrate_shield|itp_can_knock_down|itp_no_pick_up_from_ground|itp_default_ammo,itcf_carry_quiver_right_vertical,2500,weight(3)|abundance(0)|weapon_length(150)|thrust_damage(15,pierce)|max_ammo(30),imodbits_none,
   [
  (ti_on_missile_hit,
      [
          #pos1 - Missile hit position
         #param_1 - Shooter agent

          (try_begin),
			(this_or_next|multiplayer_is_dedicated_server),
			(this_or_next|multiplayer_is_server),
			(neg|game_in_multiplayer_mode),
			(init_position,pos63),
			(init_position,pos62),			 
			 
             (store_trigger_param_1,":shooter"),
			 (agent_get_team, ":shooter_team", ":shooter"),	
			 (copy_position, pos63, pos1),			 
			 (agent_get_position,pos62,":shooter"),
			 (get_distance_between_positions,":shoot_dist",pos1,pos62),
			(try_begin),
				#(lt,":shoot_dist",4000),#40 meter effective range,pretty weak vs. armor.
				(ge,":shoot_dist",2000),
				(assign,":shot_spread",450),#Wider spread.
				(store_random_in_range,":random_no",10,40),#larger range, to deal with heavy armors
				#(assign,reg0,":random_no"),
				#(display_message,"@Shot does {reg0} damage.",0xFFFFAAAA),
			(else_try),
				(lt,":shoot_dist",2000),#This is medium killing range,still deadly vs. against low-armor targets,but usually 2 or more shots are required.
				(ge,":shoot_dist",1000),
				(assign,":shot_spread",250),#Narrow spread.
				(store_random_in_range,":random_no",15,90),#larger range, to deal with heavy armors
				#(assign,reg0,":random_no"),
				#(display_message,"@Shot does {reg0} damage.",0xFFFFAAAA),				
			(else_try),
				(lt,":shoot_dist",1000),#At < 10 meters,it's pretty much insta-death for anything it hits.
				(assign,":shot_spread",150),#Narrow spread.
				(store_random_in_range,":random_no",200,300),#larger range, to deal with heavy armors
				#(assign,reg0,":random_no"),
				#(display_message,"@Shot does {reg0} damage.",0xFFFFAAAA),
			(try_end),
			(position_set_y, pos63, 0),
			(try_begin),
			 (lt,":shoot_dist",4000),
             (try_for_agents,":agent"),
				(agent_get_team, ":agent_team", ":agent"),
				(teams_are_enemies, ":agent_team", ":shooter_team"),
				(agent_is_active,":agent"),
				(agent_is_alive,":agent"),	
				(init_position,pos61),
                (agent_get_position,pos61,":agent"),
				(position_set_y, pos61, 0),
                (get_distance_between_positions,":dist",pos63,pos61),
                (try_begin),
				   (lt,":dist",":shot_spread"),
				   (position_move_z,pos61,125),
				   (position_move_y,pos61,15),
				   (particle_system_burst,"psys_blood_small",pos61,35),
					(try_for_range,":equip_type",itm_leather_armor,itm_black_armor),
						(try_begin),
							(agent_has_item_equipped,":agent",":equip_type"),
							(val_sub,":random_no",10),#Armor offers more protection than against explosive shell.
						(try_end),
					(try_end),
					(agent_deliver_damage_to_agent,":shooter",":agent",":random_no"),
                (try_end),
             (try_end),
          (try_end),
]),
]],

["cannon_ammo_end","Cannon Ammunition End (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["throwing_light_start","Light Throwing Weapons Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#Darts (low velocity, high speed rating, OK damage, uses Pierce, so it can hurt most folks- these were weird weapons).
["darts","Darts",[("dart_b",0),("dart_b_bag",ixmesh_carry)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary,itcf_throw_javelin|itcf_carry_quiver_right_vertical|itcf_show_holster_when_drawn,1000,weight(5)|spd_rtng(115)|shoot_speed(55)|thrust_damage(40,pierce)|max_ammo(30)|weapon_length(32)|accuracy(100),imodbits_none],

["war_darts","War Darts",[("dart_a",0),("dart_a_bag",ixmesh_carry)],itp_type_thrown|itp_no_pick_up_from_ground|itp_merchandise|itp_primary,itcf_throw_javelin|itcf_carry_quiver_back|itcf_show_holster_when_drawn,2000,weight(5)|spd_rtng(115)|shoot_speed(55)|thrust_damage(50,pierce)|max_ammo(30)|weapon_length(45)|accuracy(100),imodbits_none],

#Thrown spears (lower velocity than darts, high speed rating, great damage,uses Cut)
["dejawolf_atgeirr","Atgeirr (throwing spear)",[("dejawolf_atgeirr",0),("javelins_quiver_new",ixmesh_carry)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin|itcf_carry_quiver_back|itcf_show_holster_when_drawn,1000,weight(5)|spd_rtng(115)|shoot_speed(45)|thrust_damage(45,pierce)|max_ammo(10)|weapon_length(80)|accuracy(100),imodbits_none,[],[fac_kingdom_of_nords]],
["dejawolf_atgeirr_melee","Atgeirr (throwing spear)",[("dejawolf_atgeirr",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_staff,1500,weight(1)|spd_rtng(95)|swing_damage(15,cut)|thrust_damage(25,pierce)|weapon_length(80),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_bryntvari","Light Bryntvari (throwing spear)",[("dejawolf_bryntvari",0),("javelins_quiver_new",ixmesh_carry)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin|itcf_carry_quiver_back|itcf_show_holster_when_drawn,1000,weight(5)|spd_rtng(115)|shoot_speed(45)|thrust_damage(45,pierce)|max_ammo(10)|weapon_length(80)|accuracy(100),imodbits_none,[],[fac_kingdom_of_nords]],
["dejawolf_bryntvari_melee","Light Bryntvari (throwing spear)",[("dejawolf_bryntvari",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_staff,1500,weight(1)|spd_rtng(95)|swing_damage(15,cut)|thrust_damage(25,pierce)|weapon_length(80),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_bryntvari2","Heavy Bryntvari (throwing spear)",[("dejawolf_bryntvari2",0),("javelins_quiver_new",ixmesh_carry)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin|itcf_carry_quiver_back|itcf_show_holster_when_drawn,1000,weight(5)|spd_rtng(115)|shoot_speed(40)|thrust_damage(55,pierce)|max_ammo(10)|weapon_length(104)|accuracy(100),imodbits_none,[],[fac_kingdom_of_nords]],
["dejawolf_bryntvari2_melee","Heavy Bryntvari (throwing spear)",[("dejawolf_bryntvari2",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_spear,2000,weight(1)|spd_rtng(95)|swing_damage(15,cut)|thrust_damage(35,pierce)|weapon_length(104),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_krokaspjott2","Light Krokaspjott (harpoon)",[("dejawolf_krokaspjott2",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin,3500,weight(5)|spd_rtng(115)|shoot_speed(45)|thrust_damage(75,pierce)|max_ammo(3)|weapon_length(125)|accuracy(100),imodbits_none,[],[fac_kingdom_of_nords]],
["dejawolf_krokaspjott2_melee","Light Krokaspjott (harpoon)",[("dejawolf_krokaspjott2",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_spear,3500,weight(1)|spd_rtng(120)|swing_damage(15,cut)|thrust_damage(55,pierce)|weapon_length(125),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_krokaspjott1","Heavy Krokaspjott (harpoon)",[("dejawolf_krokaspjott1",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin,4000,weight(5)|spd_rtng(115)|shoot_speed(40)|thrust_damage(80,pierce)|max_ammo(3)|weapon_length(150)|accuracy(100),imodbits_none,[],[fac_kingdom_of_nords]],
["dejawolf_krokaspjott1_melee","Heavy Krokaspjott (harpoon)",[("dejawolf_krokaspjott1",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_spear,4000,weight(1)|spd_rtng(120)|swing_damage(15,cut)|thrust_damage(60,pierce)|weapon_length(150),imodbits_none,[],[fac_kingdom_of_nords]],

["dejawolf_kastad_krokaspjott","Casting Krokaspjott (harpoon)",[("dejawolf_kastad_krokaspjott",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin,3000,weight(5)|spd_rtng(115)|shoot_speed(40)|thrust_damage(80,pierce)|max_ammo(3)|weapon_length(100)|accuracy(100),imodbits_none,[],[fac_kingdom_of_nords]],
["dejawolf_kastad_krokaspjott_melee","Casting Krokaspjott (harpoon)",[("dejawolf_kastad_krokaspjott",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_spear,4000,weight(1)|spd_rtng(120)|swing_damage(15,cut)|thrust_damage(60,pierce)|weapon_length(100),imodbits_none,[],[fac_kingdom_of_nords]],

["javelin","Javelins",[("javelin",0),("javelins_quiver_new",ixmesh_carry)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin|itcf_carry_quiver_back|itcf_show_holster_when_drawn,1000,weight(5)|spd_rtng(115)|shoot_speed(50)|thrust_damage(100,cut)|max_ammo(10)|weapon_length(75)|accuracy(100),imodbits_none],
["javelin_melee","Javelin",[("javelin",0)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_staff,1000,weight(1)|spd_rtng(95)|swing_damage(15,cut)|thrust_damage(25,pierce)|weapon_length(75),imodbits_none],

["throwing_spears","Throwing Spears",[("jarid_new_b",0),("jarid_new_b_bag",ixmesh_carry)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin|itcf_carry_quiver_back|itcf_show_holster_when_drawn,2000,weight(4)|spd_rtng(115)|shoot_speed(45)|thrust_damage(130,cut)|max_ammo(5)|weapon_length(65)|accuracy(100),imodbits_none],
["throwing_spear_melee","Throwing Spear",[("jarid_new_b",0),("javelins_quiver",ixmesh_carry)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_staff,
2000,weight(1)|spd_rtng(95)|swing_damage(45,cut)|thrust_damage(40,pierce)|weapon_length(75),imodbits_none],

["jarid","Jarids",[("jarid_new",0),("jarid_quiver",ixmesh_carry)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_javelin|itcf_carry_quiver_back|itcf_show_holster_when_drawn,4000,weight(4)|spd_rtng(115)|shoot_speed(40)|thrust_damage(150,cut)|max_ammo(5)|weapon_length(65)|accuracy(100),imodbits_none],
["jarid_melee","Jarid",[("jarid_new",0),("jarid_quiver",ixmesh_carry)],itp_type_polearm|itp_primary|itp_penalty_with_shield|itp_wooden_parry,itc_staff,4000,weight(1)|spd_rtng(95)|swing_damage(60,cut)|thrust_damage(45,pierce)|weapon_length(65),imodbits_none],

#Stones (which no longer *totally* suck, but that's not saying much, but hey, they're practically free)
["stones","Stones",[("throwing_stone",0)],itp_type_thrown|itp_no_pick_up_from_ground|itp_primary,itcf_throw_stone,1,weight(4)|spd_rtng(120)|shoot_speed(40)|thrust_damage(40,blunt)|max_ammo(50)|weapon_length(8)|accuracy(100),imodbit_large_bag],

#Throwing knives,low velocity,lower damage than spears and doesn't pierce,but has an awesome reload rate
["throwing_knives","Throwing Knives",[("throwing_knife",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary,itcf_throw_knife,1000,weight(3.5)|spd_rtng(140)|shoot_speed(30)|thrust_damage(45,cut)|max_ammo(30)|weapon_length(0)|accuracy(100),imodbits_none],

["throwing_daggers","Throwing Daggers",[("throwing_dagger",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary,itcf_throw_knife,2000,weight(3.5)|spd_rtng(140)|shoot_speed(30)|thrust_damage(55,cut)|max_ammo(30)|weapon_length(0)|accuracy(100),imodbits_none],

#Throwing axes,superb damage but low velocity and only does cut
["light_throwing_axes","Light Throwing Axes",[("francisca",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_axe,
500,weight(5)|spd_rtng(130)|shoot_speed(45)|thrust_damage(80,cut)|max_ammo(10)|weapon_length(53)|accuracy(100),imodbits_none],
["light_throwing_axes_melee","Light Throwing Axe",[("francisca",0)],itp_type_one_handed_wpn|itp_primary|itp_bonus_against_shield,itc_scimitar,
500,weight(1)|spd_rtng(130)|weapon_length(53)|swing_damage(25,cut),imodbits_none],

["throwing_axes","Throwing Axes",[("throwing_axe_a",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_axe,
1500,weight(5)|spd_rtng(120)|shoot_speed(45)|thrust_damage(100,cut)|max_ammo(10)|weapon_length(53)|accuracy(100),imodbits_none],
["throwing_axes_melee","Throwing Axe",[("throwing_axe_a",0)],itp_type_one_handed_wpn|itp_primary|itp_bonus_against_shield,itc_scimitar,
1500,weight(1)|spd_rtng(120)|swing_damage(30,cut)|weapon_length(53),imodbits_none],

["heavy_throwing_axes","Heavy Throwing Axes",[("throwing_axe_b",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_axe,
3000,weight(5)|spd_rtng(120)|shoot_speed(45)|thrust_damage(140,cut)|max_ammo(10)|weapon_length(61)|accuracy(100),imodbits_none],
["heavy_throwing_axes_melee","Heavy Throwing Axe",[("throwing_axe_b",0)],itp_type_one_handed_wpn|itp_primary|itp_bonus_against_shield,itc_scimitar,
5000,weight(1)|spd_rtng(120)|swing_damage(55,cut)|weapon_length(61),imodbits_none],

["mackie_tomahawk","Tomahawk",[("mackie_tomahawk",0)],itp_type_thrown|itp_merchandise|itp_primary|itp_no_pick_up_from_ground|itp_next_item_as_melee,itcf_throw_axe,
3000,weight(5)|spd_rtng(120)|shoot_speed(45)|thrust_damage(120,cut)|max_ammo(10)|weapon_length(53)|accuracy(100),imodbits_none],
["mackie_tomahawk_melee","Tomahawk",[("mackie_tomahawk",0)],itp_type_one_handed_wpn|itp_primary|itp_bonus_against_shield,itc_scimitar,
3000,weight(1)|spd_rtng(120)|swing_damage(35,cut)|weapon_length(53),imodbits_none],

["xenoargh_bwanga","Bwanga",[("xenoargh_bwanga",0)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary|itp_next_item_as_melee,itcf_throw_axe,
3000,weight(5)|spd_rtng(120)|shoot_speed(45)|thrust_damage(145,cut)|max_ammo(3)|weapon_length(66)|accuracy(100),imodbits_none],
["xenoargh_bwanga_melee","Bwanga",[("xenoargh_bwanga",0)],itp_type_one_handed_wpn|itp_primary|itp_bonus_against_shield,itc_scimitar,
3000,weight(1)|spd_rtng(120)|swing_damage(45,cut)|weapon_length(66),imodbits_none],

["throwing_light_end","Light Throwing Weapons End (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["throwing_explosive_start","Explosive Throwing Weapons Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["xenoargh_bola","Boleadora (bola)",[("xeno_bullet",0),("xenoargh_bola",ixmesh_flying_ammo),("xenoargh_bola_store",ixmesh_inventory)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary,itcf_throw_axe,2500,weight(5)|spd_rtng(120)|shoot_speed(30)|thrust_damage(55,blunt)|max_ammo(10)|weapon_length(100)|accuracy(100),imodbits_none,
[
(ti_on_missile_hit,
      [
		#Anti-Horse script.
          #pos1 - Missile hit position
         #param_1 - Shooter agent
		 
		(try_begin),
		(init_position,pos63),
		(copy_position, pos63, pos1),	
		(position_set_y, pos63, 0),	
		(this_or_next|multiplayer_is_dedicated_server),		
		(this_or_next|multiplayer_is_server),
		(neg|game_in_multiplayer_mode),		
		(store_trigger_param_1,":shooter"),
		(agent_get_team, ":shooter_team", ":shooter"),	
		(try_for_agents,":agent"),
				(agent_get_team, ":agent_team", ":agent"),
				(teams_are_enemies, ":agent_team", ":shooter_team"),
				(agent_is_active,":agent"),
				(agent_is_alive,":agent"),		
				(try_begin),
						(agent_is_human, ":agent"),#stop if not human
					(else_try),
						(init_position,pos62),
						(agent_get_position,pos62,":agent"),
						(position_set_y, pos62, 0),
						(get_distance_between_positions,":dist",pos63,pos62),
						(try_begin),
							(lt,":dist",300),#totally unrealistic check distance, but hey, it's going to get used vs. moving targets invariably...					
							(store_trigger_param_1,":shooter"),
							(store_random_in_range,":random_no",175,350),#almost certain chance of taking a horse out on one hit.
							(agent_deliver_damage_to_agent,":shooter",":agent",":random_no"),
						(try_end),	
				(try_end),
          (try_end),
		  (try_end),
]),
]],

["xenoargh_grenade","Hand Grenades",[("xeno_bullet",0),("xeno_cannonball_explosive",ixmesh_flying_ammo),("xeno_cannonball_explosive",ixmesh_inventory)],itp_type_thrown|itp_merchandise|itp_no_pick_up_from_ground|itp_primary,itcf_throw_stone,4500,weight(5)|spd_rtng(70)|shoot_speed(30)|thrust_damage(70,pierce)|max_ammo(6)|weapon_length(100)|accuracy(75),imodbits_none,
[
(ti_on_missile_hit,
      [
			(copy_position, pos63, pos1),#Need to store it in a safe-ish place here.
			(position_get_distance_to_ground_level, ":distance", pos63),
			(try_begin),
				(lt, ":distance", 0),
				(position_set_z_to_ground_level, pos63),
				(position_move_z, pos63, 200),
			(try_end),				
			(this_or_next|multiplayer_is_dedicated_server),		  
			(this_or_next|multiplayer_is_server),
			(neg|game_in_multiplayer_mode),	
			(store_trigger_param_1,reg1),
			(try_for_range, ":unused", 1, 31),
				(copy_position, pos2, pos63),
				(store_random_in_range, ":x_offset", 1, 361),#Random Rotation of X
				(position_rotate_x, pos2, ":x_offset"),	
				(store_random_in_range, ":y_offset", 1, 361),#Random Rotation of Y
				(position_rotate_y, pos2, ":y_offset"),	
				(store_random_in_range, ":z_offset", 1, 361),#Random Rotation of Z
				(position_rotate_z, pos2, ":z_offset"),					
				(add_missile, reg1, pos2, 1500, "itm_sniper_crossbow", 0, "itm_grenade_fragments", 0),
			(try_end),	
			(particle_system_burst,"psys_explosion_dirt",pos63,200),			
			(particle_system_burst,"psys_explosive_explosion_sparks_a",pos63,35),
			(particle_system_burst,"psys_explosive_explosion_sparks_b",pos63,35),
			(play_sound_at_position, "snd_explosive_ammo_explode", pos63),			
]),
]],

["xenoargh_troop_grenade","Hand Grenades",[("xeno_bullet",0),("xeno_cannonball_explosive",ixmesh_flying_ammo),("xeno_cannonball_explosive",ixmesh_inventory)],itp_type_thrown|itp_no_pick_up_from_ground|itp_primary,itcf_throw_stone,4500,weight(5)|spd_rtng(70)|shoot_speed(70)|thrust_damage(50,pierce)|max_ammo(2)|weapon_length(100)|accuracy(75),imodbits_none,
[
(ti_on_missile_hit,
      [
			(copy_position, pos63, pos1),#Need to store it in a safe-ish place here.
			(position_get_distance_to_ground_level, ":distance", pos63),
			(try_begin),
				(lt, ":distance", 0),
				(position_set_z_to_ground_level, pos63),
				(position_move_z, pos63, 200),
			(try_end),				
			(this_or_next|multiplayer_is_dedicated_server),		  
			(this_or_next|multiplayer_is_server),
			(neg|game_in_multiplayer_mode),	
			(store_trigger_param_1,reg1),
			(try_for_range, ":unused", 1, 17),
				(copy_position, pos2, pos63),
				(store_random_in_range, ":x_offset", 1, 361),#Random Rotation of X
				(position_rotate_x, pos2, ":x_offset"),	
				(store_random_in_range, ":y_offset", 1, 361),#Random Rotation of Y
				(position_rotate_y, pos2, ":y_offset"),	
				(store_random_in_range, ":z_offset", 1, 361),#Random Rotation of Z
				(position_rotate_z, pos2, ":z_offset"),					
				(add_missile, reg1, pos2, 1500, "itm_sniper_crossbow", 0, "itm_grenade_fragments", 0),
			(try_end),	
			(particle_system_burst,"psys_explosion_dirt",pos63,200),			
			(particle_system_burst,"psys_explosive_explosion_sparks_a",pos63,35),
			(particle_system_burst,"psys_explosive_explosion_sparks_b",pos63,35),
			(play_sound_at_position, "snd_explosive_ammo_explode", pos63),			  
]),
]],

["throwing_explosive_end","Fragment",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(50,pierce)|max_ammo(50),imodbits_none],

["bows_and_xbows_start","Bows and Crossbows Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
#BOWS
#These have far faster rates of fire than crossbows, but are generally less-able to deal with armor.
#The very high end of these weapons is my best attempt to simulate the warbows of the Hundred Years War period, and are quite nasty.
#Light bows (fast,middling velocity,low damage)
["hunting_bow","Hunting Bow",[("hunting_bow",0),("hunting_bow_carry",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,
500,weight(1)|spd_rtng(140)|shoot_speed(50)|thrust_damage(25,pierce)|accuracy(107),imodbits_none],

["short_bow","Short Bow",[("short_bow",0),("short_bow_carry",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,
1000,weight(1)|spd_rtng(140)|shoot_speed(60)|thrust_damage(30,pierce)|accuracy(107),imodbits_none],

#Mongol short bow,very fast,very low damage (but it will make those pesky Khergits even more pesky)
["khergit_bow","Khergit Bow",[("khergit_bow",0),("khergit_bow_case",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bowcase_left|itcf_show_holster_when_drawn,2000,weight(1.25)|spd_rtng(135)|shoot_speed(70)|thrust_damage(33,pierce)|accuracy(107),imodbits_none,[],[fac_kingdom_of_khergits]],

#Recurve Bows (slower rate of fire, superior range, mediocre damage)
["nomad_bow","Nomad Bow",[("nomad_bow",0),("nomad_bow_case",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bowcase_left|itcf_show_holster_when_drawn,1000,weight(1.25)|spd_rtng(90)|shoot_speed(130)|thrust_damage(34,pierce)|accuracy(107),imodbits_none],

["strong_bow","Strong Bow",[("strong_bow",0),("strong_bow_case",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bowcase_left|itcf_show_holster_when_drawn,3000,weight(1.25)|spd_rtng(80)|shoot_speed(130)|thrust_damage(35,pierce)|accuracy(107),imodbits_none],

#Heavy longbows and warbows,(slower draw, much heavier and greater velocity- the top end is almost competitive with crossbows)
["long_bow","Long Bow",[("long_bow",0),("long_bow_carry",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,
2000,weight(1.75)|spd_rtng(105)|shoot_speed(100)|thrust_damage(36,pierce)|accuracy(107),imodbits_none],

["rogue_bow","Thief's Bow",[("rogue_bow",0),("rogue_bow_carry",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,3000,weight(1.25)|spd_rtng(100)|shoot_speed(105)|thrust_damage(37,pierce)|accuracy(107),imodbits_none],

["deshavi_bow","Deshavi's Bow",[("amade_latticed_flatbow",0),("amade_latticed_flatbow_carry",ixmesh_carry)],itp_type_bow|itp_primary|itp_two_handed|itp_can_penetrate_shield,itcf_shoot_bow|itcf_carry_bow_back,3000,weight(1.5)|spd_rtng(155)|shoot_speed(110)|thrust_damage(75,pierce)|accuracy(113),imodbits_none],

["mourn_bow","Mourn Bow",[("mourn_bow",0),("mourn_bow_carry",ixmesh_carry)],itp_type_bow|itp_primary|itp_two_handed|itp_can_penetrate_shield,itcf_shoot_bow|itcf_carry_bow_back,3000,weight(1.25)|spd_rtng(150)|shoot_speed(120)|thrust_damage(85,pierce)|accuracy(113),imodbits_none],

["war_bow","War Bow",[("war_bow",0),("war_bow_carry",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,
6000,weight(1.5)|spd_rtng(90)|shoot_speed(110)|thrust_damage(38,pierce)|accuracy(107),imodbits_none],

["latticed_flatbow","Latticed Flatbow",[("amade_latticed_flatbow",0),("amade_latticed_flatbow_carry",ixmesh_carry)],itp_type_bow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,10000,weight(1.5)|spd_rtng(90)|shoot_speed(110)|thrust_damage(40,pierce)|accuracy(107),imodbits_none],

#CROSSBOWS
#One-handed
#Fairly cruddy, not fast, not long range, but piercing.
["xenoargh_chu_ko_nu","Chukonu",[("xenoargh_chu_ko_nu",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed|itp_no_pick_up_from_ground,itcf_shoot_crossbow,2700,weight(2.25)|spd_rtng(220)|shoot_speed(65)|thrust_damage(35,pierce)|max_ammo(7)|accuracy(80),imodbits_none,[(ti_on_weapon_attack,[(play_sound,"snd_release_crossbow_real"),])],[fac_kingdom_of_khergits]],

["hunting_crossbow","Hunting Crossbow",[("crossbow_a",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_crossbow|itcf_carry_crossbow_back,
1000,weight(2.25)|spd_rtng(70)|shoot_speed(70)|thrust_damage(45,pierce)|max_ammo(1),imodbits_none,[(ti_on_weapon_attack,[(play_sound,"snd_release_crossbow_real"),])]],

["light_crossbow","Light Crossbow",[("crossbow_b",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_crossbow|itcf_carry_crossbow_back,
2000,weight(2.5)|spd_rtng(65)|shoot_speed(70)|thrust_damage(50,pierce)|max_ammo(1),imodbits_none,[(ti_on_weapon_attack,[(play_sound,"snd_release_crossbow_real"),])]],

#Two-handed (much greater velocity,much higher damage,low speed)
#These go through armor like paper, but you get maybe one shot at charging horses.  They're pretty much like the Real Thing.  Tradeoffs, tradeoffs...
["crossbow","Crossbow",[("crossbow_b",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed|itp_cant_reload_on_horseback,itcf_shoot_crossbow|itcf_carry_crossbow_back,2000,weight(3)|spd_rtng(55)|shoot_speed(120)|thrust_damage(55,pierce)|max_ammo(1)|accuracy(107),imodbits_none,[(ti_on_weapon_attack,[(play_sound,"snd_release_crossbow_real"),])]],

["heavy_crossbow","Heavy Crossbow",[("crossbow_c",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed|itp_cant_reload_on_horseback,itcf_shoot_crossbow|itcf_carry_crossbow_back,4500,weight(3.5)|spd_rtng(55)|shoot_speed(140)|thrust_damage(60,pierce)|max_ammo(1)|accuracy(107),imodbits_none,[(ti_on_weapon_attack,[(play_sound,"snd_release_crossbow_real"),])]],

#Exceptional weapon for siege use only,very high velocity and damage
["sniper_crossbow","Siege Crossbow",[("xenoargh_arbalest",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed|itp_cant_reload_on_horseback,itcf_shoot_crossbow|itcf_carry_crossbow_back,6000,weight(3.75)|spd_rtng(50)|shoot_speed(160)|thrust_damage(65,pierce)|max_ammo(1)|accuracy(107),imodbits_none,[(ti_on_weapon_attack,[(play_sound,"snd_release_crossbow_real"),])]],

["bows_and_xbows_end","Bows and Crossbows End (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["firearms_start","Firarms Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

#FIREARMS
#Damage is so "extreme" because M&B doesn't seem to handle firearm damage like crossbows / bows, they seem to get nerfed badly, regardless of skill

#PISTOLS (rebalanced to be actually useful- short range, but high damage and fire rate.
["flintlock_pistol","Flintlock (bullets)",[("flintlock_pistol",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield,itcf_shoot_pistol|itcf_reload_pistol|itcf_carry_revolver_right,5000,weight(1.5)|spd_rtng(80)|shoot_speed(65)|thrust_damage(80,pierce)|max_ammo(1)|accuracy(95),imodbits_none,[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 3),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Pistol Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.
	(set_fixed_point_multiplier, 100),
	(assign, ":z_fix", -4850),#-48.5 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.	
	(store_random_in_range, ":z_offset", -150, 150),#Random Rotation of Z
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)	
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.	
	(store_random_in_range, ":x_offset", -150, 151),#Random Rotation of X
	(position_rotate_x_floating, pos2, ":x_offset"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 6500, "itm_flintlock_pistol", 0, "itm_shotgun_pellets", 0),
(try_end),
(play_sound,"snd_pistol_shot"),#Firing sound
#(position_move_x,pos1,50),#Move along X
(position_move_y,pos1,50),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

["llew2_flintlock_pistol","Duelist's Pistol (bullets)",[("llew2_flintlock_pistol",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield,itcf_shoot_pistol|itcf_reload_pistol|itcf_carry_revolver_right,7000,weight(1.5)|spd_rtng(80)|shoot_speed(70)|thrust_damage(80,pierce)|max_ammo(1)|accuracy(100),imodbits_none,[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 3),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Pistol Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.
	(set_fixed_point_multiplier, 100),
	(assign, ":z_fix", -4850),#-48.5 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.	
	(store_random_in_range, ":z_offset", -125, 126),#Random Rotation of Z
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)	
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.	
	(store_random_in_range, ":x_offset", -125, 126),#Random Rotation of X
	(position_rotate_x_floating, pos2, ":x_offset"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 6000, "itm_llew2_flintlock_pistol", 0, "itm_shotgun_pellets", 0),
(try_end),
(play_sound,"snd_pistol_shot"),#Firing sound
#(position_move_x,pos1,50),#Move along X
(position_move_y,pos1,50),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],
 
["double_flintlock_pistol","Double-Barreled Flintlock (bullets)",[("xenoargh_double_dragon",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield,itcf_shoot_pistol|itcf_reload_pistol|itcf_carry_revolver_right,7500,weight(1.5)|spd_rtng(60)|shoot_speed(65)|thrust_damage(80,pierce)|max_ammo(2)|accuracy(90),imodbits_none,[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 3),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Pistol Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.
	(set_fixed_point_multiplier, 100),
	(assign, ":z_fix", -4850),#-48.5 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.	
	(store_random_in_range, ":z_offset", -200, 201),#Random Rotation of Z
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)	
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.	
	(store_random_in_range, ":x_offset", -200, 201),#Random Rotation of X
	(position_rotate_x_floating, pos2, ":x_offset"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 6000, "itm_double_flintlock_pistol", 0, "itm_shotgun_pellets", 0),
(try_end),
(play_sound,"snd_pistol_shot"),#Firing sound
#(position_move_x,pos1,50),#Move along X
(position_move_y,pos1,50),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

#The Gun-Axe.  What's fun is that I didn't even make the weapon up- they really existed :-) 
#Fires 3 shots, like a light shotgun.
["xenoargh_gun_axe","Gun-Axe (bullets)",[("xenoargh_gun_axe",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_next_item_as_melee,itcf_shoot_pistol|itcf_reload_pistol,4000,weight(1.5)|spd_rtng(60)|shoot_speed(75)|thrust_damage(75,pierce)|max_ammo(1)|accuracy(80),imodbits_none,
[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 3),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Pistol Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.
	(set_fixed_point_multiplier, 100),
	(assign, ":z_fix", -4850),#-48.5 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.	
	(store_random_in_range, ":z_offset", -100, 101),#Random Rotation of Z
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)	
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.	
	(store_random_in_range, ":x_offset", -100, 101),#Random Rotation of X
	(position_rotate_x_floating, pos2, ":x_offset"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 7500, "itm_xenoargh_gun_axe", 0, "itm_shotgun_pellets", 0),
(try_end),
(play_sound,"snd_pistol_shot"),#Firing sound
#(position_move_x,pos1,50),#Move along X
(position_move_y,pos1,50),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],
#HTH Version 
["xenoargh_gun_axe_melee","Gun-Axe (bullets)",[("xenoargh_gun_axe_hth",0)],itp_type_one_handed_wpn|itp_primary|itp_bonus_against_shield,itc_scimitar,
3000,weight(1)|spd_rtng(120)|swing_damage(55,cut)|weapon_length(60),imodbits_none],

#Llew's blunderbus- a shotgun that's a bit more accurate, long-ranged and deadly than a gun-axe.  
#This is a nasty weapon, within its range band.
["llew2_blunderbus","Blunderbus (bullets)",[("llew2_blunderbus",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_two_handed,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,8000,weight(5.5)|spd_rtng(55)|shoot_speed(80)|thrust_damage(70,pierce)|max_ammo(1)|accuracy(90),imodbits_none,
[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 6),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Rifle Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.	
	(set_fixed_point_multiplier, 100),
	(assign, ":x_fix", -350),#-3.5 degrees
	(assign, ":z_fix", 570),#5.7 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.
	(store_random_in_range, ":z_offset", -200, 201),#2 deg. of random rotation on the Z axis
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.
	(store_random_in_range, ":x_offset", -200, 201),#2 deg. of random rotation on the X axis
	(val_add, ":x_fix", ":x_offset"),#Add offset to the X fix
	(position_rotate_x_floating, pos2, ":x_fix"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 8000, "itm_llew2_blunderbus", 0, "itm_shotgun_pellets", 0),
(try_end),
(play_sound,"snd_gunshot_heavy"),#Firing sound
(position_move_y,pos1,95),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

#The Spear-Gun.  A whacky Ming-dynasty Chinese weapon.
["xenoargh_spear_gun","Spear-Gun (bullets)",[("xenoargh_spear_gun",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_two_handed|itp_next_item_as_melee,itcf_shoot_musket,2500,weight(5.5)|spd_rtng(15)|shoot_speed(60)|thrust_damage(90,pierce)|max_ammo(1)|accuracy(75),imodbits_none,[(ti_on_weapon_attack,[
(play_sound,"snd_gunshot_light"),
(position_move_y,pos1,115),
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],
#HTH Version 
["xenoargh_spear_gun_hth","Spear-Gun (bullets)",[("xenoargh_spear_gun",0)],itp_type_polearm|itp_primary|itp_wooden_parry,itc_spear,2500,weight(4)|difficulty(0)|spd_rtng(110)|weapon_length(135)|swing_damage(85,cut)|thrust_damage(50,pierce),imodbits_none],
 
["arquebus","Arquebus (bullets)",[("rrr_matchlock_musket",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,7500,weight(5.5)|spd_rtng(35)|shoot_speed(140)|thrust_damage(90,pierce)|max_ammo(1)|accuracy(95),imodbits_none,[(ti_on_weapon_attack,[
(play_sound,"snd_gunshot_light"),
(position_move_y,pos1,115),
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

["jezzail","Jezzail (bullets)",[("xeno_jezzail",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_two_handed,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,9500,weight(5.5)|spd_rtng(30)|shoot_speed(180)|thrust_damage(100,pierce)|max_ammo(1)|accuracy(107),imodbits_none,[(ti_on_weapon_attack,[
(play_sound,"snd_gunshot_heavy"),
(position_move_y,pos1,115),
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

#Llew's awesome firearm collection
["llew2_arquebus","Light Arquebus (bullets)",[("llew2_arquebus",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_two_handed,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,7000,weight(5.5)|spd_rtng(45)|shoot_speed(120)|thrust_damage(70,pierce)|max_ammo(1)|accuracy(95),imodbits_none,[(ti_on_weapon_attack,[
(play_sound,"snd_gunshot_light"),
(position_move_y,pos1,115),
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

["llew2_matchlock_1","Light Matchlock (bullets)",[("llew2_matchlock_1",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_two_handed,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,7500,weight(5.5)|spd_rtng(40)|shoot_speed(125)|thrust_damage(80,pierce)|max_ammo(1)|accuracy(95),imodbits_none,[(ti_on_weapon_attack,[
(play_sound,"snd_gunshot_light"),
(position_move_y,pos1,115),
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

["llew2_matchlock_2","Heavy Matchlock (bullets)",[("llew2_matchlock_2",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_two_handed,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,8000,weight(5.5)|spd_rtng(30)|shoot_speed(145)|thrust_damage(95,pierce)|max_ammo(1)|accuracy(100),imodbits_none,[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 3),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Rifle Version
	#Does "buck and ball" simulation, using a second weapon for the "buckshot".
	#The buckshot travels at a slower rate and will fall off faster, just like the
	#Real Thing.

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.	
	(set_fixed_point_multiplier, 100),
	(assign, ":x_fix", -350),#-3.5 degrees
	(assign, ":z_fix", 570),#5.7 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.
	(store_random_in_range, ":z_offset", -100, 101),#2 deg. of random rotation on the Z axis
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.
	(store_random_in_range, ":x_offset", -100, 101),#2 deg. of random rotation on the X axis
	(val_add, ":x_fix", ":x_offset"),#Add offset to the X fix
	(position_rotate_x_floating, pos2, ":x_fix"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 10000, "itm_llew2_blunderbus", 0, "itm_shotgun_pellets", 0),
(try_end),

(play_sound,"snd_gunshot_heavy"),
(position_move_y,pos1,115),
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

["llew2_flintlock_rifle_1","Flintlock Rifle (bullets)",[("llew2_flintlock_rifle_1",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_two_handed,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,14000,weight(5.5)|spd_rtng(25)|shoot_speed(170)|thrust_damage(115,pierce)|max_ammo(1)|accuracy(107),imodbits_none,[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 3),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Rifle Version
	#Does "buck and ball" simulation, using a second weapon for the "buckshot".
	#The buckshot travels at a slower rate and will fall off faster, just like the
	#Real Thing.

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.	
	(set_fixed_point_multiplier, 100),
	(assign, ":x_fix", -350),#-3.5 degrees
	(assign, ":z_fix", 570),#5.7 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.
	(store_random_in_range, ":z_offset", -90, 90),#2 deg. of random rotation on the Z axis
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.
	(store_random_in_range, ":x_offset", -90, 90),#2 deg. of random rotation on the X axis
	(val_add, ":x_fix", ":x_offset"),#Add offset to the X fix
	(position_rotate_x_floating, pos2, ":x_fix"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 10000, "itm_llew2_blunderbus", 0, "itm_shotgun_pellets", 0),
(try_end),

(play_sound,"snd_gunshot_heavy"),
(position_move_y,pos1,115),
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

["dragon","Dragon (bullets)",[("xenoargh_matchlock_dragon",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_no_pick_up_from_ground,itcf_shoot_pistol|itcf_reload_pistol|itcf_carry_revolver_right,7500,weight(6.0)|spd_rtng(75)|shoot_speed(55)|thrust_damage(55,pierce)|max_ammo(1)|accuracy(80),imodbits_none,
[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 16),
	(copy_position, pos2, pos1),

	#Shotgun script
	#Pistol Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.
	(set_fixed_point_multiplier, 100),
	(assign, ":z_fix", -4850),#-48.5 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.	
	(store_random_in_range, ":z_offset", -200, 201),#Random Rotation of Z
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)	
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.	
	(store_random_in_range, ":x_offset", -200, 201),#Random Rotation of X
	(position_rotate_x_floating, pos2, ":x_offset"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 5500, "itm_dragon", 0, "itm_shotgun_pellets", 0),
(try_end),
(play_sound,"snd_gunshot_light"),#Firing sound
#(position_move_x,pos1,50),#Move along X
(position_move_y,pos1,50),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],
 
["double_dragon","Double-Barreled Dragon (bullets)",[("xenoargh_double_dragon",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_bonus_against_shield|itp_no_pick_up_from_ground,itcf_shoot_pistol|itcf_reload_pistol|itcf_carry_revolver_right,7500,weight(6.0)|spd_rtng(70)|shoot_speed(55)|thrust_damage(55,pierce)|max_ammo(2)|accuracy(80),imodbits_none,
[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 16),
	(copy_position, pos2, pos1),

	#Shotgun script
	#Pistol Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.
	(set_fixed_point_multiplier, 100),
	(assign, ":z_fix", -4850),#-48.5 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.	
	(store_random_in_range, ":z_offset", -200, 201),#Random Rotation of Z
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)	
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.	
	(store_random_in_range, ":x_offset", -200, 201),#Random Rotation of X
	(position_rotate_x_floating, pos2, ":x_offset"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 5500, "itm_double_dragon", 0, "itm_shotgun_pellets", 0),
(try_end),
(play_sound,"snd_gunshot_heavy"),#Firing sound
#(position_move_x,pos1,50),#Move along X
(position_move_y,pos1,50),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]],

#Siege Gun:  basically a super-shotgun with a low rate of fire. Nasty as a one-shot weapon.
#Has an extremely large arc, but is useless at long ranges.
["siege_handgonne","Siege Handgonne (bullets)",[("rrr_arquebuse",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed|itp_cant_use_on_horseback|itp_no_pick_up_from_ground|itp_cant_reload_while_moving,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,12000,weight(8.0)|spd_rtng(60)|shoot_speed(60)|thrust_damage(100,pierce)|max_ammo(1)|accuracy(75),imodbits_none,
[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 24),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Rifle Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.	
	(set_fixed_point_multiplier, 100),
	(assign, ":x_fix", -350),#-3.5 degrees
	(assign, ":z_fix", 570),#5.7 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.
	(store_random_in_range, ":z_offset", -500, 501),#2 deg. of random rotation on the Z axis
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.
	(store_random_in_range, ":x_offset", -500, 501),#2 deg. of random rotation on the X axis
	(val_add, ":x_fix", ":x_offset"),#Add offset to the X fix
	(position_rotate_x_floating, pos2, ":x_fix"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 6000, "itm_siege_handgonne", 0, "itm_shotgun_pellets", 0),
(try_end),
(play_sound,"snd_gunshot_heavy"),#Firing sound
(position_move_y,pos1,95),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
]),]], 

["firearms_end","Firearms End (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],

["cannons_start","Cannons Start (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
 #CANNONS
 #Special weapon types that use special ammunition with specific effects, governed by scripts and mission_templates
 
#The ultra-crazy Ming-dynasty multiple rocket launcher.  
["xenoargh_rocket_projector","Nest of Bees (cannon)",[("xenoargh_rocket_projector",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed|itp_cant_use_on_horseback|itp_no_pick_up_from_ground|itp_cant_reload_while_moving,itcf_shoot_musket|itcf_reload_musket,10000,weight(6.0)|spd_rtng(20)|shoot_speed(110)|thrust_damage(30,pierce)|max_ammo(1)|accuracy(70),imodbits_none,[(ti_on_weapon_attack,[
(store_trigger_param_1,reg1),#Get the attacker Agent for add_missile
(try_for_range, ":unused", 1, 12),
	(copy_position, pos2, pos1),
	
	#Shotgun script
	#Rifle Version

	#Kludge code to fix the final angles and arrive at a satisfactory result.
	#The engine doesn't appear to use Euler angles to give perfect precision...
	#Under all circumstances, so this will be off a bit if firing way up or down.	
	(set_fixed_point_multiplier, 100),
	(assign, ":x_fix", -350),#-3.5 degrees
	(assign, ":z_fix", 570),#5.7 degrees
	#The following line is our "choke" adjustment on the Z axis.
	#Unless you need a fan pattern, You'll want to keep the change...
	#The same for X and Z, but Z needs to be applied through the special fix below.
	(store_random_in_range, ":z_offset", -300, 301),#3 deg. of random rotation on the Z axis
	(val_add, ":z_fix", ":z_offset"),#Add offset to the Z fix
	#Fix for Z axis, using floating-point adjustment of the quaternian
	#Don't mess with the next three lines...
	#Unless you really know what's going on with the math :-)
	(position_rotate_x_floating, pos2, 9000), # change axis by rotating 90 degrees
	(position_rotate_y_floating, pos2, ":z_fix"), # rotate z floating
	(position_rotate_x_floating, pos2, -9000), # change axis back
	#Choke adjustment for the X axis.  
	#Should match the choke adjustment for the Z in most cases.
	(store_random_in_range, ":x_offset", -300, 301),#3 deg. of random rotation on the X axis
	(val_add, ":x_fix", ":x_offset"),#Add offset to the X fix
	(position_rotate_x_floating, pos2, ":x_fix"), # Final adjustment of X
	
	(add_missile, reg1, pos2, 11000, "itm_xenoargh_rocket_projector", 0,"itm_explosive_rocket_arrows", 0),
(try_end),
(position_move_y,pos1,100),#Move out along Y
(particle_system_burst,"psys_pistol_smoke",pos1,15),#Smoke
(particle_system_burst,"psys_gun_sparks",pos1,15),#Flare
(play_sound,"snd_explosive_arrow_launch"),
])]],

#Grenade Mortar- long-distance grenade delivery. 
 ["handgonne","Grenade Mortar (cannon)",[("rrr_handgonne",0)],itp_type_crossbow|itp_merchandise|itp_primary|itp_two_handed|itp_cant_use_on_horseback|itp_no_pick_up_from_ground|itp_cant_reload_while_moving,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,10000,weight(6.0)|spd_rtng(60)|shoot_speed(80)|thrust_damage(50,pierce)|max_ammo(1)|accuracy(60),imodbits_none,[(ti_on_weapon_attack,[(play_sound,"snd_gunshot_heavy"),(position_move_x,pos1,27),(position_move_y,pos1,65),(particle_system_burst,"psys_pistol_smoke",pos1,15),])]],
 
#Aleph's cannon; has a bunch of special kludges to adjust for fact...
#... that AI doesn't use missile weapons correctly on large-scale Agents.
["aleph_cannon","Aleph Cannon",[("xeno_bullet",0)],itp_type_crossbow|itp_primary|itp_two_handed|itp_cant_use_on_horseback|itp_no_pick_up_from_ground,itcf_shoot_musket|itcf_reload_musket|itcf_carry_spear,12000,weight(255.0)|spd_rtng(100)|shoot_speed(110)|thrust_damage(100,pierce)|max_ammo(1)|accuracy(105),imodbits_none,[(ti_on_weapon_attack,[
  
(store_trigger_param_1,reg4),#Get the attacker Agent for add_missile
(copy_position, pos63, pos1),
(agent_get_position, pos61, reg4),
(position_move_z,pos61,100,0),#Move up along Z
(position_move_y,pos61,50,0),#Move out along Y
(position_move_x,pos61,10,0),#Move right along X
(position_copy_rotation, pos61, pos63),
(particle_system_burst,"psys_pistol_smoke",pos61,45),#Smoke
(particle_system_burst,"psys_gun_sparks",pos61,45),#Flare
(play_sound,"snd_gunshot_heavy"),
(get_player_agent_no,":player_agent"),
(try_begin),
	(neq, ":player_agent", reg4),
	(agent_ai_get_behavior_target, reg5, reg4),
	(neq, reg5, reg4),
	(agent_is_alive, reg5),
	(agent_is_active, reg5),
	(agent_get_position, pos62, reg5),
	(store_random_in_range, ":x_move", -1000, 1000),
	(position_move_x, pos62, ":x_move", 0),
	(store_random_in_range, ":y_move", -1000, 1000),
	(position_move_y, pos62, ":y_move",0),
	(store_random_in_range, ":z_move", -1000, 1000),
	(position_move_z, pos62, ":z_move",0),

	(position_get_distance_to_ground_level, ":distance", pos62),
	(try_begin),
		(lt, ":distance", 0),
		(position_set_z_to_ground_level, pos62),
		(position_move_z, pos62, 200),
	(try_end),		
	(play_sound_at_position, "snd_explosive_ammo_explode", pos62),
	
	(try_for_range, ":unused", 1, 34),
		(copy_position, pos3, pos62),
		(store_random_in_range, ":x_move", -500, 500),
		(position_move_x, pos3, ":x_move", 0),
		(store_random_in_range, ":y_move", -500, 500),
		(position_move_y, pos3, ":y_move",0),
		(store_random_in_range, ":z_move", -500, 500),
		(position_move_z, pos3, ":z_move",0),
		
		(position_get_distance_to_ground_level, ":distance", pos3),
		(try_begin),
			(lt, ":distance", 0),
			(position_set_z_to_ground_level, pos3),
			(position_move_z, pos3, 200),
		(try_end),	

		(try_for_range, ":unused", 1, 20),
			(store_random_in_range, ":x_offset", 1, 361),#Random Rotation of X
			(position_rotate_x, pos3, ":x_offset"),	
			(store_random_in_range, ":y_offset", 1, 361),#Random Rotation of Y
			(position_rotate_y, pos3, ":y_offset"),	
			(store_random_in_range, ":z_offset", 1, 361),#Random Rotation of Z
			(position_rotate_z, pos3, ":z_offset"),		
			(store_random_in_range, ":speed", 1500, 2500),
			(add_missile, reg4, pos3, ":speed", "itm_llew2_matchlock_1", 0, "itm_grenade_fragments", 0),
		(try_end),	
		(particle_system_burst,"psys_explosion_dirt",pos3,200),			
		(particle_system_burst,"psys_explosive_explosion_sparks_a",pos3,35),
		(particle_system_burst,"psys_explosive_explosion_sparks_b",pos3,35),
	(try_end),	
	
(else_try),
	(copy_position, pos2, pos61),
	(add_missile, reg4, pos2, 11000, "itm_jezzail", 0,"itm_explosive_cannon_rounds", 0),
(try_end),
  ])]],
  
  ["cannons_end","Cannons End (placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
 
 ["missile_weapons_end","Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
 
 ["weapons_end","Weapons End (Placeholder)",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,300,weight(3)|abundance(1)|weapon_length(95)|thrust_damage(1,pierce)|max_ammo(50),imodbits_none],
 
 ["torch","Torch",[("club",0)],itp_type_one_handed_wpn|itp_primary,itc_scimitar,100,weight(2.5)|spd_rtng(95)|weapon_length(95)|swing_damage(11,blunt)|thrust_damage(0,pierce),imodbits_none,
 [(ti_on_init_item,[(set_position_delta,0,60,0),(particle_system_add_new,"psys_torch_fire"),(particle_system_add_new,"psys_torch_smoke"),(set_current_color,150,130,70),(add_point_light,10,30),
])]],

["lyre","Lyre",[("lyre",0)],itp_type_shield|itp_wooden_parry|itp_civilian,itcf_carry_bow_back,100,weight(2.5)|hit_points(480)|body_armor(1)|spd_rtng(82)|weapon_length(90),0],
["lute","Lute",[("lute",0)],itp_type_shield|itp_wooden_parry|itp_civilian,itcf_carry_bow_back,100,weight(2.5)|hit_points(480)|body_armor(1)|spd_rtng(82)|weapon_length(90),0],














["black_hood","Black Hood",[("hood_black",0)],itp_type_head_armor|itp_merchandise,0,193,weight(2)|abundance(100)|head_armor(18)|body_armor(0)|leg_armor(0),imodbits_none],

["keys","Ring of Keys",[("throwing_axe_a",0)],itp_type_one_handed_wpn|itp_primary|itp_bonus_against_shield,itc_scimitar,
240,weight(5)|spd_rtng(98)|swing_damage(29,cut)|max_ammo(5)|weapon_length(53),imodbits_none],

["bride_dress","Bride Dress",[("bride_dress",0)],itp_type_body_armor|itp_covers_legs|itp_civilian,0,500,weight(3)|abundance(100)|head_armor(0)|body_armor(10)|leg_armor(10)|difficulty(0),imodbits_none],
["bride_crown","Crown of Flowers",[("bride_crown",0)],itp_type_head_armor|itp_doesnt_cover_hair|itp_civilian|itp_attach_armature,0,1,weight(0.5)|abundance(100)|head_armor(10)|body_armor(0)|leg_armor(0)|difficulty(0),imodbits_none],
["bride_shoes","Bride Shoes",[("bride_shoes",0)],itp_type_foot_armor|itp_civilian|itp_attach_armature,0,
 30,weight(1)|abundance(100)|head_armor(0)|body_armor(0)|leg_armor(8)|difficulty(0),imodbits_none],

["practice_bow_2","Practice Bow",[("hunting_bow",0),("hunting_bow_carry",ixmesh_carry)],itp_type_bow|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,1,weight(1.5)|spd_rtng(120)|shoot_speed(60)|thrust_damage(30,blunt),imodbits_none],
["practice_arrows_2","Practice Arrows",[("arena_arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows,itcf_carry_quiver_back,1,weight(1.5)|weapon_length(95)|max_ammo(80),imodbits_none],

 ["cheat_siege_handgonne","CHEATER'S Siege Handgonne (cannon)",[("rrr_arquebuse",0)],itp_type_musket|itp_primary|itp_two_handed,itcf_shoot_musket|itcf_carry_spear,1,weight(8.0)|spd_rtng(700)|shoot_speed(160)|thrust_damage(100,pierce)|max_ammo(200)|accuracy(105),imodbits_none,
 [(ti_on_weapon_attack,[(play_sound,"snd_gunshot_heavy"),(position_move_x,pos1,27),(position_move_y,pos1,65),(particle_system_burst,"psys_pistol_smoke",pos1,15),])]],

["ammo_supply","Ammunition Supplies",[("ammo_barrel",0)],itp_merchandise|itp_type_goods,0,1000,weight(20)|abundance(600),imodbits_none],
 
["cheat_explosive_cannon_rounds","CHEATER'S Explosive Cannon Round",[("xeno_bullet",0),("xeno_cannonball_explosive",ixmesh_flying_ammo),("bolt_bag_c",ixmesh_carry),("xeno_cannonball_explosive",ixmesh_inventory)],itp_type_bullets|itp_default_ammo|itp_can_penetrate_shield|itp_default_ammo,itcf_carry_quiver_right_vertical,1,weight(3)|abundance(50)|weapon_length(150)|thrust_damage(100,pierce)|max_ammo(1000),imodbits_none,
   [
 (ti_on_missile_hit,
      [
          #pos1 - Missile hit position
         #param_1 - Shooter agent
          (try_begin),
             (store_trigger_param_1,":shooter"),
			 (copy_position, pos63, pos1),
             (particle_system_burst,"psys_explosive_explosion_sparks_a",pos63,35),
			 (particle_system_burst,"psys_explosive_explosion_sparks_b",pos63,35),
			 (particle_system_burst,"psys_explosive_explosion_smoke",pos63,100),	
			(spawn_scene_prop,"spr_explosive_sound_maker",63),			 
             (try_for_agents,":agent"),
                (agent_get_position,pos62,":agent"),
                (get_distance_between_positions,":dist",pos63,pos62),
                (try_begin),
				   (lt,":dist",1000),
					(neg|agent_is_ally,":agent"),
					(agent_is_active,":agent"),
					(agent_is_alive,":agent"),
					(neq,":agent",":shooter"),				   
					(agent_set_hit_points,":agent",-1,1),
					(agent_deliver_damage_to_agent,":shooter",":agent"),
                (try_end),
             (try_end),
          (try_end),
		  ]),
]],

["cheat_latticed_flatbow","CHEATER'S Auto-bow",[("amade_latticed_flatbow",0),("amade_latticed_flatbow_carry",ixmesh_carry)],itp_type_bow|itp_primary|itp_two_handed,itcf_shoot_bow|itcf_carry_bow_back,1,weight(1.5)|spd_rtng(600)|shoot_speed(110)|thrust_damage(400,pierce)|accuracy(107),imodbits_none],

 ["cheat_arrows","CHEATER'S Arrows",[("arrow",0),("flying_missile",ixmesh_flying_ammo),("quiver",ixmesh_carry)],itp_type_arrows|itp_default_ammo,itcf_carry_quiver_back,1,weight(3)|abundance(160)|weapon_length(95)|thrust_damage(400,pierce)|max_ammo(1000),imodbits_none],
 
 ["cheater_horse","Dishonorable Templar's Steed",[("warhorse_chain_barding_templar",0)],itp_type_horse,0,70000,abundance(0)|hit_points(550)|body_armor(300)|difficulty(4)|horse_speed(150)|horse_maneuver(65)|horse_charge(115)|horse_scale(110),imodbits_none],


["items_end","Items End",[("shield_round_a",0)],0,0,1,0,0],
]
