import random

from header_common import *
from header_items import *
from header_troops import *
from header_skills import *
from ID_factions import *
from ID_items import *
from ID_scenes import *

####################################################################################################################
#Each troop contains the following fields:
#1) Troop id (string): used for referencing troops in other files. The prefix trp_ is automatically added before each troop-id .
#2) Toop name (string).
#3) Plural troop name (string).
#4) Troop flags (int). See header_troops.py for a list of available flags
#5) Scene (int) (only applicable to heroes) For example: scn_reyvadin_castle|entry(1) puts troop in reyvadin castle's first entry point
#6) Reserved (int). Put constant "reserved" or 0.
#7) Faction (int)
#8) Inventory (list): Must be a list of items
#9) Attributes (int): Example usage:
#str_6|agi_6|int_4|cha_5|level(5)
# 10) Weapon proficiencies (int): Example usage:
#wp_one_handed(55)|wp_two_handed(90)|wp_polearm(36)|wp_archery(80)|wp_crossbow(24)|wp_throwing(45)
# The function wp(x) will create random weapon proficiencies close to value x.
# To make an expert archer with other weapon proficiencies close to 60 you can use something like:
#wp_archery(160) | wp(60)
# 11) Skills (int): See header_skills.py to see a list of skills. Example:
#knows_toughness_3|knows_reflexes_2|knows_riding_2
# 12) Face code (int): You can obtain the face code by pressing ctrl+E in face generator screen
# 13) Face code (int)(2) (only applicable to regular troops,can be omitted for heroes):
# The game will create random faces between Face code 1 and face code 2 for generated troops
# 14) Troop image (string): If this variable is set,the troop will use an image rather than its 3D visual during the conversations
####################################################################################################################

# Some constant and function declarations to be used below...

def wp(x):
	n = 0
	r = 10 + int(x / 10)
#n |= wp_one_handed(x + random.randrange(r))
#n |= wp_two_handed(x + random.randrange(r))
#n |= wp_polearm(x + random.randrange(r))
#n |= wp_archery(x + random.randrange(r))
#n |= wp_crossbow(x + random.randrange(r))
#n |= wp_throwing(x + random.randrange(r))
	n |= wp_one_handed(x)
	n |= wp_two_handed(x)
	n |= wp_polearm(x)
	n |= wp_archery(x)
	n |= wp_crossbow(x)
	n |= wp_throwing(x)
	n |= wp_firearm(x)
	return n

def wpe(m,a,c,t):
	 n = 0
	 n |= wp_one_handed(m)
	 n |= wp_two_handed(m)
	 n |= wp_polearm(m)
	 n |= wp_archery(a)
	 n |= wp_crossbow(c)
	 n |= wp_throwing(t)
	 return n

def wpex(o,w,p,a,c,t):
	 n = 0
	 n |= wp_one_handed(o)
	 n |= wp_two_handed(w)
	 n |= wp_polearm(p)
	 n |= wp_archery(a)
	 n |= wp_crossbow(c)
	 n |= wp_throwing(t)
	 return n
 
def wp_melee(x):
	n = 0
	r = 10 + int(x / 10)
	#n |= wp_one_handed(x + random.randrange(r))
	#n |= wp_two_handed(x + random.randrange(r))
	#n |= wp_polearm(x + random.randrange(r))
	n |= wp_one_handed(x + 20)
	n |= wp_two_handed(x)
	n |= wp_polearm(x + 10)
	return n

#Skills
knows_common = knows_riding_1|knows_trade_2|knows_inventory_management_2|knows_prisoner_management_1|knows_leadership_1

#Default path,medium
def_attrib = str_7 | agi_5 | int_4 | cha_4
superior_attrib = str_12|agi_13|int_10| cha_10
elite_attrib = str_16|agi_16|int_16|cha_10

#Ranged path,agi biased
levy_ranged_attrib = str_7|agi_10|int_8|cha_7
recruit_ranged_attrib = str_10|agi_14|int_8|cha_7
regular_ranged_attrib = str_15|agi_18|int_8|cha_7
elite_ranged_attrib = str_18|agi_24|int_8|cha_7
expert_ranged_attrib = str_24|agi_30|int_12|cha_10

#HTH path,strength biased
levy_hth_attrib = str_10|agi_7|int_8|cha_7
recruit_hth_attrib = str_14|agi_10|int_8|cha_7
regular_hth_attrib = str_18|agi_15|int_8|cha_7
elite_hth_attrib = str_24|agi_18|int_8|cha_7
expert_hth_attrib = str_30|agi_24|int_12|cha_10

#Special Beserkir stats- crazy stats for AGI to allow for extreme speed,please don't put this on other troops unless you know what you're doing, heh
beserkir_attrib = str_30|agi_200|int_8|cha_10

#Special Assassin stats- huge STR, higher-than-usual AGI.
assassin_attrib = str_100|agi_50|int_8|cha_10

#Special Afrizarian stats- crazy stats for AGI(not quite as extreme as Bezerkirs, but still boosted).
afrizar_attrib = str_24|agi_100|int_8|cha_10

#Flesh Golem!!!
golem_attrib = str_200|agi_1|int_1|cha_1

def_attrib_multiplayer = str_14 | agi_14 | int_4 | cha_4

knows_lord_1 = knows_riding_3|knows_trade_2|knows_inventory_management_2|knows_prisoner_management_4|knows_leadership_7

knows_warrior_npc = knows_weapon_master_2|knows_riding_2|knows_inventory_management_2
knows_merchant_npc = knows_riding_2|knows_trade_3|knows_inventory_management_3 #knows persuasion
knows_tracker_npc = knows_weapon_master_1|knows_spotting_2|knows_pathfinding_2|knows_tracking_2|knows_inventory_management_2

lord_attrib = str_20|agi_20|int_20|cha_20|level(38)

knight_attrib_1 = str_21|agi_14|int_8|cha_16|level(22)
knight_attrib_2 = str_22|agi_16|int_10|cha_18|level(26)
knight_attrib_3 = str_25|agi_17|int_12|cha_20|level(30)
knight_attrib_4 = str_27|agi_19|int_13|cha_22|level(35)
knight_attrib_5 = str_30|agi_20|int_15|cha_25|level(41)
knight_skills_1 = knows_riding_3|knows_toughness_7|knows_reflexes_7|knows_prisoner_management_1|knows_leadership_3|knows_heroism_1|knows_combat_training_7|knows_defensive_tactics_1
knight_skills_2 = knows_riding_4|knows_toughness_7|knows_reflexes_7|knows_prisoner_management_2|knows_leadership_5|knows_heroism_1|knows_combat_training_7|knows_defensive_tactics_3
knight_skills_3 = knows_riding_5|knows_toughness_8|knows_reflexes_8|knows_prisoner_management_2|knows_leadership_6|knows_heroism_2|knows_combat_training_8|knows_defensive_tactics_2
knight_skills_4 = knows_riding_6|knows_toughness_9|knows_reflexes_9|knows_prisoner_management_3|knows_leadership_7|knows_heroism_2|knows_combat_training_9|knows_defensive_tactics_4
knight_skills_5 = knows_riding_7|knows_toughness_10|knows_reflexes_10|knows_prisoner_management_3|knows_leadership_9|knows_heroism_3|knows_combat_training_10|knows_defensive_tactics_6

#These face codes are generated by the in-game face generator.
#Enable edit mode and press ctrl+E in face generator screen to obtain face codes.


reserved = 0

no_scene = 0

swadian_face_younger_1 = 0x00000000000ca4d414c368a312c9391b00000000001ed3640000000000000000
swadian_face_young_1 = 0x000000000710c54326ab4e492c50950c00000000001e395b0000000000000000
swadian_face_middle_1= 0x00000006190084cf18f50f58642ee45500000000001d524c0000000000000000
swadian_face_old_1 = 0x0000000c9700c208485e6d46dcb0257600000000001cd7190000000000000000
swadian_face_older_1 = 0x0000000fcf008211579b2dd6acaaa66400000000001ed2e40000000000000000

swadian_face_younger_2 = 0x000000003a103244548c2ad6a9abcd2300000000001f5b540000000000000000
swadian_face_young_2 = 0x000000000f08a14638edb652d936440a00000000001e44ac0000000000000000
swadian_face_middle_2= 0x000000042200b311395c89bd5bb24a5b00000000001cace60000000000000000
swadian_face_old_2 = 0x0000000add0cc2c456f56e56a28ec55100000000001d28d30000000000000000
swadian_face_older_2 = 0x0000000e5f0c430412eb6db4cd91ca9e00000000001d62d50000000000000000

vaegir_face_younger_1 = 0x000000003910b50832ec8d2aec8f5c1d00000000001e4b640000000000000000
vaegir_face_young_1 = 0x00000001e110b18f44e39a428a4ac75c00000000001e17bb0000000000000000
vaegir_face_middle_1= 0x00000005280c630b49e7934b24b5b92300000000001eb7930000000000000000
vaegir_face_old_1 = 0x0000000a2808b2521b2135a6ddd0c57300000000001e0c5a0000000000000000
vaegir_face_older_1 = 0x0000000e5f0c430412eb6db4cd91ca9e00000000001d62d50000000000000000

vaegir_face_younger_2 = 0x000000003910b50832ec8d2aec8f5c1d00000000001e4b640000000000000000
vaegir_face_young_2 = 0x00000001e110b18f44e39a428a4ac75c00000000001e17bb0000000000000000
vaegir_face_middle_2= 0x00000005280c630b49e7934b24b5b92300000000001eb7930000000000000000
vaegir_face_old_2 = 0x0000000a2808b2521b2135a6ddd0c57300000000001e0c5a0000000000000000
vaegir_face_older_2 = 0x0000000e5f0c430412eb6db4cd91ca9e00000000001d62d50000000000000000

khergit_face_younger_1 = 0x000000000a00954a32db0cb7546caadb00000000001e17040000000000000000
khergit_face_young_1 = 0x00000001fa0083cc459c556d1d96a2fd00000000001dc8cb0000000000000000
khergit_face_middle_1= 0x00000006b504424d36b397653595c52200000000001e47210000000000000000
khergit_face_old_1 = 0x0000000bbd0c639417b4acccec7340ac00000000001dcb610000000000000000
khergit_face_older_1 = 0x0000000eff0083cd266baf692545499b00000000001e59eb0000000000000000

khergit_face_younger_2 = 0x000000000a00954a32db0cb7546caadb00000000001e17040000000000000000
khergit_face_young_2 = 0x00000001fa0083cc459c556d1d96a2fd00000000001dc8cb0000000000000000
khergit_face_middle_2= 0x00000006b504424d36b397653595c52200000000001e47210000000000000000
khergit_face_old_2 = 0x0000000bbd0c639417b4acccec7340ac00000000001dcb610000000000000000
khergit_face_older_2 = 0x0000000eff0083cd266baf692545499b00000000001e59eb0000000000000000

nord_face_younger_1 = 0x000000000008a151656d6eb6ad92e32400000000001ec8d50000000000000000
nord_face_young_1 = 0x00000001da04b54518a126c6f5564cac00000000001d58d30000000000000000
nord_face_middle_1= 0x000000064e10c34f489b6a94d52d9a4200000000001d3cf10000000000000000
nord_face_old_1 = 0x0000000add0cc2c456f56e56a28ec55100000000001d28d30000000000000000
nord_face_older_1 = 0x0000000eec0cb349371a6da90c6ccd1400000000001dda530000000000000000

nord_face_younger_2 = 0x000000000008a151656d6eb6ad92e32400000000001ec8d50000000000000000
nord_face_young_2 = 0x00000001da04b54518a126c6f5564cac00000000001d58d30000000000000000
nord_face_middle_2= 0x000000064e10c34f489b6a94d52d9a4200000000001d3cf10000000000000000
nord_face_old_2 = 0x0000000add0cc2c456f56e56a28ec55100000000001d28d30000000000000000
nord_face_older_2 = 0x0000000eec0cb349371a6da90c6ccd1400000000001dda530000000000000000

rhodok_face_younger_1 = 0x000000003f00818827736d26db8a64eb00000000001ce6f20000000000000000
rhodok_face_young_1 = 0x000000023f08c48630e3c52254d2299d00000000001c44dc0000000000000000
rhodok_face_middle_1= 0x00000007351044c2376998c913b2675e00000000001dc81b0000000000000000
rhodok_face_old_1 = 0x0000000cb908c0521a9951b992b4a29200000000001e87a20000000000000000
rhodok_face_older_1 = 0x0000000ecf10809438e392a6656ee73b00000000001e571c0000000000000000

rhodok_face_younger_2 = 0x000000003f00818827736d26db8a64eb00000000001ce6f20000000000000000
rhodok_face_young_2 = 0x000000023f08c48630e3c52254d2299d00000000001c44dc0000000000000000
rhodok_face_middle_2= 0x00000007351044c2376998c913b2675e00000000001dc81b0000000000000000
rhodok_face_old_2 = 0x0000000cb908c0521a9951b992b4a29200000000001e87a20000000000000000
rhodok_face_older_2 = 0x0000000ecf10809438e392a6656ee73b00000000001e571c0000000000000000

sarranid_face_younger_1 = 0x0000000023105354530a69c69d79232c00000000001dab1e0000000000000000
sarranid_face_young_1 = 0x00000001af0465d1572459492134b32600000000001e769c0000000000000000
sarranid_face_middle_1= 0x000000083f10c5c578f569665b46496c00000000001e14b30000000000000000
sarranid_face_old_1 = 0x0000000d3e00951227946cc765556ae300000000001ee5930000000000000000
sarranid_face_older_1 = 0x0000000e7b082446496bae435c9246f200000000001d26db0000000000000000

sarranid_face_younger_2 = 0x0000000023105354530a69c69d79232c00000000001dab1e0000000000000000
sarranid_face_young_2 = 0x00000001af0465d1572459492134b32600000000001e769c0000000000000000
sarranid_face_middle_2= 0x000000083f10c5c578f569665b46496c00000000001e14b30000000000000000
sarranid_face_old_2 = 0x0000000d3e00951227946cc765556ae300000000001ee5930000000000000000
sarranid_face_older_2 = 0x0000000e7b082446496bae435c9246f200000000001d26db0000000000000000

man_face_younger_1 = 0x0000000000000001124000000020000000000000001c00800000000000000000
man_face_young_1 = 0x0000000400000001124000000020000000000000001c00800000000000000000
man_face_middle_1= 0x0000000800000001124000000020000000000000001c00800000000000000000
man_face_old_1 = 0x0000000d00000001124000000020000000000000001c00800000000000000000
man_face_older_1 = 0x0000000fc0000001124000000020000000000000001c00800000000000000000

man_face_younger_2 = 0x000000003f0052064deeffffffffffff00000000001efff90000000000000000
man_face_young_2 = 0x00000003bf0052064deeffffffffffff00000000001efff90000000000000000
man_face_middle_2= 0x00000007bf0052064deeffffffffffff00000000001efff90000000000000000
man_face_old_2 = 0x0000000bff0052064deeffffffffffff00000000001efff90000000000000000
man_face_older_2 = 0x0000000fff0052064deeffffffffffff00000000001efff90000000000000000

face_afrizar_young_1 = 0x000000003f00000036db6db6db6db6db00000000001db6db0000000000000000
face_afrizar_young_2 = 0x000000003f0820c03ca192488849b88d00000000001d8d250000000000000000
face_afrizar_old_1 = 0x0000000cb80803525b6671329b65e3b600000000001d5b930000000000000000
face_afrizar_old_2 = 0x0000000cab002000476169a4e12c8f1400000000001d4c4b0000000000000000

afrizar_woman_face_young_1 = 0x00000001bb04500416996f747378b6db00000000001ed6dc0000000000000000
afrizar_woman_face_young_2 = 0x00000000231040053c1872d479bf58cc00000000001f3ad20000000000000000
afrizar_woman_face_young_3 = 0x00000001b810f00658d27214eb71469b00000000001de4e40000000000000000
afrizar_woman_face_young_4 = 0x00000001870c600234cf935ad94d9b5200000000001e3a8b0000000000000000

afrizar_woman_face_old_1 = 0x0000000e381040023c1872d479bf58cc00000000001f3ad20000000000000000
afrizar_woman_face_old_2 = 0x0000000d3f0c500426e5a9453558976300000000001d34e40000000000000000

merchant_face_1= man_face_young_1
merchant_face_2= man_face_older_2

woman_face_1= 0x000000003d000001399291d69a6cbadb00000000001dd4a20000000000000000
woman_face_2= 0x00000005990300074b5c0e2aa38d68d200000000001da7340000000000000000

european_woman_face_1	= 0x00000000000180014b5c0e2aa38d68d200000000001da7340000000000000000
european_woman_face_2	= 0x00000004bf0eb0073aa151d8dc792a9400000000001ee8e30000000000000000
european_woman_face_3	= 0x0000000000098001492172a6a46e159a00000000001e49620000000000000000
european_woman_face_4	= 0x000000053f0ab00758a6d938ca91c96400000000001e3a540000000000000000
european_woman_face_5 = 0x00000001800a70041aed5302f2aab66200000000000cd7540000000000000000
european_woman_face_6 = 0x00000001a20dc00244e345a45bd675130000000000063cbe0000000000000000
european_woman_face_7 = 0x00000001bf0e40066acd8de4138e496a00000000001dda9a0000000000000000
european_woman_face_8 = 0x000000019201a0075b5792a32430b71500000000001dd0db0000000000000000
european_woman_face_9 = 0x000000019d0e50063b112644f368d36300000000001db6e50000000000000000
european_woman_face_10 = 0x000000019405800246aa9119645248d200000000001e256e0000000000000000
european_woman_face_11 = 0x00000001bb06100446aa9119645248d200000000001e256e0000000000000000
european_woman_face_12 = 0x000000018806b00238a94ddd9bcdb8e200000000001d246d0000000000000000

rhodok_woman_face_1 = 0x00000001bf06900238a94ddd9bcdb8e200000000001d246d0000000000000000
rhodok_woman_face_2 = 0x00000001bd0e8005266251d54ad5269b00000000001f44e60000000000000000
rhodok_woman_face_3 = 0x00000001ae11a0074b4c925acf72986a00000000001ca7120000000000000000
rhodok_woman_face_4 = 0x00000001bc11b00336886ab353c626e300000000001e94a30000000000000000
rhodok_woman_face_5 = 0x00000001af114001375d6a270c6e4cde00000000001ed5090000000000000000
rhodok_woman_face_6 = 0x00000001be0e8002371c76b6a36ab79a00000000001e1c610000000000000000
rhodok_woman_face_7 = 0x00000001ba0250065b25934b2c45cadc00000000001e1ba40000000000000000
rhodok_woman_face_8 = 0x00000001be05a004299a2cb7b255ab2300000000001e348e0000000000000000
rhodok_woman_face_9 = 0x00000001bd0d800528944dd856b54ab600000000001d36630000000000000000
rhodok_woman_face_10 = 0x00000001ae0950013b9d31151c4aa89a00000000001cbb1c0000000000000000

swadian_woman_face_1 = 0x000000000001000119df65c92baa36d300000000001db31b0000000000000000
swadian_woman_face_2 = 0x0000000d7f07000745538e07924eb32300000000001c13210000000000000000

khergit_woman_face_1 = 0x000000000802c00119df65c92baa36d300000000001db31b0000000000000000
khergit_woman_face_2 = 0x0000000e7f03000719df65c92baa36d300000000001db31b0000000000000000
khergit_woman_face_3 = 0x000000019a12c002699293445ead364c00000000000e111a0000000000000000
khergit_woman_face_4 = 0x000000019612f00536db69b2938ca90a00000000001cc0940000000000000000
khergit_woman_face_5 = 0x00000001ae0ae0043b9d31151c4aa89a00000000001cbb1c0000000000000000
khergit_woman_face_6 = 0x00000001ba0f0002295b28569b954b2500000000001b26e50000000000000000
khergit_woman_face_7 = 0x00000001940f000404fd6627084624e200000000001e3b230000000000000000
khergit_woman_face_8 = 0x00000001890ef00244ec4d989c8dd67400000000001da9240000000000000000

sarranid_woman_face_1 = 0x000000000002c0014b5c0e2aa38d68d200000000001da7340000000000000000
sarranid_woman_face_2 = 0x0000000ebf00f0074b5c0e2aa38d68d200000000001da7340000000000000000
sarranid_woman_face_3 = 0x00000001a807000146aa9119645248d200000000001e256e0000000000000000


refugee_face1 = woman_face_1
refugee_face2 = woman_face_2
girl_face1= woman_face_1
girl_face2= woman_face_2

mercenary_face_1 = 0x000000000000b00236db6db6db6db6db00000000001db6db0000000000000000
mercenary_face_2 = 0x000000083c04c40f588cb5292b75ba9b00000000001ea4db0000000000000000

vaegir_face1= vaegir_face_young_1
vaegir_face2= vaegir_face_older_2

bandit_face1= man_face_young_1
bandit_face2= man_face_older_2

skeleton_face1  = 0xffffffff00000000000000000000000000000000000000000000000000000000
skeleton_face2  = 0xffffffff00000000000000000000000000000000000000000000000000000000

#NAMES:
#

tf_guarantee_all = tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield|tf_guarantee_ranged
tf_guarantee_all_wo_ranged = tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield


troops = [
["player","Player","Player",tf_mounted|tf_hero|tf_unmoveable_in_party_window,no_scene,reserved,fac_player_faction,
 [],
 str_4|agi_4|int_4|cha_4,wp(15),0,0x000000018000000136db6db6db6db6db00000000001db6db0000000000000000,0x000000018000000136db6db6db6db6db00000000001db6db0000000000000000],
["multiplayer_profile_troop_male","multiplayer_profile_troop_male","multiplayer_profile_troop_male",tf_mounted|tf_hero|tf_guarantee_all,0,0,fac_commoners,
 [itm_leather_jerkin,itm_leather_boots],
0,0,0,0x000000018000000136db6db6db6db6db00000000001db6db0000000000000000,0x000000018000000136db6db6db6db6db00000000001db6db0000000000000000],
["multiplayer_profile_troop_female","multiplayer_profile_troop_female","multiplayer_profile_troop_female",tf_mounted|tf_hero|tf_female|tf_guarantee_all,0,0,fac_commoners,
 [itm_tribal_warrior_outfit,itm_leather_boots],
0,0,0,0x000000018000000136db6db6db6db6db00000000001db6db0000000000000000,0x000000018000000136db6db6db6db6db00000000001db6db0000000000000000],
["temp_troop","Temp Troop","Temp Troop",tf_mounted|tf_hero,no_scene,reserved,fac_commoners,[],def_attrib,0,knows_common|knows_inventory_management_10,0],
##["game","Game","Game",tf_mounted|tf_hero,no_scene,reserved,fac_commoners,[],def_attrib,0,knows_common,0],
##["unarmed_troop","Unarmed Troop","Unarmed Troops",tf_mounted|tf_hero,no_scene,reserved,fac_commoners,[itm_arrows,itm_short_bow],def_attrib|str_14,0,knows_common,0],

####################################################################################################################
# Troops before this point are hardwired into the game and their order should not be changed!
####################################################################################################################
["find_item_cheat","find_item_cheat","find_item_cheat",tf_mounted|tf_hero,no_scene,reserved,fac_commoners,[],def_attrib,0,knows_common|knows_inventory_management_10,0],
["random_town_sequence","Random Town Sequence","Random Town Sequence",tf_mounted|tf_hero,no_scene,reserved,fac_commoners,[],def_attrib,0,knows_common|knows_inventory_management_10,0],
["tournament_participants","Tournament Participants","Tournament Participants",tf_mounted|tf_hero,no_scene,reserved,fac_commoners,[],def_attrib,0,knows_common|knows_inventory_management_10,0],
["tutorial_maceman","Maceman","Maceman",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_tutorial_club,itm_leather_jerkin,itm_hide_boots],
 str_6|agi_6|level(1),wp(50),knows_common,mercenary_face_1,mercenary_face_2],
["tutorial_archer","Archer","Archer",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_ranged,no_scene,reserved,fac_commoners,
 [itm_tutorial_short_bow,itm_tutorial_arrows,itm_linen_tunic,itm_hide_boots],
 str_6|agi_6|level(5),wp(100),knows_common,mercenary_face_1,mercenary_face_2],
["tutorial_swordsman","Swordsman","Swordsman",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_tutorial_sword,itm_leather_vest,itm_hide_boots],
 str_6|agi_6|level(5),wp(80),knows_common,mercenary_face_1,mercenary_face_2],

["novice_fighter","Novice Fighter","Novice Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_6|agi_6|level(5),wp(60),knows_common,mercenary_face_1,mercenary_face_2],
["regular_fighter","Regular Fighter","Regular Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_8|agi_8|level(11),wp(90),knows_common|knows_toughness_1|knows_reflexes_1|knows_riding_1,mercenary_face_1,mercenary_face_2],
["veteran_fighter","Veteran Fighter","Veteran Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,0,fac_commoners,
 [itm_hide_boots],
 str_10|agi_10|level(17),wp(110),knows_common|knows_toughness_3|knows_reflexes_2|knows_riding_2,mercenary_face_1,mercenary_face_2],
["champion_fighter","Champion Fighter","Champion Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_12|agi_11|level(22),wp(140),knows_common|knows_toughness_4|knows_reflexes_3|knows_riding_3,mercenary_face_1,mercenary_face_2],

["arena_training_fighter_1","Novice Fighter","Novice Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_6|agi_6|level(5),wp(60),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_2","Novice Fighter","Novice Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_7|agi_6|level(7),wp(70),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_3","Regular Fighter","Regular Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_8|agi_7|level(9),wp(80),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_4","Regular Fighter","Regular Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_8|agi_8|level(11),wp(90),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_5","Regular Fighter","Regular Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_9|agi_8|level(13),wp(100),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_6","Veteran Fighter","Veteran Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_10|agi_9|level(15),wp(110),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_7","Veteran Fighter","Veteran Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_10|agi_10|level(17),wp(120),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_8","Veteran Fighter","Veteran Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_11|agi_10|level(19),wp(130),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_9","Champion Fighter","Champion Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_12|agi_11|level(21),wp(140),knows_common,mercenary_face_1,mercenary_face_2],
["arena_training_fighter_10","Champion Fighter","Champion Fighters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_hide_boots],
 str_12|agi_12|level(23),wp(150),knows_common,mercenary_face_1,mercenary_face_2],

["cattle","Cattle","Cattle",0,no_scene,reserved,fac_neutral,[],def_attrib|level(1),wp(60),0,mercenary_face_1,mercenary_face_2],


#soldiers:
#This troop is the troop marked as soldiers_begin
["farmer","Farmer","Farmers",tf_mounted|tf_guarantee_armor,no_scene,reserved,fac_commoners,[itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_cleaver,itm_knife,itm_pitch_fork,itm_sickle,itm_club,itm_stones,itm_leather_cap,itm_felt_hat,itm_felt_hat,itm_linen_tunic,itm_coarse_tunic,itm_nomad_boots,itm_wrapping_boots],def_attrib|level(4),wp_melee(60)|wp_throwing(80),knows_common,man_face_middle_1,man_face_old_2],
["townsman","Townsman","Townsmen",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,no_scene,reserved,fac_commoners,[itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_stones,itm_leather_cap,itm_linen_tunic,itm_coarse_tunic,itm_leather_apron,itm_nomad_boots,itm_wrapping_boots],def_attrib|level(4),wp_melee(60)|wp_throwing(50),knows_common,mercenary_face_1,mercenary_face_2],

##########################
## MERCENARY START PATHS / HIREABLE MERCS
##########################
#All starting trees are between mercenaries_begin and mercenaries_end.Elite are past this point (non-hireable,you must upgrade to get them).
["mercenaries_begin","mercenaries_begin","mercenaries_begin",0,no_scene,reserved,fac_commoners,[], def_attrib|level(4),wp(60),knows_common,mercenary_face_1,mercenary_face_2],

#Start of standard mercenary tree.
["watchman","Watchman (recruit)","Watchmen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_ranged,no_scene,reserved,fac_commoners,[itm_bolts,itm_spiked_club,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_sword_medieval_a,itm_boar_spear,itm_hunting_crossbow,itm_light_crossbow,itm_old_green_round_shield,itm_padded_cloth,itm_leather_jerkin,itm_leather_cap,itm_padded_coif,itm_footman_helmet,itm_nomad_boots,itm_wrapping_boots],def_attrib|level(9),wp_melee(90)|wp_crossbow(100),knows_common,mercenary_face_1,mercenary_face_2],

["caravan_guard","Caravan Guard (recruit)","Caravan Guards (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_ranged,no_scene,0,fac_commoners,[itm_bolts,itm_spear,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_sword_medieval_a,itm_hunting_crossbow,itm_light_crossbow,itm_voulge,itm_old_green_round_shield,itm_leather_jerkin,itm_leather_vest,itm_hide_boots,itm_padded_coif,itm_nasal_helmet,itm_footman_helmet],recruit_ranged_attrib|level(14),wp_melee(100)|wp_crossbow(110),knows_common|knows_toughness_1|knows_combat_training_1,mercenary_face_1,mercenary_face_2],

["mercenary_skirmisher","Skirmisher (regular)","Skirmishers (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_ranged,no_scene,0,fac_commoners,[itm_bolts,itm_llew2_arquebus,itm_lui_mercenaryshort,itm_xenoargh_round_shield02_paint02,itm_wei_xiadi_rod_brigandine,itm_hide_boots,itm_nasal_helmet,itm_footman_helmet,itm_leather_gloves],regular_ranged_attrib|level(25),wp_melee(125)|wp_crossbow(140),knows_reflexes_2|knows_toughness_2|knows_combat_training_2,mercenary_face_1,mercenary_face_2],

####################### END OF MALE MERCS HERE!

#Start of standard female mercenary tree.
["mercenary_woman","Mercenary Woman (recruit)","Mercenary Women (recruit)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_ranged,no_scene,reserved,fac_commoners,[itm_bolts,itm_light_crossbow,itm_crossbow,itm_nordic_shield,itm_hide_covered_round_shield,itm_hatchet,itm_hand_axe,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_club,itm_anar_dress,itm_anar2_dress],def_attrib|level(5),wp_melee(70)|wp_crossbow(100),knows_common|knows_reflexes_1|knows_combat_training_1,european_woman_face_1,european_woman_face_2],

#Crossbow / pistolera split
["mercenary_hunter_woman","Mercenary Huntress (regular)","Mercenary Huntresses (regular)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_shield,no_scene,reserved,fac_commoners,[itm_bolts,itm_sniper_crossbow,itm_nordic_shield,itm_hide_covered_round_shield,itm_hatchet,itm_hand_axe,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_club,itm_anar_dress,itm_anar2_dress,itm_rogue_csizma],regular_ranged_attrib|level(15),wp_melee(85)|wp_crossbow(200),knows_common|knows_reflexes_1|knows_combat_training_3,european_woman_face_1,european_woman_face_2],

#HTH Females- Irregulars,light ranged and two-handed swords
["mercenary_infantry_woman","Female Irregular (recruit)","Female Irregulars (recruit)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_boots,no_scene,reserved,fac_commoners,[itm_arrows,itm_rogue_bow,itm_javelin,itm_xena_armor,itm_xena_boots],def_attrib|level(15),wp_melee(100)|wp_throwing(110),knows_common|knows_reflexes_2|knows_combat_training_1,european_woman_face_1,european_woman_face_2],

["mercenary_infantry_woman_sergeant","Female Irregular (regular)","Female Irregulars (regular)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_boots,no_scene,reserved,fac_commoners,[itm_arrows,itm_rogue_bow,itm_javelin,itm_aribeth_armor,itm_aribeth_armor_dark,itm_xena_boots],regular_hth_attrib|level(20),wp_melee(130)|wp_throwing(140),knows_common|knows_reflexes_4|knows_combat_training_2,european_woman_face_1,european_woman_face_2],

["mercenary_polearm_woman_recruit","Battle Sister (recruit)","Battle Sisters (recruit)",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_templars,[itm_xenoargh_bwanga,itm_lui_manhunter_polehammer,itm_jam_scorpion,itm_jam_voulge_b,itm_leather_gloves,itm_xenoargh_hornmask_red,itm_xena_boots,itm_xenoargh_new_female_armor_orange01,itm_heater_shield_swadian,itm_heater_shield_old,itm_heater_shield_generic,itm_heater_shield_heavy],recruit_ranged_attrib|level(15),wp_melee(125)|wp_throwing(90),knows_toughness_3|knows_combat_training_1|knows_reflexes_1,khergit_woman_face_1,khergit_woman_face_8],

["mercenary_polearm_woman_regular","Battle Sister (regular)","Battle Sisters (regular)",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_templars,[itm_xenoargh_bwanga,itm_xenoargh_bwanga,itm_lui_manhunter_polehammer,itm_jam_scorpion,itm_jam_voulge_b,itm_scale_gauntlets,itm_xenoargh_hornmask_green,itm_xena_boots,itm_xenoargh_new_female_armor_green02,itm_xenoargh_new_female_armor_green03,itm_heater_shield_swadian,itm_heater_shield_old,itm_heater_shield_generic,itm_heater_shield_heavy],regular_ranged_attrib|level(20),wp_melee(145)|wp_throwing(100),knows_toughness_4|knows_combat_training_2|knows_reflexes_1,khergit_woman_face_1,khergit_woman_face_8],

["mercenaries_end","mercenaries_end","mercenaries_end",0,no_scene,reserved,fac_commoners,[], def_attrib|level(4),wp(60),knows_common,mercenary_face_1,mercenary_face_2],

##########################
## MERCENARY UPGRADE PATHS- ELITE TROOPS
##########################

["mercenary_elite_begin","mercenary_elite_begin","mercenary_elite_begin",0,no_scene,reserved,fac_commoners,[], def_attrib|level(4),wp(60),knows_common,mercenary_face_1,mercenary_face_2],
#Standard Mercenary HTH Path
["mercenary_swordsman","Merc. Swordsman (regular)","Merc. Swordsmen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield,no_scene,reserved,fac_commoners,[itm_lui_bastardsword,itm_lui_bastardswordb,itm_lui_bastardswordc,itm_lui_bastardswordd,itm_lui_bastardsworde,itm_lui_bastardswordknight,itm_heater_shield_generic,itm_mail_hauberk,itm_haubergeon,itm_mail_chausses,itm_hide_boots,itm_kettle_hat,itm_mail_coif,itm_flat_topped_helmet,itm_helmet_with_neckguard],regular_hth_attrib|level(20),wp_melee(130),knows_common|knows_toughness_3|knows_combat_training_3,mercenary_face_1,mercenary_face_2],

["hired_blade","Hired Blade (elite)","Hired Blades (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield,no_scene,reserved,fac_commoners,[itm_lui_bastardsword,itm_lui_bastardswordb,itm_lui_bastardswordc,itm_lui_bastardswordd,itm_lui_bastardsworde,itm_lui_bastardswordknight,itm_xenoargh_shield_heater01,itm_light_mail_and_plate,itm_narf_corrazina_grey,itm_mail_and_plate,itm_mail_chausses,itm_iron_greaves,itm_narf_visored_sallet,itm_narf_visored_sallet_coif,itm_great_helmet,itm_bascinet,itm_narf_milanese_gauntlets],elite_hth_attrib|level(30),wp_melee(175),knows_common|knows_reflexes_2|knows_toughness_7|knows_combat_training_5|knows_defensive_tactics_1,mercenary_face_1,mercenary_face_2],
 
 #Standard Mercenary Ranged Path
["mercenary_crossbowman","Merc. Crossbowman (elite)","Merc. Crossbowmen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_shield,no_scene,reserved,fac_commoners,[itm_bolts,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_sword_medieval_a,itm_boar_spear,itm_crossbow,itm_old_board_shield01,itm_cuir_bouilli,itm_leather_cap,itm_padded_coif,itm_footman_helmet,itm_nomad_boots,itm_wrapping_boots],regular_ranged_attrib|level(19),wp_melee(120)|wp_crossbow(200),knows_common|knows_toughness_3|knows_combat_training_4,mercenary_face_1,mercenary_face_2],

["mercenary_grenadier","Merc. Grenadier (expert)","Merc. Grenadiers (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_helmet,no_scene,reserved,fac_commoners,[itm_bolts,itm_arquebus,itm_xenoargh_troop_grenade,itm_boar_spear,itm_banded_armor,itm_dejawolf_byzantion,itm_dejawolf_byzantion2,itm_nomad_boots,itm_wrapping_boots,itm_leather_gloves],expert_ranged_attrib|level(35),wp_melee(120)|wp_crossbow(240)|wp_throwing(200),knows_common|knows_toughness_5|knows_combat_training_5|knows_defensive_tactics_1,mercenary_face_1,mercenary_face_2],

#Cannoneers.  Somewhat crappy assault / defensive troops...
#...but they're armed with Mortars / Siege Handgonnes.
["mercenary_cannoneer","Merc. Cannoneer (elite)","Merc. Cannoneers (elite)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,no_scene,reserved,fac_commoners,[itm_universal_cannon_ammo,itm_bolts,itm_siege_handgonne,itm_handgonne,itm_voulge,itm_haubergeon,itm_hide_boots,itm_dejawolf_kettlehat1,itm_dejawolf_kettlehat2,itm_dejawolf_kettlehat3,itm_dejawolf_kettlehat4,itm_leather_gloves],regular_ranged_attrib|level(50),wp_melee(100)|wp_crossbow(180),knows_common|knows_toughness_4,mercenary_face_1,mercenary_face_2],

#Matchlock Men.  Upgrade for Skirmishers.  Good expert shooters, with pikes to fend off horsemen.
["mercenary_matchlocks","Merc. Matchlock Man (expert)","Merc. Matchlock Men (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_gloves|tf_guarantee_helmet,no_scene,reserved,fac_commoners,[itm_bolts,itm_llew2_matchlock_2,itm_lui_phalanxpike,itm_wei_xiadi_rod_coat_of_plates,itm_dejawolf_kettlehat1,itm_narf_rus_shoes,itm_narf_mail_gauntlets],expert_ranged_attrib|level(30),wp_melee(120)|wp_crossbow(280),knows_common|knows_toughness_4|knows_combat_training_7|knows_defensive_tactics_1,mercenary_face_1,mercenary_face_2],

#Standard Mercenary Cavalry Path - medium cavalry,dragoons
["mercenary_horseman","Merc. Horseman (regular)","Merc. Horsemen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,no_scene,reserved,fac_commoners,[itm_lance,itm_bastard_sword_a,itm_sword_medieval_b,itm_heater_shield_generic,itm_mail_shirt,itm_haubergeon,itm_hide_boots,itm_norman_helmet,itm_mail_coif,itm_helmet_with_neckguard,itm_saddle_horse,itm_courser,itm_hunter],regular_hth_attrib|level(20),wp_melee(150),knows_common|knows_riding_6|knows_toughness_3|knows_reflexes_3|knows_combat_training_3,mercenary_face_1,mercenary_face_2],

#Cavalry; got a buff in 1.5, basically one of the only ways to get lancers early.
["mercenary_cavalry","Merc. Cavalry (elite)","Merc. Cavalry (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield|tf_guarantee_gloves,no_scene,reserved,fac_commoners,[itm_lance,itm_mackie_morning_star,itm_xenoargh_shield_heater07,itm_banded_armor,itm_narf_mail_gauntlets,itm_splinted_leather_greaves,itm_rathos_12thcent_faceplate,itm_charger],expert_hth_attrib|level(30),wp_melee(250),knows_common|knows_riding_10|knows_toughness_5|knows_reflexes_5|knows_combat_training_5|knows_defensive_tactics_1,mercenary_face_1,mercenary_face_2],

#Dragoons:  shooters with grenades.  Pretty useful if you use them as a manuever force.
["mercenary_dragoon","Merc. Dragoon (elite)","Merc. Dragoons (elite)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_gloves,no_scene,reserved,fac_commoners,[itm_sword_khergit_1,itm_narf_mail_gauntlets,itm_rathos_mail_byrnie,itm_hide_boots,itm_jtm_sturmhaube_4bw,itm_warhorse_chain_barding04],elite_ranged_attrib|level(30),wp_melee(200)|wp_crossbow(400)|wp_throwing(225),knows_common|knows_riding_7|knows_toughness_4|knows_reflexes_3|knows_combat_training_3,mercenary_face_1,mercenary_face_2],

##########END OF MALE ELITE MERCS HERE!!!

#Standard Female Ranged Path- very elite ranged infantry
["mercenary_pistolera","Merc. Pistolera (elite)","Merc. Pistoleras (elite)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_shield|tf_guarantee_boots,no_scene,reserved,fac_commoners,[itm_bolts,itm_flintlock_pistol,itm_nordic_shield,itm_hide_covered_round_shield,itm_lui_rapier,itm_lui_rapierb,itm_lui_rapierc,itm_lui_rapierd,itm_lui_rapiere,itm_lui_rapierg,itm_rogue_armor1,itm_rogue_armor2,itm_rogue_csizma],elite_ranged_attrib|level(30),wp_melee(125)|wp_crossbow(400),knows_common|knows_reflexes_8|knows_combat_training_8,european_woman_face_1,european_woman_face_2],

["mercenary_duelist","Merc. Duelist (expert)","Merc. Duelists (expert)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_helmet,no_scene,reserved,fac_commoners,[itm_bolts,itm_llew2_flintlock_pistol,itm_plate_covered_round_shield,itm_lui_rapier,itm_lui_rapierb,itm_lui_rapierc,itm_lui_rapierd,itm_lui_rapiere,itm_lui_rapierg,itm_rogue_armor_blue,itm_rogue_armor_blue2,itm_rogue_csizma,itm_xenoargh_mask_gold,itm_xenoargh_mask_black,itm_xenoargh_mask_domino],expert_ranged_attrib|level(45),wp_melee(200)|wp_crossbow(600),knows_common|knows_reflexes_10|knows_combat_training_10,european_woman_face_1,european_woman_face_2],

#Standard Female Cavalry Path- light / medium cavalry
["mercenary_fighter_woman","Merc. Defender (elite)","Merc. Defenders (elite)",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_shield|tf_guarantee_horse,no_scene,reserved,fac_commoners,[itm_darts,itm_fur_covered_shield,itm_hide_covered_round_shield,itm_hatchet,itm_xena_armor,itm_blademaster_armor,itm_rogue_csizma,itm_xena_boots,itm_hunter,itm_xenoargh_blackleapord,itm_xenoargh_buckskin,itm_amade_bronze_winged_helm_small],regular_hth_attrib|level(15),wp_melee(160)|wp_throwing(170),knows_common|knows_riding_5|knows_reflexes_4|knows_toughness_2|knows_combat_training_2,european_woman_face_1,european_woman_face_2],

["mercenary_sword_sister","Merc. Sister (elite)","Merc. Sisters (elite)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_shield|tf_guarantee_horse|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_helmet,0,0,fac_commoners,[itm_darts,itm_sword_medieval_b,itm_sword_khergit_3,itm_plate_covered_round_shield,itm_xenoargh_round_shield02_paint03,itm_warhorse,itm_warhorse_chain,itm_warhorse_chain_barding06,itm_warhorse_chain_barding07,itm_leather_gloves,itm_blademaster_armor_chainmail,itm_rogue_csizma,itm_amade_steel_winged_helm_small,itm_amade_bronze_winged_helm_small,itm_leather_gloves,itm_mail_mittens],elite_hth_attrib|level(30),wp_melee(180)|wp_throwing(200),knows_common|knows_riding_7|knows_horse_archery_4|knows_reflexes_7|knows_toughness_3|knows_combat_training_4|knows_heroism_1,european_woman_face_1,european_woman_face_2],

["mercenary_sword_sister_expert","Sword Sister (expert)","Sword Sisters (expert)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_shield|tf_guarantee_horse|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_helmet,0,0,fac_commoners,[itm_darts,itm_sword_medieval_b,itm_sword_khergit_3,itm_plate_covered_round_shield,itm_xenoargh_round_shield02_paint03,itm_charger,itm_charger_plate,itm_leather_gloves,itm_mail_mittens,itm_dthehun_new_battle_armor05,itm_rogue_csizma,itm_xenoargh_female_crown01],expert_hth_attrib|level(50),wp_melee(220)|wp_throwing(240),knows_common|knows_riding_9|knows_horse_archery_6|knows_reflexes_10|knows_toughness_5|knows_combat_training_6,european_woman_face_1,european_woman_face_2],

#Standard Female HTH Elite - irregulars armed with ranged and two-hand swords,medium to heavy armor.Rather nasty on the high end.
["mercenary_infantry_woman_elite","Female Irregular (elite)","Female Irregulars (elite)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_boots|tf_guarantee_helmet|tf_guarantee_gloves,no_scene,reserved,fac_commoners,[itm_arrows,itm_rogue_bow,itm_javelin,itm_aribeth_armor_dark_chainmail,itm_xena_boots,itm_dejawolf_barbuta3,itm_dejawolf_barbuta4,itm_leather_gloves,itm_mail_mittens],elite_hth_attrib|level(30),wp_melee(150)|wp_throwing(180),knows_common|knows_reflexes_6|knows_toughness_2|knows_combat_training_2,european_woman_face_1,european_woman_face_2],
#Added grenades to the ultimate Irregulars, to make them even nastier assault troops.
["mercenary_infantry_woman_expert","Female Irregular (expert)","Female Irregulars (expert)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_boots|tf_guarantee_helmet|tf_guarantee_gloves,no_scene,reserved,fac_commoners,[itm_arrows,itm_rogue_bow,itm_javelin,itm_dthehun_new_battle_armor02,itm_xena_boots,itm_dejawolf_barbuta3,itm_dejawolf_barbuta4,itm_mail_mittens],expert_hth_attrib|level(50),wp_melee(200)|wp_throwing(240),knows_common|knows_reflexes_8|knows_toughness_4|knows_combat_training_4|knows_heroism_1,european_woman_face_1,european_woman_face_2],

["mercenary_polearm_woman_elite","Battle Sister (elite)","Battle Sisters (elite)",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_templars,[itm_xenoargh_bwanga,itm_xenoargh_bwanga,itm_lui_manhunter_polehammer,itm_jam_scorpion,itm_jam_voulge_b,itm_lamellar_gauntlets,itm_xenoargh_hornmask_blue,itm_xena_boots,itm_xenoargh_new_female_armor_blue04,itm_heater_shield_swadian,itm_heater_shield_old,itm_heater_shield_generic,itm_heater_shield_heavy],elite_ranged_attrib|level(32),wp_melee(180)|wp_throwing(150),knows_toughness_6|knows_combat_training_4|knows_reflexes_3,khergit_woman_face_1,khergit_woman_face_8],

["mercenary_elite_end","mercenary_elite_end","mercenary_elite_end",0,no_scene,reserved,fac_commoners,[], def_attrib|level(4),wp(60),knows_common,mercenary_face_1,mercenary_face_2],

["regional_mercenary_start","Regional Mercs Start","Regional Mercs Start",0,no_scene,reserved,fac_commoners,[], def_attrib|level(4),wp(60),knows_common,mercenary_face_1,mercenary_face_2],

#REGIONAL MERCENARIES:
#Only available in towns, these are intriguing troops that have a fairly high asking price.  Tough, nasty soldiers with special "gifts".

#Swadians:  Zweihanders and Royal Huntsmen.
#Zweihanders:  like the polearm troops, but better, faster, nastier.  You can only get them this way.
["swadian_landsknecht_expert","Landsknecht (expert)","Landsknechts (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,no_scene,reserved,fac_commoners,[itm_lui_zweilander,itm_lui_zweilanderb,itm_lui_zweilanderc,itm_lui_zweilanderd,itm_narf_armour_slashed,itm_narf_bear_paw_shoes,itm_jtm_sturmhaube_5bw,itm_narf_gauntlets],expert_hth_attrib|level(45),wp_melee(250),knows_common|knows_toughness_10|knows_combat_training_4,swadian_face_middle_1,swadian_face_old_2],

#Royal Huntsmen- the end of the Forest Bandit line.  Decent bowmen for the price, if you can get them.  
#They have grenades, but aren't particularily good with 'em.  Decent armor, good helms.
["swadian_royal_huntsman","Royal Huntsman","Royal Huntsmen",tf_mounted|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_helmet|tf_guarantee_gloves,no_scene,reserved,fac_commoners,[itm_arrows,itm_long_bow,itm_xenoargh_troop_grenade,itm_jam_voulge_b,itm_great_bardiche,itm_bardiche,itm_voulge,itm_narf_open_sallet,itm_pino_peasant_archer_mail,itm_hide_boots,itm_leather_boots,itm_leather_gloves,itm_narf_mail_gauntlets],elite_ranged_attrib|level(30),wp_melee(160)|wp_archery(200)|wp_throwing(80),knows_common|knows_reflexes_7|knows_combat_training_4,swadian_face_young_1,swadian_face_old_2],

#Rhodoks:  the Gaucho Pikemen.
#The Gaucho Pikemen are one of the nastiest things I've ever come up with.  Death to horses, period.  Hard to attack with infantry.
#Back them with shooters, and your enemies are in for a world of hurt.
["rhodok_bola_elite","Gaucho Pikeman (elite)","Gaucho Pikemen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield,no_scene,reserved,fac_commoners,[itm_xenoargh_bola,itm_lui_manhunterspear,itm_lui_raiderlordspear,itm_xenoargh_musketeer_hat_red01,itm_xenoargh_musketeer_hat_red02,itm_narf_corrazina_green,itm_splinted_leather_greaves,itm_leather_gloves,itm_narf_mail_gauntlets,itm_pavise_shield_fancy_cross],elite_hth_attrib|level(40),wp_melee(175)|wp_throwing(400),knows_toughness_5|knows_combat_training_3,rhodok_face_middle_1,rhodok_face_middle_2],

#Speaking of the shooters... heh
["rhodok_schiopettieri","Schiopettiere (elite)","Schiopettieri (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield|tf_guarantee_ranged,no_scene,reserved,fac_commoners,[itm_bolts,itm_llew2_arquebus,itm_llew2_matchlock_1,itm_llew2_matchlock_2,itm_rrr_agincourt_steel,itm_rrr_agincourt_brhil,itm_wei_xiadi_nord_spiked_mail,itm_narf_open_sallet,itm_splinted_leather_greaves,itm_narf_mail_gauntlets,itm_old_board_shield01],elite_ranged_attrib|level(35),wp_melee(145)|wp_crossbow(190),knows_toughness_5|knows_combat_training_3,rhodok_face_middle_1,rhodok_face_middle_2],

#Nords:  Felhammirs.  Think Beserkirs with giant mauls.  I think that, in terms of raw damage, they're the most deadly things in the game atm.
["nord_felhammir","Felhammir (expert)","Felhammirs (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_gloves|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet,no_scene,reserved,fac_commoners,[itm_bolts,itm_xenoargh_gun_axe,itm_rrr_mallet1,itm_maul,itm_warhammer,itm_sledgehammer,itm_xenoargh_round_shield02,itm_iron_greaves,itm_leather_gloves,itm_scale_armor,itm_xenoargh_viking_horned_helm01],beserkir_attrib|level(55),wp_melee(250),knows_reflexes_5|knows_toughness_10,nord_face_middle_1,nord_face_older_2],

#Blunderbus men.  Tough fellows with blunderbusses and swords.
["nord_blunderbus","Blunderbus Man (elite)","Blunderbus Men (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_gloves|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_ranged,no_scene,reserved,fac_commoners,[itm_bolts,itm_llew2_blunderbus,itm_sword_viking_3_small,itm_sword_viking_2_small,itm_xenoargh_round_shield02,itm_iron_greaves,itm_narf_mail_gauntlets,itm_wei_xiadi_mail_shirt01,itm_wei_xiadi_mail_shirt01,itm_nordic_footman_helmet,itm_nordic_fighter_helmet],elite_ranged_attrib|level(30),wp_melee(150)|wp_crossbow(200),knows_reflexes_4|knows_toughness_5|knows_combat_training_3,nord_face_middle_1,nord_face_older_2],

#Sarranids:  Camel Jezzails that can fire fairly accurately from camel-back.  Pretty much says it all.
["sarranid_camel_jezzailis","Camel Jezzaili (elite)","Camel Jezzailis (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield|tf_guarantee_ranged,no_scene,reserved,fac_commoners,[itm_bolts,itm_jezzail,itm_lance,itm_arabian_sword_b,itm_sarranid_cavalry_sword,itm_scimitar,itm_scimitar_b,itm_lui_cimeterre,itm_lui_cimeterreb,itm_lui_khergitexecutioneronehand,itm_xenoargh_round_shield02_paint01,itm_sarranid_elite_armor,itm_mamluke_mail,itm_sarranid_boots_c,itm_sarranid_boots_b,itm_sarranid_horseman_helmet,itm_camel,itm_camel2,itm_camel3],elite_ranged_attrib|level(40),wp_melee(150)|wp_crossbow(200),knows_riding_6|knows_horse_archery_7|knows_combat_training_6|knows_reflexes_4,sarranid_face_middle_1,sarranid_face_middle_2],

#Cataphracts:  heavily-armored cavalry with a mix of two-handed swords and guns.  Very tough. Really nasty combo troops.
["sarranid_cataphracts","Cataphract (expert)","Cataphracts (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,no_scene,reserved,fac_commoners,[itm_bolts,itm_llew2_arquebus,itm_lance,itm_two_handed_cleaver,itm_wei_xiadi_cataphract_armor,itm_gauntlets,itm_narf_shynbaulds,itm_narf_nikolskoe_helm,itm_warhorse_sarranid],expert_hth_attrib|level(45),wp_melee(225)|wp_crossbow(150),knows_riding_8|knows_horse_archery_5|knows_combat_training_6|knows_reflexes_3|knows_toughness_10,sarranid_face_middle_1,sarranid_face_middle_2], 

#Khergits:
#Chu-Ko-Nu Crossbows... silly spammers with polearms.  Crazy base of fire, short range.
["khergit_chu_ko_nu","Repeating Crossbowman (elite)","Repeating Crossbowmen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged,no_scene,reserved,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_chu_ko_nu,itm_hafted_blade_a,itm_hafted_blade_b,itm_lui_khegitnagita,itm_pinky_lamellar_vest,itm_nomad_boots,itm_vaegir_lamellar_helmet,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet],elite_ranged_attrib|level(30),wp_melee(90)|wp_crossbow(500),knows_common|knows_reflexes_3|knows_combat_training_6,khergit_face_younger_1,khergit_face_younger_2],

#Khergit rocket-arrow launchers.  Expensive as hell.  And so, so worth it.
["khergit_rocketeer_elite","Rocketeer (elite)","Rocketeers (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged,no_scene,reserved,fac_commoners,[itm_universal_cannon_ammo,itm_xenoargh_rocket_projector,itm_sword_khergit_1,itm_sword_khergit_2,itm_sword_khergit_3,itm_sword_khergit_4,itm_xenoargh_lamellar_breastplate_variant002,itm_nomad_boots,itm_vaegir_lamellar_helmet,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet],elite_ranged_attrib|level(45),wp_melee(80)|wp_archery(150)|wp_crossbow(150),knows_toughness_2|knows_reflexes_3,khergit_face_younger_1,khergit_face_younger_2],

["khergit_ronin","Ronin (expert)","Ronin (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves|tf_guarantee_horse|tf_guarantee_ranged,no_scene,reserved,fac_commoners,[itm_eastern_arrows,itm_latticed_flatbow,itm_lui_mercenarylance,itm_mackie_nodachi,itm_ssh_kote,itm_ssh_yoroi,itm_ssh_kabuto,itm_ssh_suneate,itm_hunter],assassin_attrib|level(65),wp_melee(225)|wp_archery(200),knows_riding_10|knows_toughness_6|knows_reflexes_10|knows_combat_training_8,khergit_face_younger_1,khergit_face_younger_2],

#Vaegir:
#Rhulg- heavy axemen, with a special weapon.  No upgrade, but who needs one?
["rhulg_destroyer","Rhulg Destroyer (expert)","Rhulg Destroyers (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield,no_scene,reserved,fac_commoners,[itm_taleworlds_doubleaxe,itm_old_round_shield,itm_xenoargh_round_shield02,itm_round_shield01_plain,itm_xenoargh_round_shield02,itm_narf_milanese_plate_dark,itm_narf_shynbaulds_dark,itm_narf_steel_greaves_dark,itm_rhulg_helmet,itm_narf_plate_mittens_dark],expert_hth_attrib|level(70),wp_melee(270),knows_common|knows_reflexes_5|knows_toughness_6|knows_combat_training_3,mercenary_face_1,mercenary_face_2],

["regional_mercenary_end","Regional Mercs End","Regional Mercs End",0,no_scene,reserved,fac_commoners,[], def_attrib|level(4),wp(60),knows_common,mercenary_face_1,mercenary_face_2],


#SWADIANS:
#Classic fusion of Germanic / Frankish forces.
#Swadians have the second-best knights,and great melee infantry,with a strong emphasis on polearms.
#They have not one but two excellent (if very expensive) Man At Arms classes- nasty heavy infantry, not quite Vaegir but very close.
#They are the only Official Culture that gets muskets.
#They are the only Official Culture to get cannons.
#They are, in short, well-rounded, tough, and good against practically everybody, but assembling a high-level army can be *very* expensive.

#Halberdiers- tough, reliable infantry. Elites are expensive, but very well armored.  Regulars+ get grenades, as defense / assault troops.
["swadian_polearm_levy","Swadian Halberdier (peasant levy)","Swadian Halberdiers (peasant levy)",tf_mounted|tf_guarantee_armor,0,0,fac_kingdom_of_swadians,[itm_rrr_bill1,itm_rrr_halberd1,itm_rrr_halberd2,itm_rrr_halberd3,itm_rrr_halberd4,itm_rrr_halberd5,itm_rrr_halberd6,itm_rrr_halberd7,itm_leather_cap,itm_felt_hat,itm_felt_hat,itm_padded_armor,itm_nomad_boots,itm_wrapping_boots],levy_hth_attrib|level(5),wp_melee(60),knows_common|knows_toughness_1,swadian_face_younger_1,swadian_face_middle_2],

["swadian_polearm_recruit","Swadian Halberdier (recruit)","Swadian Halberdiers (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_swadians,[itm_rrr_bill1,itm_rrr_halberd1,itm_rrr_halberd2,itm_rrr_halberd3,itm_rrr_halberd4,itm_rrr_halberd5,itm_rrr_halberd6,itm_rrr_halberd7,itm_leather_jerkin,itm_padded_leather,itm_leather_armor,itm_ankle_boots,itm_padded_coif,itm_footman_helmet],recruit_hth_attrib|level(15),wp_melee(85),knows_common|knows_toughness_2|knows_combat_training_1,swadian_face_young_1,swadian_face_old_2],

["swadian_polearm_regular","Swadian Halberdier (regular)","Swadian Halberdiers (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_xenoargh_troop_grenade,itm_rrr_bill1,itm_rrr_halberd1,itm_rrr_halberd2,itm_rrr_halberd3,itm_rrr_halberd4,itm_rrr_halberd5,itm_rrr_halberd6,itm_rrr_halberd7,itm_wei_xiadi_swa_surcoat_fancy01,itm_wei_xiadi_swa_surcoat_fancy02,itm_wei_xiadi_swa_surcoat_fancy03,itm_hide_boots,itm_ankle_boots,itm_kettle_hat,itm_mail_coif,itm_flat_topped_helmet,itm_helmet_with_neckguard],regular_hth_attrib|level(25),wp_melee(105)|wp_throwing(80),knows_common|knows_toughness_4|knows_combat_training_3,swadian_face_middle_1,swadian_face_old_2],

["swadian_polearm_elite","Swadian Halberdier (elite)","Swadian Halberdiers (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_swadians,[itm_xenoargh_troop_grenade,itm_lui_hallberd_blue,itm_lui_hallberd_green,itm_lui_hallberd_red,itm_lui_hallberd_yellow,itm_narf_armour_slashed,itm_narf_bear_paw_shoes,itm_kettle_hat,itm_mail_coif,itm_flat_topped_helmet,itm_helmet_with_neckguard,itm_narf_gauntlets],elite_hth_attrib|level(35),wp_melee(150)|wp_throwing(100),knows_common|knows_toughness_6|knows_combat_training_5,swadian_face_middle_1,swadian_face_old_2],

#Crossbows and Firearms.
#Only one level of crossbows- unlike the Rhodoks, who get a full tree.
#Gulf between arquebussiers and Cannoneers is very large, as it should be.
["swadian_ranged_levy","Swadian Skirmisher (peasant levy)","Swadian Crossbowmen (peasant levy)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_swadians,[itm_bolts,itm_light_crossbow,itm_dagger,itm_club,itm_voulge,itm_heater_shield_swadian,itm_leather_armor,itm_padded_cloth,itm_ankle_boots,itm_padded_coif,itm_arming_cap,itm_footman_helmet],recruit_ranged_attrib|level(10),wp_melee(60)|wp_crossbow(80),knows_common|knows_reflexes_1|knows_combat_training_1,swadian_face_young_1,swadian_face_middle_2],

["swadian_ranged_recruit","Swadian Crossbowman (recruit)","Swadian Crossbowmen (recruit)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_bolts,itm_crossbow,itm_dagger,itm_club,itm_voulge,itm_heater_shield_swadian,itm_leather_armor,itm_padded_cloth,itm_ankle_boots,itm_padded_coif,itm_arming_cap,itm_footman_helmet],recruit_ranged_attrib|level(15),wp_melee(80)|wp_crossbow(120),knows_common|knows_reflexes_2|knows_combat_training_2,swadian_face_young_1,swadian_face_middle_2],

["swadian_ranged_regular","Swadian Arquebussier(regular)","Swadian Arquebussiers (regular)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_swadians,[itm_bolts,itm_arquebus,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_sword_medieval_a,itm_voulge,itm_heater_shield_old,itm_leather_jerkin,itm_leather_armor,itm_hide_boots,itm_ankle_boots,itm_padded_coif,itm_nasal_helmet,itm_footman_helmet],regular_ranged_attrib|level(20),wp_melee(90)|wp_crossbow(150),knows_common|knows_toughness_1|knows_reflexes_4|knows_combat_training_4,swadian_face_young_1,swadian_face_old_2],

["swadian_ranged_elite","Swadian Sharpshooter (elite)","Swadian Sharpshooters (elite)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_bolts,itm_arquebus,itm_sword_medieval_b_small,itm_sword_medieval_a,itm_voulge,itm_heater_shield_generic,itm_haubergeon,itm_padded_leather,itm_hide_boots,itm_norman_helmet,itm_nasal_helmet,itm_kettle_hat,itm_kettle_hat,itm_leather_gloves],elite_ranged_attrib|level(30),wp_melee(100)|wp_crossbow(180),knows_common|knows_toughness_2|knows_reflexes_5|knows_combat_training_6,swadian_face_middle_1,swadian_face_older_2],

["swadian_ranged_matchlock_expert","Swadian Matchlock Sharpshooter (expert)","Swadian Matchlock Sharpshooters (expert)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_bolts,itm_llew2_matchlock_2,itm_sword_medieval_b_small,itm_sword_medieval_a,itm_voulge,itm_heater_shield_generic,itm_wei_xiadi_swa_hauberk,itm_iron_greaves,itm_norman_helmet,itm_nasal_helmet,itm_kettle_hat,itm_kettle_hat,itm_narf_mail_gauntlets],expert_ranged_attrib|level(40),wp_melee(120)|wp_crossbow(200),knows_common|knows_toughness_5|knows_reflexes_5|knows_combat_training_8,swadian_face_middle_1,swadian_face_older_2],

["swadian_ranged_expert","Swadian Handgonner (expert)","Swadian Handgonners (expert)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_universal_cannon_ammo,itm_siege_handgonne,itm_handgonne,itm_sword_medieval_b_small,itm_sword_medieval_a,itm_voulge,itm_heater_shield_generic,itm_haubergeon,itm_padded_leather,itm_hide_boots,itm_norman_helmet,itm_nasal_helmet,itm_kettle_hat,itm_kettle_hat,itm_leather_gloves],elite_ranged_attrib|str_14|level(75),wp_melee(100)|wp_crossbow(180),knows_common|knows_toughness_3|knows_reflexes_6|knows_combat_training_6,swadian_face_middle_1,swadian_face_older_2],

#Swadian Mace-Men: nasty armor-piercers.Very dangerous. Has an upgrade split to heavier versions.
["swadian_maceman_regular","Swadian Mace-man (regular)","Swadian Mace-men (regular)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_archer_mace,itm_winged_mace,itm_balanced_winged_mace,itm_mace_3,itm_mace_4,itm_lui_mace,itm_heater_shield_swadian,itm_coat_of_plates,itm_mail_with_surcoat,itm_mail_chausses,itm_iron_greaves,itm_guard_helmet,itm_helmet_with_neckguard,itm_bascinet,itm_guard_helmet,itm_leather_gloves,itm_mail_mittens],regular_hth_attrib|level(20),wp_melee(135),knows_common|knows_toughness_4|knows_combat_training_4,swadian_face_middle_1,swadian_face_older_2],

["swadian_maceman_elite","Swadian Mace-man (elite)","Swadian Mace-men (elite)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_swadians,[itm_archer_mace,itm_winged_mace,itm_balanced_winged_mace,itm_mace_3,itm_mace_4,itm_lui_mace,itm_heater_shield_swadian,itm_wei_xiadi_swa_mail_and_plate_blue,itm_iron_greaves,itm_rathos_12thcent_faceplate_bwq,itm_mail_mittens],elite_hth_attrib|level(35),wp_melee(175),knows_common|knows_toughness_6|knows_combat_training_7,swadian_face_middle_1,swadian_face_older_2],

["swadian_maceman_two_hand","Swadian Heavy Mace-man (expert)","Swadian Heavy Mace-men (expert)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_swadians,[itm_long_hafted_knobbed_mace,itm_long_hafted_spiked_mace,itm_lui_twohandedmace,itm_wei_xiadi_swa_mail_and_plate_black,itm_iron_greaves,itm_rathos_12thcent_faceplate_wbs,itm_narf_plate_mittens],expert_hth_attrib|level(40),wp_melee(250),knows_common|knows_toughness_8|knows_combat_training_7,swadian_face_middle_1,swadian_face_older_2],

#Swadian Swordsmen: Mixes it up with estocs, decent HTH troops. Only one upgrade, but the upgrade is to Expert, with ultra-nice gear.
["swadian_swordsman_regular","Swadian Swordsman (regular)","Swadian Swordsmen (regular)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_heater_shield_heavy,itm_coat_of_plates,itm_wei_xiadi_swa_surcoat_green,itm_wei_xiadi_swa_surcoat_purple,itm_wei_xiadi_swa_surcoat_v,itm_wei_xiadi_swa_tabard,itm_mail_chausses,itm_iron_greaves,itm_leather_gloves,itm_mail_mittens,itm_rathos_greathelm,itm_rathos_greathelm_blue,itm_rathos_greathelm_brass,itm_rathos_greathelm_red],regular_hth_attrib|level(20),wp_melee(135),knows_common|knows_toughness_3|knows_combat_training_6,swadian_face_middle_1,swadian_face_older_2],

["swadian_swordsman_expert","Swadian Royal Swordsman (expert)","Swadian Royal Swordsmen (expert)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_swadians,[itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_narf_shynbaulds,itm_dejawolf_flemish_armet,itm_narf_plate_mittens,itm_gauntlets,itm_narf_gothic_plate],expert_hth_attrib|level(45),wp_melee(220),knows_common|knows_toughness_8|knows_combat_training_10,swadian_face_middle_1,swadian_face_older_2],

#Swadian Knights: Expensive, but good.Kind've like Volvos.Not quite as uber as the Condotteri, as they do not get Destriers. High end is Expert, and very nasty.
["swadian_knight_regular","Swadian Sergeant (regular)","Swadian Sergeants (regular)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse,0,0,fac_kingdom_of_swadians,[itm_lance,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_bastard_sword_b,itm_morningstar,itm_morningstar2,itm_sword_medieval_c,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_coat_of_plates,itm_mail_with_surcoat,itm_mail_chausses,itm_iron_greaves,itm_guard_helmet,itm_helmet_with_neckguard,itm_bascinet,itm_guard_helmet,itm_leather_gloves,itm_hunter,itm_xenoargh_buckskin],regular_hth_attrib|level(25),wp_melee(135),knows_common|knows_riding_4|knows_toughness_3|knows_combat_training_5,swadian_face_middle_1,swadian_face_older_2],

["swadian_knight_elite","Swadian Man at Arms (elite)","Swadian Men at Arms (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_swadians,[itm_pop_gothic_lance_blue,itm_pop_gothic_lance_green,itm_pop_gothic_lance_blue,itm_pop_gothic_lance_green,itm_pop_gothic_lance_blue,itm_pop_gothic_lance_green,itm_hammer,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_bastard_sword_a,itm_sword_medieval_b,itm_sword_medieval_c_small,itm_xenoargh_shield_heater01,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_narf_early_transitional_blue,itm_narf_early_transitional_orange,itm_narf_early_transitional_white,itm_narf_splinted_greaves_spurs,itm_norman_helmet,itm_mail_coif,itm_flat_topped_helmet,itm_helmet_with_neckguard,itm_warhorse,itm_charger,itm_warhorse_chain_barding03,itm_warhorse_chain_barding04,itm_warhorse_chain_barding05],elite_hth_attrib|level(40),wp_melee(175),knows_common|knows_riding_7|knows_toughness_5|knows_combat_training_7,swadian_face_young_1,swadian_face_old_2],

["swadian_knight_expert","Swadian Royal Knight (expert)","Swadian Royal Knights (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_swadians,[itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets,itm_narf_gothic_plate,itm_warhorse,itm_charger,itm_warhorse_chain_barding03,itm_warhorse_chain_barding04,itm_warhorse_chain_barding05],expert_hth_attrib|level(55),wp_melee(225),knows_common|knows_riding_9|knows_toughness_8|knows_combat_training_8|knows_heroism_2,swadian_face_middle_1,swadian_face_older_2],

#The Grimshields.  For those days when you need the best.
["swadian_grimshield","Swadian Grimshield (champion)","Swadian Grimshields (champion)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_swadians,[itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_xenoargh_shield_heater09,itm_narf_shynbaulds_black,itm_narf_visored_sallet_coif_black,itm_narf_milanese_gauntlets_black,itm_narf_gothic_plate_black,itm_spak_warhorse_holy],expert_hth_attrib|level(75),wp_melee(400),knows_common|knows_riding_10|knows_toughness_10|knows_combat_training_10|knows_heroism_3,swadian_face_middle_1,swadian_face_older_2],

#STOCK SWADIAN CHARACTERS
["swadian_messenger","Swadian Messenger","Swadian Messengers",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_kingdom_of_swadians,[itm_sword_medieval_a,itm_leather_jerkin,itm_leather_boots,itm_courser,itm_leather_gloves,itm_light_crossbow,itm_bolts],def_attrib|agi_21|level(25),wp_melee(130),knows_common|knows_riding_7|knows_horse_archery_5,swadian_face_young_1,swadian_face_old_2],

["swadian_deserter","Swadian Deserter","Swadian Deserters",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_deserters,[itm_bolts,itm_light_crossbow,itm_hunting_crossbow,itm_dagger,itm_club,itm_voulge,itm_wooden_shield,itm_leather_jerkin,itm_padded_cloth,itm_hide_boots,itm_padded_coif,itm_nasal_helmet,itm_footman_helmet],def_attrib|level(14),wp_melee(80),knows_common|knows_riding_2|knows_toughness_1,swadian_face_young_1,swadian_face_old_2],

["swadian_prison_guard","Prison Guard","Prison Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_awlpike,itm_pike,itm_lui_zweilanderd,itm_morningstar,itm_morningstar2,itm_sword_medieval_b,itm_heater_shield_generic,itm_coat_of_plates,itm_plate_armor,itm_mail_chausses,itm_iron_greaves,itm_guard_helmet,itm_helmet_with_neckguard,itm_bascinet,itm_guard_helmet,itm_leather_gloves],def_attrib|level(25),wp_melee(130),knows_common|knows_toughness_3,swadian_face_young_1,swadian_face_old_2],

["swadian_castle_guard","Castle Guard","Castle Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_swadians,[itm_awlpike,itm_pike,itm_lui_zweilanderd,itm_morningstar,itm_morningstar2,itm_sword_medieval_b,itm_heater_shield_generic,itm_heater_shield_heavy,itm_coat_of_plates,itm_plate_armor,itm_mail_chausses,itm_iron_greaves,itm_guard_helmet,itm_helmet_with_neckguard,itm_bascinet,itm_guard_helmet,itm_leather_gloves],def_attrib|level(25),wp_melee(130),knows_common|knows_toughness_3,swadian_face_young_1,swadian_face_old_2],

#VAEGIRS:The Romans,only updated. Yup,the Romans (er,Calradians).
#Basically, they're fantasy infantry, please don't send me hate mail. The Romans never lasted long enough for us to see what this would have been like.
#I found the results so interesting in early tests that I decided to remake this entire faction.
#The Vaegirs field heavy-infantry specialists,and their troops are rather expensive to recruit and keep in the field in large numbers.
#However, they are probably the easiest force to take castles with, due to their versatile Legionaires and powerful Crushers.
#Their Pikes make them very dangerous vs. cavalry as well. It's very hard to charge a Vaegir force.

#Vaegir Infantry Path 1- classic legionnaires,with modern updates
["vaegir_legionnaire_recruit","Vaegir Legionnaire (recruit)","Vaegir Legionnaires (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_kingdom_of_vaegirs,[itm_throwing_spears,itm_jarid,itm_sword_medieval_b_small,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_splinted_greaves,itm_amade_steel_plate,itm_dejawolf_barbuta1,itm_dejawolf_barbuta2],recruit_hth_attrib|level(20),wp_melee(70)|wp_throwing(50),knows_toughness_2,vaegir_face_younger_1,vaegir_face_middle_2],

["vaegir_legionnaire_regular","Vaegir Legionnaire (regular)","Vaegir Legionnaires (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_kingdom_of_vaegirs,[itm_throwing_spears,itm_jarid,itm_sword_medieval_b_small,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_splinted_greaves,itm_amade_steel_plate,itm_dejawolf_barbuta1,itm_dejawolf_barbuta2],regular_hth_attrib|level(30),wp_melee(120)|wp_throwing(100),knows_toughness_5|knows_combat_training_1,vaegir_face_younger_1,vaegir_face_middle_2],

["vaegir_legionnaire_elite","Vaegir Legionnaire (elite)","Vaegir Legionnaires (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield|tf_guarantee_gloves,0,0,fac_kingdom_of_vaegirs,[itm_throwing_spears,itm_jarid,itm_sword_medieval_b_small,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_steel_greaves,itm_amade_steel_gauntlets,itm_amade_steel_plate_sergeant,itm_dejawolf_barbuta1,itm_dejawolf_barbuta2],elite_hth_attrib|level(40),wp_melee(175)|wp_throwing(155),knows_toughness_7|knows_combat_training_4,vaegir_face_middle_1,vaegir_face_middle_2],

#Vaegir Crushers- two-handed swordsmen,nasty assault infantry
["vaegir_crusher_recruit","Vaegir Crusher (recruit)","Vaegir Crushers (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_vaegirs,[itm_throwing_spears,itm_jarid,itm_lui_roughdaksword,itm_splinted_greaves,itm_amade_steel_plate,itm_amade_steel_winged_helm],recruit_hth_attrib|level(25),wp_melee(100)|wp_throwing(155),knows_toughness_2,vaegir_face_younger_1,vaegir_face_middle_2],

["vaegir_crusher_regular","Vaegir Crusher (regular)","Vaegir Crushers (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_vaegirs,[itm_throwing_spears,itm_jarid,itm_lui_roughdaksword,itm_splinted_greaves,itm_amade_steel_plate,itm_amade_steel_winged_helm],regular_hth_attrib|level(35),wp_melee(150)|wp_throwing(200),knows_toughness_5|knows_combat_training_1,vaegir_face_younger_1,vaegir_face_middle_2],

["vaegir_crusher_elite","Vaegir Crusher (elite)","Vaegir Crushers (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_vaegirs,[itm_throwing_spears,itm_jarid,itm_lui_roughdaksword,itm_amade_steel_greaves,itm_amade_steel_gauntlets,itm_amade_steel_plate_sergeant,itm_amade_steel_winged_helm],elite_hth_attrib|level(45),wp_melee(200)|wp_throwing(250),knows_toughness_7|knows_combat_training_4,vaegir_face_middle_1,vaegir_face_middle_2],

#Vaegir Infantry Path 3- phalanx pikemen.No shields,no backup weapon.Needs script to give 'em short swords during sieges.
["vaegir_pikeman_recruit","Vaegir Pikeman (recruit)","Vaegir Pikemen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_vaegirs,[itm_lui_vaegirpike,itm_splinted_greaves,itm_amade_steel_plate,itm_amade_steel_winged_helm],recruit_hth_attrib|level(20),wp_melee(70),knows_toughness_2,vaegir_face_younger_1,vaegir_face_middle_2],

["vaegir_pikeman_regular","Vaegir Pikeman (regular)","Vaegir Pikemen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_vaegirs,[itm_lui_vaegirpike,itm_amade_steel_greaves,itm_amade_steel_gauntlets,itm_amade_steel_plate,itm_amade_steel_winged_helm],regular_hth_attrib|level(30),wp_melee(120),knows_toughness_5|knows_combat_training_1,vaegir_face_younger_1,vaegir_face_middle_2],

["vaegir_pikeman_elite","Vaegir Pikeman (elite)","Vaegir Pikemen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_vaegirs,[itm_lui_vaegirpike,itm_amade_steel_greaves,itm_amade_steel_gauntlets,itm_amade_steel_plate_sergeant,itm_amade_steel_winged_helm],elite_hth_attrib|level(40),wp_melee(150),knows_toughness_7|knows_combat_training_4,vaegir_face_middle_1,vaegir_face_middle_2],

#Vaegir Bowmen- these are cheap bowmen,meant for large skirmishing forces.Even the elites are terrible in HTH,and they're all incredibly fragile.
#The Vaegirs don't have crossbows at all,which puts them at a severe disadvantage in range duels.
["vaegir_archer_levy","Vaegir Archer (peasant levy)","Vaegir Archers (peasant levy)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_vaegirs,[itm_eastern_arrows,itm_axe,itm_sword_khergit_1,itm_nomad_bow,itm_nomad_bow,itm_short_bow,itm_leather_jerkin,itm_leather_vest,itm_nomad_boots],levy_ranged_attrib|level(5),wp_melee(50)|wp_archery(70),knows_combat_training_1,vaegir_face_young_1,vaegir_face_older_2],

["vaegir_archer_recruit","Vaegir Archer (recruit)","Vaegir Archers (recruit)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_vaegirs,[itm_eastern_arrows,itm_axe,itm_sword_khergit_1,itm_nomad_bow,itm_nomad_bow,itm_leather_jerkin,itm_leather_vest,itm_nomad_boots],recruit_ranged_attrib|level(10),wp_melee(60)|wp_archery(120),knows_combat_training_3,vaegir_face_young_1,vaegir_face_older_2],

["vaegir_archer_regular","Vaegir Archer (regular)","Vaegir Archers (regular)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_vaegirs,[itm_eastern_arrows,itm_axe,itm_sword_khergit_1,itm_strong_bow,itm_nomad_bow,itm_leather_jerkin,itm_leather_vest,itm_nomad_boots],regular_ranged_attrib|level(15),wp_melee(70)|wp_archery(150),knows_combat_training_5,vaegir_face_young_1,vaegir_face_older_2],

["vaegir_archer_elite","Vaegir Archer (elite)","Vaegir Archers (elite)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_vaegirs,[itm_eastern_arrows,itm_axe,itm_sword_khergit_1,itm_strong_bow,itm_nomad_bow,itm_amade_steel_plate,itm_amade_steel_winged_helm,itm_nomad_boots],elite_ranged_attrib|level(40),wp_melee(100)|wp_archery(200),knows_toughness_2|knows_combat_training_7,vaegir_face_young_1,vaegir_face_older_2],

#Vaegir Light Horse Cavalry- as the name suggests,they're light forces,armed only with spears and their distinctive armor.
#They aren't generally a match for heavy shock cavalry from other nations,but are suitable for raiding and cleanup...
#...and they're reasonable assault infantry,in a pinch.
["vaegir_horseman_recruit","Vaegir Horseman (recruit)","Vaegir Horsemen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_kingdom_of_vaegirs,[itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_nomad_boots,itm_amade_bronze_plate,itm_amade_bronze_winged_helm,itm_sumpter_horse,itm_saddle_horse,itm_xenoargh_appaloosa01],recruit_hth_attrib|level(20),wp_melee(120),knows_riding_3|knows_toughness_1|knows_combat_training_1,vaegir_face_young_1,vaegir_face_older_2],

["vaegir_horseman_regular","Vaegir Horseman (regular)","Vaegir Horsemen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_helmet,0,0,fac_kingdom_of_vaegirs,[itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_nomad_boots,itm_amade_bronze_gauntlets,itm_amade_bronze_plate,itm_amade_bronze_winged_helm,itm_saddle_horse,itm_courser,itm_xenoargh_blackleapord],regular_hth_attrib|level(35),wp_melee(140),knows_riding_5|knows_toughness_2|knows_combat_training_2,vaegir_face_young_1,vaegir_face_older_2],

["vaegir_horseman_elite","Vaegir Horseman (elite)","Vaegir Horsemen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_gloves|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_kingdom_of_vaegirs,[itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_plate,itm_amade_bronze_winged_helm,itm_courser,itm_hunter,itm_xenoargh_buckskin],elite_hth_attrib|level(50),wp_melee(180),knows_riding_7|knows_toughness_3|knows_combat_training_3,vaegir_face_young_1,vaegir_face_older_2],

#Praetorians.  Leaders of the Vaegir host, deadly center of the vanguard.
["vaegir_praetorian","Vaegir Praetorian (Champion)","Vaegir Praetorians (Champion)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_vaegirs,[itm_throwing_spears,itm_jarid,itm_lui_roughdaksword,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_plate_champion,itm_amade_vaegir_champion_helm],expert_hth_attrib|level(75),wp_melee(400)|wp_throwing(350),knows_toughness_10|knows_combat_training_10,vaegir_face_middle_1,vaegir_face_middle_2],

#STOCK VAEGIR CHARACTERS
["vaegir_messenger","Vaegir Messenger","Vaegir Messengers",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse,0,0,fac_kingdom_of_vaegirs,[itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_nomad_boots,itm_amade_bronze_plate,itm_amade_bronze_winged_helm,itm_sumpter_horse,itm_saddle_horse],def_attrib|agi_21|level(25),wp_melee(130),knows_common|knows_riding_7|knows_horse_archery_5,vaegir_face_young_1,vaegir_face_older_2],

["vaegir_deserter","Vaegir Deserter","Vaegir Deserters",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_deserters,[itm_eastern_arrows,itm_spiked_mace,itm_axe,itm_falchion,itm_short_bow,itm_short_bow,itm_hunting_bow,itm_javelin,itm_javelin,itm_steppe_cap,itm_nomad_cap,itm_leather_vest,itm_leather_vest,itm_nomad_armor,itm_nomad_boots],def_attrib|str_10|level(14),wp_melee(80),knows_toughness_1,vaegir_face_young_1,vaegir_face_older_2],

["vaegir_prison_guard","Prison Guard","Prison Guards",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_vaegirs,[itm_lui_midnight,itm_lui_kingswordtwohdark,itm_lui_roughdaksword,itm_lui_zweilander,itm_amade_steel_greaves,itm_amade_steel_gauntlets,itm_amade_steel_plate_sergeant,itm_amade_steel_winged_helm],regular_hth_attrib|level(45),wp_melee(200),knows_reflexes_8|knows_toughness_10,vaegir_face_middle_1,vaegir_face_middle_2],

["vaegir_castle_guard","Castle Guard","Castle Guards",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_vaegirs,[itm_lui_midnight,itm_lui_kingswordtwohdark,itm_lui_roughdaksword,itm_lui_zweilander,itm_amade_steel_greaves,itm_amade_steel_gauntlets,itm_amade_steel_plate_sergeant,itm_amade_steel_winged_helm],regular_hth_attrib|level(45),wp_melee(200),knows_reflexes_8|knows_toughness_10,vaegir_face_middle_1,vaegir_face_middle_2],

#KHERGITS:Classic fusion of Mongols, Chinese and Russian steppe peoples.
#They have three full infantry paths now, amongst other things, stolen from the Vaegirs, with much modification.
#They are more balanced now between archer masses and cavalry masses, with blocks of spearmen as an option.
#Their overall mix is still a work in progress.

#Khergit Archer Infantry
#These are mediocre archers for deploying in blocks, with an Elite level that is a real cut above.
["khergit_infantry_archer_levy","Khergit Archer (peasant levy)","Khergit Archers (peasant levy)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_ranged,0,0,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_chu_ko_nu,itm_eastern_arrows,itm_club,itm_khergit_bow,itm_steppe_cap,itm_nomad_cap,itm_leather_vest,itm_nomad_boots],levy_ranged_attrib|level(5),wp_melee(50)|wp_archery(90)|wp_crossbow(90),knows_common|knows_reflexes_1|knows_combat_training_1,khergit_face_younger_1,khergit_face_old_2],

["khergit_infantry_archer_recruit","Khergit Archer (recruit)","Khergit Archers (recruit)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_chu_ko_nu,itm_eastern_arrows,itm_sword_khergit_1,itm_sword_khergit_2,itm_sword_khergit_3,itm_sword_khergit_4,itm_khergit_bow,itm_nomad_bow,itm_xenoargh_lamellar_leather_variant001,itm_nomad_boots,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet,itm_vaegir_fur_cap,itm_nomad_cap],recruit_ranged_attrib|str_12|level(10),wp_melee(70)|wp_archery(125)|wp_crossbow(125),knows_common|knows_reflexes_3|knows_combat_training_3,khergit_face_younger_1,khergit_face_old_2],

["khergit_infantry_archer_regular","Khergit Archer (regular)","Khergit Archers (regular)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_chu_ko_nu,itm_eastern_arrows,itm_sword_khergit_1,itm_sword_khergit_2,itm_sword_khergit_3,itm_sword_khergit_4,itm_strong_bow,itm_xenoargh_lamellar_breastplate_variant002,itm_nomad_boots,itm_vaegir_lamellar_helmet,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet],regular_ranged_attrib|level(20),wp_melee(80)|wp_archery(175)|wp_crossbow(175),knows_common|knows_reflexes_5|knows_combat_training_5,khergit_face_younger_1,khergit_face_old_2],

["khergit_infantry_archer_elite","Khergit Archer (elite)","Khergit Archers (elite)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_chu_ko_nu,itm_universal_cannon_ammo,itm_xenoargh_rocket_projector,itm_eastern_arrows,itm_sword_khergit_1,itm_sword_khergit_2,itm_sword_khergit_3,itm_sword_khergit_4,itm_strong_bow,itm_xenoargh_lamellar_breastplate_variant002,itm_nomad_boots,itm_vaegir_lamellar_helmet,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet],elite_ranged_attrib|level(30),wp_melee(80)|wp_archery(200)|wp_crossbow(200),knows_common|knows_reflexes_7|knows_combat_training_7,khergit_face_younger_1,khergit_face_old_2],

["khergit_infantry_archer_expert","Khergit Archer (expert)","Khergit Archers (expert)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_chu_ko_nu,itm_universal_cannon_ammo,itm_xenoargh_rocket_projector,itm_eastern_arrows,itm_sword_khergit_1,itm_sword_khergit_2,itm_sword_khergit_3,itm_sword_khergit_4,itm_strong_bow,itm_xenoargh_lamellar_breastplate_variant002,itm_nomad_boots,itm_vaegir_lamellar_helmet,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet],expert_ranged_attrib|level(40),wp_melee(80)|wp_archery(250)|wp_crossbow(250),knows_common|knows_reflexes_9|knows_combat_training_9,khergit_face_younger_1,khergit_face_old_2],

#Khergit Nanigata Infantry
#These are fairly weak, cheap polearm troops, but they get a fairly nice high-level Elite.  And hand grenades.
["khergit_infantry_nanigata_levy","Khergit Spearman (peasant levy)","Khergit Spearmen (peasant levy)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_khergits,[itm_hafted_blade_a,itm_hafted_blade_b,itm_lui_khegitnagita,itm_steppe_cap,itm_nomad_cap,itm_steppe_armor,itm_nomad_boots],levy_hth_attrib|level(5),wp_melee(90),knows_common|knows_reflexes_1|knows_toughness_1|knows_combat_training_1,khergit_face_younger_2,khergit_face_old_1],

["khergit_infantry_nanigata_recruit","Khergit Spearman (recruit)","Khergit Spearmen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_khergits,[itm_hafted_blade_a,itm_hafted_blade_b,itm_lui_khegitnagita,itm_nomad_cap,itm_vaegir_fur_cap,itm_xenoargh_lamellar_breastplate_variant003,itm_nomad_boots],recruit_hth_attrib|level(10),wp_melee(125),knows_common|knows_reflexes_2|knows_toughness_2|knows_combat_training_1,khergit_face_younger_2,khergit_face_old_1],

["khergit_infantry_nanigata_regular","Khergit Spearman (regular)","Khergit Spearmen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_khergits,[itm_xenoargh_troop_grenade,itm_hafted_blade_a,itm_hafted_blade_b,itm_lui_khegitnagita,itm_xenoargh_lamellar_breastplate_variant004,itm_khergit_guard_boots,itm_lamellar_helmet_b],regular_hth_attrib|level(20),wp_melee(150)|wp_throwing(125),knows_common|knows_reflexes_5|knows_toughness_4|knows_combat_training_2,khergit_face_younger_2,khergit_face_old_1],

["khergit_infantry_nanigata_elite","Khergit Spearman (elite)","Khergit Spearmen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_khergits,[itm_xenoargh_troop_grenade,itm_hafted_blade_a,itm_hafted_blade_b,itm_lui_khegitnagita,itm_xenoargh_lamellar_breastplate_variant004,itm_khergit_guard_boots,itm_lamellar_helmet_a],elite_hth_attrib|level(30),wp_melee(175)|wp_throwing(125),knows_common|knows_reflexes_8|knows_toughness_6|knows_combat_training_6,khergit_face_younger_2,khergit_face_old_1],

#Khergit Poleaxe Infantry
#These are fairly decent polearm infantry.
["khergit_infantry_polearm_recruit","Khergit Halberdier (recruit)","Khergit Halberdiers (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_khergits,[itm_lui_kjergithallberd,itm_lui_kjergithallberdstrong,itm_steppe_cap,itm_nomad_cap,itm_xenoargh_lamellar_leather_variant005,itm_nomad_boots],recruit_hth_attrib|level(10),wp_melee(125),knows_common|knows_reflexes_1|knows_toughness_1|knows_combat_training_1,khergit_face_younger_2,khergit_face_old_1],

["khergit_infantry_polearm_regular","Khergit Halberdier (regular)","Khergit Halberdiers (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_kingdom_of_khergits,[itm_xenoargh_troop_grenade,itm_lui_kjergithallberd,itm_lui_kjergithallberdstrong,itm_nomad_cap,itm_vaegir_fur_cap,itm_xenoargh_lamellar_breastplate_variant006,itm_nomad_boots],regular_hth_attrib|level(15),wp_melee(150)|wp_throwing(100),knows_common|knows_reflexes_4|knows_toughness_2|knows_combat_training_4,khergit_face_younger_2,khergit_face_old_1],

["khergit_infantry_polearm_elite","Khergit Halberdier (elite)","Khergit Halberdiers (elite)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_khergits,[itm_xenoargh_troop_grenade,itm_lui_kjergithallberd,itm_lui_kjergithallberdstrong,itm_xenoargh_lamellar_breastplate_variant001,itm_nomad_boots,itm_vaegir_lamellar_helmet,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet],elite_hth_attrib|level(30),wp_melee(175)|wp_throwing(125),knows_common|knows_reflexes_6|knows_toughness_3|knows_combat_training_6,khergit_face_younger_2,khergit_face_old_1],

#Khergit Archer Cavalry
#The Khergits wouldn't be Khergits without their swarms of cheap cavalry.
#Like the Infantry Archer class, they start with some fairly crappy troops at the low end, with a strong finish.
#They are the only Khergit tree that gets an Expert class, and their Experts are the cheapest in the game.
["khergit_cavalry_archer_recruit","Khergit Horse Archer (recruit)","Khergit Horse Archers (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_horse,0,0,fac_kingdom_of_khergits,[itm_eastern_arrows,itm_club,itm_hunting_bow,itm_steppe_cap,itm_nomad_cap,itm_leather_vest,itm_nomad_boots,itm_steppe_horse],recruit_ranged_attrib|level(10),wp_melee(50)|wp_archery(90),knows_riding_5|knows_horse_archery_5|knows_reflexes_2|knows_combat_training_2,khergit_face_younger_1,khergit_face_old_2],

["khergit_cavalry_archer_regular","Khergit Horse Archer (regular)","Khergit Horse Archers (regular)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_kingdom_of_khergits,[itm_eastern_arrows,itm_sword_khergit_1,itm_sword_khergit_2,itm_sword_khergit_3,itm_sword_khergit_4,itm_nomad_bow,itm_xenoargh_lamellar_breastplate_variant001,itm_nomad_boots,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet,itm_vaegir_fur_cap,itm_nomad_cap,itm_steppe_horse],regular_ranged_attrib|str_12|level(10),wp_melee(70)|wp_archery(125),knows_riding_7|knows_horse_archery_7|knows_reflexes_5|knows_combat_training_5,khergit_face_younger_1,khergit_face_old_2],

["khergit_cavalry_archer_expert","Khergit Horse Archer (expert)","Khergit Horse Archers (expert)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse,0,0,fac_kingdom_of_khergits,[itm_eastern_arrows,itm_sword_khergit_1,itm_sword_khergit_2,itm_sword_khergit_3,itm_sword_khergit_4,itm_strong_bow,itm_nomad_bow,itm_xenoargh_lamellar_breastplate_variant002,itm_nomad_boots,itm_vaegir_lamellar_helmet,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet,itm_steppe_horse],expert_ranged_attrib|level(35),wp_melee(125)|wp_archery(200),knows_riding_10|knows_horse_archery_10|knows_reflexes_8|knows_combat_training_7,khergit_face_younger_1,khergit_face_old_2],

#Khergit Lancer Cavalry
#These are mixed-arms irregulars, with lances, maces and swords.
#They're generally better-armored than the other Khergit classes.  High-end Elite has armored warhorse, and is really scary.
["khergit_lancer_recruit","Khergit Lancer (recruit)","Khergit Lancers (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_spear_gun,itm_eastern_arrows,itm_light_lance,itm_winged_mace,itm_spear,itm_lance,itm_lance,itm_nomad_bow,itm_sword_khergit_2,itm_xenoargh_shield_small03,itm_xenoargh_shield_small01,itm_spear,itm_leather_steppe_cap_a,itm_leather_steppe_cap_b,itm_nomad_robe,itm_nomad_vest,itm_nomad_boots,itm_hide_boots,itm_spiked_helmet,itm_nomad_cap,itm_steppe_horse,itm_hunter],recruit_hth_attrib|level(15),wp_melee(125)|wp_archery(100)|wp_crossbow(100),knows_riding_5|knows_horse_archery_3|knows_reflexes_3|knows_combat_training_2|knows_toughness_1,khergit_face_young_1,khergit_face_older_2],

["khergit_lancer_regular","Khergit Lancer (regular)","Khergit Lancers (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_spear_gun,itm_eastern_arrows,itm_sword_khergit_2,itm_winged_mace,itm_spear,itm_winged_mace,itm_spear,itm_lance,itm_lance,itm_khergit_bow,itm_xenoargh_shield_small03,itm_xenoargh_shield_small03,itm_xenoargh_shield_small01,itm_eastern_arrows,itm_eastern_arrows,itm_javelin,itm_leather_steppe_cap_b,itm_xenoargh_lamellar_breastplate_variant003,itm_xenoargh_lamellar_breastplate_variant004,itm_hide_boots,itm_xenoargh_shield_small03,itm_xenoargh_shield_small01,itm_steppe_horse],regular_hth_attrib|level(25),wp_melee(150)|wp_archery(110)|wp_crossbow(120),knows_riding_7|knows_horse_archery_5|knows_reflexes_4|knows_combat_training_4|knows_toughness_2,khergit_face_young_1,khergit_face_older_2],

["khergit_lancer_elite","Khergit Lancer (elite)","Khergit Lancers (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_khergits,[itm_bolts,itm_xenoargh_spear_gun,itm_eastern_arrows,itm_sword_khergit_4,itm_winged_mace,itm_spear,itm_lance,itm_lance,itm_khergit_bow,itm_strong_bow,itm_short_bow,itm_eastern_arrows,itm_eastern_arrows,itm_khergit_elite_armor,itm_hide_boots,itm_lamellar_gauntlets,itm_xenoargh_shield_small01,itm_xenoargh_shield_small02,itm_warhorse_steppe],elite_hth_attrib|level(45),wp_melee(200)|wp_archery(140)|wp_crossbow(160),knows_riding_9|knows_horse_archery_7|knows_reflexes_5|knows_combat_training_5|knows_toughness_3,khergit_face_middle_1,khergit_face_older_2],

#STOCK KHERGIT CHARACTERS
["khergit_messenger","Khergit Messenger","Khergit Messengers",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_kingdom_of_khergits,[itm_sword_khergit_2,itm_leather_jerkin,itm_leather_boots,itm_courser,itm_leather_gloves,itm_short_bow,itm_eastern_arrows],def_attrib|agi_21|level(25),wp_melee(130),knows_common|knows_riding_7|knows_horse_archery_5,khergit_face_young_1,khergit_face_older_2],

["khergit_deserter","Khergit Deserter","Khergit Deserters",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_deserters,[itm_eastern_arrows,itm_spiked_mace,itm_axe,itm_sword_khergit_1,itm_short_bow,itm_short_bow,itm_hunting_bow,itm_javelin,itm_javelin,itm_steppe_cap,itm_nomad_cap,itm_leather_vest,itm_leather_vest,itm_nomad_armor,itm_nomad_boots],def_attrib|str_10|level(14),wp_melee(80),knows_toughness_1,khergit_face_young_1,khergit_face_older_2],

["khergit_prison_guard","Prison Guard","Prison Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_khergits,[itm_sword_khergit_3,itm_xenoargh_shield_small01,itm_xenoargh_shield_small03,itm_lamellar_vest_khergit,itm_lamellar_armor,itm_xenoargh_lamellar_breastplate_variant006,itm_hide_boots,itm_iron_greaves,itm_leather_warrior_cap],def_attrib|level(24),wp_melee(130),knows_reflexes_3|knows_toughness_3,khergit_face_middle_1,khergit_face_older_2],

["khergit_castle_guard","Castle Guard","Castle Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_khergits,[itm_sword_khergit_4,itm_xenoargh_shield_small01,itm_xenoargh_shield_small03,itm_lamellar_vest_khergit,itm_lamellar_armor,itm_xenoargh_lamellar_breastplate_variant003,itm_hide_boots,itm_iron_greaves,itm_leather_warrior_cap],def_attrib|level(24),wp_melee(130),knows_reflexes_3|knows_toughness_3,khergit_face_middle_1,khergit_face_older_2],

#NORDS
#Viking / Breton / Irish mixture,with emphasis on 1066-era troops.
#I have taken great liberties with historical accuracy in the interest of Fun.
#The Valkyries and Beserkirs are made of whole cloth, totally silly (but fun),but otherwise the Nords make more sense now.
#You will note that certain Nords get horses, but aren't garaunteed them.
#This is historically accurate.Vikings often rode horses, and used them in combat,but didn't get into cavalry like other cultures.
#The Nords have been generally rebalanced to have the best cheap infantry and medium infantry in the game, and the most spectacular specialists.
#The Nords are also the only Official Culture that get a female tree, the Valkyries.
#Basically though, the Nords are specialists with medium-inf 'glue' and do not have the heavy-heavy troops of the other factions.

#Nord Archers
#These are the finest archers in the land. Period. This is because they are masters of the mighty longbow and warbow.
#They are also fairly inexpensive, because their equipment is never quite supurb...
#...until you hit Expert, which is a bit painful- but their Elites are quite nice enough.
#That said, if you want to see an arrow-storm in Blood and Steel, they are the way to go, and can be quite devastating.
["nord_archer_recruit","Nord Longbowman (recruit)","Nord Longbowmen (recruit)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_nords,[itm_arrows,itm_axe,itm_long_bow,itm_leather_jerkin,itm_shirt,itm_leather_boots,itm_leather_gloves,itm_nasal_helmet,itm_xenoargh_viking_horned_helm04,itm_leather_cap],recruit_ranged_attrib|level(10),wp_melee(80)|wp_archery(100),knows_toughness_1|knows_reflexes_3,nord_face_young_1,nord_face_old_2],

["nord_archer_regular","Nord Longbowman (regular)","Nord Longbowmen (regular)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_nords,[itm_arrows,itm_one_handed_war_axe_a,itm_sword_viking_1,itm_long_bow,itm_war_bow,itm_leather_jerkin,itm_padded_leather,itm_leather_boots,itm_leather_gloves,itm_nasal_helmet,itm_xenoargh_viking_horned_helm04,itm_xenoargh_viking_horned_helm03,itm_leather_cap],regular_ranged_attrib|level(15),wp_melee(80)|wp_archery(150),knows_toughness_2|knows_reflexes_4|knows_combat_training_1,nord_face_middle_1,nord_face_older_2],

["nord_archer_elite","Nord Longbowman (elite)","Nord Longbowmen (elite)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_arrows,itm_one_handed_war_axe_a,itm_sword_viking_1,itm_war_bow,itm_padded_leather,itm_byrnie,itm_leather_boots,itm_leather_gloves,itm_nasal_helmet,itm_xenoargh_viking_horned_helm03,itm_xenoargh_viking_horned_helm02,itm_leather_cap],elite_ranged_attrib|level(25),wp_melee(100)|wp_archery(200),knows_toughness_2|knows_reflexes_5|knows_combat_training_3,nord_face_middle_1,nord_face_older_2],

["nord_archer_expert","Nord Longbowman (expert)","Nord Longbowmen (expert)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_arrows,itm_one_handed_war_axe_a,itm_sword_viking_1,itm_war_bow,itm_padded_leather,itm_byrnie,itm_leather_boots,itm_splinted_leather_greaves,itm_leather_gloves,itm_mail_mittens,itm_xenoargh_viking_horned_helm01,itm_xenoargh_viking_horned_helm02],expert_ranged_attrib|level(40),wp_melee(120)|wp_archery(250),knows_toughness_3|knows_reflexes_6|knows_combat_training_5,nord_face_middle_1,nord_face_older_2],

#Nord Assault Infantry
#These are fast medium infantry, armed with two-handed axes.Nasty in numbers, because of their speed and powerful attacks.
#The important part about them, however, is that this is the path to the BESERKIRS, who are some of the deadliest infantry in the game, period.
["nord_axeman_levy","Nord Axeman (peasant levy)","Nord Axemen (peasant levy)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_kingdom_of_nords,[itm_bolts,itm_xenoargh_gun_axe,itm_lui_vikingaxebastard,itm_lui_vikingaxeoneh,itm_lui_vikingaxeonehb,itm_lui_vikingaxeonehc,itm_axe,itm_hatchet,itm_battle_axe,itm_throwing_axes,itm_war_axe,itm_round_shield01_viking01,itm_blue_tunic,itm_shirt,itm_coarse_tunic,itm_hide_boots,itm_nomad_boots,itm_leather_cap,itm_skullcap],levy_hth_attrib|level(5),wp_melee(50)|wp_throwing(75)|wp_crossbow(75),knows_reflexes_1|knows_combat_training_1,nord_face_younger_1,nord_face_old_2],

["nord_axeman_recruit","Nord Axeman (recruit)","Nord Axemen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_kingdom_of_nords,[itm_bolts,itm_xenoargh_gun_axe,itm_lui_vikingaxebastard,itm_lui_vikingaxeoneh,itm_lui_vikingaxeonehb,itm_lui_vikingaxeonehc,itm_fighting_axe,itm_one_handed_war_axe_a,itm_axe,itm_hatchet,itm_battle_axe,itm_war_axe,itm_round_shield01_viking01,itm_throwing_axes,itm_mail_hauberk,itm_byrnie,itm_haubergeon,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_horned_helm01,itm_xenoargh_viking_horned_helm04],recruit_hth_attrib|level(20),wp_melee(70)|wp_throwing(100)|wp_crossbow(100),knows_toughness_2|knows_reflexes_2|knows_combat_training_2,nord_face_young_1,nord_face_old_2],

["nord_axeman_regular","Nord Axeman (regular)","Nord Axemen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_bolts,itm_xenoargh_gun_axe,itm_lui_vikingaxebastard,itm_lui_vikingaxeoneh,itm_lui_vikingaxeonehb,itm_lui_vikingaxeonehc,itm_fighting_axe,itm_one_handed_war_axe_a,itm_axe,itm_hatchet,itm_battle_axe,itm_war_axe,itm_round_shield01_viking01,itm_throwing_axes,itm_mail_hauberk,itm_byrnie,itm_haubergeon,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_horned_helm01,itm_xenoargh_viking_horned_helm04,itm_narf_mail_gauntlets,itm_leather_gloves],regular_hth_attrib|level(25),wp_melee(125)|wp_throwing(140)|wp_crossbow(140),knows_toughness_3|knows_reflexes_3|knows_combat_training_3,nord_face_young_1,nord_face_old_2],

["nord_axeman_elite","Nord Axeman (elite)","Nord Axemen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_nords,[itm_bolts,itm_xenoargh_gun_axe,itm_lui_vikingaxebastard,itm_lui_vikingaxeoneh,itm_lui_vikingaxeonehb,itm_lui_vikingaxeonehc,itm_fighting_axe,itm_one_handed_war_axe_a,itm_axe,itm_hatchet,itm_battle_axe,itm_war_axe,itm_round_shield01_viking01,itm_throwing_axes,itm_pino_haubergeon_jco,itm_narf_kuyak_a,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_horned_helm01,itm_xenoargh_viking_horned_helm04,itm_narf_wisby_gauntlets_red,itm_narf_mail_gauntlets],elite_hth_attrib|level(35),wp_melee(175)|wp_throwing(200)|wp_crossbow(200),knows_toughness_4|knows_reflexes_4|knows_combat_training_4,nord_face_young_1,nord_face_old_2],

#Beserkirs use special attribs.Please don't use them elsewhere by copy-pasta accidents unless you really want everybody to be this fast, lol
["nord_beserkir","Beserkir (expert)","Beserkirs (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_gloves|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_bolts,itm_xenoargh_gun_axe,itm_throwing_axes,itm_heavy_throwing_axes,itm_throwing_axes,itm_heavy_throwing_axes,itm_throwing_axes,itm_heavy_throwing_axes,itm_mackie_celtic_axe,itm_round_shield01_viking04,itm_iron_greaves,itm_narf_wisby_gauntlets_red,itm_narf_mail_gauntlets,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_xenoargh_viking_steel_winged_helm02],beserkir_attrib|level(50),wp_melee(240)|wp_throwing(260)|wp_crossbow(260),knows_reflexes_5|knows_toughness_5|knows_combat_training_5,nord_face_middle_1,nord_face_older_2],

#Nord Spearmen. Reasonable fighting troops, with Piercing throwing weapons, but nothing spectacular. Less flukey than the Axemen, though.
["nord_spearman_recruit","Nord Spearman (recruit)","Nord Spearmen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_kingdom_of_nords,[itm_dejawolf_atgeirr,itm_dejawolf_bryntvari,itm_dejawolf_bryntvari2,itm_dejawolf_krokaspjott2,itm_dejawolf_krokaspjott1,itm_dejawolf_kastad_krokaspjott,itm_dejawolf_fjadraspjot,itm_dejawolf_hoggkesja,itm_dejawolf_kastspjottmidtaggir,itm_dejawolf_langr_bryntvari,itm_dejawolf_langr_hoggspjott1,itm_dejawolf_langr_svia,itm_dejawolf_spjotkesja,itm_dejawolf_svia2,itm_dejawolf_svia,itm_round_shield01_viking02,itm_throwing_axes,itm_mail_hauberk,itm_byrnie,itm_haubergeon,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_horned_helm01,itm_xenoargh_viking_horned_helm04],recruit_hth_attrib|level(20),wp_melee(70)|wp_throwing(120),knows_toughness_1|knows_reflexes_1|knows_combat_training_2,nord_face_young_1,nord_face_old_2],

["nord_spearman_regular","Nord Spearman (regular)","Nord Spearmen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_dejawolf_atgeirr,itm_dejawolf_bryntvari,itm_dejawolf_bryntvari2,itm_dejawolf_krokaspjott2,itm_dejawolf_krokaspjott1,itm_dejawolf_kastad_krokaspjott,itm_dejawolf_fjadraspjot,itm_dejawolf_hoggkesja,itm_dejawolf_kastspjottmidtaggir,itm_dejawolf_langr_bryntvari,itm_dejawolf_langr_hoggspjott1,itm_dejawolf_langr_svia,itm_dejawolf_spjotkesja,itm_dejawolf_svia2,itm_dejawolf_svia,itm_round_shield01_viking02,itm_throwing_axes,itm_mail_hauberk,itm_byrnie,itm_haubergeon,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_horned_helm01,itm_xenoargh_viking_horned_helm04,itm_leather_gloves,itm_mail_mittens,itm_narf_mail_gauntlets],regular_hth_attrib|level(25),wp_melee(150)|wp_throwing(170),knows_toughness_2|knows_reflexes_2|knows_combat_training_3,nord_face_young_1,nord_face_old_2],

["nord_spearman_elite","Nord Spearman (elite)","Nord Spearmen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_nords,[itm_xenoargh_troop_grenade,itm_dejawolf_atgeirr,itm_dejawolf_bryntvari,itm_dejawolf_bryntvari2,itm_dejawolf_krokaspjott2,itm_dejawolf_krokaspjott1,itm_dejawolf_kastad_krokaspjott,itm_dejawolf_fjadraspjot,itm_dejawolf_hoggkesja,itm_dejawolf_kastspjottmidtaggir,itm_dejawolf_langr_bryntvari,itm_dejawolf_langr_hoggspjott1,itm_dejawolf_langr_svia,itm_dejawolf_spjotkesja,itm_dejawolf_svia2,itm_dejawolf_svia,itm_round_shield01_viking02,itm_throwing_axes,itm_pino_peasant_leather_plate,itm_wei_xiadi_nord_cuir_bouilli,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_horned_helm02,itm_xenoargh_viking_horned_helm03,itm_mail_mittens,itm_narf_mail_gauntlets],elite_hth_attrib|level(35),wp_melee(200)|wp_throwing(220),knows_toughness_3|knows_reflexes_3|knows_combat_training_4,nord_face_middle_1,nord_face_old_2],

["nord_spearman_expert","Nord Spearman (expert)","Nord Spearmen (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_nords,[itm_xenoargh_troop_grenade,itm_dejawolf_atgeirr,itm_dejawolf_bryntvari,itm_dejawolf_bryntvari2,itm_dejawolf_krokaspjott2,itm_dejawolf_krokaspjott1,itm_dejawolf_kastad_krokaspjott,itm_dejawolf_fjadraspjot,itm_dejawolf_hoggkesja,itm_dejawolf_kastspjottmidtaggir,itm_dejawolf_langr_bryntvari,itm_dejawolf_langr_hoggspjott1,itm_dejawolf_langr_svia,itm_dejawolf_spjotkesja,itm_dejawolf_svia2,itm_dejawolf_svia,itm_round_shield01_viking02,itm_throwing_axes,itm_pino_peasant_leather_plate,itm_wei_xiadi_nord_cuir_bouilli,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_horned_helm02,itm_xenoargh_viking_horned_helm03,itm_mail_mittens,itm_narf_mail_gauntlets,itm_narf_wisby_gauntlets_red],expert_hth_attrib|level(50),wp_melee(270)|wp_throwing(300),knows_toughness_4|knows_reflexes_4|knows_combat_training_6,nord_face_middle_1,nord_face_old_2],

#Nord Huscarls. Irregulars- they are a mix of abilities, with axes, longbows, and swords. 
#Two levels, regular and elite.
["nord_huscarl_regular","Nord Huscarl (regular)","Nord Huscarl (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_ranged|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking03,itm_mail_hauberk,itm_byrnie,itm_haubergeon,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm01,itm_xenoargh_viking_steel_winged_helm04,itm_leather_gloves,itm_mail_mittens,itm_xenoargh_baydark,itm_xenoargh_blackleapord,itm_hunter],regular_hth_attrib|level(30),wp_melee(150)|wp_crossbow(130),knows_toughness_4|knows_reflexes_4|knows_combat_training_4|knows_riding_5,nord_face_young_1,nord_face_old_2],

["nord_huscarl_elite","Nord Huscarl (elite)","Nord Huscarl (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_shield|tf_guarantee_ranged|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_nords,[itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking03,itm_dejawolf_vikingbyrnie,itm_narf_kuyak_b,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_mail_mittens,itm_narf_mail_gauntlets,itm_narf_wisby_gauntlets_red,itm_warhorse_chain,itm_warhorse_chain_barding01],elite_hth_attrib|level(40),wp_melee(200)|wp_crossbow(170),knows_toughness_6|knows_reflexes_6|knows_combat_training_7|knows_riding_7,nord_face_middle_1,nord_face_old_2],

#Nord Valkyries. Irregular cavalry. Always armed with throwing darts or light axes.
#Always equipped with swords and shields and helmets.*NOT* guaranteed armor- in fact, they're frequently, er, nearly naked.
#However, that just makes them even more badass, right? Right.
#They are cheap because frankly I suspect they will die in droves due to lack of armor.
#But they have one *hell* of a final jump, basically becoming Hell's Angels (or, uh, Odin's Seekers) at the high end.
#They aren't truly competitive with their closest rivals (Mercenary Sword Sisters) but they are nasty, and avoid the double-whammy cost hit for Elite Mounted Mercs.
#Between them, Huscarls, Beserkirs and the archers, let alone the regulars, the Nords are in it to win it.
["nord_valkyrie_regular","Nord Valkyrie (regular)","Nord Valkerie (regular)",tf_mounted|tf_female|tf_guarantee_ranged|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_armor,0,0,fac_kingdom_of_nords,[itm_darts,itm_anar_dress,itm_throwing_axes,itm_heavy_throwing_axes,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_octopus,itm_throwing_axes,itm_xenoargh_viking_gold_winged_helm01,itm_xenoargh_viking_gold_winged_helm04,itm_leather_gloves,itm_mail_mittens,itm_saddle_horse,itm_sumpter_horse,itm_xenoargh_bayappy,itm_xenoargh_baydark],regular_ranged_attrib|level(10),wp_melee(150)|wp_throwing(130),knows_toughness_3|knows_reflexes_3|knows_combat_training_7|knows_riding_3,european_woman_face_1,european_woman_face_2],

["nord_valkyrie_elite","Nord Valkyrie (elite)","Nord Valkerie (elite)",tf_mounted|tf_female|tf_guarantee_ranged|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_nords,[itm_war_darts,itm_blademaster_armor,itm_xena_boots,itm_throwing_axes,itm_heavy_throwing_axes,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_octopus,itm_throwing_axes,itm_xenoargh_viking_gold_winged_helm01,itm_xenoargh_viking_gold_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_hunter,itm_xenoargh_buckskin],elite_ranged_attrib|level(20),wp_melee(200)|wp_throwing(170),knows_toughness_4|knows_reflexes_4|knows_combat_training_8|knows_riding_6,european_woman_face_1,european_woman_face_2],

#Note: does not follow previous trend, this troop is a major jump.
["nord_valkyrie_expert","Nord Seeker of the Slain (expert)","Nord Seekers of the Slain (expert)",tf_mounted|tf_female|tf_guarantee_ranged|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_boots,0,0,fac_kingdom_of_nords,[itm_dthehun_new_battle_armor01,itm_xena_boots,itm_war_darts,itm_war_darts,itm_throwing_axes,itm_heavy_throwing_axes,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_octopus,itm_throwing_axes,itm_xenoargh_viking_gold_winged_helm01,itm_xenoargh_viking_gold_winged_helm02,itm_leather_gloves,itm_mail_mittens,itm_lamellar_gauntlets,itm_warhorse_chain,itm_warhorse_chain_barding02],beserkir_attrib|level(50),wp_melee(300)|wp_throwing(250),knows_toughness_8|knows_reflexes_8|knows_combat_training_10|knows_riding_8,european_woman_face_1,european_woman_face_2],

#The Valgarde.  For when you really absolutely must kill everything in sight.  They're basically Beserkirs with heavy armor, practically unstoppable.  They form the core of a Nord's retinue.
["nord_valgarde","Valgarde (Champion)","Valgarde (Champion)",tf_mounted|tf_guarantee_boots|tf_guarantee_gloves|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_bolts,itm_xenoargh_gun_axe,itm_mackie_bearded_axe,itm_round_shield01_viking04,itm_iron_greaves,itm_narf_wisby_gauntlets_black,itm_ssh_valgarde_armor,itm_dejawolf_valgarde_helm],beserkir_attrib|level(75),wp_melee(350),knows_reflexes_8|knows_toughness_8|knows_combat_training_8,nord_face_middle_1,nord_face_older_2],

#STOCK NORD CHARACTERS
["nord_messenger","Nord Messenger","Nord Messengers",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_kingdom_of_nords,[itm_sword_viking_2,itm_leather_jerkin,itm_leather_boots,itm_courser,itm_leather_gloves,itm_short_bow,itm_arrows],def_attrib|agi_21|level(25),wp_melee(130),knows_common|knows_riding_7|knows_horse_archery_5,nord_face_younger_1,nord_face_younger_2],

["nord_deserter","Nord Deserter","Nord Deserters",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_deserters,[itm_arrows,itm_spiked_mace,itm_axe,itm_falchion,itm_short_bow,itm_short_bow,itm_hunting_bow,itm_javelin,itm_javelin,itm_steppe_cap,itm_nomad_cap,itm_leather_vest,itm_leather_vest,itm_nomad_armor,itm_nomad_boots],def_attrib|str_10|level(14),wp_melee(80),knows_toughness_1,nord_face_young_1,nord_face_young_2],

["nord_prison_guard","Prison Guard","Prison Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_ashwood_pike,itm_battle_fork,itm_battle_axe,itm_fighting_axe,itm_xenoargh_round_shield02,itm_mail_hauberk,itm_mail_chausses,itm_iron_greaves,itm_nordic_helmet,itm_nordic_helmet,itm_nordic_helmet,itm_spiked_helmet,itm_leather_gloves],def_attrib|level(24),wp_melee(130),knows_reflexes_3|knows_toughness_3,nord_face_middle_1,nord_face_middle_2],

["nord_castle_guard","Castle Guard","Castle Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_nords,[itm_ashwood_pike,itm_battle_fork,itm_battle_axe,itm_fighting_axe,itm_xenoargh_round_shield02,itm_xenoargh_shield_small04,itm_mail_hauberk,itm_rathos_surcoat_white,itm_mail_chausses,itm_iron_greaves,itm_nordic_helmet,itm_nordic_helmet,itm_nordic_helmet,itm_spiked_helmet,itm_leather_gloves],def_attrib|level(24),wp_melee(130),knows_reflexes_3|knows_toughness_3,nord_face_older_1,nord_face_older_2],


#RHODOKS
#A strong infantry / cavalry force, with Italian / Spanish influences.
#The Rhodoks are the masters of defensive warfare.Their pavise spearment and crossbows are equally dangerous on the field or in a castle.
#The Rhodoks are also the only Official Culture to have troops carrying pistols and rapiers, which gives them an interesting assault troop.
#They also have an unusual mixed class carrying bladed stave weapons and two-handed axes, giving them a medium-level punch.
#Lastly, the Rhodok Condottierri are the finest shock cavalry in the game, bar none, giving the Rhodoks many good choices.

#Rhodok spearmen are the backbone of Rhodok forces.
#They have one of the cheapest Experts available, and all but the levies are guaranteed shields.
#These are strong troops on the defensive, and much less of the "speed bump" they were in Native.
["rhodok_spearman_levy","Rhodok Spearman (peasant levy)","Rhodok Spearmen (peasant levy)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_rhodoks,[itm_battle_fork,itm_pitch_fork,itm_military_fork,itm_boar_spear,itm_war_spear,itm_awlpike,itm_jam_boarspear,itm_old_board_shield01,itm_shirt,itm_coarse_tunic,itm_wrapping_boots,itm_nomad_boots,itm_head_wrappings],levy_hth_attrib|level(5),wp_melee(55),knows_toughness_1|knows_combat_training_1,rhodok_face_younger_1,rhodok_face_younger_2],

["rhodok_spearman_recruit","Rhodok Spearman (recruit)","Rhodok Spearmen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_battle_fork,itm_pitch_fork,itm_military_fork,itm_boar_spear,itm_war_spear,itm_awlpike,itm_jam_boarspear,itm_old_board_shield01,itm_old_board_shield_rhodok,itm_leather_cap,itm_common_hood,itm_leather_vest,itm_leather_vest,itm_wrapping_boots,itm_nomad_boots],recruit_hth_attrib|level(10),wp_melee(80),knows_toughness_2|knows_combat_training_2,rhodok_face_young_1,rhodok_face_young_2],

["rhodok_spearman_regular","Rhodok Spearman (regular)","Rhodok Spearmen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_boar_spear,itm_war_spear,itm_awlpike,itm_jam_boarspear,itm_old_board_shield_rhodok,itm_leather_cap,itm_leather_vest,itm_ragged_outfit,itm_padded_cloth,itm_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_nomad_boots,itm_leather_gloves],regular_hth_attrib|level(15),wp_melee(105),knows_toughness_3|knows_combat_training_3,rhodok_face_middle_1,rhodok_face_middle_2],

["rhodok_spearman_elite","Rhodok Spearman (elite)","Rhodok Spearmen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_xenoargh_troop_grenade,itm_lui_manhunterspear,itm_lui_onehandedphalanxpike,itm_pavise_shield_plain,itm_dejawolf_kettlehat1,itm_dejawolf_kettlehat2,itm_dejawolf_kettlehat3,itm_dejawolf_kettlehat4,itm_leather_cap,itm_xenoargh_hard_leather02,itm_nomad_boots,itm_leather_gloves,itm_mail_mittens],elite_hth_attrib|level(20),wp_melee(115),knows_toughness_5|knows_combat_training_5,rhodok_face_old_1,rhodok_face_old_2],

["rhodok_spearman_expert","Rhodok Spearman (expert)","Rhodok Spearmen (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield|tf_guarantee_gloves,0,0,fac_kingdom_of_rhodoks,[itm_xenoargh_troop_grenade,itm_lui_manhunterspear,itm_lui_onehandedphalanxpike,itm_pavise_shield_fancy_cross,itm_jtm_sturmhaube_7b,itm_xenoargh_metal_leather02,itm_nomad_boots,itm_leather_gloves,itm_mail_mittens,itm_gauntlets],expert_hth_attrib|level(35),wp_melee(200),knows_toughness_9|knows_combat_training_6,rhodok_face_older_1,rhodok_face_older_2],

#The Rhodoks wouldn't be Rhodoks without their crossbows.
#They are, in fact, the only Official Culture that includes an extensive crossbow tree now.
#They are largely unchanged from the Native setup, other than giving them all godendags instead of a mixed bag, because that makes them reasonable anti-cavalry in a pinch.
["rhodok_crossbowman_recruit","Rhodok Crossbowman (recruit)","Rhodok Crossbowmen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged,0,0,fac_kingdom_of_rhodoks,[itm_club_with_spike_head,itm_old_board_shield01,itm_crossbow,itm_bolts,itm_leather_jerkin,itm_ragged_outfit,itm_nomad_boots],recruit_ranged_attrib|level(10),wp_melee(50)|wp_crossbow(100),knows_toughness_1|knows_combat_training_1,rhodok_face_younger_1,rhodok_face_younger_2],

["rhodok_crossbowman_regular","Rhodok Crossbowman (regular)","Rhodok Crossbowmen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_club_with_spike_head,itm_old_board_shield_rhodok,itm_old_board_shield01,itm_crossbow,itm_bolts,itm_leather_cap,itm_leather_jerkin,itm_ragged_outfit,itm_nomad_boots],regular_ranged_attrib|level(20),wp_melee(70)|wp_crossbow(150),knows_toughness_3|knows_combat_training_3,rhodok_face_young_1,rhodok_face_young_2],

["rhodok_crossbowman_elite","Rhodok Crossbowman (elite)","Rhodok Crossbowmen (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_club_with_spike_head,itm_pavise_shield_painted01,itm_heavy_crossbow,itm_bolts,itm_leather_cap,itm_leather_jerkin,itm_padded_leather,itm_nomad_boots],elite_ranged_attrib|level(30),wp_melee(100)|wp_crossbow(300),knows_toughness_5|knows_combat_training_4,rhodok_face_middle_1,rhodok_face_middle_2],

["rhodok_crossbowman_expert","Rhodok Sharpshooter (expert)","Rhodok Sharpshooters (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_club_with_spike_head,itm_pavise_shield_painted02,itm_sniper_crossbow,itm_bolts,itm_kettle_hat,itm_xenoargh_hard_leather01,itm_leather_boots],expert_ranged_attrib|level(40),wp_melee(110)|wp_crossbow(400),knows_toughness_6|knows_combat_training_6,rhodok_face_middle_1,rhodok_face_middle_2],

#Rhodok Pistoleros.
#These are light assault troops with a strong ranged component.
#They are the only Official Culture with pistols. They have only two upgrades.
["rhodok_pistolero_regular","Rhodok Pistolero (regular)","Rhodok Pistoleros (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_bolts,itm_flintlock_pistol,itm_lui_rapier,itm_lui_rapierb,itm_lui_rapierc,itm_lui_rapierd,itm_lui_rapiere,itm_lui_rapierg,itm_xenoargh_musketeer_hat_black01,itm_xenoargh_musketeer_hat_black02,itm_light_leather,itm_light_leather_boots,itm_old_board_shield01,itm_old_board_shield_rhodok,itm_pavise_shield_plain],regular_ranged_attrib|level(15),wp_melee(110)|wp_crossbow(150),knows_reflexes_4|knows_combat_training_4,rhodok_face_young_1,rhodok_face_young_2],

["rhodok_pistolero_elite","Rhodok Pistolero (elite)","Rhodok Pistoleros (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_bolts,itm_double_flintlock_pistol,itm_lui_rapier,itm_lui_rapierb,itm_lui_rapierc,itm_lui_rapierd,itm_lui_rapiere,itm_lui_rapierg,itm_xenoargh_musketeer_hat_red01,itm_xenoargh_musketeer_hat_red02,itm_brigandine_red,itm_splinted_leather_greaves,itm_pavise_shield_painted01],elite_ranged_attrib|level(30),wp_melee(150)|wp_crossbow(250),knows_reflexes_6|knows_combat_training_7,rhodok_face_middle_1,rhodok_face_middle_2],

#Rhodok Stormtroops.
#These are mixed-arms troops, with a bizarre variety of polearms.
#Unlike the rest of the Rhodok force, they don't have shields, so they tend to die like flies.
#They are cheap to start, though, and like the Swadian Halberdiers, offer a fine finish for the patient.
["rhodok_stormtroop_recruit","Rhodok Stormtroop (recruit)","Rhodok Stormtroops (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_rhodoks,[itm_bardiche,itm_great_bardiche,itm_voulge,itm_great_long_bardiche,itm_glaive,itm_jam_fauchard,itm_jam_voulge_b,itm_lui_glaiveb,itm_lui_glaivec,itm_lui_glaivenew,itm_rrr_glaive1,itm_rrr_glaive2,itm_rrr_glaive3,itm_leather_cap,itm_common_hood,itm_leather_vest,itm_leather_vest,itm_wrapping_boots,itm_nomad_boots],recruit_hth_attrib|level(10),wp_melee(80),knows_toughness_2|knows_combat_training_1,rhodok_face_younger_1,rhodok_face_younger_2],

["rhodok_stormtroop_regular","Rhodok Stormtroop (regular)","Rhodok Stormtroops (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_rhodoks,[itm_bardiche,itm_great_bardiche,itm_voulge,itm_great_long_bardiche,itm_glaive,itm_jam_fauchard,itm_jam_voulge_b,itm_lui_glaiveb,itm_lui_glaivec,itm_lui_glaivenew,itm_rrr_glaive1,itm_rrr_glaive2,itm_rrr_glaive3,itm_leather_cap,itm_wei_xiadi_surcoat_rhodok01,itm_wei_xiadi_surcoat_rhodok02,itm_nomad_boots,itm_leather_gloves],regular_hth_attrib|level(15),wp_melee(105),knows_toughness_4|knows_combat_training_2,rhodok_face_young_1,rhodok_face_young_2],

["rhodok_stormtroop_elite","Rhodok Stormtroop (elite)","Rhodok Stormtroops (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_rhodoks,[itm_bardiche,itm_great_bardiche,itm_voulge,itm_great_long_bardiche,itm_glaive,itm_jam_fauchard,itm_jam_voulge_b,itm_lui_glaiveb,itm_lui_glaivec,itm_lui_glaivenew,itm_rrr_glaive1,itm_rrr_glaive2,itm_rrr_glaive3,itm_dejawolf_kettlehat1,itm_dejawolf_kettlehat2,itm_dejawolf_kettlehat3,itm_dejawolf_kettlehat4,itm_leather_cap,itm_narf_corrazina_green,itm_nomad_boots,itm_leather_gloves,itm_mail_mittens],elite_hth_attrib|level(35),wp_melee(150)|wp_throwing(75),knows_toughness_6|knows_combat_training_3,rhodok_face_middle_1,rhodok_face_middle_2],

["rhodok_stormtroop_expert","Rhodok Stormtroop (expert)","Rhodok Stormtroops (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_rhodoks,[itm_bardiche,itm_great_bardiche,itm_voulge,itm_great_long_bardiche,itm_glaive,itm_jam_fauchard,itm_jam_voulge_b,itm_lui_glaiveb,itm_lui_glaivec,itm_lui_glaivenew,itm_rrr_glaive1,itm_rrr_glaive2,itm_rrr_glaive3,itm_jtm_sturmhaube_3,itm_narf_armour_stripes,itm_nomad_boots,itm_leather_gloves,itm_mail_mittens,itm_gauntlets],expert_hth_attrib|level(45),wp_melee(220)|wp_throwing(125),knows_toughness_8|knows_combat_training_4,rhodok_face_older_1,rhodok_face_older_2],

#Rhodok Condottieri
#When you care enough to send the very best...
#These are, without any doubt, the heaviest cavalry available to any Official Culture.
#They are also terrifically expensive. But for the price, you get the best of the best.
["rhodok_condottieri_elite","Rhodok Condottiere (elite)","Rhodok Condottieri (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_pop_gothic_lance_blue,itm_pop_gothic_lance_green,itm_pop_gothic_lance_blue,itm_pop_gothic_lance_green,itm_pop_gothic_lance_blue,itm_pop_gothic_lance_green,itm_hammer,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_narf_corrazina_grey,itm_narf_corrazina_red,itm_iron_greaves,itm_narf_plate_mittens,itm_gauntlets,itm_narf_open_sallet_coif,itm_narf_open_sallet,itm_mail_coif,itm_charger,itm_warhorse_chain_barding05,itm_warhorse_chain_barding06],elite_hth_attrib|level(50),wp_melee(175),knows_riding_5|knows_toughness_8|knows_combat_training_4,rhodok_face_young_1,rhodok_face_young_2],

["rhodok_condottieri_expert","Rhodok Condottiere (expert)","Rhodok Condottieri (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_rhodoks,[itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_narf_milanese_plate,itm_charger_plate],expert_hth_attrib|level(65),wp_melee(225),knows_riding_7|knows_toughness_10|knows_combat_training_6,rhodok_face_middle_1,rhodok_face_middle_2],

#STOCK RHODOK CHARACTERS
["rhodok_messenger","Rhodok Messenger","Rhodok Messengers",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_kingdom_of_rhodoks,[itm_sword_medieval_b,itm_leather_jerkin,itm_leather_boots,itm_courser,itm_leather_gloves,itm_short_bow,itm_eastern_arrows],def_attrib|agi_21|level(25),wp_melee(130),knows_common|knows_riding_7|knows_horse_archery_5,rhodok_face_young_1,rhodok_face_young_2],

["rhodok_deserter","Rhodok Deserter","Rhodok Deserters",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_deserters,[itm_eastern_arrows,itm_spiked_mace,itm_axe,itm_falchion,itm_short_bow,itm_short_bow,itm_hunting_bow,itm_javelin,itm_javelin,itm_steppe_cap,itm_nomad_cap,itm_leather_vest,itm_leather_vest,itm_nomad_armor,itm_nomad_boots],def_attrib|str_10|level(14),wp_melee(80),knows_toughness_1,rhodok_face_middle_1,rhodok_face_middle_2],

["rhodok_prison_guard","Prison Guard","Prison Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_rhodoks,[itm_ashwood_pike,itm_battle_fork,itm_battle_axe,itm_fighting_axe,itm_old_board_shield_rhodok,itm_mail_hauberk,itm_byrnie,itm_mail_chausses,itm_iron_greaves,itm_guard_helmet,itm_leather_gloves],def_attrib|level(24),wp_melee(130),knows_reflexes_3|knows_toughness_3,rhodok_face_middle_1,rhodok_face_middle_2],

["rhodok_castle_guard","Castle Guard","Castle Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_rhodoks,[itm_ashwood_pike,itm_battle_fork,itm_battle_axe,itm_fighting_axe,itm_pavise_shield_plain,itm_mail_hauberk,itm_byrnie,itm_mail_chausses,itm_iron_greaves,itm_guard_helmet,itm_leather_gloves],def_attrib|level(24),wp_melee(130),knows_reflexes_3|knows_toughness_3,rhodok_face_older_1,rhodok_face_older_2],


#SARRANIDS
#A strong mixed-arms faction, with strong differences in style from the Swadians and Rhodoks.
#The Sarranids are roughly based on Arab/Turk/Persian troops, but I've taken a lot of liberties.
#They feature weak-but-cheap irregulars- mix of missiles and hand weapons, quite nasty in bunches,
#Missile troops, starting with weak bowmen but going quickly to firearms, in the form of the exceedingly-dangerous Jezzailis,
#Strong-but-expensive infantry swordsmen, armed with crushing swords (cleavers),
#Strong cavalry forces, which ride camels past the Recruit level, and are very dangerous in masses,
#And last-but-not least, their gimmick troops, the Afrizarians, who are freaky-fast guys with spears and shields.
#They also have a specialist troop, at the top of the sword tree, the Assassin,
#And last but not least the Mamlukes get a high end character that is pretty nice.
#Like the Swadians and Rhodoks, the Sarranids have a broad mix of arms, and can be expected to handle most other factions well.

#Sarranid Irregulars
#These are mixed-arms troops, with hand weapons, shields, and missile weapons in a large variety.
#They aren't guaranteed missile weapons, but will have them frequently.
#They are fairly weak early, although the missiles have good effect during siege assaults.
#Top end is fairly nasty, though, with medium armors.
["sarranid_irregular_levy","Sarranid Irregular (peasant levy)","Sarranid Irregulars (peasant levy)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_sarranids,[itm_javelin,itm_darts,itm_war_darts,itm_javelin,itm_darts,itm_war_darts,itm_scythe,itm_hatchet,itm_pickaxe,itm_club,itm_heater_shield_swadian,itm_sarranid_felt_hat,itm_turban,itm_sarranid_boots_a,itm_sarranid_cloth_robe,itm_sarranid_cloth_robe_b],levy_hth_attrib|level(4),wp_melee(60)|wp_throwing(130),knows_reflexes_1,sarranid_face_younger_1,sarranid_face_younger_2],

["sarranid_irregular_recruit","Sarranid Irregular (recruit)","Sarranid Irregulars (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_kingdom_of_sarranids,[itm_javelin,itm_darts,itm_war_darts,itm_javelin,itm_darts,itm_war_darts,itm_sarranid_axe_a,itm_sarranid_axe_b,itm_lui_kjergitaxe,itm_lui_warbastardaxea,itm_xenoargh_shield_kite01_06,itm_desert_turban,itm_sarranid_leather_armor,itm_turban,itm_sarranid_boots_a,itm_sarranid_boots_b],recruit_hth_attrib|level(10),wp_melee(75)|wp_throwing(160),knows_reflexes_4,sarranid_face_young_1,sarranid_face_young_2],

["sarranid_irregular_regular","Sarranid Irregular (regular)","Sarranid Irregulars (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_kingdom_of_sarranids,[itm_xenoargh_troop_grenade,itm_javelin,itm_darts,itm_war_darts,itm_javelin,itm_darts,itm_war_darts,itm_sarranid_axe_a,itm_sarranid_axe_b,itm_lui_kjergitaxe,itm_lui_warbastardaxea,itm_sarranid_mail_coif,itm_xenoargh_shield_kite01_05,itm_sarranid_boots_b,itm_sarranid_leather_armor],regular_hth_attrib|level(20),wp_melee(125)|wp_throwing(180),knows_reflexes_6,sarranid_face_young_1,sarranid_face_young_2],

["sarranid_irregular_elite","Sarranid Irregular (elite)","Sarranid Irregular (elite)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_sarranids,[itm_xenoargh_troop_grenade,itm_javelin,itm_darts,itm_war_darts,itm_javelin,itm_darts,itm_war_darts,itm_sarranid_mail_shirt,itm_sarranid_warrior_cap,itm_sarranid_boots_c,itm_sarranid_boots_b,itm_sarranid_axe_a,itm_sarranid_axe_b,itm_lui_kjergitaxe,itm_lui_warbastardaxea,itm_xenoargh_shield_kite01_01],elite_hth_attrib|level(30),wp_melee(150)|wp_throwing(200),knows_reflexes_8,sarranid_face_middle_1,sarranid_face_middle_2],

#The Immortals.  A big step up.  Pure HTH fury.
["sarranid_irregular_expert","Sarranid Immortal (expert)","Sarranid Immortals (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_kingdom_of_sarranids,[itm_xenoargh_bwanga,itm_sarranid_two_handed_axe_a,itm_sarranid_two_handed_axe_b,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_wei_xiadi_sar_plain_surcoat,itm_sar_helmet_immortal,itm_narf_wisby_gauntlets_black],expert_hth_attrib|level(40),wp_melee(275)|wp_throwing(250),knows_reflexes_10|knows_toughness_5|knows_heroism_2,sarranid_face_middle_1,sarranid_face_middle_2],

#Dervishes.  For when Immortals aren't quite cutting it.
["sarranid_dervish","Sarranid Dervish (champion)","Sarranid Dervishes (champion)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_gloves|tf_guarantee_shield,0,0,fac_kingdom_of_sarranids,[itm_xeno_grom,itm_spak_hermitage_shield_2,itm_sarranid_plate_boots,itm_youhou_assassin_armor,itm_sar_helmet_immortal,itm_narf_wisby_gauntlets_black],assassin_attrib|level(60),wp_melee(400),knows_reflexes_10|knows_toughness_8|knows_heroism_3,sarranid_face_middle_1,sarranid_face_middle_2],

#Sarranid Bowmen / Jezzails
#Weakly-armored bowmen, moving up to Jezzails at the Regular level.
#These are cheap-but-effective missile troops, and the Jezzaillis have the best range in the game, other than cannons, and the best accuracy.
["sarranid_archer_recruit","Sarranid Archer (recruit)","Sarranid Archers (recruit)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_sarranids,[itm_eastern_arrows,itm_nomad_bow,itm_arabian_sword_a,itm_archers_vest,itm_sarranid_boots_b,itm_sarranid_helmet1,itm_turban,itm_desert_turban],recruit_ranged_attrib|level(10),wp_melee(60)|wp_archery(100),knows_reflexes_2,sarranid_face_young_1,sarranid_face_young_2],

["sarranid_archer_regular","Sarranid Jezzaili (regular)","Sarranid Jezzailis (regular)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_sarranids,[itm_bolts,itm_jezzail,itm_arabian_sword_a,itm_archers_vest,itm_sarranid_boots_b,itm_sarranid_helmet1,itm_turban,itm_desert_turban],regular_ranged_attrib|level(25),wp_melee(80)|wp_crossbow(150),knows_reflexes_3,sarranid_face_middle_1,sarranid_face_middle_2],

["sarranid_archer_elite","Sarranid Jezzaili (elite)","Sarranid Jezzailis (elite)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_sarranids,[itm_bolts,itm_jezzail,itm_arabian_sword_a,itm_sarranid_mail_shirt,itm_sarranid_boots_b,itm_sarranid_helmet1,itm_turban,itm_desert_turban],elite_ranged_attrib|level(35),wp_melee(100)|wp_crossbow(200),knows_reflexes_4,sarranid_face_older_1,sarranid_face_older_2],

["sarranid_archer_expert","Sarranid Sniper (expert)","Sarranid Snipers (expert)",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_sarranids,[itm_bolts,itm_jezzail,itm_lui_cimeterre,itm_lui_cimeterreb,itm_wei_xiadi_sarranid_mamluk_robes,itm_sarranid_boots_b,itm_sarranid_helmet1,itm_turban,itm_desert_turban],expert_ranged_attrib|level(45),wp_melee(100)|wp_crossbow(275),knows_reflexes_7,sarranid_face_old_1,sarranid_face_old_2],

#Sarranid Swordsmen
#Odd swordsman class, starts at Regulars, armed soley with military cleavers / huge scimitars.
#Ends with the Assassin class, which is a gimmick troop.
#Assasins get a lot of hitpoints and damage bonuses (supernatural STR) and are thus hard to kill, and fast, and armed with unusual weapons.
["sarranid_swordsman_regular","Sarranid Swordsman (regular)","Sarranid Swordsmen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_kingdom_of_sarranids,[itm_military_cleaver_b,itm_military_cleaver_c,itm_khergit_sword_two_handed_a,itm_khergit_sword_two_handed_b,itm_lui_bastardcimitar,itm_lui_khergitexecutioner,itm_two_handed_cleaver,itm_sarranid_mail_coif,itm_xenoargh_shield_kite01_05,itm_sarranid_boots_b,itm_sarranid_leather_armor],regular_hth_attrib|level(20),wp_melee(125)|wp_throwing(80),knows_toughness_2|knows_reflexes_4,sarranid_face_young_1,sarranid_face_young_2],

["sarranid_swordsman_elite","Sarranid Swordsman (elite)","Sarranid Swordsmen (elite)",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_sarranids,[itm_military_cleaver_b,itm_military_cleaver_c,itm_khergit_sword_two_handed_a,itm_khergit_sword_two_handed_b,itm_lui_bastardcimitar,itm_lui_khergitexecutioner,itm_two_handed_cleaver,itm_sarranid_mail_shirt,itm_sarranid_warrior_cap,itm_sarranid_boots_c,itm_sarranid_boots_b,itm_sarranid_axe_a,itm_sarranid_axe_b,itm_lui_kjergitaxe,itm_lui_warbastardaxea,itm_xenoargh_shield_kite01_01],elite_hth_attrib|level(30),wp_melee(150)|wp_throwing(110),knows_toughness_4|knows_reflexes_6,sarranid_face_middle_1,sarranid_face_middle_2],

["sarranid_swordsman_expert","Sarranid Assassin (expert)","Sarranid Assassins (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_sarranids,[itm_xenoargh_bwanga,itm_xenoargh_bwanga,itm_youhou_assassin_sword,itm_lui_khergitlordexecutioner,itm_narf_splinted_greaves_spurs,itm_youhou_assassin_armor,itm_youhou_assassin_armor_red,itm_youhou_assassin_hood,itm_youhou_assassin_hood_red,itm_narf_wisby_gauntlets_black,itm_narf_wisby_gauntlets_red],assassin_attrib|level(55),wp_melee(250)|wp_throwing(200),knows_toughness_6|knows_reflexes_10|knows_combat_training_8,sarranid_face_older_1,sarranid_face_older_2],

#Sarranid Afrizarian Mercenaries
#These are mercenary troops from Afrizar, deep to the south of Calradia.
#These are a special spear class, with special stats.
#They are cheap, despite their stats, because they wear lighter armors at best, and frequently don't wear helmets.
["sarranid_afrizar_regular","Afrizarian Mercenary (regular)","Afrizarian Mercenaries (regular)",tf_mounted|tf_guarantee_shield,0,0,fac_kingdom_of_sarranids,[itm_jarid,itm_jarid,itm_jarid,itm_jarid,itm_bamboo_spear,itm_fur_covered_shield],afrizar_attrib|level(20),wp_melee(175)|wp_throwing(130),knows_toughness_2|knows_reflexes_10|knows_combat_training_2,face_afrizar_young_1,face_afrizar_young_2],

["sarranid_afrizar_elite","Afrizarian Mercenary (elite)","Afrizarian Mercenaries (elite)",tf_mounted|tf_guarantee_shield|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_kingdom_of_sarranids,[itm_jarid,itm_jarid,itm_jarid,itm_jarid,itm_bamboo_spear,itm_fur_covered_shield,itm_wei_xiadi_sar_leather_armor,itm_sarranid_mail_coif,itm_sarranid_boots_b,itm_leather_gloves],afrizar_attrib|level(25),wp_melee(200)|wp_throwing(180),knows_toughness_4|knows_reflexes_10|knows_combat_training_4,face_afrizar_young_1,face_afrizar_young_2],

#These guys are a big step up.  Afrizars with decent armor and big maces.  Fairly scary.
["sarranid_afrizar_expert","Afrizarian Mercenary (expert)","Afrizarian Mercenaries (expert)",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_sarranids,[itm_jarid,itm_sarranid_two_handed_mace_1,itm_fur_covered_shield,itm_wei_xiadi_sar_leather_chain,itm_sarranid_mail_coif,itm_sarranid_warrior_cap,itm_sarranid_boots_c,itm_sarranid_boots_b,itm_narf_wisby_gauntlets_black],afrizar_attrib|level(45),wp_melee(225)|wp_throwing(200),knows_toughness_6|knows_reflexes_10|knows_combat_training_6,face_afrizar_young_1,face_afrizar_young_2],

#Sarranid Horse / Camel riders.
#These are comparatively lighter than Rhodoks / Swadians, but camels are nasty beasts and make them very dangerous in the field. 
["sarranid_horseman_regular","Sarranid Horseman (regular)","Sarranid Horsemen (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_sarranids,[itm_lance,itm_arabian_sword_b,itm_sarranid_cavalry_sword,itm_scimitar,itm_scimitar_b,itm_lui_cimeterre,itm_lui_cimeterreb,itm_lui_khergitexecutioneronehand,itm_xenoargh_shield_small01,itm_sarranid_mail_shirt,itm_wei_xiadi_sar_hauberk,itm_wei_xiadi_sar_byrnie,itm_sarranid_boots_c,itm_sarranid_boots_b,itm_sarranid_horseman_helmet,itm_arabian_horse_a,itm_arabian_horse_b],regular_hth_attrib|level(20),wp_melee(125),knows_riding_4|knows_reflexes_4|knows_combat_training_2,sarranid_face_young_1,sarranid_face_young_2],
 
["sarranid_horseman_elite","Sarranid Camel Rider (elite)","Sarranid Camel Riders (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_sarranids,[itm_bolts,itm_jezzail,itm_lance,itm_arabian_sword_b,itm_sarranid_cavalry_sword,itm_scimitar,itm_scimitar_b,itm_lui_cimeterre,itm_lui_cimeterreb,itm_lui_khergitexecutioneronehand,itm_xenoargh_shield_small01,itm_sarranid_elite_armor,itm_mamluke_mail,itm_sarranid_boots_c,itm_sarranid_boots_b,itm_sarranid_horseman_helmet,itm_camel,itm_camel2,itm_camel3],elite_hth_attrib|level(30),wp_melee(150),knows_riding_6|knows_reflexes_6|knows_combat_training_3,sarranid_face_middle_1,sarranid_face_middle_2], 

#New Sarranid plated horses were too tempting to leave out. 
["sarranid_horseman_expert","Sarranid Mamluke (expert)","Sarranid Mamlukes (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_sarranids,[itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_leather_gloves,itm_mail_mittens,itm_warhorse_sarranid],expert_hth_attrib|level(45),wp_melee(210),knows_riding_8|knows_toughness_3|knows_reflexes_8|knows_combat_training_5,sarranid_face_middle_1,sarranid_face_middle_2],

#STOCK SARRANID CHARACTERS
["sarranid_messenger","Sarranid Messenger","Sarranid Messengers",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_kingdom_of_sarranids,[itm_lance,itm_arabian_sword_b,itm_scimitar_b,itm_mace_4,itm_xenoargh_shield_small01,itm_sarranid_mail_shirt,itm_mail_chausses,itm_sarranid_helmet1,itm_courser,itm_hunter],def_attrib|level(20),wp_melee(100),knows_common|knows_riding_4|knows_toughness_2,sarranid_face_young_1,sarranid_face_old_2],

["sarranid_deserter","Sarranid Deserter","Sarranid Deserters",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_deserters,[itm_lance,itm_arabian_sword_b,itm_scimitar_b,itm_mace_4,itm_xenoargh_shield_small01,itm_sarranid_mail_shirt,itm_mail_chausses,itm_desert_turban,itm_arabian_horse_a],def_attrib|level(20),wp_melee(100),knows_common|knows_riding_4|knows_toughness_2,sarranid_face_young_1,sarranid_face_old_2], 

["sarranid_prison_guard","Prison Guard","Prison Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_sarranids,[itm_arabian_sword_b,itm_scimitar_b,itm_war_spear,itm_mace_4,itm_sarranid_boots_c,itm_arabian_armor_b,itm_sarranid_mail_coif,itm_sarranid_helmet1,itm_sarranid_horseman_helmet,itm_mail_boots,itm_iron_greaves,itm_mail_mittens,itm_leather_gloves,itm_xenoargh_shield_kite01_02],def_attrib|level(25),wp_melee(135)|wp_throwing(100),knows_common|knows_toughness_3,sarranid_face_middle_1,sarranid_face_older_2],

["sarranid_castle_guard","Castle Guard","Castle Guards",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_sarranids,[itm_arabian_sword_b,itm_scimitar_b,itm_war_spear,itm_mace_4,itm_sarranid_boots_c,itm_sarranid_boots_d,itm_arabian_armor_b,itm_sarranid_mail_coif,itm_sarranid_helmet1,itm_sarranid_horseman_helmet,itm_mail_boots,itm_iron_greaves,itm_mail_mittens,itm_leather_gloves,itm_xenoargh_shield_kite01_02],def_attrib|level(25),wp_melee(135)|wp_throwing(100),knows_common|knows_toughness_3,sarranid_face_middle_1,sarranid_face_older_2],

#Still very easy... but not naked
["looter","Looter (peasant levy)","Looters (peasant levy)",tf_mounted|tf_guarantee_armor,0,0,fac_outlaws,[itm_hatchet,itm_club,itm_butchering_knife,itm_falchion,itm_stones,itm_woolen_cap,itm_woolen_cap,itm_nomad_boots,itm_wrapping_boots,itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_wrapping_boots,itm_fur_hat,itm_leather_cap],def_attrib|level(4),wp_melee(20)|wp_throwing(60),0,bandit_face1,bandit_face2],

#Not quite so easy, and can have bows
["looter_normal","Scum (recruit)","Scum (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_outlaws,[itm_eastern_arrows,itm_short_bow,itm_hatchet,itm_club,itm_butchering_knife,itm_falchion,itm_old_round_shield,itm_hide_covered_round_shield,itm_rawhide_coat,itm_wei_xiadi_rod_thick_coat,itm_stones,itm_nomad_armor,itm_nomad_armor,itm_woolen_cap,itm_woolen_cap,itm_nomad_boots,itm_wrapping_boots,itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_wrapping_boots,itm_fur_hat,itm_leather_cap],recruit_ranged_attrib|level(8),wp_melee(80)|wp_archery(100),knows_reflexes_1,bandit_face1,bandit_face2],

#Definitely a little harder, but they're still speed bumps.
["looter_hard","Desperate Man (regular)","Desperate Men (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_outlaws,[itm_eastern_arrows,itm_short_bow,itm_nomad_bow,itm_strong_bow,itm_hatchet,itm_club,itm_butchering_knife,itm_falchion,itm_old_round_shield,itm_hide_covered_round_shield,itm_rawhide_coat,itm_wei_xiadi_rod_thick_coat,itm_nomad_armor,itm_nomad_armor,itm_woolen_cap,itm_woolen_cap,itm_nomad_boots,itm_wrapping_boots,itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_wrapping_boots,itm_fur_hat,itm_leather_cap],regular_ranged_attrib|level(15),wp_melee(125)|wp_archery(150),knows_reflexes_2,bandit_face1,bandit_face2],

#Bandits
#These are now seen elsewhere than just in the default scenes. They are mixed in with Looters on anything but Casual Gamer mode.
#The maces make them rather nasty at close range, but they aren't that bad.
["bandit","Bandit (peasant levy)","Bandits (peasant levy)",tf_mounted|tf_guarantee_armor|tf_guarantee_ranged,0,0,fac_outlaws,[itm_eastern_arrows,itm_short_bow,itm_falchion,itm_nordic_shield,itm_rawhide_coat,itm_wei_xiadi_rod_thick_coat,itm_leather_cap,itm_leather_jerkin,itm_nomad_armor,itm_nomad_boots,itm_wrapping_boots],def_attrib|level(10),wp_melee(60)|wp_archery(100),knows_reflexes_1,bandit_face1,bandit_face2],

["bandit_normal","Thug (recruit)","Thugs (recruit)",tf_mounted|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_ranged,0,0,fac_outlaws,[itm_eastern_arrows,itm_bolts,itm_flintlock_pistol,itm_spiked_mace,itm_spiked_club,itm_falchion,itm_short_bow,itm_nordic_shield,itm_rawhide_coat,itm_wei_xiadi_rod_thick_coat,itm_leather_cap,itm_leather_jerkin,itm_nomad_armor,itm_nomad_boots,itm_wrapping_boots],recruit_ranged_attrib|level(15),wp_melee(100)|wp_archery(100)|wp_crossbow(100),knows_reflexes_2,bandit_face1,bandit_face2],

#Nasty guys, armed with blunderbusses, significantly better armor, and maces.  A big step up.
["bandit_hard","Blackguard (regular)","Blackguards (regular)",tf_mounted|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_helmet|tf_guarantee_ranged,0,0,fac_outlaws,[itm_bolts,itm_llew2_blunderbus,itm_spiked_mace,itm_spiked_club,itm_mace_1,itm_morningstar2,itm_morningstar2,itm_nordic_shield,itm_helmet_with_neckguard,itm_pino_peasant_man_spiked_mail,itm_nomad_boots],regular_ranged_attrib|level(30),wp_melee(135)|wp_archery(150)|wp_crossbow(175),knows_combat_training_2|knows_reflexes_5,bandit_face1,bandit_face2],

#Brigands
#These guys are pretty much just copies of the Bandits, but mounted. I gave them a pistol and they're archers now.
["brigand","Brigand (peasant levy)","Brigands (peasant levy)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_outlaws,[itm_bolts,itm_flintlock_pistol,itm_spiked_mace,itm_sword_viking_1,itm_falchion,itm_old_round_shield,itm_hide_covered_round_shield,itm_leather_cap,itm_leather_jerkin,itm_nomad_boots,itm_woolen_cap,itm_woolen_cap,itm_nomad_boots,itm_wrapping_boots,itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_wrapping_boots,itm_fur_hat,itm_leather_cap,itm_saddle_horse,itm_sumpter_horse,itm_xenoargh_appaloosa01,itm_xenoargh_bayappy,itm_xenoargh_baydark,itm_xenoargh_blackleapord],levy_ranged_attrib|level(15),wp_melee(70)|wp_crossbow(80),knows_riding_2|knows_reflexes_2,bandit_face1,bandit_face2],

["brigand_normal","Highwayman (recruit)","Highwaymen (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_ranged|tf_guarantee_shield,0,0,fac_outlaws,[itm_bolts,itm_flintlock_pistol,itm_spiked_mace,itm_sword_viking_1,itm_falchion,itm_old_round_shield,itm_hide_covered_round_shield,itm_leather_cap,itm_leather_jerkin,itm_nomad_boots,itm_woolen_cap,itm_woolen_cap,itm_nomad_boots,itm_wrapping_boots,itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_wrapping_boots,itm_fur_hat,itm_leather_cap,itm_saddle_horse,itm_sumpter_horse,itm_xenoargh_appaloosa01,itm_xenoargh_bayappy,itm_xenoargh_baydark,itm_xenoargh_blackleapord],recruit_ranged_attrib|level(20),wp_melee(125)|wp_crossbow(110),knows_riding_2|knows_reflexes_3,bandit_face1,bandit_face2],

#Significantly better armor.  Light arquebusses, for a bit more punch.
["brigand_hard","Desperado (regular)","Desperados (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_outlaws,[itm_bolts,itm_llew2_arquebus,itm_spiked_mace,itm_sword_viking_1,itm_falchion,itm_old_round_shield,itm_hide_covered_round_shield,itm_leather_cap,itm_pino_peasant_man_leather_mail,itm_nomad_boots,itm_woolen_hose,itm_nomad_boots,itm_hunter,itm_xenoargh_buckskin],regular_ranged_attrib|level(30),wp_melee(130)|wp_crossbow(140),knows_riding_4|knows_reflexes_5,bandit_face1,bandit_face2],

#Mountain Bandits.  
#Somewhat nasty, due to the horses, spears and shields.
#They get horses past the levies, just like their pictures.
["mountain_bandit","Mountain Bandit (peasant levy)","Mountain Bandits (peasant levy)",tf_mounted|tf_guarantee_armor|tf_guarantee_boots|tf_guarantee_horse,0,0,fac_outlaws,[itm_eastern_arrows,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_short_bow,itm_javelin,itm_old_green_round_shield,itm_felt_hat,itm_head_wrappings,itm_skullcap,itm_khergit_armor,itm_nomad_armor,itm_rawhide_coat,itm_wei_xiadi_rod_thick_coat,itm_nomad_vest,itm_hide_boots,itm_nomad_boots,itm_wooden_shield,itm_nordic_shield,itm_sumpter_horse,itm_xenoargh_baydark],levy_ranged_attrib|level(10),wp_melee(70)|wp_throwing(60)|wp_archery(60),knows_riding_2|knows_toughness_2,rhodok_face_young_1,rhodok_face_old_2],

["mountain_bandit_normal","Mountain Thief (recruit)","Mountain Thieves (recruit)",tf_mounted|tf_guarantee_armor|tf_guarantee_boots|tf_guarantee_horse,0,0,fac_outlaws,[itm_eastern_arrows,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_short_bow,itm_javelin,itm_old_green_round_shield,itm_felt_hat,itm_head_wrappings,itm_skullcap,itm_khergit_armor,itm_nomad_armor,itm_pino_peasant_man_leather,itm_nomad_vest,itm_hide_boots,itm_nomad_boots,itm_wooden_shield,itm_nordic_shield,itm_sumpter_horse,itm_xenoargh_baydark],recruit_ranged_attrib|level(15),wp_melee(110)|wp_throwing(80)|wp_archery(80),knows_riding_3|knows_toughness_4,rhodok_face_young_1,rhodok_face_old_2],

#These guys are quite a bit nastier, with better armor and more toughness overall.
["mountain_bandit_hard","Mountain Robber (regular)","Mountain Robbers (regular)",tf_mounted|tf_guarantee_armor|tf_guarantee_boots|tf_guarantee_horse|tf_guarantee_shield|tf_guarantee_ranged,0,0,fac_outlaws,[itm_eastern_arrows,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_short_bow,itm_javelin,itm_old_green_round_shield,itm_felt_hat,itm_head_wrappings,itm_skullcap,itm_pino_peasant_leather_scale,itm_hide_boots,itm_nomad_boots,itm_wooden_shield,itm_nordic_shield,itm_sumpter_horse,itm_xenoargh_baydark],regular_ranged_attrib|level(30),wp_melee(135)|wp_throwing(115)|wp_archery(115),knows_riding_4|knows_toughness_6,rhodok_face_young_1,rhodok_face_old_2],

#Forest Bandits
#These guys didn't require much to be more effective. They're still virtually worthless vs. cavalry.
["forest_bandit","Forest Thief (peasant levy)","Forest Thieves (peasant levy)",tf_mounted|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_boots,0,0,fac_outlaws,[itm_eastern_arrows,itm_axe,itm_hatchet,itm_quarter_staff,itm_short_bow,itm_hunting_bow,itm_common_hood,itm_black_hood,itm_pino_peasant_archer,itm_padded_leather,itm_leather_jerkin,itm_ragged_outfit,itm_hide_boots,itm_leather_boots],levy_ranged_attrib|level(10),wp_melee(90)|wp_archery(115),knows_reflexes_2,swadian_face_young_1,swadian_face_old_2],

["forest_bandit_normal","Honest Thief (recruit)","Honest Thieves (recruit)",tf_mounted|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_boots,0,0,fac_outlaws,[itm_arrows,itm_axe,itm_hatchet,itm_quarter_staff,itm_long_bow,itm_common_hood,itm_black_hood,itm_pino_peasant_archer,itm_padded_leather,itm_leather_jerkin,itm_ragged_outfit,itm_hide_boots,itm_leather_boots],recruit_ranged_attrib|level(15),wp_melee(125)|wp_archery(140),knows_combat_training_1|knows_reflexes_3,swadian_face_young_1,swadian_face_old_2],

["forest_bandit_hard","Merry Man (regular)","Merry Men (regular)",tf_mounted|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_boots,0,0,fac_outlaws,[itm_arrows,itm_axe,itm_hatchet,itm_quarter_staff,itm_long_bow,itm_common_hood,itm_black_hood,itm_xenoargh_green_leather_armor,itm_hide_boots,itm_leather_boots],regular_ranged_attrib|level(20),wp_melee(145)|wp_archery(170),knows_combat_training_2|knows_reflexes_3,swadian_face_young_1,swadian_face_old_2],

#Basically unchanged, only they get harder by removing crap. Mixing them with Forest Bandits = infantry nasties on hard
["sea_raider","Sea Raider (recruit)","Sea Raiders (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_outlaws,[itm_arrows,itm_lui_vikingaxebastard,itm_lui_vikingaxeoneh,itm_lui_vikingaxeonehb,itm_lui_vikingaxeonehc,itm_sword_viking_1,itm_sword_viking_2,itm_fighting_axe,itm_battle_axe,itm_spear,itm_xenoargh_round_shield02,itm_long_bow,itm_javelin,itm_throwing_axes,itm_nordic_helmet,itm_nordic_helmet,itm_nasal_helmet,itm_leather_jerkin,itm_byrnie,itm_leather_jerkin,itm_leather_boots,itm_nomad_boots],recruit_hth_attrib|level(15),wp_melee(110)|wp_archery(90),knows_toughness_2|knows_riding_1|knows_reflexes_2,nord_face_young_1,nord_face_old_2],

["sea_raider_normal","Disgraced Nord (regular)","Disgraced Nords (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet,0,0,fac_outlaws,[itm_arrows,itm_lui_vikingaxebastard,itm_lui_vikingaxeoneh,itm_lui_vikingaxeonehb,itm_lui_vikingaxeonehc,itm_sword_viking_1,itm_sword_viking_2,itm_fighting_axe,itm_battle_axe,itm_spear,itm_xenoargh_round_shield02_paint02,itm_long_bow,itm_javelin,itm_throwing_axes,itm_nordic_helmet,itm_nordic_helmet,itm_nasal_helmet,itm_leather_jerkin,itm_byrnie,itm_pino_peasant_leather_ring,itm_leather_boots,itm_nomad_boots,itm_leather_gloves,itm_mail_mittens],regular_hth_attrib|level(20),wp_melee(135)|wp_archery(115),knows_toughness_3|knows_riding_1|knows_reflexes_3|knows_combat_training_1,nord_face_young_1,nord_face_old_2],

["sea_raider_hard","Sea Raider Captain (elite)","Sea Raider Captains (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_gloves|tf_guarantee_horse,0,0,fac_outlaws,[itm_arrows,itm_lui_vikingaxebastard,itm_lui_vikingaxeoneh,itm_lui_vikingaxeonehb,itm_lui_vikingaxeonehc,itm_sword_viking_1,itm_sword_viking_2,itm_fighting_axe,itm_battle_axe,itm_spear,itm_xenoargh_round_shield02_paint03,itm_long_bow,itm_javelin,itm_throwing_axes,itm_nordic_helmet,itm_nordic_helmet,itm_nasal_helmet,itm_dejawolf_vikingbyrnie,itm_byrnie,itm_pino_haubergeon_jco,itm_leather_boots,itm_nomad_boots,itm_leather_gloves,itm_mail_mittens,itm_hunter,itm_warhorse_chain],elite_hth_attrib|level(35),wp_melee(165)|wp_archery(135),knows_toughness_5|knows_riding_5|knows_reflexes_5|knows_combat_training_3,nord_face_young_1,nord_face_old_2],

#Basically unchanged, they just get less crappy at the high end.
["steppe_bandit","Steppe Bandit (recruit)","Steppe Bandits (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_outlaws,[itm_eastern_arrows,itm_sword_khergit_1,itm_winged_mace,itm_spear,itm_light_lance,itm_nomad_bow,itm_nomad_bow,itm_short_bow,itm_jarid,itm_leather_steppe_cap_a,itm_leather_steppe_cap_b,itm_khergit_armor,itm_nomad_armor,itm_steppe_armor,itm_leather_vest,itm_wei_xiadi_kher_ragged_outfit,itm_hide_boots,itm_nomad_boots,itm_leather_covered_round_shield,itm_leather_covered_round_shield,itm_saddle_horse,itm_steppe_horse,itm_steppe_horse,itm_xenoargh_appaloosa01],recruit_ranged_attrib|level(15),wp_melee(100)|wp_archery(115)|wp_throwing(115),knows_riding_4|knows_horse_archery_3|knows_reflexes_5,khergit_face_young_1,khergit_face_old_2],

["steppe_bandit_normal","Steppe Thug (regular)","Steppe Thugs (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_ranged,0,0,fac_outlaws,[itm_eastern_arrows,itm_sword_khergit_1,itm_winged_mace,itm_spear,itm_light_lance,itm_nomad_bow,itm_nomad_bow,itm_short_bow,itm_jarid,itm_leather_steppe_cap_a,itm_leather_steppe_cap_b,itm_khergit_armor,itm_nomad_armor,itm_steppe_armor,itm_leather_vest,itm_wei_xiadi_kher_ragged_outfit,itm_hide_boots,itm_nomad_boots,itm_leather_covered_round_shield,itm_leather_covered_round_shield,itm_saddle_horse,itm_steppe_horse,itm_steppe_horse,itm_xenoargh_appaloosa01],regular_ranged_attrib|level(20),wp_melee(130)|wp_archery(145)|wp_throwing(145),knows_riding_6|knows_horse_archery_5|knows_reflexes_7,khergit_face_young_1,khergit_face_old_2],

#Much better stats and body armor.
["steppe_bandit_hard","Steppe Outlaw (elite)","Steppe Outlaws (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_ranged|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_outlaws,[itm_eastern_arrows,itm_sword_khergit_1,itm_winged_mace,itm_spear,itm_light_lance,itm_nomad_bow,itm_nomad_bow,itm_short_bow,itm_jarid,itm_leather_steppe_cap_a,itm_leather_steppe_cap_b,itm_pino_peasant_archer_scale,itm_hide_boots,itm_nomad_boots,itm_scale_gauntlets,itm_lamellar_gauntlets,itm_leather_covered_round_shield,itm_leather_covered_round_shield,itm_saddle_horse,itm_steppe_horse,itm_steppe_horse,itm_xenoargh_appaloosa01],elite_ranged_attrib|level(30),wp_melee(140)|wp_archery(165)|wp_throwing(165),knows_riding_7|knows_horse_archery_6|knows_reflexes_8,khergit_face_young_1,khergit_face_old_2],

#The Taiga Bandits are now the Ruskis- fairly cheap, nasty irregulars with good armor for their cost and great shields.
["ruski_soldat","Ruski Soldat (recruit)","Ruski Soldats (recruit)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_outlaws,[itm_mackie_double_axe,itm_eastern_arrows,itm_nomad_bow,itm_jarid,itm_javelin,itm_plate_covered_round_shield,itm_steel_shield,itm_plate_shield,itm_narf_rus_helm,itm_narf_gnezdovo_helm_b,itm_narf_gnezdovo_helm_a,itm_narf_rus_lamellar_a,itm_narf_rus_lamellar_b,itm_narf_rus_shoes],recruit_hth_attrib|level(15),wp_melee(110)|wp_archery(115)|wp_throwing(115),knows_reflexes_1|knows_combat_training_1|knows_toughness_2,vaegir_face_young_1,vaegir_face_old_2],

["ruski_soldat_normal","Ruski Soldat (regular)","Ruski Soldats (regular)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield|tf_guarantee_gloves,0,0,fac_outlaws,[itm_mackie_double_axe,itm_eastern_arrows,itm_nomad_bow,itm_jarid,itm_javelin,itm_plate_covered_round_shield,itm_steel_shield,itm_plate_shield,itm_narf_rus_cav_boots,itm_leather_gloves,itm_narf_tagancha_helm_b,itm_narf_nikolskoe_helm,itm_narf_kuyak_a,itm_narf_kuyak_b],regular_hth_attrib|level(20),wp_melee(155)|wp_archery(125)|wp_throwing(125),knows_reflexes_2|knows_combat_training_2|knows_toughness_4,vaegir_face_young_1,vaegir_face_old_2],

["ruski_soldat_hard","Ruski Soldat (elite)","Ruski Soldats (elite)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_shield|tf_guarantee_gloves,0,0,fac_outlaws,[itm_mackie_double_axe,itm_eastern_arrows,itm_nomad_bow,itm_jarid,itm_javelin,itm_plate_covered_round_shield,itm_steel_shield,itm_plate_shield,itm_narf_rus_cav_boots,itm_lamellar_gauntlets,itm_narf_rus_scale,itm_narf_tagancha_helm_a,itm_narf_novogrod_helm],elite_hth_attrib|level(40),wp_melee(200)|wp_archery(175)|wp_throwing(165),knows_reflexes_4|knows_combat_training_4|knows_toughness_8,vaegir_face_young_1,vaegir_face_old_2],

#These guys are so powerful with camels, I actually nerfed them.
["desert_bandit","Desert Bandit","Desert Bandits",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_outlaws,[itm_eastern_arrows,itm_arabian_sword_a,itm_winged_mace,itm_spear,itm_light_lance,itm_jarid,itm_nomad_bow,itm_short_bow,itm_jarid,itm_sarranid_cloth_robe,itm_sarranid_cloth_robe,itm_wei_xiadi_sar_jellaba_blue,itm_wei_xiadi_sar_jellaba_white,itm_desert_turban,itm_turban,itm_leather_steppe_cap_b,itm_leather_gloves,itm_leather_covered_round_shield,itm_leather_covered_round_shield,itm_arabian_horse_a,itm_arabian_horse_b,itm_arabian_horse_a,itm_arabian_horse_b,itm_camel,itm_camel2,itm_camel3],recruit_hth_attrib|level(12),wp_melee(100)|wp_archery(65)|wp_throwing(65),knows_riding_4|knows_horse_archery_3,sarranid_face_young_1,sarranid_face_old_1],

["desert_bandit_normal","Desert Thief","Desert Thieves",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_outlaws,[itm_eastern_arrows,itm_arabian_sword_a,itm_winged_mace,itm_spear,itm_light_lance,itm_jarid,itm_nomad_bow,itm_short_bow,itm_jarid,itm_sarranid_cloth_robe,itm_sarranid_cloth_robe,itm_wei_xiadi_sar_jellaba_blue,itm_wei_xiadi_sar_jellaba_white,itm_desert_turban,itm_turban,itm_leather_steppe_cap_b,itm_leather_gloves,itm_scale_gauntlets,itm_leather_covered_round_shield,itm_leather_covered_round_shield,itm_arabian_horse_a,itm_arabian_horse_b,itm_arabian_horse_a,itm_arabian_horse_b,itm_camel,itm_camel2,itm_camel3],regular_hth_attrib|level(20),wp_melee(135)|wp_archery(95)|wp_throwing(95),knows_riding_5|knows_horse_archery_4,sarranid_face_young_1,sarranid_face_old_1],

["desert_bandit_hard","Desert Nomad Warrior","Desert Nomad Warriors",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_shield|tf_guarantee_helmet|tf_guarantee_gloves,0,0,fac_outlaws,[itm_eastern_arrows,itm_arabian_sword_a,itm_winged_mace,itm_spear,itm_light_lance,itm_jarid,itm_nomad_bow,itm_short_bow,itm_jarid,itm_wei_xiadi_sar_leather_armor,itm_skirmisher_armor,itm_desert_turban,itm_turban,itm_leather_steppe_cap_b,itm_leather_covered_round_shield,itm_leather_covered_round_shield,itm_camel,itm_camel2,itm_camel3],elite_hth_attrib|level(25),wp_melee(175)|wp_archery(115)|wp_throwing(115),knows_riding_7|knows_horse_archery_7,sarranid_face_young_1,sarranid_face_old_1],


#Manhunters:  light horsemen with maces.  They eventually don't suck.
["manhunter","Manhunter (regular)","Manhunters (regular)",tf_mounted|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_manhunters,[itm_winged_mace,itm_spiked_mace,itm_woolen_cap,itm_rawhide_coat,itm_wei_xiadi_rod_thick_coat,itm_old_blue_round_shield,itm_nomad_boots,itm_sumpter_horse,itm_xenoargh_blackleapord,itm_courser],regular_hth_attrib|level(10),wp_melee(120),knows_riding_3|knows_toughness_6,bandit_face1,bandit_face2],

["slave_driver","Slave Driver (regular)","Slave Drivers (regular)",tf_mounted|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_boots,0,0,fac_slavers,[itm_winged_mace,itm_spiked_mace,itm_nordic_archer_helmet,itm_pino_peasant_leather_ring,itm_old_blue_round_shield,itm_nomad_boots,itm_hunter,itm_xenoargh_buckskin],regular_hth_attrib|level(15),wp_melee(130),knows_riding_4|knows_toughness_7,bandit_face1,bandit_face2],

["slave_hunter","Slave Hunter (elite)","Slave Hunters (elite)",tf_mounted|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_boots|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_slavers,[itm_sarranid_two_handed_mace_1,itm_lui_twohandedmace,itm_long_hafted_spiked_mace,itm_narf_combed_morion,itm_narf_combed_morion2,itm_wei_xiadi_mail_shirt01,itm_wei_xiadi_mail_shirt02,itm_nordic_shield,itm_nomad_boots,itm_hunter,itm_xenoargh_buckskin,itm_leather_gloves,itm_mail_mittens],elite_hth_attrib|level(20),wp_melee(155),knows_riding_6|knows_toughness_8,bandit_face1,bandit_face2],

["slave_crusher","Slave Crusher (elite)","Slave Crushers (elite)",tf_mounted|tf_guarantee_armor|tf_guarantee_horse|tf_guarantee_boots|tf_guarantee_helmet|tf_guarantee_gloves|tf_guarantee_shield,0,0,fac_slavers,[itm_sarranid_two_handed_mace_1,itm_lui_twohandedmace,itm_long_hafted_spiked_mace,itm_pino_haubergeon_jco,itm_old_blue_round_shield,itm_leather_boots,itm_mail_mittens,itm_warhorse_chain,itm_narf_combed_morion,itm_narf_combed_morion2],elite_hth_attrib|level(30),wp_melee(180),knows_riding_7|knows_toughness_9,bandit_face1,bandit_face2],

["slaver_chief","Slaver Chief (expert)","Slaver Chiefs (expert)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_gloves|tf_guarantee_shield,0,0,fac_slavers,[itm_sarranid_two_handed_mace_1,itm_lui_twohandedmace,itm_long_hafted_spiked_mace,itm_coat_of_plates_red,itm_coat_of_plates,itm_scale_armor,itm_old_blue_round_shield,itm_scale_gauntlets,itm_mail_boots,itm_leather_boots,itm_iron_greaves,itm_warhorse,itm_warhorse_chain,itm_narf_combed_morion,itm_narf_combed_morion2],expert_hth_attrib|level(40),wp_melee(250),knows_riding_8|knows_toughness_10,bandit_face1,bandit_face2],

#CANNON WALKERS
["aleph","Aleph","Alephs",tf_cannon_walker|tf_mounted|tf_guarantee_ranged|tf_guarantee_helmet,0,0,fac_neutral,[itm_aleph_cannon,itm_universal_cannon_ammo,itm_aleph_suit],str_100|agi_10|int_15|cha_15|level(70),wp_crossbow(500),knows_toughness_10|knows_combat_training_10,0],

#DEATH KNIGHTS
["death_knight","Death Knight (?)","Death Knight (?)",tf_death_knight|tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_death_knights,[itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spak_knight_shield,itm_mail_chausses,itm_narf_steel_greaves,itm_spak_demon_hood,itm_spak_horned_helm,itm_spak_skull_helm,itm_rhulg_helmet,itm_spak_dark_gauntlets,itm_narf_plate_mittens_dark,itm_narf_milanese_plate_dark,itm_spak_horny_charger_plate],expert_hth_attrib|level(75),wp_melee(275),knows_riding_8|knows_toughness_7|knows_combat_training_5|knows_reflexes_1,skeleton_face1],

["death_knight_footman","Death Knight Footman (?)","Death Knight Footmen (?)",tf_death_knight|tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_death_knights,[itm_spiked_club,itm_mace_1,itm_morningstar,itm_mackie_panabas,itm_spak_asmodeus_shield,itm_mail_chausses,itm_narf_steel_greaves,itm_xenoargh_evil_helm,itm_xenoargh_evil_helm_white,itm_xenoargh_evil_helm_yellow,itm_spak_dark_gauntlets,itm_narf_plate_mittens_dark,itm_xenoargh_dark_armor,itm_xenoargh_dark_armor_red,itm_xenoargh_dark_armor_yellow],expert_hth_attrib|level(85),wp_melee(275),knows_reflexes_4|knows_toughness_8|knows_combat_training_5,skeleton_face1],

["death_knight_crossbowman","Death Knight Crossbowman (?)","Death Knight Crossbowmen (?)",tf_death_knight|tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield|tf_guarantee_ranged,0,0,fac_death_knights,[itm_bolts,itm_sniper_crossbow,itm_double_flintlock_pistol,itm_xenoargh_gun_axe,itm_arquebus,itm_jezzail,itm_spak_butcher,itm_spak_evil_katana,itm_spak_bronze_shield,itm_mail_chausses,itm_narf_steel_greaves,itm_xenoargh_evil_helm,itm_xenoargh_evil_helm_white,itm_xenoargh_evil_helm_yellow,itm_spak_dark_gauntlets,itm_narf_plate_mittens_dark,itm_xenoargh_dark_armor02],expert_ranged_attrib|level(75),wp_melee(200)|wp_crossbow(275),knows_toughness_5|knows_combat_training_3|knows_reflexes_5,skeleton_face1],

#SKELETON ARMIES
["death_knight_skeleton_archer","Death Knight Skeleton Archer (?)","Death Knight Skeleton Archers (?)",tf_skeleton|tf_always_fall_dead|tf_mounted|tf_guarantee_helmet|tf_guarantee_ranged|tf_no_capture_alive,0,0,fac_death_knights,[itm_eastern_arrows,itm_strong_bow,itm_spiked_club,itm_nordic_archer_helmet],regular_ranged_attrib|level(25),wp_melee(75)|wp_archery(150),knows_combat_training_2|knows_reflexes_2,skeleton_face1],

["death_knight_skeleton_warrior","Death Knight Skeleton Warrior (?)","Death Knight Skeleton Warriors (?)",tf_skeleton|tf_always_fall_dead|tf_mounted|tf_guarantee_helmet|tf_no_capture_alive|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_death_knights,[itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_old_board_shield01,itm_xenoargh_viking_horned_helm02,itm_skeleton_armor03],elite_hth_attrib|level(35),wp_melee(150),knows_combat_training_4|knows_reflexes_2|knows_toughness_2,skeleton_face1],

["death_knight_skeleton_rider","Death Knight Skeleton Rider (?)","Death Knight Skeleton Riders (?)",tf_skeleton|tf_always_fall_dead|tf_mounted|tf_guarantee_helmet|tf_no_capture_alive|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_death_knights,[itm_rrr_spear1,itm_heater_shield_swadian,itm_narf_combed_morion,itm_skeleton_armor01,itm_skeleton_armor02,itm_barf_skeletal_horse],elite_hth_attrib|level(40),wp_melee(170),knows_riding_4|knows_combat_training_5|knows_reflexes_2|knows_toughness_4,skeleton_face1],

#Remnants!
["death_knight_remnant","Remnant (?)","Remnants (?)",tf_remnant|tf_always_fall_dead|tf_mounted,0,0,fac_death_knights,[itm_mackie_mangler,itm_mackie_mangler_short],regular_hth_attrib|level(40),wp_melee(150),knows_reflexes_10|knows_toughness_4,skeleton_face1],

#FLEEEESH GOOOOLEM
["death_knight_flesh_golem","Flesh Golem (?)","Flesh Golems (?)",tf_flesh_golem|tf_always_fall_dead|tf_mounted,0,0,fac_death_knights,[itm_xeno_golem_axe,itm_spak_golem_shield],golem_attrib|level(100),wp_melee(150),knows_toughness_10,skeleton_face1],

#TEMPLARS
["templar_paragon","Templar Paragon (?)","Templar Paragons (?)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield|tf_guarantee_horse,0,0,fac_templars,[itm_lui_empirehammer,itm_lui_empirewarhammer,itm_lui_empirewarhammerb,itm_lui_empirewarhammerc,itm_lui_knighthammer,itm_lui_knighthammerempire,itm_narf_early_transitional_templar,itm_narf_wisby_gauntlets_red,itm_narf_wisby_gauntlets_black,itm_narf_splinted_greaves_spurs,itm_narf_sugarloaf_templar,itm_narf_sugarloaf_coif_templar,itm_winged_great_helmet,itm_great_helmet,itm_warhorse_chain_barding_templar,itm_spak_knight_shield_templar],expert_hth_attrib|level(75),wp_melee(325),knows_riding_10|knows_toughness_10|knows_combat_training_10|knows_reflexes_10,rhodok_face_middle_1,rhodok_face_middle_2],

["templar_trooper","Templar Defenders (?)","Templar Defenders (?)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield|tf_guarantee_ranged,0,0,fac_templars,[itm_bolts,itm_sniper_crossbow,itm_rrr_spear4,itm_rrr_spear1,itm_rrr_spear3,itm_awlpike,itm_awlpike_long,itm_narf_early_transitional_templar,itm_narf_wisby_gauntlets_red,itm_narf_wisby_gauntlets_black,itm_narf_splinted_greaves_spurs,itm_dejawolf_klappvisier,itm_narf_hounskull,itm_spak_knight_shield_templar],expert_hth_attrib|level(75),wp_melee(375)|wp_crossbow(275),knows_toughness_10|knows_combat_training_10|knows_reflexes_10,rhodok_face_middle_1,rhodok_face_middle_2],

["templar_pikeman","Templar Pikeman (?)","Templar Pikemen (?)",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet,0,0,fac_templars,[itm_ashwood_pike,itm_lui_largespear,itm_lui_vaegirpike,itm_wei_xiadi_templar_surcoat_red,itm_wei_xiadi_templar_surcoat_white,itm_narf_wisby_gauntlets_red,itm_narf_wisby_gauntlets_black,itm_narf_splinted_greaves_spurs,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2],expert_hth_attrib|level(75),wp_melee(375),knows_toughness_10|knows_combat_training_10|knows_reflexes_10,rhodok_face_middle_1,rhodok_face_middle_2],

["sister_of_mercy","Sister Of Mercy (?)","Sisters of Mercy (?)",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_helmet|tf_guarantee_shield,0,0,fac_templars,[itm_xenoargh_troop_grenade,itm_lui_manhunter_polehammer,itm_jam_scorpion,itm_jam_voulge_b,itm_narf_wisby_gauntlets_red,itm_narf_wisby_gauntlets_black,itm_xenoargh_hornmask_black,itm_xena_boots,itm_xenoargh_female_armor_sisters_of_mercy1,itm_xenoargh_female_armor_sisters_of_mercy2,itm_xenoargh_female_armor_sisters_of_mercy3,itm_xenoargh_female_armor_sisters_of_mercy4,itm_spak_knight_shield_templar],afrizar_attrib|level(75),wp_melee(275)|wp_throwing(275),knows_toughness_10|knows_combat_training_10|knows_reflexes_10,khergit_woman_face_1,khergit_woman_face_8],

#Rhodok tribal,Hunter,warrior,veteran,warchief






#["undead_walker","undead_walker","undead_walkers",tf_mounted|tf_undead|tf_allways_fall_dead,0,0,fac_undeads,
# [],
# def_attrib|level(3),wp(60),knows_common,undead_face1,undead_face2],
#["undead_horseman","undead_horseman","undead_horsemen",tf_mounted|tf_undead|tf_allways_fall_dead|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse,0,0,fac_undeads,
# [],
# def_attrib|level(19),wp(100),knows_common,undead_face1,undead_face2],
#["undead_nomad","undead_nomad","undead_nomads",tf_mounted|tf_undead|tf_allways_fall_dead|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_black_khergits,
# [],
# def_attrib|level(21),wp(100),knows_common|knows_riding_4,khergit_face1,khergit_face2],
#["undead","undead","undead",tf_mounted|tf_undead|tf_allways_fall_dead,0,0,fac_undeads,
# [],
# def_attrib|level(3),wp(60),knows_common,undead_face1,undead_face2],
#["hell_knight","hell_knight","hell_knights",tf_mounted|tf_undead|tf_allways_fall_dead|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_undeads,
# [],
# def_attrib|level(23),wp(100),knows_common|knows_riding_3,undead_face1,undead_face2],

["follower_woman","Camp Follower (peasant levy)","Camp Follower (peasant levy)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_ranged,0,0,fac_commoners,[itm_bolts,itm_light_crossbow,itm_nordic_shield,itm_hide_covered_round_shield,itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_dress,itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_skullcap,itm_wrapping_boots],recruit_ranged_attrib|level(5),wp_melee(70)|wp_crossbow(100),knows_common|knows_reflexes_3|knows_toughness_1|knows_combat_training_2,refugee_face1,refugee_face2],

["hunter_woman","Huntress (recruit)","Huntresses (recruit)",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_ranged,0,0,fac_commoners,[itm_bolts,itm_crossbow,itm_nordic_shield,itm_hide_covered_round_shield,itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_leather_jerkin,itm_leather_vest,itm_skullcap,itm_wrapping_boots],regular_ranged_attrib|level(10),wp_melee(85)|wp_crossbow(150),knows_common|knows_reflexes_3|knows_toughness_2|knows_combat_training_3,refugee_face1,refugee_face2],

["fighter_woman","Grove Keepers (regular)","Grove Keepers (regular)",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield|tf_guarantee_ranged|tf_guarantee_helmet,0,0,fac_commoners,[itm_bolts,itm_heavy_crossbow,itm_fur_covered_shield,itm_hide_covered_round_shield,itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_leather_gloves,itm_xenoargh_leaf_helm,itm_xenoargh_new_female_armor02,itm_xenoargh_new_female_armor03,itm_xena_boots],elite_ranged_attrib|level(30),wp_melee(135)|wp_crossbow(200),knows_common|knows_riding_3|knows_reflexes_5|knows_toughness_3|knows_combat_training_5,refugee_face1,refugee_face2],

["sword_sister","Blade (elite)","Blades (elite)",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_shield|tf_guarantee_ranged|tf_guarantee_helmet,0,0,fac_commoners,[itm_bolts,itm_sniper_crossbow,itm_lui_highelflongsword,itm_plate_covered_round_shield,itm_dthehun_new_battle_armor04,itm_xena_boots,itm_xenoargh_shell_helm,itm_narf_wisby_gauntlets_black],assassin_attrib|level(50),wp_melee(200)|wp_crossbow(350),knows_common|knows_riding_5|knows_reflexes_9|knows_toughness_6|knows_combat_training_7,refugee_face1,refugee_face2],

["refugee","Refugee","Refugees",tf_mounted|tf_female|tf_guarantee_armor,0,0,fac_commoners,[itm_knife,itm_pitch_fork,itm_sickle,itm_hatchet,itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_club,itm_dress,itm_robe,itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_headcloth,itm_woolen_hood,itm_wrapping_boots],def_attrib|level(1),wp(45),knows_common,refugee_face1,refugee_face2],

["peasant_woman","Peasant Woman","Peasant Women",tf_mounted|tf_female|tf_guarantee_armor,0,0,fac_commoners,[itm_knife,itm_pitch_fork,itm_sickle,itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_hatchet,itm_club,itm_dress,itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_headcloth,itm_woolen_hood,itm_wrapping_boots],def_attrib|level(1),wp(40),knows_common,refugee_face1,refugee_face2],

["caravan_master","Caravan Master","Caravan Masters",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_commoners,[itm_winged_mace,itm_coat_of_plates_red,itm_steel_shield,itm_scale_gauntlets,itm_mail_boots,itm_iron_greaves,itm_warhorse_chain,itm_segmented_helmet],def_attrib|level(9),wp_melee(100),knows_common|knows_riding_4|knows_toughness_3,mercenary_face_1,mercenary_face_2],

["kidnapped_girl","Kidnapped Girl","Kidnapped Girls",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,reserved,fac_commoners,[itm_dress,itm_leather_boots],def_attrib|level(2),wp_melee(50),knows_common|knows_riding_2,woman_face_1,woman_face_2],

#This troop is the troop marked as soldiers_end and town_walkers_begin
 ["town_walker_1","Townsman","Townsmen",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_short_tunic,itm_linen_tunic,itm_fur_coat,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_arena_tunic_white,itm_leather_apron,itm_shirt,itm_arena_tunic_green,itm_arena_tunic_blue,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_hide_boots,itm_ankle_boots,itm_leather_boots,itm_fur_hat,itm_leather_cap,itm_straw_hat,itm_felt_hat],
 def_attrib|level(4),wp(60),knows_common,man_face_young_1,man_face_old_2],
 ["town_walker_2","Townswoman","Townswomen",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_blue_dress,itm_dress,itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_peasant_dress,itm_woolen_hose,itm_blue_hose,itm_wimple_a,itm_wimple_with_veil,itm_female_hood],
 def_attrib|level(2),wp(40),knows_common,woman_face_1,woman_face_2],
 ["khergit_townsman","Townsman","Townsmen",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_khergits,
 [itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_sarranid_felt_hat,itm_turban,itm_wrapping_boots,itm_sarranid_cloth_robe,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe],
 def_attrib|level(4),wp(60),knows_common,khergit_face_younger_1,khergit_face_middle_2],
 ["khergit_townswoman","Townswoman","Townswomen",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_blue_dress,itm_dress,itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_peasant_dress,itm_woolen_hose,itm_blue_hose,itm_wimple_a,itm_wimple_with_veil,itm_female_hood],
 def_attrib|level(2),wp(40),knows_common,khergit_woman_face_1,khergit_woman_face_2],
 ["sarranid_townsman","Townsman","Townsmen",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_sarranids,
 [itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_sarranid_felt_hat,itm_turban,itm_wrapping_boots,itm_sarranid_cloth_robe,itm_sarranid_cloth_robe_b,itm_wei_xiadi_sar_jellaba_blue,itm_wei_xiadi_sar_jellaba_white,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02],
 def_attrib|level(4),wp(60),knows_common,sarranid_face_younger_1,sarranid_face_middle_2],
 ["sarranid_townswoman","Townswoman","Townswomen",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_sarranid_common_dress,itm_sarranid_common_dress_b,itm_sarranid_felt_head_cloth,itm_sarranid_felt_head_cloth_b],
 def_attrib|level(2),wp(40),knows_common,sarranid_woman_face_1,sarranid_woman_face_2],

#This troop is the troop marked as town_walkers_end and village_walkers_begin
 ["village_walker_1","Villager","Villagers",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_leather_vest,itm_leather_apron,itm_shirt,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_hide_boots,itm_ankle_boots,itm_leather_boots,itm_fur_hat,itm_leather_cap,itm_straw_hat,itm_felt_hat],
 def_attrib|level(4),wp(60),knows_common,man_face_younger_1,man_face_older_2],
 ["village_walker_2","Villager","Villagers",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_mackie_billhook,itm_mackie_beefsplitter01,itm_mackie_beefsplitter02,itm_blue_dress,itm_dress,itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_peasant_dress,itm_woolen_hose,itm_blue_hose,itm_wimple_a,itm_wimple_with_veil,itm_female_hood],
 def_attrib|level(2),wp(40),knows_common,woman_face_1,woman_face_2],

#This troop is the troop marked as village_walkers_end and spy_walkers_begin
 ["spy_walker_1","Townsman","Townsmen",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_commoners,
 [itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_robe,itm_leather_apron,itm_shirt,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_hide_boots,itm_ankle_boots,itm_leather_boots,itm_fur_hat,itm_leather_cap,itm_straw_hat,itm_felt_hat],
 def_attrib|level(4),wp(60),knows_common,man_face_middle_1,man_face_old_2],
 ["spy_walker_2","Townswoman","Townswomen",tf_mounted|tf_female|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_commoners,
 [itm_blue_dress,itm_dress,itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_peasant_dress,itm_woolen_hose,itm_blue_hose,itm_wimple_a,itm_wimple_with_veil,itm_female_hood],
 def_attrib|level(2),wp(40),knows_common,woman_face_1,woman_face_2],
# Ryan END

#This troop is the troop marked as spy_walkers_end
# Zendar
["tournament_master","Tournament Master","Tournament Master",tf_mounted|tf_hero,scn_zendar_center|entry(1),reserved, fac_commoners,[itm_nomad_armor,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008414401e28f534c8a2d09],
["trainer","Trainer","Trainer",tf_mounted|tf_hero,scn_zendar_center|entry(2),reserved, fac_commoners,[itm_leather_jerkin,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000430c701ea98836781647f],
["Constable_Hareck","Constable Hareck","Constable Hareck",tf_mounted|tf_hero,scn_zendar_center|entry(5),reserved, fac_commoners,[itm_leather_jacket,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x00000000000c41c001fb15234eb6dd3f],

# Ryan BEGIN
["Ramun_the_slave_trader","Ramun,the slave trader","Ramun,the slave trader",tf_mounted|tf_hero,no_scene,reserved,fac_commoners,[itm_leather_jacket,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x0000000fd5105592385281c55b8e44eb00000000001d9b220000000000000000,0x0000000fd5105592385281c55b8e44eb00000000001d9b220000000000000000],

["guide","Quick Jimmy","Quick Jimmy",tf_mounted|tf_hero,no_scene,0, fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,0x00000000000c318301f24e38a36e38e3],
# Ryan END

["Xerina","Xerina","Xerina",tf_mounted|tf_hero|tf_female,scn_the_happy_boar|entry(5),reserved, fac_commoners,[itm_leather_jerkin,itm_hide_boots],def_attrib|str_15|agi_15|level(39),wp(312),knows_toughness_5|knows_riding_6|knows_reflexes_8,0x00000001ac0820074920561d0b51e6ed00000000001d40ed0000000000000000,0x00000001ac0820074920561d0b51e6ed00000000001d40ed0000000000000000],
["Dranton","Dranton","Dranton",tf_mounted|tf_hero,scn_the_happy_boar|entry(2),reserved, fac_commoners,[itm_leather_vest,itm_hide_boots],def_attrib|str_15|agi_14|level(42),wp(324),knows_toughness_7|knows_riding_4|knows_reflexes_4,0x0000000a460c3002470c50f3502879f800000000001ce0a00000000000000000,0x0000000a460c3002470c50f3502879f800000000001ce0a00000000000000000],
["Kradus","Kradus","Kradus",tf_mounted|tf_hero,scn_the_happy_boar|entry(3),reserved, fac_commoners,[itm_padded_leather,itm_hide_boots],def_attrib|str_15|agi_14|level(43),wp(270),knows_toughness_7|knows_riding_4|knows_reflexes_4,0x0000000f5b1052c61ce1a9521db1375200000000001ed31b0000000000000000,0x0000000f5b1052c61ce1a9521db1375200000000001ed31b0000000000000000],


#Tutorial
["tutorial_trainer","Training Ground Master","Training Ground Master",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008414401e28f534c8a2d09],
["tutorial_student_1","{!}tutorial_student_1","{!}tutorial_student_1",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_neutral,
 [itm_practice_sword,itm_practice_shield,itm_leather_jerkin,itm_padded_leather,itm_leather_armor,itm_ankle_boots,itm_padded_coif,itm_footman_helmet],
 def_attrib|level(2),wp(20),knows_common,sarranid_face_young_1,swadian_face_old_2],
["tutorial_student_2","{!}tutorial_student_2","{!}tutorial_student_2",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_neutral,
 [itm_practice_sword,itm_practice_shield,itm_leather_jerkin,itm_padded_leather,itm_leather_armor,itm_ankle_boots,itm_padded_coif,itm_footman_helmet],
 def_attrib|level(2),wp(20),knows_common,sarranid_face_young_1,swadian_face_old_2],
["tutorial_student_3","{!}tutorial_student_3","{!}tutorial_student_3",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_neutral,
 [itm_practice_staff,itm_leather_jerkin,itm_padded_leather,itm_leather_armor,itm_ankle_boots,itm_padded_coif,itm_footman_helmet],
 def_attrib|level(2),wp(20),knows_common,sarranid_face_young_1,swadian_face_old_2],
["tutorial_student_4","{!}tutorial_student_4","{!}tutorial_student_4",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_neutral,
 [itm_practice_staff,itm_leather_jerkin,itm_padded_leather,itm_leather_armor,itm_ankle_boots,itm_padded_coif,itm_footman_helmet],
 def_attrib|level(2),wp(20),knows_common,sarranid_face_young_1,swadian_face_old_2],


#Salt mine
["Galeas","Galeas","Galeas",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_jacket,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x0000000cea009241072c91eae48f272e00000000001da6dc0000000000000000,0x0000000cea009241072c91eae48f272e00000000001da6dc0000000000000000],

#Dhorak keep

["farmer_from_bandit_village","Farmer","Farmers",tf_mounted|tf_guarantee_armor,no_scene,reserved,fac_commoners,
 [itm_linen_tunic,itm_coarse_tunic,itm_shirt,itm_nomad_boots,itm_wrapping_boots],
 def_attrib|level(4),wp(60),knows_common,man_face_middle_1,man_face_older_2],

["trainer_1","Trainer","Trainer",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,scn_training_ground_ranged_melee_1|entry(6),reserved, fac_commoners,[itm_leather_jerkin,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x0000000d0d1030c74ae8d661b651c6840000000000000e220000000000000000,0x0000000d0d1030c74ae8d661b651c6840000000000000e220000000000000000],
["trainer_2","Trainer","Trainer",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,scn_training_ground_ranged_melee_2|entry(6),reserved, fac_commoners,[itm_nomad_vest,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x0000000e5a04360428ec253846640b5d0000000000000ee80000000000000000,0x0000000e5a04360428ec253846640b5d0000000000000ee80000000000000000],
["trainer_3","Trainer","Trainer",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,scn_training_ground_ranged_melee_3|entry(6),reserved, fac_commoners,[itm_padded_leather,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x0000000e4a0445822ca1a11ab1e9eaea0000000000000f510000000000000000,0x0000000e4a0445822ca1a11ab1e9eaea0000000000000f510000000000000000],
["trainer_4","Trainer","Trainer",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,scn_training_ground_ranged_melee_4|entry(6),reserved, fac_commoners,[itm_leather_jerkin,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x0000000e600452c32ef8e5bb92cf1c970000000000000fc20000000000000000,0x0000000e600452c32ef8e5bb92cf1c970000000000000fc20000000000000000],
["trainer_5","Trainer","Trainer",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,scn_training_ground_ranged_melee_5|entry(6),reserved, fac_commoners,[itm_leather_vest,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x0000000e77082000150049a34c42ec960000000000000e080000000000000000,0x0000000e77082000150049a34c42ec960000000000000e080000000000000000],

# Ransom brokers.
["ransom_broker_1","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_vest,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000074010449322eacd4aea6a172500000000001e2a520000000000000000,0x000000074010449322eacd4aea6a172500000000001e2a520000000000000000],
["ransom_broker_2","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_tabard,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000077e0c5482695a519d2438371400000000000e14a40000000000000000,0x000000077e0c5482695a519d2438371400000000000e14a40000000000000000],
["ransom_broker_3","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_vest,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000076f08340f59abb139d256c86b00000000001a48dc0000000000000000,0x000000076f08340f59abb139d256c86b00000000001a48dc0000000000000000],
["ransom_broker_4","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_short_tunic,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x00000007540093903a96e6049391a91d00000000001348e60000000000000000,0x00000007540093903a96e6049391a91d00000000001348e60000000000000000],
["ransom_broker_5","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x0000000d76042208684d94a66375b69300000000001e3d590000000000000000,0x0000000d76042208684d94a66375b69300000000001e3d590000000000000000],
["ransom_broker_6","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x00000004210410c013528cd65b2e3b4800000000001da92a0000000000000000,0x00000004210410c013528cd65b2e3b4800000000001da92a0000000000000000],
["ransom_broker_7","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x0000000c1a0032c845224dd96d4a98e700000000001e26db0000000000000000,0x0000000c1a0032c845224dd96d4a98e700000000001e26db0000000000000000],
["ransom_broker_8","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_fur_coat,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x0000000c2a10950f62556de8667b2b6300000000001a265a0000000000000000,0x0000000c2a10950f62556de8667b2b6300000000001a265a0000000000000000],
["ransom_broker_9","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_vest,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x00000003f50c22513adb1944ea6c817200000000001dd2ad0000000000000000,0x00000003f50c22513adb1944ea6c817200000000001dd2ad0000000000000000],
["ransom_broker_10","Ransom_Broker","Ransom_Broker",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_jacket,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000093804540c32db562a9d89ab2500000000001e05220000000000000000,0x000000093804540c32db562a9d89ab2500000000001e05220000000000000000],

# Tavern traveler.
["tavern_traveler_1","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_fur_coat,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000090f08c34e26f3b2135c8db86100000000000927650000000000000000,0x000000090f08c34e26f3b2135c8db86100000000000927650000000000000000],
["tavern_traveler_2","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_tabard,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000091510350c28996d44a1c9baec00000000000d5d2c0000000000000000,0x000000091510350c28996d44a1c9baec00000000000d5d2c0000000000000000],
["tavern_traveler_3","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_vest,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x00000003690cb19336e972673345b79c00000000001e031c0000000000000000,0x00000003690cb19336e972673345b79c00000000001e031c0000000000000000],
["tavern_traveler_4","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x00000003780803d449935668e555549f000000000004e4f60000000000000000,0x00000003780803d449935668e555549f000000000004e4f60000000000000000],
["tavern_traveler_5","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_short_tunic,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000036510c0cd471336b26475a2a200000000001d931a0000000000000000,0x000000036510c0cd471336b26475a2a200000000001d931a0000000000000000],
["tavern_traveler_6","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_fur_coat,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000034000754448592f311a69c53c000000000011e6e30000000000000000,0x000000034000754448592f311a69c53c000000000011e6e30000000000000000],
["tavern_traveler_7","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_jacket,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000037a10158f5b1b865b995598d500000000001deb610000000000000000,0x000000037a10158f5b1b865b995598d500000000001deb610000000000000000],
["tavern_traveler_8","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_tabard,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000035000a4d029636dc5094e54a500000000001ee2e10000000000000000,0x000000035000a4d029636dc5094e54a500000000001ee2e10000000000000000],
["tavern_traveler_9","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_fur_coat,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000034a1031512b6b5354ea565895000000000006d5a40000000000000000,0x000000034a1031512b6b5354ea565895000000000006d5a40000000000000000],
["tavern_traveler_10","Traveller","Traveller",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_jacket,itm_hide_boots],def_attrib|level(5),wp(20),knows_common,0x000000034810911426db55c45d11e7ae000000000011c89d0000000000000000,0x000000034810911426db55c45d11e7ae000000000011c89d0000000000000000],

# Tavern traveler.
["tavern_bookseller_1","Book_Merchant","Book_Merchant",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_fur_coat,itm_hide_boots,
 itm_book_tactics,itm_book_persuasion,itm_book_wound_treatment_reference,itm_book_leadership,
 itm_book_intelligence,itm_book_training_reference,itm_book_surgery_reference],def_attrib|level(5),wp(20),knows_common,0x000000037110654764aa563a56a1a46e00000000001eaa940000000000000000,0x000000037110654764aa563a56a1a46e00000000001eaa940000000000000000],
["tavern_bookseller_2","Book_Merchant","Book_Merchant",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_fur_coat,itm_hide_boots,
 itm_book_wound_treatment_reference,itm_book_leadership,itm_book_intelligence,itm_book_trade,
 itm_book_engineering,itm_book_weapon_mastery],def_attrib|level(5),wp(20),knows_common,0x000000034e0883c244d38ca0e369d75c00000000001cbc640000000000000000,0x000000034e0883c244d38ca0e369d75c00000000001cbc640000000000000000],

# Tavern minstrel.
["tavern_minstrel_1","Wandering Minstrel","Minstrel",tf_mounted|tf_hero|tf_guarantee_shield|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_leather_jacket,itm_hide_boots,itm_lute],def_attrib|level(5),wp(20),knows_common,0x000000036410008e27adcd48a6d63b6c00000000000e5ce10000000000000000,0x000000036410008e27adcd48a6d63b6c00000000000e5ce10000000000000000],#lute
["tavern_minstrel_2","Wandering Bard","Minstrel",tf_mounted|tf_hero|tf_guarantee_shield|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_tunic_with_green_cape,itm_hide_boots,itm_lyre],def_attrib|level(5),wp(20),knows_common,0x000000034b0051cb46e30cd723754a7400000000000e2d5a0000000000000000,0x000000034b0051cb46e30cd723754a7400000000000e2d5a0000000000000000], #early harp/lyre
["tavern_minstrel_3","Wandering Ashik","Minstrel",tf_mounted|tf_hero|tf_guarantee_shield|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_nomad_robe,itm_hide_boots,itm_lute],def_attrib|level(5),wp(20),knows_common,0x000000035804a44648744d3aa3292711000000000010b9230000000000000000,0x000000035804a44648744d3aa3292711000000000010b9230000000000000000],#lute/oud or rebab
["tavern_minstrel_4","Wandering Skald","Minstrel",tf_mounted|tf_hero|tf_guarantee_shield|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_fur_coat,itm_hide_boots,itm_lyre],def_attrib|level(5),wp(20),knows_common,0x00000003410c918a266589d7b1b6daa300000000001636e40000000000000000,0x00000003410c918a266589d7b1b6daa300000000001636e40000000000000000],#No instrument or lyre
["tavern_minstrel_5","Wandering Troubadour","Minstrel",tf_mounted|tf_hero|tf_guarantee_shield|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,[itm_short_tunic,itm_hide_boots,itm_lute],def_attrib|level(5),wp(20),knows_common,0x0000000368046307448f84c75b6936b200000000001db4630000000000000000,0x0000000368046307448f84c75b6936b200000000001db4630000000000000000],#Lute or Byzantine/Occitan lyra

#NPC system changes begin
#Companions
["kingdom_heroes_including_player_begin", "kingdom_heroes_including_player_begin", "kingdom_heroes_including_player_begin", tf_hero,0,reserved, fac_kingdom_of_swadians,[],lord_attrib,wp(220),knows_lord_1,0x000000000010918a01f248377289467d],

#DEATH KNIGHTS LEADERS
["death_knights_lord", "Baron Von Death", "Von Death", tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_death_knights,[itm_mackie_morning_star_long,itm_narf_milanese_plate_dark,itm_spak_horny_charger_plate,itm_narf_steel_greaves_dark,itm_narf_wisby_gauntlets_black,itm_spak_demon_hood], assassin_attrib,wp(300),knight_skills_5|knows_trainer_5,0x0000000a7f10b394189c71d6d386e8ac00000000001e24eb0000000000000000,0x0000000a7f10b394189c71d6d386e8ac00000000001e24eb0000000000000000],

#TEMPLAR LEADERS
["templars_lord", "Wilhelm Peace-Of-God", "Wilhelm", tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_templars,[itm_xeno_war_pick02,itm_narf_early_transitional_templar,itm_warhorse_chain_barding_templar,itm_narf_steel_greaves_dark,itm_narf_wisby_gauntlets_black,itm_narf_sugarloaf_coif_templar], assassin_attrib,wp(300),knight_skills_5|knows_trainer_5,0x0000000a7f10b394189c71d6d386e8ac00000000001e24eb0000000000000000,0x0000000a7f10b394189c71d6d386e8ac00000000001e24eb0000000000000000],

["npc1","Borcha","Borcha",tf_mounted|tf_hero|tf_guarantee_horse,0,reserved,fac_player_supporters_faction,[itm_khergit_armor,itm_nomad_boots,itm_knife,itm_sumpter_horse,itm_flintlock_pistol,itm_bolts],
 str_8|agi_12|int_12|cha_7|level(2),wp(40)|wp_polearm(125)|wp_crossbow(550),knows_tracker_npc|knows_pathfinding_3|knows_looting_4|knows_tracking_1|knows_riding_4|knows_toughness_3|knows_reflexes_3|knows_combat_training_1|knows_leadership_10,#skills 2/3 player at that level
 0x00000004bf08b143259d061a9046e23500000000001db52c0000000000000000],
["npc2","Marnid","Marnid",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_player_supporters_faction,[itm_linen_tunic,itm_hide_boots,itm_club,itm_dragon,itm_bolts],
 str_7|agi_12|int_11|cha_12|level(1),wp(40)|wp_crossbow(500),knows_merchant_npc|
 knows_riding_2|knows_trade_4|knows_weapon_master_1|knows_toughness_1|knows_defensive_tactics_1|knows_wound_treatment_1|knows_first_aid_1|knows_leadership_10,
 0x000000018300c001570b893712c8d28d00000000001dc8990000000000000000],
["npc3","Ymira","Ymira",tf_mounted|tf_female|tf_hero,0,reserved,fac_player_supporters_faction,[itm_dress,itm_woolen_hose,itm_knife],
 str_6|agi_14|int_13|cha_15|level(1),wp(60)|wp_archery(100),knows_merchant_npc|
 knows_wound_treatment_1|knows_trade_4|knows_first_aid_3|knows_surgery_1|knows_reflexes_4|knows_riding_3|knows_leadership_10,
 0x00000000b21110055b6b76346baf267200000000001e37630000000000000000],
["npc4","Rolf","Rolf",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_player_supporters_faction,[itm_leather_jerkin,itm_nomad_boots,itm_sword_medieval_a],
 str_13|agi_9|int_13|cha_10|level(2),wp(50)|wp_crossbow(100)|wp_one_handed(150),knows_warrior_npc|
 knows_weapon_master_2|knows_riding_2|knows_reflexes_5|knows_first_aid_1|knows_surgery_1|knows_combat_training_1|knows_leadership_10,
 0x000000057f1025c02c75c6a8a58ad72e00000000001e1a890000000000000000],
["npc5","Baheshtur","Baheshtur",tf_mounted|tf_guarantee_horse|tf_hero,0,reserved, fac_player_supporters_faction,[itm_nomad_vest,itm_nomad_boots,itm_sword_khergit_1,itm_xenoargh_blackleapord],
 str_12|agi_13|int_9|cha_7|level(2),wp(90)|wp_archery(200),knows_warrior_npc|
 knows_riding_5|knows_horse_archery_3|knows_weapon_master_2|knows_toughness_2|knows_reflexes_2|knows_combat_training_2|knows_leadership_10,
 0x00000008b110918b5c6f972328324a6200000000001cd3310000000000000000],
["npc6","Firentis","Firentis",tf_mounted|tf_guarantee_horse|tf_hero,0,reserved, fac_player_supporters_faction,[itm_brigandine_red,itm_nomad_boots,itm_sword_medieval_a,itm_xenoargh_bayappy,itm_dejawolf_krokaspjott2],
 str_14|agi_14|int_10|cha_5|level(2),wp(40)|wp_two_handed(225)|wp_one_handed(200),knows_warrior_npc|
 knows_riding_3|knows_weapon_master_2|knows_toughness_4|knows_reflexes_1|knows_trainer_1|knows_combat_training_4|knows_leadership_10,
0x000000022200b2036a1895d0748f3ca300000000001c0f0b0000000000000000],
["npc7","Deshavi","Deshavi",tf_mounted|tf_guarantee_ranged|tf_female|tf_hero,0,reserved, fac_player_supporters_faction,[itm_ragged_outfit,itm_wrapping_boots,itm_deshavi_bow,itm_arrows,itm_arrows,itm_quarter_staff],
 str_8|agi_16|int_10|cha_6|level(1),wp(60)|wp_archery(600),knows_tracker_npc|
 knows_tracking_2|knows_riding_2|knows_reflexes_4|knows_spotting_3|knows_pathfinding_3|knows_leadership_10,
 0x00000001fc08100533a15297634d44f400000000001e02db0000000000000000],
["npc8","Matheld","Matheld",tf_mounted|tf_guarantee_horse|tf_female|tf_hero,0,reserved, fac_player_supporters_faction,[itm_tribal_warrior_outfit,itm_nomad_boots,itm_dejawolf_langr_hoggspjott1,itm_xenoargh_shield_small03,itm_xenoargh_buckskin],
 str_9|agi_10|int_9|cha_10|level(2),wp(50)|wp_polearm(225)|wp_throwing(100)|wp_archery(110),knows_warrior_npc|
 knows_weapon_master_3|knows_riding_3|knows_toughness_2|knows_reflexes_2|knows_combat_training_2|knows_heroism_1|knows_leadership_10,
 0x00000005800e700637db8a14e351e76e00000000001c42db0000000000000000],
["npc9","Alayen","Alayen",tf_mounted|tf_hero|tf_guarantee_horse,0,reserved, fac_player_supporters_faction,[itm_mail_shirt,itm_nomad_boots,itm_hunter,itm_boar_spear,itm_xenoargh_shield_heater01],
 str_13|agi_11|int_7|cha_8|level(3),wp(40)|wp_one_handed(125)|wp_two_handed(220)|wp_polearm(150),knows_warrior_npc|
 knows_weapon_master_1|knows_riding_4|knows_toughness_4|knows_reflexes_1|knows_combat_training_3|knows_heroism_1|knows_leadership_10,
 0x000000002100a00f36db6db6db6db6db00000000001db6db0000000000000000],
["npc10","Bunduk","Bunduk",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_player_supporters_faction,[itm_padded_leather,itm_nomad_boots,itm_crossbow,itm_bolts,itm_pickaxe],
 str_12|agi_15|int_9|cha_11|level(2),wp(60)|wp_one_handed(125)|wp_crossbow(200),knows_warrior_npc|
 knows_weapon_master_3|knows_toughness_3|knows_trainer_2|knows_defensive_tactics_1|knows_combat_training_3|knows_leadership_10,
 0x0000000a3f082006572c91c71c8d46cb00000000001e468a0000000000000000],
["npc11","Katrin","Katrin",tf_mounted|tf_female|tf_hero,0,reserved, fac_player_supporters_faction,[itm_leather_apron,itm_falchion,itm_wrapping_boots],
 str_8|agi_11|int_10|cha_10|level(2),wp(70)|wp_crossbow(120),knows_merchant_npc|
 knows_weapon_master_1|knows_first_aid_1|knows_wound_treatment_2|knows_toughness_3|knows_inventory_management_5|knows_persuasion_3|knows_defensive_tactics_1|knows_leadership_10,
 0x0000000d7f0620045915aa226b4d975200000000001ea49e0000000000000000],
["npc12","Jeremus","Jeremus",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_player_supporters_faction,[itm_pilgrim_disguise,itm_nomad_boots,itm_staff],
 str_11|agi_7|int_13|cha_7|level(1),wp(30)|wp_polearm(100),knows_merchant_npc|
 knows_toughness_1|knows_surgery_4|knows_wound_treatment_3|knows_first_aid_3|knows_persuasion_3|knows_leadership_10,
 0x000000079100400e4f8ba62a9cd5d36d00000000001e36250000000000000000],
["npc13","Nizar","Nizar",tf_mounted|tf_guarantee_horse|tf_hero,0,reserved, fac_player_supporters_faction,[itm_nomad_robe,itm_nomad_boots,itm_scimitar,itm_courser,itm_xenoargh_bwanga],
 str_12|agi_11|int_9|cha_8|level(2),wp(60)|wp_one_handed(175)|wp_throwing(200),knows_warrior_npc|
 knows_riding_4|knows_reflexes_5|knows_toughness_2|knows_heroism_1|knows_weapon_master_4|knows_leadership_10,
 0x00000004bf04c1085f4e9592de4e574c00000000001e369c0000000000000000],
["npc14","Lezalit","Lezalit",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_player_supporters_faction,[itm_nobleman_outfit,itm_nomad_boots,itm_sword_medieval_b_small,itm_dejawolf_bryntvari],
 str_9|agi_8|int_13|cha_8|level(3),wp(70)|wp_one_handed(150)|wp_crossbow(125),knows_warrior_npc|
 knows_trainer_6|knows_weapon_master_2|knows_leadership_10|knows_heroism_1|knows_defensive_tactics_4,
 0x000000018e10759144d5d1d6eb55e96a00000000001db0db0000000000000000],
["npc15","Artimenner","Artimenner",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_player_supporters_faction,[itm_rich_outfit,itm_nomad_boots,itm_sword_medieval_b_small,itm_xenoargh_bola],
 str_9|agi_9|int_15|cha_8|level(4),wp(80)|wp_polearm(200),knows_warrior_npc|
 knows_heroism_4|knows_defensive_tactics_1|knows_engineer_5|knows_trade_3|knows_tracking_1|knows_spotting_1|knows_leadership_10,
 0x0000000f2e1031862b4b9123594eab5300000000001d55360000000000000000],
["npc16","Klethi","Klethi",tf_mounted|tf_female|tf_hero,0,reserved, fac_player_supporters_faction,[itm_peasant_dress,itm_nomad_boots,itm_dagger,itm_throwing_knives],str_7|agi_13|int_8|cha_7|level(1),wp(80)|wp_throwing(500),knows_tracker_npc|
 knows_riding_2|knows_reflexes_6|knows_looting_5|knows_leadership_10,
 0x000000000012f00734efa60aa38c597100000000001cc6d50000000000000000],
#NPC system changes end

#The Kings.  Upgraded their equipment / skills considerably.  Look out for Ragnar!
["kingdom_of_swadians_lord", "King Harlaus", "Harlaus", tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_charger,itm_rich_outfit,itm_blue_hose, itm_narf_shynbaulds,itm_narf_milanese_plate, itm_gauntlets, itm_balanced_winged_mace, itm_xenoargh_shield_heater02, itm_narf_hounskull],knight_attrib_5,wp(220),knight_skills_5|knows_trainer_5,0x0000000f45043105241acd2b5a66a86900000000001e98310000000000000000,0x0000000f45043105241acd2b5a66a86900000000001e98310000000000000000],

["kingdom_of_vaegirs_lord", "King Yaroglek", "Yaroglek", tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_charger_plate, itm_nobleman_outfit, itm_leather_boots,itm_taleworlds_doubleaxe,itm_spak_hermitage_shield_1,itm_narf_milanese_plate_dark,itm_narf_shynbaulds_dark,itm_narf_steel_greaves_dark,itm_rhulg_helmet,itm_narf_plate_mittens_dark], knight_attrib_5,wp(220),knight_skills_5|knows_trainer_4,0x0000000ef50061400a2269f919dee11700000000001cc57d0000000000000000,0x0000000ef50061400a2269f919dee11700000000001cc57d0000000000000000],

["kingdom_of_khergits_lord", "Sanjar Khan", "Sanjar", tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_warhorse_steppe,itm_nomad_robe, itm_leather_boots,itm_splinted_greaves, itm_wei_xiadi_samurai_armor02, itm_scale_gauntlets, itm_nodachi, itm_ssh_kabuto], assassin_attrib,wp(300),knight_skills_5|knows_trainer_6,0x0000000cee0081cc44be2d14d370c65c00000000001ed6df0000000000000000,0x0000000cee0081cc44be2d14d370c65c00000000001ed6df0000000000000000],

["kingdom_of_nords_lord", "King Ragnar", "Ragnar", tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_warhorse_chain, itm_nobleman_outfit, itm_narf_splinted_greaves_spurs, itm_wei_xiadi_nord_spiked_mail, itm_gauntlets, itm_mackie_double_axe, itm_xenoargh_shield_small04, itm_xenoargh_viking_horned_helm01],beserkir_attrib,wp(300),knight_skills_5|knows_trainer_4,0x0000000e0c0c328a068e8c18557b12a500000000001c0fe80000000000000000,0x0000000e0c0c328a068e8c18557b12a500000000001c0fe80000000000000000],
["kingdom_of_rhodoks_lord", "King Graveth", "Graveth", tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_charger_plate, itm_tabard, itm_leather_boots,itm_splinted_leather_greaves, itm_narf_gothic_plate, itm_gauntlets, itm_rrr_spear6, itm_xenoargh_shield_heater02, itm_narf_open_sallet], knight_attrib_5,wp(220),knight_skills_5|knows_trainer_5,0x0000000efc04219225848dac5d50d62400000000001d48b80000000000000000,0x0000000efc04219225848dac5d50d62400000000001d48b80000000000000000],
["kingdom_of_sarranids_lord", "Sultan Hakim", "Hakim", tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_warhorse_sarranid,itm_wei_xiadi_sarranid_mamluk_armor,itm_splinted_leather_greaves,itm_sarranid_mail_coif, itm_narf_wisby_gauntlets_black, itm_lui_sultancimitar, itm_xenoargh_shield_small02], assassin_attrib,wp(300),knight_skills_5|knows_trainer_5,0x0000000a7f10b394189c71d6d386e8ac00000000001e24eb0000000000000000,0x0000000a7f10b394189c71d6d386e8ac00000000001e24eb0000000000000000],

#Swadian Lords.
#Equipment much-upgraded, as well as some skills.  Much tougher heroes.
["knight_1_1","Count Klargus","Klargus",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_5,wp(230),knight_skills_5|knows_trainer_1|knows_trainer_3,0x0000000c3e08601414ab4dc6e39296b200000000001e231b0000000000000000,0x0000000c3e08601414ab4dc6e39296b200000000001e231b0000000000000000],
["knight_1_2","Count Delinard","Delinard",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_5,wp(240),knight_skills_5,0x0000000c0f0c320627627238dcd6599400000000001c573d0000000000000000,0x0000000c0f0c320627627238dcd6599400000000001c573d0000000000000000],
["knight_1_3","Count Haringoth","Haringoth",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_5,wp(260),knight_skills_5|knows_trainer_3,0x0000000ab700410214ce89db276aa2f400000000001d36730000000000000000,0x0000000ab700410214ce89db276aa2f400000000001d36730000000000000000],
["knight_1_4","Count Clais","Clais",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_5,wp(180),knight_skills_5|knows_trainer_4,0x0000000bd00ca194546469ca6c4e450e00000000001ebac40000000000000000,0x0000000bd00ca194546469ca6c4e450e00000000001ebac40000000000000000],
["knight_1_5","Count Deglan","Deglan",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_4,wp(200),knight_skills_4|knows_trainer_6,0x0000000c0c1064864ba34e2ae291992b00000000001da8720000000000000000,0x0000000c0c1064864ba34e2ae291992b00000000001da8720000000000000000],
["knight_1_6","Count Tredian","Tredian",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_5,wp(240),knight_skills_4|knows_trainer_4,0x0000000a0a08c38736db74c6a396a8e500000000001db8eb0000000000000000,0x0000000a0a08c38736db74c6a396a8e500000000001db8eb0000000000000000],
["knight_1_7","Count Grainwad","Grainwad",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_5,wp(290),knight_skills_4|knows_trainer_4,0x000000001e003500589dae4094aa291c00000000001e37a80000000000000000,0x000000001e003500589dae4094aa291c00000000001e37a80000000000000000],
["knight_1_8","Count Ryis","Ryis",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_4,wp(250),knight_skills_4,0x0000000db30865054aa9aa431a48d74600000000001ed5240000000000000000,0x0000000db30865054aa9aa431a48d74600000000001ed5240000000000000000],

#Swadian younger knights
["knight_1_9","Count Plais","Plais",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_3,wp(160),knight_skills_3,0x000000046d08600458739a9a1476199800000000001fb6f10000000000000000,0x000000046d08600458739a9a1476199800000000001fb6f10000000000000000],
["knight_1_10","Count Mirchaud","Mirchaud",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_3,wp(190),knight_skills_3,0x0000000c0610351048e325361d7236cd00000000001d532a0000000000000000,0x0000000c0610351048e325361d7236cd00000000001d532a0000000000000000],
["knight_1_11","Count Stamar","Stamar",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_3,wp(220),knight_skills_3,0x0000000c03104490280a8cb2a24196ab00000000001eb4dc0000000000000000,0x0000000c03104490280a8cb2a24196ab00000000001eb4dc0000000000000000],
["knight_1_12","Count Meltor","Meltor",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_3,wp(130),knight_skills_3,0x000000072a08a5442b2c6cc98c8dbaac00000000001d389b0000000000000000,0x000000072a08a5442b2c6cc98c8dbaac00000000001d389b0000000000000000],
["knight_1_13","Count Beranz","Beranz",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_2,wp(160),knight_skills_2,0x0000000c380c30c2392a8e5322a5392c00000000001e5c620000000000000000,0x0000000c380c30c2392a8e5322a5392c00000000001e5c620000000000000000],
["knight_1_14","Count Rafard","Rafard",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_2,wp(190),knight_skills_3|knows_trainer_6,0x00000006ff10c00532d45203954e192200000000001e47630000000000000000,0x00000006ff10c00532d45203954e192200000000001e47630000000000000000],
["knight_1_15","Count Regas","Regas",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_4,wp(140),knight_skills_2,0x0000000c5c0840034895654c9b660c5d00000000001e34530000000000000000,0x0000000c5c0840034895654c9b660c5d00000000001e34530000000000000000],
["knight_1_16","Count Devlian","Devlian",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_1,wp(130),knight_skills_2,0x000000027708b44657a1ba3ad456e8cb00000000001e325a0000000000000000,0x000000027708b44657a1ba3ad456e8cb00000000001e325a0000000000000000],
["knight_1_17","Count Rafarch","Rafarch",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_2,wp(190),knight_skills_1|knows_trainer_4,0x0000000c010c42c14d9d6918bdb336e200000000001dd6a30000000000000000,0x0000000c010c42c14d9d6918bdb336e200000000001dd6a30000000000000000],
["knight_1_18","Count Rochabarth","Rochabarth",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_3,wp(210),knight_skills_1,0x0000000c150045c6365d8565932a8d6400000000001ec6940000000000000000,0x0000000c150045c6365d8565932a8d6400000000001ec6940000000000000000],
["knight_1_19","Count Despin","Despin",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets], knight_attrib_1,wp(120),knight_skills_1,0x00000004460082033d9b6d4a92ada53500000000001cc1180000000000000000,0x00000004460082033d9b6d4a92ada53500000000001cc1180000000000000000],
["knight_1_20","Count Montewar","Montewar",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_swadians,[itm_narf_gothic_plate,itm_red_gambeson,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_spak_warhorse_imperial,itm_charger,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_balanced_winged_mace,itm_mace_2,itm_mace_3,itm_mace_4,itm_xenoargh_shield_kite01_05,itm_xenoargh_shield_kite01_01,itm_xenoargh_shield_kite01_02,itm_mail_chausses,itm_narf_shynbaulds,itm_narf_hounskull,itm_full_helm,itm_great_helmet,itm_winged_great_helmet,itm_dejawolf_flemish_armet,itm_dejawolf_greathelm1,itm_dejawolf_greathelm2,itm_narf_plate_mittens,itm_gauntlets],knight_attrib_2,wp(150),knight_skills_1,0x0000000c4d0840d24a9b2ab4ac2a332400000000001d34db0000000000000000,0x0000000c4d0840d24a9b2ab4ac2a332400000000001d34db0000000000000000],





#Vaegir Lords.
["knight_2_1","Boyar Vuldrat","Vuldrat",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_1,wp(130),knight_skills_1|knows_trainer_3,0x0000000c990061c33d9b6d4a92ada53500000000001cc1180000000000000000,0x0000000c990061c33d9b6d4a92ada53500000000001cc1180000000000000000],
["knight_2_2","Boyar Naldera","Naldera",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_2,wp(160),knight_skills_2,0x0000000b6a00b5d249b68b46a98e176400000000001d95a40000000000000000,0x0000000b6a00b5d249b68b46a98e176400000000001d95a40000000000000000],
["knight_2_3","Boyar Meriga","Meriga",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm],knight_attrib_3,wp(190),knight_skills_3,0x0000000c131031c546a38a2765b4c86000000000001e58d30000000000000000,0x0000000c131031c546a38a2765b4c86000000000001e58d30000000000000000],
["knight_2_4","Boyar Khavel","Khavel",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_4,wp(220),knight_skills_4,0x0000000c2f0832c748f272540d8ab65900000000001d34e60000000000000000,0x0000000c2f0832c748f272540d8ab65900000000001d34e60000000000000000],
["knight_2_5","Boyar Doru","Doru",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm],knight_attrib_5,wp(250),knight_skills_5,0x0000000e310061435d76bb5f55bad9ad00000000001ed8ec0000000000000000,0x0000000e310061435d76bb5f55bad9ad00000000001ed8ec0000000000000000],
["knight_2_6","Boyar Belgaru","Belgaru",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm],knight_attrib_1,wp(130),knight_skills_1|knows_trainer_3,0x0000000a0100421038da7157aa4e430a00000000001da8bc0000000000000000,0x0000000a0100421038da7157aa4e430a00000000001da8bc0000000000000000],
["knight_2_7","Boyar Ralcha","Ralcha",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm],knight_attrib_2,wp(160),knight_skills_2|knows_trainer_4,0x0000000f8410b153335ba9390b2d277500000000001d89120000000000000000,0x0000000f8410b153335ba9390b2d277500000000001d89120000000000000000],
["knight_2_8","Boyar Vlan","Vlan",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_3,wp(200),knight_skills_3|knows_trainer_5,0x0000000c00046581234e8da2cdd248db00000000001f569c0000000000000000,0x0000000c00046581234e8da2cdd248db00000000001f569c0000000000000000],
["knight_2_9","Boyar Mleza","Mleza",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_4,wp(230),knight_skills_4,0x0000000c160451d2136469c4d9b159ad00000000001e28f10000000000000000,0x0000000c160451d2136469c4d9b159ad00000000001e28f10000000000000000],
["knight_2_10","Boyar Nelag","Nelag",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_5,wp(260),knight_skills_5|knows_trainer_6,0x0000000f7c00520e66b76edd5cd5eb6e00000000001f691e0000000000000000,0x0000000f7c00520e66b76edd5cd5eb6e00000000001f691e0000000000000000],
["knight_2_11","Boyar Crahask","Crahask",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_1,wp(130),knight_skills_1,0x000000021d0831d236acd6991b74d69d00000000001e476c0000000000000000,0x000000021d0831d236acd6991b74d69d00000000001e476c0000000000000000],
["knight_2_12","Boyar Bracha","Bracha",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_2,wp(170),knight_skills_2,0x000000036d04624b2509d5d53944c6a300000000001d5b320000000000000000,0x000000036d04624b2509d5d53944c6a300000000001d5b320000000000000000],
["knight_2_13","Boyar Druli","Druli",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm],knight_attrib_3,wp(190),knight_skills_3,0x0000000c680432d3392230cb926d56ca00000000001da69b0000000000000000,0x0000000c680432d3392230cb926d56ca00000000001da69b0000000000000000],
["knight_2_14","Boyar Marmun","Marmun",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_4,wp(220),knight_skills_4|knows_trainer_6,0x0000000c27046000471bd2e93375b52c00000000001dd5220000000000000000,0x0000000c27046000471bd2e93375b52c00000000001dd5220000000000000000],
["knight_2_15","Boyar Gastya","Gastya",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm],knight_attrib_5,wp(250),knight_skills_5,0x0000000de50052123b6bb36de5d6eb7400000000001dd72c0000000000000000,0x0000000de50052123b6bb36de5d6eb7400000000001dd72c0000000000000000],
["knight_2_16","Boyar Harish","Harish",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm],knight_attrib_1,wp(120),knight_skills_1,0x000000049f00c00539233512e287391d00000000001db7200000000000000000,0x000000049f00c00539233512e287391d00000000001db7200000000000000000],
["knight_2_17","Boyar Taisa","Taisa",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm],knight_attrib_2,wp(150),knight_skills_2,0x0000000a070c4387374bd19addd2a4ab00000000001e32cc0000000000000000,0x0000000a070c4387374bd19addd2a4ab00000000001e32cc0000000000000000],
["knight_2_18","Boyar Valishin","Valishin",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_3,wp(180),knight_skills_3,0x00000005a70062c23d9b6d4a92ada53500000000001cc1180000000000000000,0x00000005a70062c23d9b6d4a92ada53500000000001cc1180000000000000000],
["knight_2_19","Boyar Rudin","Rudin",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_4,wp(210),knight_skills_4|knows_trainer_4,0x0000000e070050853b0a6e4994ae272a00000000001db4e10000000000000000,0x0000000e070050853b0a6e4994ae272a00000000001db4e10000000000000000],
["knight_2_20","Boyar Kumipa","Kumipa",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_vaegirs,[itm_amade_bronze_plate,itm_nobleman_outfit,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_blue_gambeson,itm_rich_outfit,itm_nomad_boots,itm_warhorse_chain,itm_warhorse_sarranid,itm_rrr_spear1,itm_rrr_spear2,itm_rrr_spear3,itm_rrr_spear4,itm_rrr_spear5,itm_rrr_spear6,itm_steel_shield,itm_steel_shield_herald01,itm_steel_shield_herald02,itm_steel_shield_herald03,itm_amade_bronze_greaves,itm_amade_bronze_gauntlets,itm_amade_bronze_winged_helm], knight_attrib_5,wp(240),knight_skills_5|knows_trainer_5,0x00000003c00031c63b0a6e4994ae272a00000000001db4e10000000000000000,0x00000003c00031c63b0a6e4994ae272a00000000001db4e10000000000000000],

#khergit civilian clothes: itm_leather_vest,itm_nomad_vest,itm_nomad_robe,itm_lamellar_vest,itm_tribal_warrior_outfit
["knight_3_1","Alagur Noyan","Alagur",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_1,wp(130),knight_skills_1|knows_trainer_3,0x000000043000318b54b246b7094dc39c00000000001d31270000000000000000,0x000000043000318b54b246b7094dc39c00000000001d31270000000000000000],
["knight_3_2","Tonju Noyan", "Tonju",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe],knight_attrib_2,wp(160),knight_skills_2,0x0000000c280461004929b334ad632aa200000000001e05120000000000000000,0x0000000c280461004929b334ad632aa200000000001e05120000000000000000],
["knight_3_3","Belir Noyan", "Belir",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_3,wp(190),knight_skills_3|knows_trainer_5,0x00000000310062c53b0a6e4994ae272a00000000001db4e10000000000000000,0x00000000310062c53b0a6e4994ae272a00000000001db4e10000000000000000],
["knight_3_4","Asugan Noyan","Asugan",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_4,wp(220),knight_skills_4,0x0000000c23085386391b5ac72a96d95c00000000001e37230000000000000000,0x0000000c23085386391b5ac72a96d95c00000000001e37230000000000000000],
["knight_3_5","Brula Noyan", "Brula",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_5,wp(250),knight_skills_5,0x0000000efe0051ca4b377b4964b6eb6500000000001f696c0000000000000000,0x0000000efe0051ca4b377b4964b6eb6500000000001f696c0000000000000000],
["knight_3_6","Imirza Noyan","Imirza",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe],knight_attrib_1,wp(130),knight_skills_1,0x00000006f600418b54b246b7094dc31a00000000001d37270000000000000000,0x00000006f600418b54b246b7094dc31a00000000001d37270000000000000000],
["knight_3_7","Urumuda Noyan","Urumuda",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe],knight_attrib_2,wp(160),knight_skills_2,0x0000000bdd00510a44be2d14d370c65c00000000001ed6df0000000000000000,0x0000000bdd00510a44be2d14d370c65c00000000001ed6df0000000000000000],
["knight_3_8","Kramuk Noyan","Kramuk",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_3,wp(190),knight_skills_3,0x0000000abc00518b5af4ab4b9c8e596400000000001dc76d0000000000000000,0x0000000abc00518b5af4ab4b9c8e596400000000001dc76d0000000000000000],
["knight_3_9","Chaurka Noyan","Chaurka",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_4,wp(220),knight_skills_4,0x00000004f70441c921a30ea68b54971500000000001e54db0000000000000000,0x00000004f70441c921a30ea68b54971500000000001e54db0000000000000000],
["knight_3_10","Sebula Noyan","Sebula",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe],knight_attrib_5,wp(250),knight_skills_5|knows_trainer_6,0x0000000a3b00418c5b36c686d920a76100000000001c436f0000000000000000,0x0000000a3b00418c5b36c686d920a76100000000001c436f0000000000000000],
["knight_3_11","Tulug Noyan","Tulug",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_1,wp(150),knight_skills_1,0x00000007d100534b44962d14d370c65c00000000001ed6df0000000000000000,0x00000007d100534b44962d14d370c65c00000000001ed6df0000000000000000],
["knight_3_12","Nasugei Noyan","Nasugei",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe],knight_attrib_2,wp(190),knight_skills_2,0x0000000bf400610c5b33d3c9258edb6c00000000001eb96d0000000000000000,0x0000000bf400610c5b33d3c9258edb6c00000000001eb96d0000000000000000],
["knight_3_13","Urubay Noyan","Urubay",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_3,wp(200),knight_skills_3|knows_trainer_3,0x0000000bfd0061c65b6eb33b25d2591d00000000001f58eb0000000000000000,0x0000000bfd0061c65b6eb33b25d2591d00000000001f58eb0000000000000000],
["knight_3_14","Hugu Noyan", "Hugu",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_4,wp(300),knight_skills_4|knows_trainer_6,0x0000000b6900514144be2d14d370c65c00000000001ed6df0000000000000000,0x0000000b6900514144be2d14d370c65c00000000001ed6df0000000000000000],
["knight_3_15","Tansugai Noyan","Tansugai",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_5,wp(240),knight_skills_5|knows_trainer_4,0x0000000c360c524b6454465b59b9d93500000000001ea4860000000000000000,0x0000000c360c524b6454465b59b9d93500000000001ea4860000000000000000],
["knight_3_16","Tirida Noyan","Tirida",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_1,wp(120),knight_skills_1,0x0000000c350c418438ab85b75c61b8d300000000001d21530000000000000000,0x0000000c350c418438ab85b75c61b8d300000000001d21530000000000000000],
["knight_3_17","Ulusamai Noyan","Ulusamai",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_2,wp(150),knight_skills_2,0x00000004bc08c1c647264ab6e68dc4d500000000001e42590000000000000000,0x00000004bc08c1c647264ab6e68dc4d500000000001e42590000000000000000],
["knight_3_18","Karaban Noyan","Karaban",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe],knight_attrib_3,wp(180),knight_skills_3|knows_trainer_4,0x0000000c0810500347ae7acd0d3ad74a00000000001e289a0000000000000000,0x0000000c0810500347ae7acd0d3ad74a00000000001e289a0000000000000000],
["knight_3_19","Akadan Noyan","Akadan",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_4,wp(210),knight_skills_4|knows_trainer_5,0x0000000c1500510528f50d52d20b152300000000001d66db0000000000000000,0x0000000c1500510528f50d52d20b152300000000001d66db0000000000000000],
["knight_3_20","Dundush Noyan","Dundush",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_khergits,[itm_wei_xiadi_samurai_armor01,itm_wei_xiadi_samurai_armor02,itm_wei_xiadi_kher_kaftan,itm_wei_xiadi_kher_robe,itm_nomad_boots,itm_eastern_arrows,itm_strong_bow,itm_katana,itm_nodachi,itm_lamellar_gauntlets,itm_ssh_kabuto,itm_warhorse_steppe], knight_attrib_5,wp(240),knight_skills_5,0x0000000f7800620d66b76edd5cd5eb6e00000000001f691e0000000000000000,0x0000000f7800620d66b76edd5cd5eb6e00000000001f691e0000000000000000],

#Nord Lords.
#Again, significant upgrades.  Most importantly, half of them are Beserkirs.
["knight_4_1","Jarl Aedin","Aedin",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(130),knight_skills_1,0x0000000e53007254340eb1d91159392d00000000001eb75a0000000000000000,0x0000000e53007254340eb1d91159392d00000000001eb75a0000000000000000],
["knight_4_2","Jarl Irya","Irya",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01], knight_attrib_2,wp(160),knight_skills_2|knows_trainer_3,0x0000000cc610b18368e29744e9a5985b00000000001db2a10000000000000000,0x0000000cc610b18368e29744e9a5985b00000000001db2a10000000000000000],
["knight_4_3","Jarl Olaf","Olaf",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(190),knight_skills_3,0x0000000c03043289245a314b744b30a400000000001eb2a90000000000000000,0x0000000c03043289245a314b744b30a400000000001eb2a90000000000000000],
["knight_4_4","Jarl Reamald","Reamald",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01], beserkir_attrib,wp(210),knight_skills_4,0x0000000ebf10a1ca3d6955b26a8939a300000000001e39b60000000000000000,0x0000000ebf10a1ca3d6955b26a8939a300000000001e39b60000000000000000],
["knight_4_5","Jarl Turya","Turya",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],knight_attrib_5,wp(250),knight_skills_5,0x0000000ff508330546dc4a59422d450c00000000001e51340000000000000000,0x0000000ff508330546dc4a59422d450c00000000001e51340000000000000000],
["knight_4_6","Jarl Gundur","Gundur",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(130),knight_skills_1,0x0000000fb000a1813d9b6d4a92ada53500000000001cc1180000000000000000,0x0000000fb000a1813d9b6d4a92ada53500000000001cc1180000000000000000],
["knight_4_7","Jarl Harald","Harald",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],knight_attrib_2,wp(160),knight_skills_2|knows_trainer_4,0x0000000f690042873d9b6d4a92ada53500000000001cc1180000000000000000,0x0000000f690042873d9b6d4a92ada53500000000001cc1180000000000000000],
["knight_4_8","Jarl Knudarr","Knudarr",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(190),knight_skills_3,0x0000000b430071c53b026e4994ae272a00000000001db4e10000000000000000,0x0000000b430071c53b026e4994ae272a00000000001db4e10000000000000000],
["knight_4_9","Jarl Haeda","Haeda",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01], knight_attrib_4,wp(220),knight_skills_4|knows_trainer_5,0x00000002110c74c1345bd21349b1b67300000000001c90c80000000000000000,0x00000002110c74c1345bd21349b1b67300000000001c90c80000000000000000],
["knight_4_10","Jarl Turegor","Turegor",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(250),knight_skills_5|knows_trainer_6,0x00000003e500a2063d9b6d4a92ada53500000000001cc1180000000000000000,0x00000003e500a2063d9b6d4a92ada53500000000001cc1180000000000000000],
["knight_4_11","Jarl Logarson","Logarson",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],knight_attrib_1,wp(140),knight_skills_1,0x000000020e106005471d4ae69ccacb1d00000000001dca550000000000000000,0x000000020e106005471d4ae69ccacb1d00000000001dca550000000000000000],
["knight_4_12","Jarl Aeric","Aeric",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(200),knight_skills_2,0x000000019500720824936cc51cb5bb2500000000001dd4d80000000000000000,0x000000019500720824936cc51cb5bb2500000000001dd4d80000000000000000],
["knight_4_13","Jarl Faarn","Faarn",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01], knight_attrib_3,wp(250),knight_skills_3|knows_trainer_3,0x000000057000b2c439233512e287391d00000000001db7200000000000000000,0x000000057000b2c439233512e287391d00000000001db7200000000000000000],
["knight_4_14","Jarl Bulba","Bulba",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(200),knight_skills_4,0x0000000c0700414f2cb6aa36ea50a69d00000000001dc55c0000000000000000,0x0000000c0700414f2cb6aa36ea50a69d00000000001dc55c0000000000000000],
["knight_4_15","Jarl Rayeck","Rayeck",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],knight_attrib_5,wp(290),knight_skills_5|knows_trainer_6,0x00000003d20861831715d1aa9221372300000000001ec6630000000000000000,0x00000003d20861831715d1aa9221372300000000001ec6630000000000000000],
["knight_4_16","Jarl Dirigun","Dirigun",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(120),knight_skills_1,0x00000008c300b24239233512e287391d00000000001db7200000000000000000,0x00000008c300b24239233512e287391d00000000001db7200000000000000000],
["knight_4_17","Jarl Marayirr","Marayirr",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],knight_attrib_2,wp(150),knight_skills_2|knows_trainer_4,0x0000000c2f0442036d232a2324b5b81400000000001e55630000000000000000,0x0000000c2f0442036d232a2324b5b81400000000001e55630000000000000000],
["knight_4_18","Jarl Gearth","Gearth",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(180),knight_skills_3,0x000000078d00418866e22e3d9735a72600000000001eacad0000000000000000,0x000000078d00418866e22e3d9735a72600000000001eacad0000000000000000],
["knight_4_19","Jarl Surdun","Surdun",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01], knight_attrib_4,wp(210),knight_skills_4|knows_trainer_5,0x000000050308325124e26d4a6295965a00000000001d23e40000000000000000,0x000000050308325124e26d4a6295965a00000000001d23e40000000000000000],
["knight_4_20","Jarl Gerlad","Gerlad",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_nords,[itm_dejawolf_vikingbyrnie,itm_wei_xiadi_scale_armor01,itm_wei_xiadi_scale_armor02,itm_rich_outfit,itm_short_tunic,itm_woolen_hose,itm_bolts,itm_xenoargh_gun_axe,itm_lui_nordicbastard,itm_lui_nordictwoh,itm_lui_bastardswordc,itm_lui_bastardswordnew,itm_lui_claymorebastard,itm_lui_claymorebroad,itm_lui_gothiconehander,itm_round_shield01_viking01,itm_mail_hauberk,itm_leather_boots,itm_nomad_boots,itm_xenoargh_viking_steel_winged_helm02,itm_xenoargh_viking_steel_winged_helm03,itm_leather_gloves,itm_mail_mittens,itm_warhorse_chain,itm_warhorse_chain_barding01],beserkir_attrib,wp(240),knight_skills_5,0x000000026c00b2813b6bb36de5d6eb7400000000001dd72c0000000000000000,0x000000026c00b2813b6bb36de5d6eb7400000000001dd72c0000000000000000],

#The Rhodok heroes didn't need a huge upgrade, other than finally wearing armor as good as the Condottieri they lead...
["knight_5_1","Count Matheas","Matheas",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_nobleman_outfit,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_1,wp(130),knight_skills_1|knows_trainer_3,0x0000000a1b0c20483adcbaa5ac9a34a200000000001ca2d40000000000000000,0x0000000a1b0c20483adcbaa5ac9a34a200000000001ca2d40000000000000000],
["knight_5_2","Count Gutlans","Gutlans",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_rich_outfit,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_2,wp(160),knight_skills_2|knows_trainer_4,0x0000000c390c659229136db45a75251300000000001f16930000000000000000,0x0000000c390c659229136db45a75251300000000001f16930000000000000000],
["knight_5_3","Count Laruqen","Laruqen",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_red_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_3,wp(190),knight_skills_3,0x0000000c2f10415108b1aacba27558d300000000001d329c0000000000000000,0x0000000c2f10415108b1aacba27558d300000000001d329c0000000000000000],
["knight_5_4","Count Raichs","Raichs",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_wei_xiadi_rod_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_4,wp(220),knight_skills_4,0x000000003c002110345c59d56975ba1200000000001e24e40000000000000000,0x000000003c002110345c59d56975ba1200000000001e24e40000000000000000],
["knight_5_5","Count Reland","Reland",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_wei_xiadi_rod_padded_cloth,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_5,wp(250),knight_skills_5,0x00000000060460c454826e471092299a00000000001d952d0000000000000000,0x00000000060460c454826e471092299a00000000001d952d0000000000000000],
["knight_5_6","Count Tarchias","Tarchias",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_blue_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_1,wp(130),knight_skills_1,0x0000000f9100b00648d24d36cd964b1d00000000001e2dac0000000000000000,0x0000000f9100b00648d24d36cd964b1d00000000001e2dac0000000000000000],
["knight_5_7","Count Gharmall","Gharmall",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_nobleman_outfit,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_2,wp(160),knight_skills_2,0x0000000d3a0465c443d46e4c8b91291a00000000001ca51b0000000000000000,0x0000000d3a0465c443d46e4c8b91291a00000000001ca51b0000000000000000],
["knight_5_8","Count Talbar","Talbar",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_red_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_3,wp(190),knight_skills_3|knows_trainer_3,0x0000000c2c0844d42914d19b2369b4ea00000000001e331b0000000000000000,0x0000000c2c0844d42914d19b2369b4ea00000000001e331b0000000000000000],
["knight_5_9","Count Rimusk","Rimusk",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_wei_xiadi_rod_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_4,wp(220),knight_skills_4|knows_trainer_6,0x00000000420430c32331b5551c4724a100000000001e39a40000000000000000,0x00000000420430c32331b5551c4724a100000000001e39a40000000000000000],
["knight_5_10","Count Falsevor","Falsevor",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_wei_xiadi_rod_padded_cloth,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_5,wp(250),knight_skills_5|knows_trainer_4,0x00000000370061063d9b6d4a92ada53500000000001cc1180000000000000000,0x00000000370061063d9b6d4a92ada53500000000001cc1180000000000000000],
["knight_5_11","Count Etrosq","Etrosq",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_blue_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_1,wp(130),knight_skills_1,0x000000043e0cc4874752adb6eb3228d500000000001c955c0000000000000000,0x000000043e0cc4874752adb6eb3228d500000000001c955c0000000000000000],
["knight_5_12","Count Kurnias","Kurnias",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_rich_outfit,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_2,wp(160),knight_skills_2|knows_trainer_5,0x00000004880c93d056ec8da85e3126ed00000000001d4ce60000000000000000,0x00000004880c93d056ec8da85e3126ed00000000001d4ce60000000000000000],
["knight_5_13","Count Tellrog","Tellrog",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_nobleman_outfit,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_3,wp(190),knight_skills_3,0x000000003f10a00562a4954ae731588a00000000001d6b530000000000000000,0x000000003f10a00562a4954ae731588a00000000001d6b530000000000000000],
["knight_5_14","Count Tribidan","Tribidan",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_red_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_4,wp(220),knight_skills_4,0x00000005730845823baa77556c4e331a00000000001cb9110000000000000000,0x00000005730845823baa77556c4e331a00000000001cb9110000000000000000],
["knight_5_15","Count Gerluchs","Gerluchs",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_wei_xiadi_rod_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_5,wp(250),knight_skills_5,0x000000063b00c106370c4d4732b536de00000000001db9280000000000000000,0x000000063b00c106370c4d4732b536de00000000001db9280000000000000000],
["knight_5_16","Count Fudreim","Fudreim",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_wei_xiadi_rod_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_1,wp(120),knight_skills_1,0x0000000c06046151435b5122a37756a400000000001c46e50000000000000000,0x0000000c06046151435b5122a37756a400000000001c46e50000000000000000],
["knight_5_17","Count Nealcha","Nealcha",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_wei_xiadi_rod_padded_cloth,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_2,wp(150),knight_skills_2,0x000000006710a1d3465c89a6a452356300000000001cda550000000000000000,0x000000006710a1d3465c89a6a452356300000000001cda550000000000000000],
["knight_5_18","Count Fraichin","Fraichin",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_blue_gambeson,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_3,wp(180),knight_skills_3,0x000000073d0c33c3452aa967276dc95c00000000001dad350000000000000000,0x000000073d0c33c3452aa967276dc95c00000000001dad350000000000000000],
["knight_5_19","Count Trimbau","Trimbau",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_rich_outfit,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate],knight_attrib_4,wp(210),knight_skills_4|knows_trainer_5,0x0000000038043194092ab4b2d9adb44c00000000001e072c0000000000000000,0x0000000038043194092ab4b2d9adb44c00000000001e072c0000000000000000],
["knight_5_20","Count Reichsin","Reichsin",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_rhodoks,[itm_narf_milanese_plate,itm_nobleman_outfit,itm_nomad_boots,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_pop_gothic_lance_red,itm_pop_gothic_lance_yellow,itm_lui_estoc,itm_lui_estocc,itm_lui_estocd,itm_lui_newestoc,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_plate_covered_round_shield,itm_xenoargh_shield_heater01,itm_mail_chausses,itm_narf_steel_greaves,itm_dejawolf_klappvisier,itm_narf_visored_sallet_coif,itm_dejawolf_greatbascinet1,itm_narf_plate_mittens,itm_gauntlets,itm_charger_plate], knight_attrib_5,wp(240),knight_skills_5|knows_trainer_6,0x000000003600420515a865b45c64d64c00000000001d544b0000000000000000,0x000000003600420515a865b45c64d64c00000000001d544b0000000000000000],

#Sarranid Lords.
#Funnily enough, they were the best-equipped Lords in Native, when Sarranid armor was mysteriously both lighter and more effective than practically anything else.
#Now they're middling, but half of them have Assassin stats.  Times change.
["knight_6_1","Emir Uqais","Uqais",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(130),knight_skills_1|knows_trainer_3,0x0000000aba1040cf47550e7b53b6238200000000001de69b0000000000000000,0x0000000aba1040cf47550e7b53b6238200000000001de69b0000000000000000],
["knight_6_2","Emir Hamezan","Hamezan",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],knight_attrib_2,wp(160),knight_skills_2|knows_trainer_4,0x0000000d3a10115147550e7b53b6238200000000001de69b0000000000000000,0x0000000d3a10115147550e7b53b6238200000000001de69b0000000000000000],
["knight_6_3","Emir Atis","Atis",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(190),knight_skills_3,0x000000002208428579723147247ad4e500000000001f14d40000000000000000,0x000000002208428579723147247ad4e500000000001f14d40000000000000000],
["knight_6_4","Emir Nuwas","Nuwas",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(220),knight_skills_4,0x00000009bf084285050caa7d285be51a00000000001d11010000000000000000,0x00000009bf084285050caa7d285be51a00000000001d11010000000000000000],
["knight_6_5","Emir Mundhalir","Mundhalir",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(250),knight_skills_5,0x0000000baa082003330175aae175da9c00000000001e02150000000000000000,0x0000000baa082003330175aae175da9c00000000001e02150000000000000000],
["knight_6_6","Emir Ghanawa","Ghanawa",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(130),knight_skills_1,0x0000000c3d00a3834733294c89b128e200000000001e59510000000000000000,0x0000000c3d00a3834733294c89b128e200000000001e59510000000000000000],
["knight_6_7","Emir Nuam","Nuam",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(160),knight_skills_2,0x0000000fbf10034020504bbbda9135d500000000001f62380000000000000000,0x0000000fbf10034020504bbbda9135d500000000001f62380000000000000000],
["knight_6_8","Emir Dhiyul","Dhiyul",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(190),knight_skills_3|knows_trainer_3,0x0000000c5004100b336dcd3ca2cacae300000000001f47640000000000000000,0x0000000c5004100b336dcd3ca2cacae300000000001f47640000000000000000],
["knight_6_9","Emir Lakhem","Lakhem",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(220),knight_skills_4|knows_trainer_6,0x00000004de0050c4549dd5ca6f4dd56500000000001e291b0000000000000000,0x00000004de0050c4549dd5ca6f4dd56500000000001e291b0000000000000000],
["knight_6_10","Emir Ghulassen","Ghulassen",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(250),knight_skills_5|knows_trainer_4,0x00000004660421c66ce99256b4ad4b3300000000001d392c0000000000000000,0x00000004660421c66ce99256b4ad4b3300000000001d392c0000000000000000],
["knight_6_11","Emir Azadun","Azadun",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(130),knight_skills_1,0x0000000fff08134726c28af8dc96e4da00000000001e541d0000000000000000,0x0000000fff08134726c28af8dc96e4da00000000001e541d0000000000000000],
["knight_6_12","Emir Quryas","Quryas",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],knight_attrib_2,wp(160),knight_skills_2|knows_trainer_5,0x00000004b5104084635b74ba5491a7a400000000001e46d60000000000000000,0x00000004b5104084635b74ba5491a7a400000000001e46d60000000000000000],
["knight_6_13","Emir Amdar","Amdar",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(190),knight_skills_3,0x00000002801001435b734d4ad94eba9400000000001eb8eb0000000000000000,0x00000002801001435b734d4ad94eba9400000000001eb8eb0000000000000000],
["knight_6_14","Emir Hiwan","Hiwan",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid], knight_attrib_4,wp(220),knight_skills_4,0x000000073e0ca5c63a5b921ac22db8e200000000001cca530000000000000000,0x000000073e0ca5c63a5b921ac22db8e200000000001cca530000000000000000],
["knight_6_15","Emir Muhnir","Muhnir",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(250),knight_skills_5,0x000000001b0c4185369a6938cecde95600000000001f25210000000000000000,0x000000001b0c4185369a6938cecde95600000000001f25210000000000000000],

["knight_6_16","Emir Ayyam","Ayyam",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid], knight_attrib_1,wp(120),knight_skills_1,0x00000007770841c80a01e1c5eb51ffff00000000001f12d80000000000000000,0x00000007770841c80a01e1c5eb51ffff00000000001f12d80000000000000000],
["knight_6_17","Emir Raddoun","Raddoun",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(150),knight_skills_2,0x000000007f0462c32419f47a1aba8bcf00000000001e7e090000000000000000,0x000000007f0462c32419f47a1aba8bcf00000000001e7e090000000000000000],
["knight_6_18","Emir Tilimsan","Tilimsan",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid], knight_attrib_3,wp(180),knight_skills_3,0x00000001b410110070d975caac91aca500000000001c27530000000000000000,0x00000001b410110070d975caac91aca500000000001c27530000000000000000],
["knight_6_19","Emir Dhashwal","Dhashwal",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid],assassin_attrib,wp(210),knight_skills_4|knows_trainer_5,0x000000070a08918016ac36bc8b6e4a9900000000001dd45d0000000000000000,0x000000070a08918016ac36bc8b6e4a9900000000001dd45d0000000000000000],
["knight_6_20","Emir Biliya","Biliya",tf_mounted|tf_hero|tf_guarantee_all_wo_ranged,0,reserved, fac_kingdom_of_sarranids,[itm_plate_armor,itm_wei_xiadi_sarranid_mamluk_armor,itm_wei_xiadi_rich_tunic02,itm_wei_xiadi_sar_robe01,itm_wei_xiadi_sar_robe02,itm_sarranid_boots_a,itm_heavy_lance,itm_lui_cimeterrec,itm_lui_sultancimitar,itm_xenoargh_shield_small02,itm_sarranid_boots_d,itm_sarranid_boots_c,itm_sarranid_veiled_helmet,itm_narf_wisby_gauntlets_black,itm_mail_mittens,itm_warhorse_sarranid], knight_attrib_5,wp(240),knight_skills_5|knows_trainer_6,0x000000037d0050c0281a899ac956b94b00000000001ec8910000000000000000,0x000000037d0050c0281a899ac956b94b00000000001ec8910000000000000000],




["kingdom_of_swadians_pretender", "Lady Isolla of Suno","Isolla", tf_hero|tf_female,0,reserved, fac_kingdom_of_swadians,[itm_charger,itm_rich_outfit, itm_blue_hose, itm_iron_greaves, itm_mail_shirt, itm_sword_medieval_c_small, itm_xenoargh_shield_small02,itm_bascinet],lord_attrib,wp(220),knows_lord_1,0x00000000ef02300237dc71b90c31631200000000001e371b0000000000000000,0x00000000ef02300237dc71b90c31631200000000001e371b0000000000000000],
#claims pre-salic descent

["kingdom_of_vaegirs_pretender", "Prince Valdym the Bastard","Valdym", tf_hero,0,reserved, fac_kingdom_of_vaegirs,[itm_warhorse_chain, itm_nobleman_outfit, itm_leather_boots,itm_mail_chausses,itm_lamellar_armor,itm_military_pick, itm_heater_shield_old, itm_flat_topped_helmet], lord_attrib,wp(220),knows_lord_1,0x00000000200492142452ed631b30365c00000000001c94e80000000000000000,0x00000000200492142452ed631b30365c00000000001c94e80000000000000000],
#had his patrimony falsified

["kingdom_of_khergits_pretender", "Dustum Khan","Dustum", tf_hero,0,reserved, fac_kingdom_of_khergits,[itm_warhorse_steppe,itm_nomad_robe, itm_leather_boots,itm_splinted_greaves, itm_khergit_guard_armor, itm_sword_khergit_2,itm_xenoargh_shield_small02,itm_segmented_helmet], lord_attrib,wp(220),knows_lord_1,0x000000067f04610b30d556b51238f66100000000001c256d0000000000000000,0x000000067f04610b30d556b51238f66100000000001c256d0000000000000000],
#of the family

["kingdom_of_nords_pretender", "Lethwin Far-Seeker","Lethwin", tf_hero,0,reserved, fac_kingdom_of_nords,[itm_warhorse_chain, itm_tabard, itm_leather_boots,itm_mail_boots,itm_brigandine_red, itm_sword_medieval_c, itm_xenoargh_shield_heater01, itm_kettle_hat],lord_attrib,wp(220),knows_lord_1,0x00000004340c71841d89949529a6776a00000000001c910a0000000000000000,0x00000004340c71841d89949529a6776a00000000001c910a0000000000000000],
#dispossessed and wronged

["kingdom_of_rhodoks_pretender", "Lord Kastor of Veluca", "Kastor", tf_hero,0,reserved, fac_kingdom_of_rhodoks,[itm_charger_plate, itm_nobleman_outfit, itm_leather_boots,itm_splinted_leather_greaves,itm_mail_hauberk, itm_sword_medieval_c, itm_heater_shield_heavy,itm_spiked_helmet], lord_attrib,wp(220),knows_lord_1,0x0000000bed1061051da9abc49ecce25e00000000001e98680000000000000000,0x0000000bed1061051da9abc49ecce25e00000000001e98680000000000000000],
#republican

["kingdom_of_sarranids_pretender", "Arwa the Pearled One","Arwa", tf_hero|tf_female,0,reserved, fac_kingdom_of_sarranids,[itm_warhorse_sarranid,itm_sarranid_mail_shirt,itm_sarranid_boots_c,itm_sarranid_cavalry_sword, itm_xenoargh_shield_small02],lord_attrib,wp(220),knows_lord_1,0x000000053f010004072d51c293a9a70b00000000001dd6a90000000000000000,0x000000053f010004072d51c293a9a70b00000000001dd6a90000000000000000],





#Royal family members

["knight_1_1_wife","Error - knight_1_1_wife should not appear in game","knight_1_1_wife",tf_mounted|tf_hero|tf_female,0,reserved,fac_commoners,[itm_lady_dress_ruby ,itm_turret_hat_ruby, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_1,european_woman_face_2],

#Swadian ladies - eight mothers,eight daughters,four sisters
["kingdom_of_swadians_lady_1","Lady Anna","Anna",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[ itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_1,european_woman_face_1],
["kingdom_of_swadians_lady_2","Lady Nelda","Nelda",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[ 
itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_1,european_woman_face_1],
["knight_1_lady_3","Lady Bela","Bela",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians, [itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_2,european_woman_face_2],
["knight_1_lady_4","Lady Elina","Elina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians, [itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_2,european_woman_face_2],
["kingdom_l_lady_5","Lady Constanis","Constanis",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_3,european_woman_face_3],
["kingdom_of_swadians_lady_6","Lady Vera","Vera",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_3,european_woman_face_3],
["kingdom_of_swadians_lady_7","Lady Auberina","Auberina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_4,european_woman_face_4],
["kingdom_of_swadians_lady_8","Lady Tibal","Tibal",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[ itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_4,european_woman_face_4],
["kingdom_of_swadians_lady_9","Lady Magar","Magar",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_5,european_woman_face_5],
["kingdom_of_swadians_lady_10","Lady Thedosa","Thedosa",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_5,european_woman_face_5],
["kingdom_of_swadians_lady_11","Lady Melisar","Melisar",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_6,european_woman_face_6],
["kingdom_of_swadians_lady_12","Lady Irena","Irena",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_6,european_woman_face_6],
["kingdom_l_lady_13","Lady Philenna","Philenna",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[ itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_7,european_woman_face_7],
["kingdom_of_swadians_lady_14","Lady Sonadel","Sonadel",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_7,european_woman_face_7],
["kingdom_of_swadians_lady_15","Lady Boadila","Boadila",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_8,european_woman_face_8],
["kingdom_of_swadians_lady_16","Lady Elys","Elys",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_8,european_woman_face_8],
["kingdom_of_swadians_lady_17","Lady Johana","Johana",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_9,european_woman_face_9],
["kingdom_of_swadians_lady_18","Lady Bernatys","Bernatys",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_10,european_woman_face_10],
["kingdom_of_swadians_lady_19","Lady Enricata","Enricata",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_11,european_woman_face_11],
["kingdom_of_swadians_lady_20","Lady Gaeta","Gaeta",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_lady_dress_green, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_12,european_woman_face_12],

#Vaegir ladies
["kingdom_of_vaegirs_lady_1","Lady Junitha","Junitha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_1,european_woman_face_1],
["kingdom_of_vaegirs_lady_2","Lady Katia","Katia",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_1,european_woman_face_1],
["kingdom_of_vaegirs_lady_3","Lady Seomis","Seomis",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [ itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_1,rhodok_woman_face_1],
["kingdom_of_vaegirs_lady_4","Lady Drina","Drina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_1,rhodok_woman_face_1],
["kingdom_of_vaegirs_lady_5","Lady Nesha","Nesha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_2,european_woman_face_2],
["kingdom_of_vaegirs_lady_6","Lady Tabath","Tabath",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[ itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_2,european_woman_face_2],
["kingdom_of_vaegirs_lady_7","Lady Pelaeka","Pelaeka",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [ itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_2,rhodok_woman_face_2],
["kingdom_of_vaegirs_lady_8","Lady Haris","Haris",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_2,rhodok_woman_face_2],
["kingdom_of_vaegirs_lady_9","Lady Vayen","Vayen",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_3,european_woman_face_3],
["kingdom_of_vaegirs_lady_10","Lady Joaka","Joaka",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_3,european_woman_face_3],
["kingdom_of_vaegirs_lady_11","Lady Tejina","Tejina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_3,rhodok_woman_face_3],
["kingdom_of_vaegirs_lady_12","Lady Olekseia","Olekseia",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_3,rhodok_woman_face_3],
["kingdom_of_vaegirs_lady_13","Lady Myntha","Myntha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_4,european_woman_face_4],
["kingdom_of_vaegirs_lady_14","Lady Akilina","Akilina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[ itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_4,european_woman_face_4],
["kingdom_of_vaegirs_lady_15","Lady Sepana","Sepana",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [ itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_4,rhodok_woman_face_4],
["kingdom_of_vaegirs_lady_16","Lady Iarina","Iarina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_4,rhodok_woman_face_4],
["kingdom_of_vaegirs_lady_17","Lady Sihavan","Sihavan",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_5,european_woman_face_5],
["kingdom_of_vaegirs_lady_18","Lady Erenchina","Erenchina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs,[itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_6,european_woman_face_6],
["kingdom_of_vaegirs_lady_19","Lady Tamar","Tamar",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_5,rhodok_woman_face_5],
["kingdom_of_vaegirs_lady_20","Lady Valka","Valka",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_vaegirs, [itm_green_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_6,rhodok_woman_face_6],


["kingdom_of_khergits_lady_1","Lady Borge","Borge",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_brown_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_1,khergit_woman_face_1],
["kingdom_of_khergits_lady_2","Lady Tuan","Tuan",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_green_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_2,khergit_woman_face_2],
["kingdom_of_khergits_lady_3","Lady Mahraz","Mahraz",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_red_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_3,khergit_woman_face_3],
["kingdom_of_khergits_lady_4","Lady Ayasu","Ayasu",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_red_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_4,khergit_woman_face_4],
["kingdom_of_khergits_lady_5","Lady Ravin","Ravin",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_green_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_5,khergit_woman_face_5],
["kingdom_of_khergits_lady_6","Lady Ruha","Ruha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_green_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_6,khergit_woman_face_6],
["kingdom_of_khergits_lady_7","Lady Chedina","Chedina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_brown_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_7,khergit_woman_face_7],
["kingdom_of_khergits_lady_8","Lady Kefra","Kefra",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_brown_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_8,khergit_woman_face_8],
["kingdom_of_khergits_lady_9","Lady Nirvaz","Nirvaz",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_brown_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_1,khergit_woman_face_1],
["kingdom_of_khergits_lady_10","Lady Dulua","Dulua",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_brown_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_2,khergit_woman_face_2],
["kingdom_of_khergits_lady_11","Lady Selik","Selik",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_brown_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_3,khergit_woman_face_3],
["kingdom_of_khergits_lady_12","Lady Thalatha","Thalatha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_brown_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_4,khergit_woman_face_4],
["kingdom_of_khergits_lady_13","Lady Yasreen","Yasreen",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_brown_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_5,khergit_woman_face_5],
["kingdom_of_khergits_lady_14","Lady Nadha","Nadha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_brown_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_6,khergit_woman_face_6],
["kingdom_of_khergits_lady_15","Lady Zenur","Zenur",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_brown_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_7,khergit_woman_face_7],
["kingdom_of_khergits_lady_16","Lady Arjis","Arjis",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_brown_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_8,khergit_woman_face_8],
["kingdom_of_khergits_lady_17","Lady Atjahan","Atjahan",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_brown_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_1,khergit_woman_face_1],
["kingdom_of_khergits_lady_18","Lady Qutala","Qutala",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits,[itm_brown_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_2,khergit_woman_face_2],
["kingdom_of_khergits_lady_19","Lady Hindal","Hindal",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_brown_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_3,khergit_woman_face_3],
["kingdom_of_khergits_lady_20","Lady Mechet","Mechet",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_khergits, [itm_brown_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_4,khergit_woman_face_4],



["kingdom_of_nords_lady_1","Lady Jadeth","Jadeth",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_1,european_woman_face_1],
["kingdom_of_nords_lady_2","Lady Miar","Miar",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_2,european_woman_face_2],
["kingdom_of_nords_lady_3","Lady Dria","Dria",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_3,european_woman_face_3],
["kingdom_of_nords_lady_4","Lady Glunde","Glunde",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_4,european_woman_face_4],
["kingdom_of_nords_lady_5","Lady Loeka","Loeka",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_5,european_woman_face_5],
["kingdom_of_nords_lady_6","Lady Bryn","Bryn",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_6,european_woman_face_6],
["kingdom_of_nords_lady_7","Lady Eir","Eir",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_7,european_woman_face_7],
["knight_4_2b_daughter","Lady Thera","Thera",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_8,european_woman_face_8],
["kingdom_of_nords_lady_9","Lady Hild","Hild",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress , itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_9,european_woman_face_9],
["knight_4_2c_wife","Lady Endegrid","Endegrid",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_10,european_woman_face_10],
["kingdom_of_nords_lady_11","Lady Herjasa","Herjasa",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_11,european_woman_face_11],
["knight_4_2c_daughter","Lady Svipul","Svipul",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_12,european_woman_face_12],
["knight_4_1b_wife","Lady Ingunn","Ingunn",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_1,european_woman_face_1],
["kingdom_of_nords_lady_14","Lady Kaeteli","Kaeteli",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_2,european_woman_face_2],
["knight_4_1b_daughter","Lady Eilif","Eilif",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_3,european_woman_face_3],
["knight_4_2b_daughter","Lady Gudrun","Gudrun",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_4,european_woman_face_4],
["kingdom_of_nords_lady_17","Lady Bergit","Bergit",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress , itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_5,european_woman_face_5],
["knight_4_2c_wife","Lady Aesa","Aesa",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords,[itm_court_dress ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_6,european_woman_face_6],
["knight_4_1c_daughter","Lady Alfrun","Alfrun",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_7,european_woman_face_7],
["kingdom_of_nords_lady_20","Lady Afrid","Afrid",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_nords, [itm_peasant_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,european_woman_face_8,european_woman_face_8],


["kingdom_of_rhodoks_lady_1","Lady Brina","Brina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_1,rhodok_woman_face_1],
["kingdom_of_rhodoks_lady_2","Lady Aliena","Aliena",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_2,rhodok_woman_face_2],
["kingdom_of_rhodoks_lady_3","Lady Aneth","Aneth",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_3,rhodok_woman_face_3],
["kingdom_of_rhodoks_lady_4","Lady Reada","Reada",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_4,rhodok_woman_face_4],
["kingdom_of_rhodoks_5_wife","Lady Saraten","Saraten",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_5,rhodok_woman_face_5],
["kingdom_of_rhodoks_2b_wife","Lady Baotheia","Baotheia",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_6,rhodok_woman_face_6],
["kingdom_of_rhodoks_1c_daughter","Lady Eleandra","Eleandra",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_7,rhodok_woman_face_7],
["kingdom_of_rhodoks_2c_daughter","Lady Meraced","Meraced",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby ,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_8,rhodok_woman_face_8],
["kingdom_of_rhodoks_1c_wife","Lady Adelisa","Adelisa",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_9,rhodok_woman_face_9],
["kingdom_of_rhodoks_2c_wife","Lady Calantina","Calantina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_10,rhodok_woman_face_10],
["kingdom_of_rhodoks_1c_daughter","Lady Forbesa","Forbesa",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby , itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_1,rhodok_woman_face_1],
["kingdom_of_rhodoks_2c_daughter","Lady Claudora","Claudora",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby , itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_2,rhodok_woman_face_2],
["kingdom_of_rhodoks_1b_wife","Lady Anais","Anais",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_3,rhodok_woman_face_3],
["kingdom_of_rhodoks_2b_wife","Lady Miraeia","Miraeia",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_4,rhodok_woman_face_4],
["kingdom_of_rhodoks_1c_daughter","Lady Agasia","Agasia",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby , itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_5,rhodok_woman_face_5],
["kingdom_of_rhodoks_lady_16","Lady Geneiava","Geneiava",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby , itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_6,rhodok_woman_face_6],
["kingdom_of_rhodoks_1c_wife","Lady Gwenael","Gwenael",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_7,rhodok_woman_face_7],
["kingdom_of_rhodoks_2c_wife","Lady Ysueth","Ysueth",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks,[itm_lady_dress_green,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_8,rhodok_woman_face_8],
["kingdom_of_rhodoks_1c_daughter","Lady Ellian","Ellian",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby , itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_9,rhodok_woman_face_9],
["kingdom_of_rhodoks_lady_20","Lady Timethi","Timethi",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_rhodoks, [ itm_lady_dress_ruby , itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,rhodok_woman_face_10,rhodok_woman_face_10],

#Sarranid ladies
["kingdom_of_sarranids_lady_1","Lady Rayma","Rayma",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress, itm_sarranid_head_cloth,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,sarranid_woman_face_1,sarranid_woman_face_1],
["kingdom_of_sarranids_lady_2","Lady Thanaikha","Thanaikha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,sarranid_woman_face_2,sarranid_woman_face_2],
["kingdom_of_sarranids_lady_3","Lady Sulaha","Sulaha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids, [itm_sarranid_lady_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,sarranid_woman_face_3,sarranid_woman_face_3],
["kingdom_of_sarranids_lady_4","Lady Shatha","Shatha",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids, [itm_sarranid_lady_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,afrizar_woman_face_young_1,afrizar_woman_face_young_1],
["kingdom_of_sarranids_lady_5","Lady Bawthan","Bawthan",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,afrizar_woman_face_young_2,afrizar_woman_face_young_2],
["kingdom_of_sarranids_lady_6","Lady Mahayl","Mahayl",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,afrizar_woman_face_young_3,afrizar_woman_face_young_3],
["kingdom_of_sarranids_lady_7","Lady Isna","Isna",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,afrizar_woman_face_young_4,afrizar_woman_face_young_4],
["kingdom_of_sarranids_lady_8","Lady Siyafan","Siyafan",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_1,khergit_woman_face_1],
["kingdom_of_sarranids_lady_9","Lady Ifar","Ifar",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_1,khergit_woman_face_1],
["kingdom_of_sarranids_lady_10","Lady Yasmin","Yasmin",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_2,khergit_woman_face_2],
["kingdom_of_sarranids_lady_11","Lady Dula","Dula",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_3,khergit_woman_face_3],
["kingdom_of_sarranids_lady_12","Lady Ruwa","Ruwa",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_4,khergit_woman_face_4],
["kingdom_of_sarranids_lady_13","Lady Luqa","Luqa",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_5,khergit_woman_face_5],
["kingdom_of_sarranids_lady_14","Lady Zandina","Zandina",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_6,khergit_woman_face_6],
["kingdom_of_sarranids_lady_15","Lady Lulya","Lulya",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b,itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_7,khergit_woman_face_7],
["kingdom_of_sarranids_lady_16","Lady Zahara","Zahara",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,khergit_woman_face_8,khergit_woman_face_8],
["kingdom_of_sarranids_lady_17","Lady Safiya","Safiya",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,sarranid_woman_face_1,sarranid_woman_face_1],
["kingdom_of_sarranids_lady_18","Lady Khalisa","Khalisa",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,sarranid_woman_face_2,sarranid_woman_face_2],
["kingdom_of_sarranids_lady_19","Lady Janab","Janab",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress_b, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,afrizar_woman_face_young_1,afrizar_woman_face_young_1],
["kingdom_of_sarranids_lady_20","Lady Sur","Sur",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_sarranids,[itm_sarranid_lady_dress, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,afrizar_woman_face_young_2,afrizar_woman_face_young_2],

["heroes_end","{!}heroes end","{!}heroes end",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_saddle_horse,itm_leather_jacket,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008318101f390c515555594],



#Seneschals
["town_1_seneschal","{!}Town 1 Seneschal","{!}Town 1 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_coarse_tunic,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000c218501ef4f5d2ccb0026],
["town_2_seneschal","{!}Town 2 Seneschal","{!}Town 2 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_padded_leather,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000c03cc01cc34a9a467fdfd],
["town_3_seneschal","{!}Town 3 Seneschal","{!}Town 3 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_coarse_tunic,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000c500e01dbb2115a55f3cd],
["town_4_seneschal","{!}Town 4 Seneschal","{!}Town 4 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000008035201e6eebaf3f3eb2b],
["town_5_seneschal","{!}Town 5 Seneschal","{!}Town 5 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_leather_jerkin,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000000249101e7898999ac54c6],
["town_6_seneschal","{!}Town 6 Seneschal","{!}Town 6 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x000000000010360b01cef8b57553d34e],
["town_7_seneschal","{!}Town 7 Seneschal","{!}Town7 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_leather_jerkin,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000000018101f9487aa831dce4],
["town_8_seneschal","{!}Town 8 Seneschal","{!}Town 8 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x000000000004715201ea236c60a2bcae],
["town_9_seneschal","{!}Town 9 Seneschal","{!}Town 9 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_coarse_tunic,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["town_10_seneschal","{!}Town 10 Seneschal","{!}Town 10 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_leather_jerkin,itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000010230c01ef41badb50465e],
["town_11_seneschal","{!}Town 11 Seneschal","{!}Town 11 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_leather_jacket,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008061301fb89acfb95332f],
["town_12_seneschal","{!}Town 12 Seneschal","{!}Town 12 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_coarse_tunic,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000c13ce01dc4723ab936c82],
["town_13_seneschal","{!}Town 13 Seneschal","{!}Town 13 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_leather_jerkin,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000008548e01d952a9b25d6d5a],
["town_14_seneschal","{!}Town 14 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_15_seneschal","{!}Town 15 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_16_seneschal","{!}Town 16 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_17_seneschal","{!}Town17 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_18_seneschal","{!}Town 18 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_19_seneschal","{!}Town 19 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_20_seneschal","{!}Town 20 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_21_seneschal","{!}Town 21 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_22_seneschal","{!}Town 22 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_black_fortress_seneschal","{!}Town 22 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],
["town_holy_city_seneschal","{!}Town 22 Seneschal","{!}Town 14 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004728b01c293c694944b05],

["castle_1_seneschal","{!}Castle 1 Seneschal","{!}Castle 1 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_coarse_tunic,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x000000000010360b01cef8b57553d34e],
["castle_2_seneschal","{!}Castle 2 Seneschal","{!}Castle 2 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_nomad_armor, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000008061301fb89acfb95332f],
["castle_3_seneschal","{!}Castle 3 Seneschal","{!}Castle 3 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008548e01d952a9b25d6d5a],
["castle_4_seneschal","{!}Castle 4 Seneschal","{!}Castle 4 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_linen_tunic, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004715201ea236c60a2bcae],
["castle_5_seneschal","{!}Castle 5 Seneschal","{!}Castle 5 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_jerkin,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x00000000000c500e01dbb2115a55f3cd],
["castle_6_seneschal","{!}Castle 6 Seneschal","{!}Castle 6 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_coarse_tunic,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000c03cc01cc34a9a467fdfd],
["castle_7_seneschal","{!}Castle 7 Seneschal","{!}Castle 7 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000c13ce01dc4723ab936c82],
["castle_8_seneschal","{!}Castle 8 Seneschal","{!}Castle 8 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x00000000000c218501ef4f5d2ccb0026],
["castle_9_seneschal","{!}Castle 9 Seneschal","{!}Castle 9 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_jacket,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008035201e6eebaf3f3eb2b],
["castle_10_seneschal","{!}Castle 10 Seneschal","{!}Castle 10 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_11_seneschal","{!}Castle 11 Seneschal","{!}Castle 11 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_12_seneschal","{!}Castle 2 Seneschal","{!}Castle 2 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_nomad_armor, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000008061301fb89acfb95332f],
["castle_13_seneschal","{!}Castle 3 Seneschal","{!}Castle 3 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008548e01d952a9b25d6d5a],
["castle_14_seneschal","{!}Castle 4 Seneschal","{!}Castle 4 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_linen_tunic, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004715201ea236c60a2bcae],
["castle_15_seneschal","{!}Castle 5 Seneschal","{!}Castle 5 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_jerkin,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x00000000000c500e01dbb2115a55f3cd],
["castle_16_seneschal","{!}Castle 6 Seneschal","{!}Castle 6 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_coarse_tunic,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000c03cc01cc34a9a467fdfd],
["castle_17_seneschal","{!}Castle 7 Seneschal","{!}Castle 7 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000c13ce01dc4723ab936c82],
["castle_18_seneschal","{!}Castle 8 Seneschal","{!}Castle 8 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x00000000000c218501ef4f5d2ccb0026],
["castle_19_seneschal","{!}Castle 9 Seneschal","{!}Castle 9 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_jacket,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008035201e6eebaf3f3eb2b],
["castle_20_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_21_seneschal","{!}Castle 11 Seneschal","{!}Castle 11 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_22_seneschal","{!}Castle 2 Seneschal","{!}Castle 2 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_nomad_armor, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000008061301fb89acfb95332f],
["castle_23_seneschal","{!}Castle 3 Seneschal","{!}Castle 3 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008548e01d952a9b25d6d5a],
["castle_24_seneschal","{!}Castle 4 Seneschal","{!}Castle 4 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_linen_tunic, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004715201ea236c60a2bcae],
["castle_25_seneschal","{!}Castle 5 Seneschal","{!}Castle 5 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_jerkin,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x00000000000c500e01dbb2115a55f3cd],
["castle_26_seneschal","{!}Castle 6 Seneschal","{!}Castle 6 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_coarse_tunic,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000c03cc01cc34a9a467fdfd],
["castle_27_seneschal","{!}Castle 7 Seneschal","{!}Castle 7 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000c13ce01dc4723ab936c82],
["castle_28_seneschal","{!}Castle 8 Seneschal","{!}Castle 8 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x00000000000c218501ef4f5d2ccb0026],
["castle_29_seneschal","{!}Castle 9 Seneschal","{!}Castle 9 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_jacket,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008035201e6eebaf3f3eb2b],
["castle_30_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_31_seneschal","{!}Castle 11 Seneschal","{!}Castle 11 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_32_seneschal","{!}Castle 2 Seneschal","{!}Castle 2 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_nomad_armor, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000008061301fb89acfb95332f],
["castle_33_seneschal","{!}Castle 3 Seneschal","{!}Castle 3 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008548e01d952a9b25d6d5a],
["castle_34_seneschal","{!}Castle 4 Seneschal","{!}Castle 4 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_linen_tunic, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x000000000004715201ea236c60a2bcae],
["castle_35_seneschal","{!}Castle 5 Seneschal","{!}Castle 5 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_jerkin,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x00000000000c500e01dbb2115a55f3cd],
["castle_36_seneschal","{!}Castle 6 Seneschal","{!}Castle 6 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_coarse_tunic,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000000000c03cc01cc34a9a467fdfd],
["castle_37_seneschal","{!}Castle 7 Seneschal","{!}Castle 7 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_blue_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000c13ce01dc4723ab936c82],
["castle_38_seneschal","{!}Castle 8 Seneschal","{!}Castle 8 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,0x00000000000c218501ef4f5d2ccb0026],
["castle_39_seneschal","{!}Castle 9 Seneschal","{!}Castle 9 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_jacket,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000000008035201e6eebaf3f3eb2b],
["castle_40_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_41_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_42_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_43_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_44_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_45_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_46_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_47_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["castle_48_seneschal","{!}Castle 20 Seneschal","{!}Castle 20 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["black_castle_1_seneschal","{!}Black Castle 1 Seneschal","{!}Black Castle 1 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["black_castle_2_seneschal","{!}Black Castle 2 Seneschal","{!}Black Castle 2 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],
["black_castle_3_seneschal","{!}Black Castle 3 Seneschal","{!}Black Castle 3 Seneschal",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_padded_leather, itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,0x00000000000440c601e1cd45cfb38550,0x00000000000440c601e1cd45cfb38550],

#Arena Masters
["town_1_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_1_arena|entry(52),reserved,fac_commoners,[itm_coarse_tunic, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_2_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_2_arena|entry(52),reserved,fac_commoners,[itm_linen_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_3_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_3_arena|entry(52),reserved,fac_commoners,[itm_nomad_armor,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_4_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_4_arena|entry(52),reserved,fac_commoners,[itm_coarse_tunic, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_5_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_5_arena|entry(52),reserved,fac_commoners,[itm_linen_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_6_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_6_arena|entry(52),reserved,fac_commoners,[itm_leather_jerkin, itm_leather_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_7_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_7_arena|entry(52),reserved,fac_commoners,[itm_padded_leather, itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_8_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_8_arena|entry(52),reserved,fac_commoners,[itm_linen_tunic,itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_9_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_9_arena|entry(52),reserved,fac_commoners,[itm_padded_leather, itm_leather_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_10_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_10_arena|entry(52),reserved, fac_commoners,[itm_nomad_armor,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_11_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_11_arena|entry(52),reserved, fac_commoners,[itm_coarse_tunic, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_12_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_12_arena|entry(52),reserved, fac_commoners,[itm_leather_jerkin, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_13_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_13_arena|entry(52),reserved, fac_commoners,[itm_coarse_tunic, itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_14_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_14_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_15_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_15_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_16_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_16_arena|entry(52),reserved, fac_commoners,[itm_fur_coat, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_17_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_17_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_18_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_18_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_19_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_19_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_20_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_20_arena|entry(52),reserved, fac_commoners,[itm_fur_coat, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_21_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_21_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_22_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_22_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_black_fortress_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_22_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],
["town_holy_city_arena_master","Tournament Master","{!}Tournament Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,scn_town_22_arena|entry(52),reserved, fac_commoners,[itm_padded_leather, itm_hide_boots], def_attrib|level(2),wp(20),knows_common,man_face_middle_1,man_face_older_2],





# Armor Merchants
#arena_masters_end = zendar_armorer

["town_1_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_linen_tunic, itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_2_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_straw_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_3_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_arena_tunic_red,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_4_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_leather_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_5_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_linen_tunic,itm_nomad_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_6_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_fur_coat,itm_nomad_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_7_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_leather_jerkin,itm_blue_hose],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_8_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_padded_leather,itm_leather_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_9_armorer","Armorer", "{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_nomad_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_10_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_leather_jerkin,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_11_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_fur_coat,itm_leather_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_12_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth, itm_nomad_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_13_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_leather_jacket,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_14_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06, itm_headcloth],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_15_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_leather_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_16_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_fur_coat, itm_nomad_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_17_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_fur_coat,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_18_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06, itm_headcloth],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_19_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_leather_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_20_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_fur_coat, itm_nomad_boots ],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_21_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_fur_coat,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_22_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_sarranid_common_dress, itm_sarranid_head_cloth],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_black_fortress_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_sarranid_common_dress, itm_sarranid_head_cloth],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_holy_city_armorer","Armorer","{!}Armorer", tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_sarranid_common_dress, itm_sarranid_head_cloth],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],

# Weapon merchants

["town_1_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_linen_tunic, itm_hide_boots,itm_straw_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_2_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_shirt,itm_nomad_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_3_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_fur_coat,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_4_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_shirt,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_5_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_leather_jerkin,itm_wrapping_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_6_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_linen_tunic, itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_7_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_shirt,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_8_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_wrapping_boots,itm_straw_hat],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_9_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_leather_jerkin,itm_leather_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_10_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_linen_tunic,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_11_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_leather_jacket, itm_woolen_hose],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_12_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_shirt, itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_13_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_arena_tunic_red,itm_wrapping_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_14_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_arena_tunic_blue,itm_wrapping_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_15_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_leather_jacket, itm_woolen_hose],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_16_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_shirt, itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_17_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_arena_tunic_green,itm_wrapping_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_18_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_linen_tunic,itm_wrapping_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_19_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_leather_jacket, itm_sarranid_boots_a],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_20_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_shirt, itm_sarranid_boots_a],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_21_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_arena_tunic_green,itm_sarranid_boots_a],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_22_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_linen_tunic,itm_sarranid_boots_a],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_black_fortress_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_linen_tunic,itm_sarranid_boots_a],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],
["town_holy_city_weaponsmith","Weaponsmith","{!}Weaponsmith",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face,0,0,fac_commoners,[itm_linen_tunic,itm_sarranid_boots_a],def_attrib|level(5),wp(20),knows_inventory_management_10,mercenary_face_1,mercenary_face_2],

#Tavern keepers

["town_1_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_1_tavern|entry(9),0,fac_commoners,[itm_leather_apron,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_common,0x00000007a91051ca45ec85ba924eb8ad00000000001ecf5b0000000000000000,0x00000007a91051ca45ec85ba924eb8ad00000000001ecf5b0000000000000000],
["town_2_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_2_tavern|entry(9),0,fac_commoners,[itm_leather_apron,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000005df0432d35715da658f49577200000000000edb180000000000000000,0x00000005df0432d35715da658f49577200000000000edb180000000000000000],
["town_3_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_3_tavern|entry(9),0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x0000000166017001499b0e389b6d5adb00000000001db4eb0000000000000000,0x0000000166017001499b0e389b6d5adb00000000001db4eb0000000000000000],
["town_4_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_4_tavern|entry(9),0,fac_commoners,[itm_leather_apron,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x000000023400c001468b8da6e375b89b00000000001e36db0000000000000000,0x000000023400c001468b8da6e375b89b00000000001e36db0000000000000000],
["town_5_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_5_tavern|entry(9),0,fac_commoners,[itm_leather_apron,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x00000004f010b3004d5c93232d6ebaa200000000001e429c0000000000000000,0x00000004f010b3004d5c93232d6ebaa200000000001e429c0000000000000000],
["town_6_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_6_tavern|entry(9),0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x000000034401d005499b0e389b6d5adb00000000001db4eb0000000000000000,0x000000034401d005499b0e389b6d5adb00000000001db4eb0000000000000000],
["town_7_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_7_tavern|entry(9),0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_leather_boots, itm_headcloth],def_attrib|level(2),wp(20),knows_common,0x000000015a02d004395b31b8a36d4ad200000000001dc4eb0000000000000000,0x000000015a02d004395b31b8a36d4ad200000000001dc4eb0000000000000000],
["town_8_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_8_tavern|entry(9),0,fac_commoners,[itm_leather_apron, itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x0000000760086581546e6f155989a8d300000000000e21a60000000000000000,0x0000000760086581546e6f155989a8d300000000000e21a60000000000000000],
["town_9_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_9_tavern|entry(9),0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x0000000090056002491c96bd32ae4cb6000000000009b8d50000000000000000,0x0000000090056002491c96bd32ae4cb6000000000009b8d50000000000000000],
["town_10_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_10_tavern|entry(9),0, fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x00000004f20e9006252397266d45380200000000000a5c5b0000000000000000,0x00000004f20e9006252397266d45380200000000000a5c5b0000000000000000],
["town_11_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_11_tavern|entry(9),0, fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x000000025109d00348ac3dcd16d1b4a400000000001db6d20000000000000000,0x000000025109d00348ac3dcd16d1b4a400000000001db6d20000000000000000],
["town_12_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_12_tavern|entry(9),0, fac_commoners,[itm_leather_apron,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x0000000500080544395c8de2eb2a33090000000000192ad50000000000000000,0x0000000500080544395c8de2eb2a33090000000000192ad50000000000000000],
["town_13_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_13_tavern|entry(9),0, fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_hide_boots,itm_headcloth],def_attrib|level(2),wp(20),knows_common,0x000000017c09000746d568ba65a946ec00000000000632690000000000000000,0x000000017c09000746d568ba65a946ec00000000000632690000000000000000],
["town_14_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_14_tavern|entry(9),0, fac_commoners,[itm_shirt,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000003020c34cb3922653a9c71cad200000000000f375c0000000000000000,0x00000003020c34cb3922653a9c71cad200000000000f375c0000000000000000],
["town_15_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_15_tavern|entry(9),0, fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,0x00000002b20ce00739149a0b9b8f289500000000001ca4620000000000000000,0x00000002b20ce00739149a0b9b8f289500000000001ca4620000000000000000],
["town_16_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_16_tavern|entry(9),0, fac_commoners,[itm_leather_apron,itm_hide_boots],def_attrib|level(2),wp(20),knows_common,0x00000002f60c638d3cd49a450cbb595b00000000000ad69a0000000000000000,0x00000002f60c638d3cd49a450cbb595b00000000000ad69a0000000000000000],
["town_17_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_17_tavern|entry(9),0, fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_hide_boots,itm_headcloth],def_attrib|level(2),wp(20),knows_common,0x000000030401a002368a766cf4b12552000000000005c8550000000000000000,0x000000030401a002368a766cf4b12552000000000005c8550000000000000000],
["town_18_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_18_tavern|entry(9),0, fac_commoners,[itm_shirt,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,0x00000001360890432d2a54c55badc71500000000000994ec0000000000000000,0x00000001360890432d2a54c55badc71500000000000994ec0000000000000000],
["town_19_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_19_tavern|entry(9),0, fac_commoners,[itm_sarranid_dress_a,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common,0x00000000b300d00644e36ec4644eeacd000000000005d6e50000000000000000,0x00000000b300d00644e36ec4644eeacd000000000005d6e50000000000000000],
["town_20_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_20_tavern|entry(9),0, fac_commoners,[itm_sarranid_cloth_robe,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common,0x000000002100b002489391a6e371badb00000000001e26db0000000000000000,0x000000002100b002489391a6e371badb00000000001e26db0000000000000000],
["town_21_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero|tf_female,scn_town_21_tavern|entry(9),0, fac_commoners,[itm_sarranid_common_dress,itm_sarranid_boots_a,itm_headcloth],def_attrib|level(2),wp(20),knows_common,0x000000050f090001526b4e1ae54d631d000000000011da9b0000000000000000,0x000000050f090001526b4e1ae54d631d000000000011da9b0000000000000000],
["town_22_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_22_tavern|entry(9),0, fac_commoners,[itm_sarranid_cloth_robe_b,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common,0x00000000cf0cc3871492f666a3b9a7b200000000000533110000000000000000,0x00000000cf0cc3871492f666a3b9a7b200000000000533110000000000000000],
["town_black_fortress_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_22_tavern|entry(9),0, fac_commoners,[itm_sarranid_cloth_robe_b,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common,0x00000000cf0cc3871492f666a3b9a7b200000000000533110000000000000000,0x00000000cf0cc3871492f666a3b9a7b200000000000533110000000000000000],
["town_holy_city_tavernkeeper","Tavern_Keeper","{!}Tavern_Keeper",tf_mounted|tf_hero, scn_town_22_tavern|entry(9),0, fac_commoners,[itm_sarranid_cloth_robe_b,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common,0x00000000cf0cc3871492f666a3b9a7b200000000000533110000000000000000,0x00000000cf0cc3871492f666a3b9a7b200000000000533110000000000000000],

#Goods Merchants

["town_1_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_1_store|entry(9),0,fac_commoners,[itm_coarse_tunic, itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_2_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_2_store|entry(9),0,fac_commoners,[itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_3_merchant","Merchant","{!}Merchant",tf_mounted|tf_female|tf_hero|tf_randomize_face,scn_town_3_store|entry(9),0,fac_commoners,[itm_dress, itm_leather_boots, itm_straw_hat ],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_4_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_4_store|entry(9),0,fac_commoners,[itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_5_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_5_store|entry(9),0,fac_commoners,[itm_nomad_armor,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_6_merchant","Merchant","{!}Merchant",tf_mounted|tf_female|tf_hero|tf_randomize_face,scn_town_6_store|entry(9),0,fac_commoners,[itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06, itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_7_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_7_store|entry(9),0,fac_commoners,[itm_leather_jerkin,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_8_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_8_store|entry(9),0,fac_commoners,[itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_9_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_9_store|entry(9),0,fac_commoners,[itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_10_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_10_store|entry(9),0,fac_commoners, [itm_leather_jerkin,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_11_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_11_store|entry(9),0,fac_commoners, [itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_12_merchant","Merchant","{!}Merchant",tf_mounted|tf_female|tf_hero|tf_randomize_face,scn_town_12_store|entry(9),0,fac_commoners, [itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06, itm_leather_boots, itm_female_hood ],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_13_merchant","Merchant","{!}Merchant",tf_mounted|tf_female|tf_hero|tf_randomize_face,scn_town_13_store|entry(9),0,fac_commoners, [itm_dress, itm_leather_boots, itm_straw_hat ],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_14_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_14_store|entry(9),0,fac_commoners, [itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_15_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_15_store|entry(9),0,fac_commoners, [itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_16_merchant","Merchant","{!}Merchant",tf_mounted|tf_female|tf_hero|tf_randomize_face,scn_town_16_store|entry(9),0,fac_commoners, [itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06, itm_leather_boots, itm_female_hood ],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_17_merchant","Merchant","{!}Merchant",tf_mounted|tf_female|tf_hero|tf_randomize_face,scn_town_17_store|entry(9),0,fac_commoners, [itm_dress, itm_leather_boots, itm_straw_hat ],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_18_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_18_store|entry(9),0,fac_commoners, [itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_19_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_19_store|entry(9),0,fac_commoners, [itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_20_merchant","Merchant","{!}Merchant",tf_mounted|tf_female|tf_hero|tf_randomize_face,scn_town_20_store|entry(9),0,fac_commoners, [itm_sarranid_common_dress_b, itm_sarranid_boots_a,itm_sarranid_felt_head_cloth_b],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_21_merchant","Merchant","{!}Merchant",tf_mounted|tf_female|tf_hero|tf_randomize_face,scn_town_21_store|entry(9),0,fac_commoners, [itm_sarranid_dress_a, itm_sarranid_boots_a, itm_sarranid_felt_head_cloth],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_22_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_22_store|entry(9),0,fac_commoners, [itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_black_fortress_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_22_store|entry(9),0,fac_commoners, [itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_holy_city_merchant","Merchant","{!}Merchant",tf_hero|tf_randomize_face,scn_town_22_store|entry(9),0,fac_commoners, [itm_leather_apron,itm_leather_boots ],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],

["salt_mine_merchant","Barezan","Barezan", tf_hero,scn_salt_mine|entry(1),0,fac_commoners,[itm_leather_apron,itm_leather_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,0x00000000000c528601ea69b6e46dbdb6],

# Horse Merchants

["town_1_horse_merchant","Horse Merchant","{!}Town 1 Horse Merchant",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_blue_dress, itm_blue_hose, itm_female_hood],def_attrib|level(2),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_2_horse_merchant","Horse Merchant","{!}Town 2 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_linen_tunic,itm_nomad_boots,],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_3_horse_merchant","Horse Merchant","{!}Town 3 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_nomad_armor,itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_4_horse_merchant","Horse Merchant","{!}Town 4 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_leather_jerkin,itm_nomad_boots], def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_5_horse_merchant","Horse Merchant","{!}Town 5 Horse Merchant",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_dress, itm_woolen_hose, itm_woolen_hood],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_6_horse_merchant","Horse Merchant","{!}Town 6 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic, itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_7_horse_merchant","Horse Merchant","{!}Town 7 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic, itm_leather_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_8_horse_merchant","Horse Merchant","{!}Town 8 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic, itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_9_horse_merchant","Horse Merchant","{!}Town 9 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_leather_jerkin,itm_woolen_hose], def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_10_horse_merchant","Horse Merchant","{!}Town 10 Horse Merchant",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_blue_dress,itm_blue_hose, itm_straw_hat],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_11_horse_merchant","Horse Merchant","{!}Town 11 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_nomad_armor, itm_leather_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_12_horse_merchant","Horse Merchant","{!}Town 12 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_leather_jacket, itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_13_horse_merchant","Horse Merchant","{!}Town 13 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots], def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_14_horse_merchant","Horse Merchant","{!}Town 14 Horse Merchant",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_peasant_dress,itm_blue_hose, itm_headcloth],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_15_horse_merchant","Horse Merchant","{!}Town 15 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_nomad_armor, itm_leather_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_16_horse_merchant","Horse Merchant","{!}Town 16 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_leather_jacket, itm_hide_boots],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_17_horse_merchant","Horse Merchant","{!}Town 17 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots], def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_18_horse_merchant","Horse Merchant","{!}Town 18 Horse Merchant",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_peasant_dress,itm_blue_hose, itm_headcloth],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_19_horse_merchant","Horse Merchant","{!}Town 15 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_nomad_armor, itm_sarranid_boots_a],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_20_horse_merchant","Horse Merchant","{!}Town 16 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe, itm_sarranid_boots_a],def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_21_horse_merchant","Horse Merchant","{!}Town 17 Horse Merchant",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe_b,itm_sarranid_boots_a], def_attrib|level(5),wp(20),knows_inventory_management_10,man_face_young_1,man_face_older_2],
["town_22_horse_merchant","Horse Merchant","{!}Town 18 Horse Merchant",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_sarranid_common_dress_b,itm_blue_hose, itm_sarranid_felt_head_cloth_b],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_black_fortress_horse_merchant","Horse Merchant","{!}Town 18 Horse Merchant",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_sarranid_common_dress_b,itm_blue_hose, itm_sarranid_felt_head_cloth_b],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],
["town_holy_city_horse_merchant","Horse Merchant","{!}Town 18 Horse Merchant",tf_mounted|tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots|tf_randomize_face|tf_female,0,0,fac_commoners,[itm_sarranid_common_dress_b,itm_blue_hose, itm_sarranid_felt_head_cloth_b],def_attrib|level(5),wp(20),knows_inventory_management_10,woman_face_1,woman_face_2],


#Town Mayors#itm_nobleman_outfit itm_gambeson itm_blue_gambeson itm_red_gambeson itm_nobleman_outfit itm_rich_outfit
["town_1_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_nobleman_outfit,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,mercenary_face_2],
["town_2_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_3_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_leather_boots],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_4_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_fur_coat,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_blue_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_5_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_nobleman_outfit,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_6_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_7_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_rich_outfit,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_8_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_9_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_nobleman_outfit,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_leather_boots],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_10_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_leather_jerkin,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_blue_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_11_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_leather_jacket,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_12_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_leather_boots],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_13_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_nobleman_outfit,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_14_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_blue_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_15_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_leather_jacket,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_16_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_fur_coat,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_leather_boots],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_17_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_nobleman_outfit,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_18_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_blue_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_blue_hose],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_19_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_sarranid_cloth_robe,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_20_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_sarranid_cloth_robe,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_21_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_sarranid_cloth_robe,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_22_mayor","Guild_Master","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_sarranid_cloth_robe,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_black_fortress_mayor","Servant of the Dark","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_sarranid_cloth_robe,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],
["town_holy_city_mayor","Servant of the Light","{!}Guild_Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[ itm_sarranid_cloth_robe,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common, man_face_middle_1,mercenary_face_2],


#Village stores
["village_1_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_2_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_3_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_4_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_5_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_6_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_7_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_8_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_wrapping_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_9_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_10_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_11_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_12_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_13_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_14_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_15_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_16_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_leather_warrior_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_17_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_18_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_leather_warrior_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_19_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_20_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_leather_warrior_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_21_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_22_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_23_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_24_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_25_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_26_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_27_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_wrapping_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_28_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_29_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_30_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_31_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_32_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_33_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_34_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_35_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_36_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_37_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_38_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_39_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_40_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_41_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_42_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_43_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_44_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_45_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_46_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_47_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_48_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_49_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_50_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_51_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_52_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_53_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_54_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_55_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_56_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_57_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_wrapping_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_58_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_59_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_60_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_61_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_62_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_63_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_64_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_65_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_66_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_67_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_wrapping_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_68_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_69_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_70_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_71_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_72_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_73_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_74_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_75_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_76_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_77_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_wrapping_boots,itm_felt_hat],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_78_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_79_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_80_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_81_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_82_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_83_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_84_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_85_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_86_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_87_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_88_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_fur_coat,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_89_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_90_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_91_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_92_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_93_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_94_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_95_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe_b,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_96_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_97_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_98_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_99_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_100_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_101_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe_b,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_102_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_103_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe,itm_wrapping_boots,itm_leather_cap],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_104_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_nomad_boots,itm_fur_hat],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_105_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe_b,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_106_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_coarse_tunic,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_107_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_108_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe_b,itm_hide_boots],def_attrib|level(2),wp(20),knows_inventory_management_10,man_face_old_1,man_face_older_2],
["village_109_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_sarranid_cloth_robe_b,itm_nomad_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
["village_110_elder","Village_Elder","{!}village_1_elder",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[itm_robe,itm_wrapping_boots],def_attrib|level(2),wp(20),knows_inventory_management_10, man_face_old_1,man_face_older_2],
# Place extra merchants before this point
["merchants_end","merchants_end","merchants_end",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[],def_attrib|level(2),wp(20),knows_inventory_management_10,0],

#Used for player enterprises
["town_1_master_craftsman","Shop Master","Shop Master",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_neutral,[itm_leather_apron,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,man_face_old_1,man_face_older_2],




# Chests
["zendar_chest","{!}Zendar Chest","{!}Zendar Chest",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,
 [],def_attrib|level(18),wp(60),knows_common,0],
["tutorial_chest_1","{!}Melee Weapons Chest","{!}Melee Weapons Chest",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[itm_tutorial_sword,itm_tutorial_axe,itm_tutorial_spear,itm_tutorial_club,itm_tutorial_battle_axe],def_attrib|level(18),wp(60),knows_common,0],
["tutorial_chest_2","{!}Ranged Weapons Chest","{!}Ranged Weapons Chest",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[itm_tutorial_short_bow,itm_tutorial_arrows,itm_tutorial_crossbow,itm_tutorial_bolts,itm_tutorial_throwing_daggers],def_attrib|level(18),wp(60),knows_common,0],
["bonus_chest_1","{!}Bonus Chest","{!}Bonus Chest",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[itm_wei_xiadi_samurai_armor01,itm_strange_short_sword],def_attrib|level(18),wp(60),knows_common,0],
["bonus_chest_2","{!}Bonus Chest","{!}Bonus Chest",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[itm_ssh_suneate,itm_katana],def_attrib|level(18),wp(60),knows_common,0],
["bonus_chest_3","{!}Bonus Chest","{!}Bonus Chest",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[itm_ssh_kabuto,itm_nodachi],def_attrib|level(18),wp(60),knows_common,0],

["household_possessions","{!}household_possessions","{!}household_possessions",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[],def_attrib|level(18),wp(60),knows_inventory_management_10,0], 

# These are used as arrays in the scripts.
["temp_array_a","{!}temp_array_a","{!}temp_array_a",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[],def_attrib|level(18),wp(60),knows_common,0],
["temp_array_b","{!}temp_array_b","{!}temp_array_b",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[],def_attrib|level(18),wp(60),knows_common,0],
["temp_array_c","{!}temp_array_c","{!}temp_array_c",tf_mounted|tf_hero|tf_inactive,0,reserved, fac_neutral,[],def_attrib|level(18),wp(60),knows_common,0],

["stack_selection_amounts","{!}stack_selection_amounts","{!}stack_selection_amounts",tf_mounted|tf_hero|tf_inactive,0,reserved,fac_neutral,[],def_attrib,0,knows_common,0],
["stack_selection_ids","{!}stack_selection_ids","{!}stack_selection_ids",tf_mounted|tf_hero|tf_inactive,0,reserved,fac_neutral,[],def_attrib,0,knows_common,0],

["notification_menu_types","{!}notification_menu_types","{!}notification_menu_types",tf_mounted|tf_hero|tf_inactive,0,reserved,fac_neutral,[],def_attrib,0,knows_common,0],
["notification_menu_var1","{!}notification_menu_var1","{!}notification_menu_var1",tf_mounted|tf_hero|tf_inactive,0,reserved,fac_neutral,[],def_attrib,0,knows_common,0],
["notification_menu_var2","{!}notification_menu_var2","{!}notification_menu_var2",tf_mounted|tf_hero|tf_inactive,0,reserved,fac_neutral,[],def_attrib,0,knows_common,0],

["banner_background_color_array","{!}banner_background_color_array","{!}banner_background_color_array",tf_mounted|tf_hero|tf_inactive,0,reserved,fac_neutral,[],def_attrib,0,knows_common,0],

["multiplayer_data","{!}multiplayer_data","{!}multiplayer_data",tf_mounted|tf_hero|tf_inactive,0,reserved,fac_neutral,[],def_attrib,0,knows_common,0],

##["black_khergit_guard","Black Khergit Guard","Black Khergit Guard",tf_mounted|tf_guarantee_ranged|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_helmet|tf_guarantee_armor|tf_guarantee_horse,0,0,fac_black_khergits,
## [itm_arrows,itm_nomad_sabre,itm_scimitar,itm_winged_mace,itm_lance,itm_khergit_bow,itm_khergit_guard_boots,itm_khergit_guard_armor,itm_nomad_shield,itm_steppe_horse,itm_warhorse],
## def_attrib|level(28),wp(140),knows_riding_6|knows_toughness_4|knows_horse_archery_6,khergit_face1,khergit_face2],


# Add Extra Quest NPCs below this point

["local_merchant","Local Merchant","Local Merchants",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["tax_rebel","Peasant Rebel","Peasant Rebels",tf_mounted|tf_guarantee_armor,0,reserved,fac_commoners,
 [itm_cleaver,itm_knife,itm_pitch_fork,itm_sickle,itm_club,itm_stones,itm_leather_cap,itm_felt_hat,itm_felt_hat,itm_linen_tunic,itm_coarse_tunic,itm_nomad_boots,itm_wrapping_boots],
 def_attrib|level(4),wp(60),knows_common,vaegir_face1,vaegir_face2],
["trainee_peasant","Peasant","Peasants",tf_mounted|tf_guarantee_armor,0,reserved,fac_commoners,
 [itm_cleaver,itm_knife,itm_pitch_fork,itm_sickle,itm_club,itm_stones,itm_leather_cap,itm_felt_hat,itm_felt_hat,itm_linen_tunic,itm_coarse_tunic,itm_nomad_boots,itm_wrapping_boots],
 def_attrib|level(4),wp(60),knows_common,vaegir_face1,vaegir_face2],
["fugitive","Nervous Man","Nervous Men",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_wrapping_boots,itm_fur_hat,itm_leather_cap,itm_sword_medieval_b,itm_throwing_daggers],
 def_attrib|str_24|agi_25|level(26),wp(180),knows_common|knows_toughness_9,man_face_middle_1,man_face_old_2],
 
["belligerent_drunk","Belligerent Drunk","Belligerent Drunks",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_short_tunic,itm_linen_tunic,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_coarse_tunic,itm_tabard,itm_leather_vest,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_wrapping_boots,itm_fur_hat,itm_leather_cap,itm_sword_viking_1],
 def_attrib|str_20|agi_8|level(15),wp(120),knows_common|knows_toughness_9, bandit_face1,bandit_face2],

["hired_assassin","Assassin","Assassins",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,#they look like belligerent drunks #Now they don't, heh.
 [itm_youhou_assassin_sword,itm_lui_khergitlordexecutioner,itm_narf_splinted_greaves_spurs,itm_youhou_assassin_armor,itm_youhou_assassin_armor_red,itm_youhou_assassin_hood,itm_youhou_assassin_hood_red,itm_narf_wisby_gauntlets_black,itm_narf_wisby_gauntlets_red],
 def_attrib|str_20|agi_16|level(20),wp(180),knows_common|knows_toughness_3, bandit_face1,bandit_face2],

["fight_promoter","Rough-Looking Character","Rough-Looking Character",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,
 [itm_short_tunic,itm_linen_tunic,itm_coarse_tunic,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_tabard,itm_leather_vest,itm_woolen_hose,itm_nomad_boots,itm_blue_hose,itm_wrapping_boots,itm_fur_hat,itm_leather_cap,itm_sword_viking_1],
 def_attrib|str_20|agi_16|level(20),wp(180),knows_common|knows_toughness_3, bandit_face1,bandit_face2], 
 
["spy","Ordinary Townsman","Ordinary Townsmen",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse,0,0,fac_neutral,
 [itm_sword_viking_1,itm_leather_jerkin,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_leather_boots,itm_courser,itm_leather_gloves],
 def_attrib|agi_11|level(20),wp(130),knows_common,man_face_middle_1,man_face_older_2],
 
["spy_partner","Unremarkable Townsman","Unremarkable Townsmen",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse,0,0,fac_neutral,
 [itm_sword_medieval_b,itm_leather_jerkin,itm_leather_boots,itm_courser,itm_leather_gloves],
 def_attrib|agi_11|level(10),wp(130),knows_common,vaegir_face1,vaegir_face2],

 ["nurse_for_lady","Nurse","Nurse",tf_mounted|tf_female|tf_guarantee_armor,0,reserved,fac_commoners,
 [itm_robe,itm_black_hood,itm_wrapping_boots],
 def_attrib|level(4),wp(60),knows_common,woman_face_1,woman_face_2],
 ["temporary_minister","Minister","Minister",tf_mounted|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_commoners,
 [itm_rich_outfit,itm_wrapping_boots],
 def_attrib|level(4),wp(60),knows_common,man_face_middle_1,man_face_older_2],
 
 
##["conspirator","Conspirator","Conspirators",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse,0,0,fac_neutral,
## [itm_sword,itm_leather_jerkin,itm_leather_boots,itm_hunter,itm_leather_gloves],
## def_attrib|agi_11|level(10),wp(130),knows_common,vaegir_face1,vaegir_face2],
##["conspirator_leader","Conspirator","Conspirators",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_gloves|tf_guarantee_horse,0,0,fac_neutral,
## [itm_sword,itm_leather_jerkin,itm_leather_boots,itm_hunter,itm_leather_gloves],
## def_attrib|agi_11|level(10),wp(130),knows_common,vaegir_face1,vaegir_face2],
##["peasant_rebel","Peasant Rebel","Peasant Rebels",tf_mounted|tf_guarantee_armor,0,reserved,fac_peasant_rebels,
## [itm_cleaver,itm_knife,itm_pitch_fork,itm_sickle,itm_club,itm_stones,itm_leather_cap,itm_felt_hat,itm_felt_hat,itm_linen_tunic,itm_coarse_tunic,itm_nomad_boots,itm_wrapping_boots],
## def_attrib|level(4),wp(60),knows_common,vaegir_face1,vaegir_face2],
##["noble_refugee","Noble Refugee","Noble Refugees",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_noble_refugees,
## [itm_sword,itm_leather_jacket,itm_hide_boots,itm_saddle_horse,itm_leather_jacket,itm_leather_cap],
## def_attrib|level(9),wp(100),knows_common,swadian_face1,swadian_face2],
##["noble_refugee_woman","Noble Refugee Woman","Noble Refugee Women",tf_mounted|tf_female|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_noble_refugees,
## [itm_knife,itm_dagger,itm_hunting_crossbow,itm_dress,itm_robe,itm_woolen_dress01,itm_woolen_dress02,itm_woolen_dress03,itm_woolen_dress04,itm_woolen_dress05,itm_woolen_dress06,itm_headcloth,itm_woolen_hood,itm_wrapping_boots],
## def_attrib|level(3),wp(45),knows_common,refugee_face1,refugee_face2],


["quick_battle_6_player","{!}quick_battle_6_player","{!}quick_battle_6_player",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,reserved, fac_player_faction,[itm_padded_cloth,itm_nomad_boots,itm_splinted_leather_greaves,itm_skullcap,itm_sword_medieval_b, itm_crossbow,itm_bolts,itm_plate_covered_round_shield], knight_attrib_1,wp(130),knight_skills_1,0x000000000008010b01f041a9249f65fd],

#Multiplayer ai troops
["swadian_crossbowman_multiplayer_ai","Swadian Crossbowman","Swadian Crossbowmen",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_swadians,
 [itm_bolts,itm_crossbow,itm_sword_medieval_a,itm_heater_shield_old,
itm_leather_jerkin,itm_leather_armor,itm_ankle_boots,itm_footman_helmet],
 def_attrib|level(19),wp_melee(90)|wp_crossbow(100),knows_common|knows_toughness_4|knows_reflexes_6,swadian_face_young_1,swadian_face_old_2],
["swadian_infantry_multiplayer_ai","Swadian Infantry","Swadian Infantry",tf_mounted|tf_guarantee_all_wo_ranged,0,0,fac_kingdom_of_swadians,
 [itm_pike,itm_bastard_sword_a,itm_heater_shield_generic,
itm_studded_leather_coat,itm_ankle_boots,itm_flat_topped_helmet],
 def_attrib|level(19),wp_melee(105),knows_common|knows_toughness_5|knows_reflexes_4,swadian_face_middle_1,swadian_face_old_2],
["swadian_man_at_arms_multiplayer_ai","Swadian Man at Arms","Swadian Men at Arms",tf_mounted|tf_guarantee_all_wo_ranged,0,0,fac_kingdom_of_swadians,
 [itm_lance,itm_bastard_sword_a,itm_xenoargh_shield_heater01,
itm_mail_with_surcoat,itm_hide_boots,itm_norman_helmet,itm_hunter],
 def_attrib|level(19),wp_melee(100),knows_common|knows_riding_4|knows_toughness_4|knows_reflexes_1,swadian_face_young_1,swadian_face_old_2],
["vaegir_archer_multiplayer_ai","Vaegir Archer","Vaegir Archers",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_vaegirs,
 [itm_arrows,itm_scimitar,itm_nomad_bow,
itm_leather_vest,itm_nomad_boots,itm_spiked_helmet,itm_nomad_cap],
 def_attrib|str_12|level(19),wp_melee(70)|wp_archery(110),knows_toughness_4|knows_reflexes_6,vaegir_face_young_1,vaegir_face_older_2],
["vaegir_spearman_multiplayer_ai","Vaegir Spearman","Vaegir Spearmen",tf_mounted|tf_guarantee_all_wo_ranged,0,0,fac_kingdom_of_vaegirs,
 [itm_padded_leather,itm_nomad_boots,itm_spiked_helmet,itm_nomad_cap,itm_spear,itm_xenoargh_shield_kite01_05,itm_mace_1,itm_javelin],
 def_attrib|str_12|level(19),wp_melee(90),knows_toughness_4|knows_reflexes_6,vaegir_face_young_1,vaegir_face_older_2],
["vaegir_horseman_multiplayer_ai","Vaegir Horseman","Vaegir Horsemen",tf_mounted|tf_guarantee_all_wo_ranged,0,0,fac_kingdom_of_vaegirs,
 [itm_battle_axe,itm_scimitar,itm_lance,itm_xenoargh_shield_kite01_03,
 itm_studded_leather_coat,itm_lamellar_vest,itm_nomad_boots,itm_spiked_helmet,itm_saddle_horse],
 def_attrib|level(19),wp(100),knows_riding_4|knows_toughness_4,vaegir_face_young_1,vaegir_face_older_2],
["khergit_dismounted_lancer_multiplayer_ai","Khergit Dismounted Lancer","Khergit Dismounted Lancer",tf_mounted|tf_guarantee_all_wo_ranged,0,0,fac_kingdom_of_khergits,
 [itm_sword_khergit_4,itm_winged_mace,itm_spear,itm_lamellar_vest_khergit,itm_lamellar_armor,itm_hide_boots,itm_leather_gloves,itm_xenoargh_shield_small01,itm_xenoargh_shield_small02],
 def_attrib|level(19),wp(100),knows_toughness_4,khergit_face_middle_1,khergit_face_older_2],
["khergit_veteran_horse_archer_multiplayer_ai","Khergit Horse Archer","Khergit Horse Archers",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_khergits,
 [itm_sword_khergit_3,itm_khergit_bow,itm_arrows,itm_xenoargh_shield_small01,itm_tribal_warrior_outfit,itm_hide_boots,itm_steppe_horse],
 def_attrib|level(21),wp(90)|wp_archery(130),knows_riding_6|knows_horse_archery_5,khergit_face_middle_1,khergit_face_older_2],
["khergit_lancer_multiplayer_ai","Khergit Lancer","Khergit Lancers",tf_mounted|tf_guarantee_all_wo_ranged,0,0,fac_kingdom_of_khergits,
 [itm_sword_khergit_4,itm_winged_mace,itm_spear,itm_lance,itm_lance,
itm_lamellar_vest_khergit,itm_lamellar_armor,itm_hide_boots,itm_leather_gloves,itm_xenoargh_shield_small01,itm_xenoargh_shield_small02,itm_courser],
 def_attrib|level(23),wp(120),knows_riding_6|knows_toughness_2|knows_horse_archery_1,khergit_face_middle_1,khergit_face_older_2],
["nord_veteran_multiplayer_ai","Nord Footman","Nord Footmen",tf_mounted|tf_guarantee_all_wo_ranged,0,0,fac_kingdom_of_nords,
 [itm_sword_viking_2,itm_one_handed_battle_axe_b,itm_two_handed_axe,itm_xenoargh_round_shield02,itm_throwing_axes,
itm_nordic_helmet,itm_nasal_helmet,itm_byrnie,itm_mail_hauberk,itm_splinted_leather_greaves,itm_leather_boots,itm_leather_gloves],
 def_attrib|level(19),wp(130),knows_toughness_3|knows_reflexes_5,nord_face_young_1,nord_face_older_2],
["nord_scout_multiplayer_ai","Nord Scout","Nord Scouts",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_nords,
 [itm_javelin,itm_sword_viking_1,itm_two_handed_axe,itm_spear,itm_old_round_shield,
itm_skullcap,itm_leather_jerkin,itm_leather_boots,itm_saddle_horse],
 def_attrib|level(19),wp(100),knows_riding_5|knows_toughness_2|knows_horse_archery_2,nord_face_young_1,nord_face_older_2],
["nord_archer_multiplayer_ai","Nord Archer","Nord Archers",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_nords,
 [itm_arrows,itm_two_handed_axe,itm_sword_viking_2,itm_short_bow,
itm_leather_jerkin,itm_shirt,itm_leather_boots,itm_nasal_helmet,itm_leather_cap],
 def_attrib|str_11|level(19),wp_melee(80)|wp_archery(110),knows_toughness_4|knows_reflexes_6,nord_face_young_1,nord_face_old_2],
["rhodok_veteran_crossbowman_multiplayer_ai","Rhodok Crossbowman","Rhodok Crossbowmen",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_rhodoks,
 [itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_club_with_spike_head,itm_maul,itm_pavise_shield_plain,itm_heavy_crossbow,itm_bolts,
itm_leather_cap,itm_padded_leather,itm_nomad_boots],
 def_attrib|level(19),wp_melee(100)|wp_crossbow(120),knows_common|knows_toughness_4|knows_reflexes_6,rhodok_face_middle_1,rhodok_face_older_2],
["rhodok_veteran_spearman_multiplayer_ai","Rhodok Spearman","Rhodok Spearmen",tf_mounted|tf_guarantee_all_wo_ranged,0,0,fac_kingdom_of_rhodoks,
 [itm_ashwood_pike,itm_war_spear,itm_pike,itm_club_with_spike_head,itm_sledgehammer,itm_pavise_shield_plain,itm_sword_medieval_a,
itm_leather_cap,itm_byrnie,itm_ragged_outfit,itm_nomad_boots],
 def_attrib|level(19),wp(115),knows_common|knows_toughness_5|knows_reflexes_3,rhodok_face_young_1,rhodok_face_older_2],
["rhodok_scout_multiplayer_ai","Rhodok Scout","Rhodok Scouts",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_rhodoks,
 #TODO: Change weapons,copied from Nord Scout
 [itm_javelin,itm_sword_viking_1,itm_two_handed_axe,itm_spear,itm_old_round_shield,
itm_skullcap,itm_leather_jerkin,itm_leather_boots,itm_saddle_horse],
 def_attrib|level(19),wp(100),knows_riding_5|knows_toughness_2|knows_horse_archery_2,rhodok_face_young_1,rhodok_face_older_2],
["sarranid_infantry_multiplayer_ai","Sarranid Infantry","Sarranid Infantries",tf_mounted|tf_guarantee_shield|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet,0,0,fac_kingdom_of_sarranids,
 [itm_sarranid_mail_shirt,itm_sarranid_horseman_helmet,itm_sarranid_boots_b,itm_sarranid_boots_c,itm_splinted_leather_greaves,itm_arabian_sword_b,itm_mace_3,itm_spear,itm_xenoargh_shield_kite01_01],
 def_attrib|level(20),wp_melee(105),knows_common|knows_riding_3|knows_toughness_2,swadian_face_middle_1,swadian_face_old_2],
["sarranid_archer_multiplayer_ai","Sarranid Archer","Sarranid Archers",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_sarranids,
 [itm_arrows,itm_nomad_bow,itm_arabian_sword_a,itm_archers_vest,itm_sarranid_boots_b,itm_sarranid_helmet1,itm_turban,itm_desert_turban],
 def_attrib|level(19),wp_melee(90)|wp_crossbow(100),knows_common|knows_riding_2|knows_toughness_1,swadian_face_young_1,swadian_face_old_2],
["sarranid_horseman_multiplayer_ai","Sarranid Horseman","Sarranid Horsemen",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_sarranids,
 [itm_lance,itm_arabian_sword_b,itm_scimitar_b,itm_mace_4,itm_xenoargh_shield_small01,
itm_sarranid_mail_shirt,itm_sarranid_boots_b,itm_sarranid_boots_c,itm_sarranid_horseman_helmet,itm_courser,itm_hunter],
 def_attrib|level(20),wp_melee(100),knows_common|knows_riding_4|knows_toughness_2,swadian_face_young_1,swadian_face_old_2],

 
 
#Multiplayer troops (they must have the base items only,nothing else)
["swadian_crossbowman_multiplayer","Swadian Crossbowman","Swadian Crossbowmen",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_swadians,
 [itm_bolts,itm_crossbow,itm_sword_medieval_b_small,itm_heater_shield_swadian,itm_red_shirt,itm_ankle_boots],
 def_attrib_multiplayer|level(19),wpe(90,60,180,90),knows_common|knows_toughness_2|knows_reflexes_5|knows_riding_1,swadian_face_young_1,swadian_face_old_2],
["swadian_infantry_multiplayer","Swadian Infantry","Swadian Infantry",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_swadians,
 [itm_sword_medieval_a,itm_heater_shield_swadian,itm_red_tunic,itm_ankle_boots],
 def_attrib_multiplayer|level(20),wpex(105,130,110,40,60,110),knows_common|knows_toughness_5|knows_reflexes_6|knows_riding_1,swadian_face_middle_1,swadian_face_old_2],
["swadian_man_at_arms_multiplayer","Swadian Man at Arms","Swadian Men at Arms",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_swadians,
 [itm_lance,itm_sword_medieval_a,itm_heater_shield_swadian,
itm_red_tunic,itm_ankle_boots,itm_saddle_horse],
 def_attrib_multiplayer|level(20),wp_melee(110),knows_common|knows_riding_5|knows_toughness_3|knows_reflexes_3,swadian_face_young_1,swadian_face_old_2],
#["swadian_mounted_crossbowman_multiplayer","Swadian Mounted Crossbowman","Swadian Mounted Crossbowmen",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_swadians,
# [itm_arrows,itm_light_crossbow,itm_xenoargh_shield_heater01,itm_bastard_sword_a,
#itm_red_shirt,itm_hide_boots,itm_saddle_horse],
# def_attrib_multiplayer|level(20),wp_melee(100)|wp_crossbow(120),knows_common|knows_riding_4|knows_toughness_3|knows_horse_archery_2|knows_reflexes_2,swadian_face_young_1,swadian_face_old_2],
["vaegir_archer_multiplayer","Vaegir Archer","Vaegir Archers",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_vaegirs,
 [itm_arrows,itm_scimitar,itm_nomad_bow,
itm_linen_tunic,itm_hide_boots],
 def_attrib_multiplayer|str_12|level(19),wpe(80,150,60,80),knows_toughness_2|knows_reflexes_4|knows_riding_1,vaegir_face_young_1,vaegir_face_older_2],
["vaegir_spearman_multiplayer","Vaegir Spearman","Vaegir spearman",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_ranged|tf_guarantee_shield,0,0,fac_kingdom_of_vaegirs,
 [itm_spear,itm_xenoargh_shield_kite01_06,itm_mace_1,
itm_linen_tunic,itm_hide_boots],
 def_attrib_multiplayer|str_12|level(19),wpex(110,100,130,30,50,120),knows_toughness_4|knows_reflexes_6|knows_riding_1,vaegir_face_young_1,vaegir_face_older_2],
["vaegir_horseman_multiplayer","Vaegir Horseman","Vaegir Horsemen",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_vaegirs,
 [itm_scimitar,itm_lance,itm_xenoargh_shield_kite01_03,
itm_linen_tunic,itm_hide_boots,itm_saddle_horse],
 def_attrib_multiplayer|level(19),wpe(110,90,60,110),knows_riding_5|knows_toughness_3,vaegir_face_young_1,vaegir_face_older_2],
["khergit_veteran_horse_archer_multiplayer","Khergit Horse Archer","Khergit Horse Archers",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_khergits,
 [itm_sword_khergit_1,itm_nomad_bow,itm_arrows,itm_xenoargh_shield_small03,
itm_coarse_tunic,itm_leather_steppe_cap_a,itm_hide_boots,itm_steppe_horse],
 def_attrib_multiplayer|level(21),wpe(80,150,60,100),knows_riding_6|knows_horse_archery_5|knows_reflexes_3,khergit_face_middle_1,khergit_face_older_2],
["khergit_lancer_multiplayer","Khergit Lancer","Khergit Lancers",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_khergits,
 [itm_sword_khergit_1,itm_lance,itm_xenoargh_shield_small03,
itm_coarse_tunic,itm_leather_steppe_cap_a,itm_hide_boots,itm_steppe_horse],
 def_attrib_multiplayer|level(21),wp(115),knows_riding_6|knows_toughness_3|knows_horse_archery_1|knows_reflexes_5,khergit_face_middle_1,khergit_face_older_2],
["nord_archer_multiplayer","Nord Archer","Nord Archers",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_nords,
 [itm_arrows,itm_sword_viking_2_small,itm_short_bow,
itm_blue_tunic,itm_leather_boots],
 def_attrib_multiplayer|str_11|level(15),wpe(90,150,60,80),knows_toughness_2|knows_reflexes_4|knows_riding_1,nord_face_young_1,nord_face_old_2],
["nord_veteran_multiplayer","Nord Huscarl","Nord Huscarls",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_nords,
 [itm_sword_viking_1,itm_one_handed_war_axe_a,itm_old_round_shield,
itm_blue_tunic,itm_leather_boots],
 def_attrib_multiplayer|level(24),wpex(110,135,100,40,60,140),knows_toughness_4|knows_reflexes_6|knows_riding_1,nord_face_young_1,nord_face_older_2],
["nord_scout_multiplayer","Nord Scout","Nord Scouts",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_nords,
 [itm_javelin,itm_sword_viking_1,itm_spear,itm_xenoargh_shield_small03,
itm_blue_tunic,itm_leather_boots,itm_saddle_horse],
 def_attrib_multiplayer|level(19),wp(105),knows_riding_6|knows_toughness_2|knows_horse_archery_2|knows_reflexes_3,vaegir_face_young_1,vaegir_face_older_2],
["rhodok_veteran_crossbowman_multiplayer","Rhodok Crossbowman","Rhodok Crossbowmen",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_rhodoks,
 [itm_crossbow,itm_bolts,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_old_board_shield01,
itm_tunic_with_green_cape,itm_ankle_boots],
 def_attrib_multiplayer|level(20),wpe(100,60,180,90),knows_common|knows_toughness_2|knows_reflexes_5|knows_riding_1,rhodok_face_middle_1,rhodok_face_older_2],
["rhodok_sergeant_multiplayer","Rhodok Sergeant","Rhodok Sergeants",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_rhodoks,
 [itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_hammer,itm_military_pick,itm_morningstar,itm_morningstar2,itm_spiked_mace,itm_hammer,itm_fine_military_hammer,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_fine_military_hammer2,itm_spiked_hammer,itm_spiked_hammer2,itm_old_board_shield01,itm_spear,
itm_green_tunic,itm_ankle_boots],
 def_attrib_multiplayer|level(20),wpex(110,100,140,30,50,110),knows_common|knows_toughness_4|knows_reflexes_6|knows_riding_1,rhodok_face_middle_1,rhodok_face_older_2],
["rhodok_horseman_multiplayer","Rhodok Horseman","Rhodok Horsemen",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_rhodoks,
 [itm_sword_medieval_a,itm_xenoargh_shield_heater01,itm_light_lance,
itm_green_tunic,itm_ankle_boots,itm_saddle_horse],
 def_attrib_multiplayer|level(20),wp(100),knows_riding_4|knows_toughness_3|knows_reflexes_3,rhodok_face_middle_1,rhodok_face_older_2],
["sarranid_archer_multiplayer","Sarranid Archer","Sarranid Archers",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_sarranids,
 [itm_arrows,itm_arabian_sword_a,itm_nomad_bow,
itm_sarranid_cloth_robe,itm_sarranid_boots_b],
 def_attrib_multiplayer|str_12|level(19),wpe(80,150,60,80),knows_toughness_2|knows_reflexes_5|knows_riding_1,vaegir_face_young_1,vaegir_face_older_2],
["sarranid_footman_multiplayer","Sarranid Footman","Sarranid footman",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_sarranids,
 [itm_bamboo_spear,itm_xenoargh_shield_kite01_06,itm_arabian_sword_a,
itm_sarranid_cloth_robe,itm_sarranid_boots_b],
 def_attrib_multiplayer|str_12|level(19),wpex(110,100,130,30,50,120),knows_toughness_4|knows_reflexes_6|knows_riding_1,vaegir_face_young_1,vaegir_face_older_2],
["sarranid_mamluke_multiplayer","Sarranid Mamluke","Sarranid Mamluke",tf_mounted|tf_guarantee_all,0,0,fac_kingdom_of_sarranids,
 [itm_arabian_sword_a,itm_lance,itm_xenoargh_shield_small03,
itm_sarranid_cloth_robe,itm_sarranid_boots_b,itm_saddle_horse],
 def_attrib_multiplayer|level(19),wpe(110,90,60,110),knows_riding_5|knows_toughness_3,vaegir_face_young_1,vaegir_face_older_2],

["multiplayer_end","{!}multiplayer_end","{!}multiplayer_end",0,0,0,fac_kingdom_of_rhodoks,[],0,0,0,0,0],

#replacable troop,not used
["nurse","Nurse","{!}nurse",tf_mounted|tf_hero|tf_female,0,reserved,fac_kingdom_of_swadians,[itm_lady_dress_ruby ,itm_turret_hat_ruby, itm_leather_boots],def_attrib|level(2),wp(50),knows_common|knows_riding_2,0x000000055910200107632d675a92b92d00000000001e45620000000000000000,0x000000055910200107632d675a92b92d00000000001e45620000000000000000],
#erase later added to avoid errors

#Player history array
["log_array_entry_type","{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["log_array_entry_time","{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["log_array_actor","{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["log_array_center_object", "{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["log_array_center_object_lord", "{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["log_array_center_object_faction","{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["log_array_troop_object","{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["log_array_troop_object_faction", "{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],
["log_array_faction_object","{!}Local Merchant","{!}Local Merchant",tf_mounted|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_commoners,[itm_leather_apron,itm_leather_boots,itm_butchering_knife],def_attrib|level(5),wp(40),knows_common,merchant_face_1,merchant_face_2],

["quick_battle_troop_1","Rodrigo de Braganca","Rodrigo de Braganca",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_swadians,
 [itm_long_hafted_knobbed_mace,itm_wooden_shield,itm_iron_staff,itm_throwing_daggers,
itm_felt_hat,itm_fur_coat,itm_light_leather_boots,itm_leather_gloves],
 str_9|agi_15|int_12|cha_12|level(15),wpex(109,33,132,15,32,100),knows_riding_3|knows_reflexes_5|knows_weapon_master_3|knows_toughness_3,0x0000000e240070cd598bb02b9556428c00000000001eabce0000000000000000,0x0000000e240070cd598bb02b9556428c00000000001eabce0000000000000000],
["quick_battle_troop_2","Usiatra","Usiatra",tf_mounted|tf_hero|tf_female,0,0,fac_kingdom_of_swadians,
 [itm_nomad_bow,itm_arrows,itm_scimitar,itm_xenoargh_shield_small02,itm_sumpter_horse,
itm_leather_armor,itm_splinted_greaves],
 str_12|agi_14|int_11|cha_18|level(22),wpex(182,113,112,159,82,115),knows_horse_archery_2|knows_riding_3|knows_reflexes_4|knows_weapon_master_4|knows_toughness_4,0x000000007f004000719b69422165b71300000000001d5d1d0000000000000000,0x000000007f004000719b69422165b71300000000001d5d1d0000000000000000],
["quick_battle_troop_3","Hegen","Hegen",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_swadians,
 [itm_heavy_lance,itm_sword_two_handed_b,itm_sword_medieval_c,itm_heater_shield_generic,itm_warhorse,
itm_guard_helmet,itm_coat_of_plates,itm_mail_mittens,itm_mail_boots],
 str_18|agi_16|int_12|cha_11|level(24),wpex(90,152,102,31,33,34),knows_riding_5|knows_reflexes_5|knows_weapon_master_5|knows_toughness_6,0x000000018000324428db8a431491472400000000001e44a90000000000000000,0x000000018000324428db8a431491472400000000001e44a90000000000000000],
["quick_battle_troop_4","Konrad","Konrad",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_swadians,
 [itm_sword_two_handed_a,itm_mace_4,itm_xenoargh_shield_kite01_02,
itm_bascinet_3,itm_scale_armor,itm_mail_mittens,itm_mail_boots],
 str_18|agi_15|int_12|cha_12|level(24),wpex(130,150,130,30,50,90),knows_riding_2|knows_reflexes_5|knows_weapon_master_5|knows_toughness_6,0x000000081700205434db6df4636db8e400000000001db6e30000000000000000,0x000000081700205434db6df4636db8e400000000001db6e30000000000000000],
["quick_battle_troop_5","Sverre","Sverre",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_swadians,
 [itm_long_axe,itm_sword_viking_1,itm_light_throwing_axes,itm_xenoargh_round_shield02,
itm_nordic_fighter_helmet,itm_byrnie,itm_leather_gloves,itm_leather_boots],
 str_15|agi_15|int_12|cha_12|level(21),wpex(110,130,110,80,15,110),knows_riding_1|knows_reflexes_5|knows_weapon_master_5|knows_toughness_5,0x000000048a00024723134e24cb51c91b00000000001dc6aa0000000000000000,0x000000048a00024723134e24cb51c91b00000000001dc6aa0000000000000000],
["quick_battle_troop_6","Borislav","Borislav",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_swadians,
 [itm_strong_bow,itm_arrows,itm_arrows,itm_shortened_spear,
itm_leather_warrior_cap,itm_leather_jerkin,itm_leather_gloves,itm_ankle_boots],
 str_12|agi_15|int_15|cha_9|level(18),wpex(70,70,100,140,15,100),knows_horse_archery_2|knows_riding_2|knows_reflexes_5|knows_weapon_master_3|knows_toughness_2,0x000000089e00444415136e36e34dc8e400000000001d46d90000000000000000,0x000000089e00444415136e36e34dc8e400000000001d46d90000000000000000],
["quick_battle_troop_7","Stavros","Stavros",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_swadians,
 [itm_heavy_crossbow,itm_bolts,itm_sword_medieval_b_small,itm_pavise_shield_plain,
itm_nasal_helmet,itm_padded_leather,itm_leather_gloves,itm_leather_boots],
 str_12|agi_15|int_15|cha_12|level(21),wpex(100,70,70,30,140,80),knows_horse_archery_2|knows_riding_2|knows_reflexes_5|knows_weapon_master_5|knows_toughness_4,0x0000000e1400659226e34dcaa46e36db00000000001e391b0000000000000000,0x0000000e1400659226e34dcaa46e36db00000000001e391b0000000000000000],
["quick_battle_troop_8","Gamara","Gamara",tf_mounted|tf_hero|tf_female,0,0,fac_kingdom_of_swadians,
 [itm_throwing_spears,itm_throwing_spears,itm_scimitar,itm_leather_covered_round_shield,
itm_desert_turban,itm_skirmisher_armor,itm_leather_gloves,itm_sarranid_boots_b],
 str_12|agi_15|int_12|cha_12|level(18),wpex(100,40,100,85,15,130),knows_horse_archery_2|knows_riding_2|knows_reflexes_5|knows_weapon_master_4|knows_toughness_2,0x000000015400300118d36636db6dc8e400000000001db6db0000000000000000,0x000000015400300118d36636db6dc8e400000000001db6db0000000000000000],
["quick_battle_troop_9","Aethrod","Aethrod",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_swadians,
 [itm_nomad_bow,itm_arrows,itm_arrows,itm_scimitar_b,
itm_splinted_greaves,itm_lamellar_vest],
 str_16|agi_21|int_12|cha_14|level(26),wpex(182,113,112,159,82,115),knows_horse_archery_2|knows_riding_2|knows_reflexes_7|knows_weapon_master_4|knows_toughness_4,0x000000000000210536db6db6db6db6db00000000001db6db0000000000000000,0x000000000000210536db6db6db6db6db00000000001db6db0000000000000000],
["quick_battle_troop_10","Zaira","Zaira",tf_mounted|tf_hero|tf_female,0,0,fac_kingdom_of_swadians,
 [itm_sarranid_cavalry_sword,itm_strong_bow,itm_arrows,itm_arrows,itm_arabian_horse_b,
itm_sarranid_felt_head_cloth_b,itm_sarranid_common_dress,itm_sarranid_boots_b],
 str_13|agi_18|int_15|cha_9|level(18),wpex(126,19,23,149,41,26),knows_horse_archery_6|knows_riding_6|knows_weapon_master_2|knows_toughness_1,0x0000000502003001471a6a24dc6594cb00000000001da4840000000000000000,0x0000000502003001471a6a24dc6594cb00000000001da4840000000000000000],
["quick_battle_troop_11","Argo Sendnar","Argo Sendnar",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_swadians,
 [itm_morningstar,itm_morningstar2,itm_xenoargh_round_shield02,itm_war_spear,itm_courser,
itm_leather_gloves,itm_fur_hat,itm_leather_boots,itm_leather_jacket],
 str_15|agi_12|int_14|cha_20|level(28),wpex(101,35,136,15,17,19),knows_riding_4|knows_reflexes_2|knows_weapon_master_4|knows_toughness_5,0x0000000e800015125adb702de3459a9c00000000001ea6d00000000000000000,0x0000000e800015125adb702de3459a9c00000000001ea6d00000000000000000],
["quick_battle_troops_end","{!}quick_battle_troops_end","{!}quick_battle_troops_end",0,0,0,fac_kingdom_of_rhodoks,[],0,0,0,0,0],

["tutorial_fighter_1","Novice Fighter","Fighters",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_vaegirs,
 [itm_linen_tunic,itm_nomad_boots],
 def_attrib|level(1),wp_melee(10),knows_reflexes_1|knows_toughness_2,0x000000088c1073144252b1929a85569300000000000496a50000000000000000,0x000000088c1073144252b1929a85569300000000000496a50000000000000000],
["tutorial_fighter_2","Novice Fighter","Fighters",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_vaegirs,
 [itm_green_tunic,itm_nomad_boots],
 def_attrib|level(1),wp_melee(10),knows_reflexes_1|knows_toughness_2,0x000000088b08049056ab56566135c46500000000001dda1b0000000000000000,0x000000088b08049056ab56566135c46500000000001dda1b0000000000000000],
["tutorial_fighter_3","Regular Fighter","Fighters",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_vaegirs,
 [itm_green_tunic,itm_nomad_boots],
 def_attrib|level(9),wp_melee(50),knows_reflexes_1|knows_toughness_2,0x00000008bc00400654914a3b0d0de74d00000000001d584e0000000000000000,0x00000008bc00400654914a3b0d0de74d00000000001d584e0000000000000000],
["tutorial_fighter_4","Veteran Fighter","Fighters",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_vaegirs,
 [itm_linen_tunic,itm_nomad_boots],
 def_attrib|level(16),wp_melee(110),knows_reflexes_1|knows_toughness_3,0x000000089910324a495175324949671800000000001cd8ab0000000000000000,0x000000089910324a495175324949671800000000001cd8ab0000000000000000],
["tutorial_archer_1","Archer","Archers",tf_mounted|tf_guarantee_ranged|tf_guarantee_boots|tf_guarantee_armor,0,0,fac_kingdom_of_vaegirs,
 [itm_leather_jerkin,itm_leather_vest,itm_nomad_boots,itm_vaegir_spiked_helmet,itm_vaegir_fur_helmet,itm_vaegir_fur_cap,itm_nomad_cap],
 def_attrib|str_12|level(19),wp_melee(70)|wp_archery(110),knows_toughness_1|knows_reflexes_2,vaegir_face_young_1,vaegir_face_older_2],
["tutorial_master_archer","Archery Trainer","Archery Trainer",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_vaegirs,
 [itm_linen_tunic,itm_nomad_boots],
 def_attrib|str_12|level(19),wp_melee(70)|wp_archery(110),knows_toughness_1|knows_reflexes_2,0x0000000ea508540642f34d461d2d54a300000000001d5d9a0000000000000000,0x0000000ea508540642f34d461d2d54a300000000001d5d9a0000000000000000],
["tutorial_rider_1","Rider","{!}Vaegir Knights",tf_mounted|tf_guarantee_boots|tf_guarantee_gloves|tf_guarantee_armor|tf_guarantee_helmet|tf_guarantee_horse|tf_guarantee_shield,0,0,fac_kingdom_of_vaegirs,
 [itm_green_tunic,itm_hunter,itm_saddle_horse,itm_leather_gloves],
 def_attrib|level(24),wp(130),knows_riding_4|knows_toughness_3,vaegir_face_middle_1,vaegir_face_older_2],
["tutorial_rider_2","Horse archer","{!}Khergit Horse Archers",tf_mounted|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_ranged|tf_guarantee_horse,0,0,fac_kingdom_of_khergits,
 [itm_tribal_warrior_outfit,itm_nomad_robe,itm_hide_boots,itm_xenoargh_shield_small03,itm_steppe_horse],
 def_attrib|level(14),wp(80)|wp_archery(110),knows_riding_5|knows_toughness_1|knows_horse_archery_4,khergit_face_young_1,khergit_face_older_2],
["tutorial_master_horseman","Riding Trainer","Riding Trainer",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_vaegirs,
 [itm_leather_vest,itm_nomad_boots],
 def_attrib|str_12|level(19),wp_melee(70)|wp_archery(110),knows_toughness_1|knows_reflexes_2,0x0000000ea0084140478a692894ba185500000000001d4af30000000000000000,0x0000000ea0084140478a692894ba185500000000001d4af30000000000000000],
 
["swadian_merchant","Merchant of Praven","{!}Prominent",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_kingdom_of_nords,[itm_sword_two_handed_a,itm_nobleman_outfit,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_leather_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,mercenary_face_2],
["vaegir_merchant","Merchant of Reyvadin","{!}Prominent",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_kingdom_of_rhodoks,[itm_sword_two_handed_a,itm_nobleman_outfit,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_woolen_hose],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,mercenary_face_2],
["khergit_merchant","Merchant of Tulga","{!}Prominent",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_kingdom_of_swadians,[itm_sword_two_handed_a,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,mercenary_face_2],
["nord_merchant","Merchant of Sargoth","{!}Prominent",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_kingdom_of_vaegirs,[itm_sword_two_handed_a,itm_red_gambeson,itm_wei_xiadi_rod_gambeson,itm_wei_xiadi_rod_padded_cloth,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_nomad_boots],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,mercenary_face_2],
["rhodok_merchant","Merchant of Jelkala","{!}Prominent",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_kingdom_of_khergits,[itm_sword_two_handed_a,itm_leather_jerkin,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_blue_hose],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,mercenary_face_2],
["sarranid_merchant","Merchant of Shariz","{!}Prominent",tf_mounted|tf_hero|tf_randomize_face|tf_guarantee_armor|tf_guarantee_boots,0,reserved,fac_kingdom_of_sarranids,[itm_sword_two_handed_a,itm_sarranid_cloth_robe,itm_wei_xiadi_rich_tunic01,itm_wei_xiadi_rich_tunic02,itm_sarranid_boots_a],def_attrib|level(2),wp(20),knows_common,man_face_middle_1,mercenary_face_2],
["startup_merchants_end","startup_merchants_end","startup_merchants_end",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[],def_attrib|level(2),wp(20),knows_inventory_management_10,0],
 
["sea_raider_leader","Sea Raider Captain","Sea Raiders",tf_mounted|tf_hero|tf_guarantee_boots|tf_guarantee_armor|tf_guarantee_shield,0,0,fac_outlaws,
 [itm_arrows,itm_sword_viking_1,itm_sword_viking_2,itm_fighting_axe,itm_battle_axe,itm_spear,itm_nordic_shield,itm_nordic_shield,itm_nordic_shield,itm_wooden_shield,itm_long_bow,itm_javelin,itm_throwing_axes,
itm_nordic_helmet,itm_nordic_helmet,itm_nasal_helmet,itm_leather_jerkin,itm_byrnie,itm_leather_jerkin,itm_leather_boots,itm_nomad_boots],
 def_attrib|level(24),wp(110),knows_toughness_2|knows_riding_1|knows_reflexes_2,nord_face_young_1,nord_face_old_2],

["looter_leader","Robber","Looters",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_outlaws,
 [itm_hatchet,itm_club,itm_butchering_knife,itm_falchion,itm_rawhide_coat,itm_wei_xiadi_rod_thick_coat,itm_stones,itm_nomad_armor,itm_nomad_armor,itm_woolen_cap,itm_woolen_cap,itm_nomad_boots,itm_wrapping_boots],
 def_attrib|level(4),wp(20),knows_common,0x00000001b80032473ac49738206626b200000000001da7660000000000000000,0x00000001b80032473ac49738206626b200000000001da7660000000000000000],
 
["bandit_leaders_end","bandit_leaders_end","bandit_leaders_end",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[],def_attrib|level(2),wp(20),knows_inventory_management_10,0],

["relative_of_merchant","Merchant's Brother","{!}Prominent",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_kingdom_of_vaegirs,
 [itm_linen_tunic,itm_nomad_boots],
 def_attrib|level(1),wp_melee(10),knows_reflexes_1|knows_toughness_2,swadian_face_younger_2,swadian_face_younger_1],
 
["relative_of_merchants_end","relative_of_merchants_end","relative_of_merchants_end",tf_mounted|tf_hero|tf_guarantee_armor|tf_guarantee_boots,0,0,fac_commoners,[],def_attrib|level(2),wp(20),knows_inventory_management_10,0],
]
###DO NOT ADD NEW TROOPS BELOW THIS LINE!

#####################################################################################
#####################################################################################
#####################################################################################
#Troop upgrade declarations
upgrade(troops,"farmer","townsman")
upgrade(troops,"townsman","watchman")

#Standard Mercenary Start
upgrade2(troops,"watchman","caravan_guard","mercenary_skirmisher")
#Upgraded Skirmishers with heavier weapons and armor
upgrade(troops,"mercenary_skirmisher","mercenary_matchlocks")

#Standard Mercenary Split
upgrade2(troops,"caravan_guard","mercenary_swordsman","mercenary_crossbowman")

#Standard Mercenary Ranged Path
upgrade2(troops,"mercenary_crossbowman","mercenary_grenadier","mercenary_cannoneer")

#Standard Mercenary HTH / Cavalry Split
upgrade2(troops,"mercenary_swordsman","hired_blade","mercenary_horseman")
upgrade2(troops,"hired_blade","templar_trooper","templar_pikeman")

#Standard Mercenary Cavalry Split
upgrade2(troops,"mercenary_horseman","mercenary_cavalry","mercenary_dragoon")
upgrade(troops,"mercenary_cavalry","templar_paragon")

#Female split,ranged vs. mounted; now with a split between the old Hunter Woman / Sword Sister trees
#Yup, only took two years to get them back into the game design, lol.
upgrade2(troops, "follower_woman", "mercenary_woman", "hunter_woman")
upgrade(troops, "hunter_woman", "fighter_woman")
upgrade(troops, "fighter_woman", "sword_sister")
upgrade2(troops,"mercenary_woman","mercenary_hunter_woman","mercenary_fighter_woman")

#Female Ranged Path
upgrade(troops,"mercenary_hunter_woman","mercenary_pistolera")
upgrade(troops,"mercenary_pistolera","mercenary_duelist")

#Female Mounted Path
upgrade(troops,"mercenary_fighter_woman","mercenary_sword_sister")
upgrade(troops,"mercenary_sword_sister","mercenary_sword_sister_expert")

#Female HTH Path
upgrade(troops,"mercenary_infantry_woman","mercenary_infantry_woman_sergeant")
upgrade(troops,"mercenary_infantry_woman_sergeant","mercenary_infantry_woman_elite")
upgrade(troops,"mercenary_infantry_woman_elite","mercenary_infantry_woman_expert")

#The Battle Sister Path
upgrade(troops,"mercenary_polearm_woman_recruit","mercenary_polearm_woman_regular")
upgrade(troops,"mercenary_polearm_woman_regular","mercenary_polearm_woman_elite")
upgrade(troops,"mercenary_polearm_woman_elite","sister_of_mercy")

#New Swadian trees!
upgrade(troops,"swadian_polearm_levy","swadian_polearm_recruit")
upgrade(troops,"swadian_polearm_recruit","swadian_polearm_regular")
upgrade(troops,"swadian_polearm_regular","swadian_polearm_elite")

upgrade(troops,"swadian_ranged_levy","swadian_ranged_recruit")
upgrade(troops,"swadian_ranged_recruit","swadian_ranged_regular")
upgrade(troops,"swadian_ranged_regular","swadian_ranged_elite")
upgrade2(troops,"swadian_ranged_elite","swadian_ranged_expert","swadian_ranged_matchlock_expert")

upgrade2(troops,"swadian_maceman_regular","swadian_maceman_elite","swadian_maceman_two_hand")

upgrade(troops,"swadian_swordsman_regular","swadian_swordsman_expert")

upgrade(troops,"swadian_knight_regular","swadian_knight_elite")
upgrade(troops,"swadian_knight_elite","swadian_knight_expert")


#New Vaegir trees!
upgrade(troops,"vaegir_legionnaire_recruit","vaegir_legionnaire_regular")
upgrade(troops,"vaegir_legionnaire_regular","vaegir_legionnaire_elite")

upgrade(troops,"vaegir_crusher_recruit","vaegir_crusher_regular")
upgrade(troops,"vaegir_crusher_regular","vaegir_crusher_elite")

upgrade(troops,"vaegir_pikeman_recruit","vaegir_pikeman_regular")
upgrade(troops,"vaegir_pikeman_regular","vaegir_pikeman_elite")

upgrade(troops,"vaegir_archer_levy","vaegir_archer_recruit")
upgrade(troops,"vaegir_archer_recruit","vaegir_archer_regular")
upgrade(troops,"vaegir_archer_regular","vaegir_archer_elite")

upgrade(troops,"vaegir_horseman_recruit","vaegir_horseman_regular")
upgrade(troops,"vaegir_horseman_regular","vaegir_horseman_elite")


#New Khergit trees!
upgrade(troops,"khergit_infantry_archer_levy","khergit_infantry_archer_recruit")
upgrade(troops,"khergit_infantry_archer_recruit","khergit_infantry_archer_regular")
upgrade(troops,"khergit_infantry_archer_regular","khergit_infantry_archer_elite")
upgrade(troops,"khergit_infantry_archer_elite","khergit_infantry_archer_expert")

upgrade(troops,"khergit_infantry_nanigata_levy","khergit_infantry_nanigata_recruit")
upgrade(troops,"khergit_infantry_nanigata_recruit","khergit_infantry_nanigata_regular")
upgrade(troops,"khergit_infantry_nanigata_regular","khergit_infantry_nanigata_elite")

upgrade(troops,"khergit_infantry_polearm_recruit","khergit_infantry_polearm_regular")
upgrade(troops,"khergit_infantry_polearm_regular","khergit_infantry_polearm_elite")

upgrade(troops,"khergit_cavalry_archer_recruit","khergit_cavalry_archer_regular")
upgrade(troops,"khergit_cavalry_archer_regular","khergit_cavalry_archer_expert")

upgrade(troops,"khergit_lancer_recruit","khergit_lancer_regular")
upgrade(troops,"khergit_lancer_regular","khergit_lancer_elite")

#New Nord Tree!
upgrade(troops,"nord_archer_recruit","nord_archer_regular")
upgrade(troops,"nord_archer_regular","nord_archer_elite")
upgrade(troops,"nord_archer_elite","nord_archer_expert")

upgrade(troops,"nord_axeman_levy","nord_axeman_recruit")
upgrade(troops,"nord_axeman_recruit","nord_axeman_regular")
upgrade(troops,"nord_axeman_regular","nord_axeman_elite")
upgrade(troops,"nord_axeman_elite","nord_beserkir")

upgrade(troops,"nord_spearman_recruit","nord_spearman_regular")
upgrade(troops,"nord_spearman_regular","nord_spearman_elite")
upgrade(troops,"nord_spearman_elite","nord_spearman_expert")

upgrade(troops,"nord_huscarl_regular","nord_huscarl_elite")

upgrade(troops,"nord_valkyrie_regular","nord_valkyrie_elite")
upgrade(troops,"nord_valkyrie_elite","nord_valkyrie_expert")

#New Rhodok Tree!
upgrade(troops,"rhodok_spearman_levy","rhodok_spearman_recruit")
upgrade(troops,"rhodok_spearman_recruit","rhodok_spearman_regular")
upgrade(troops,"rhodok_spearman_regular","rhodok_spearman_elite")
upgrade(troops,"rhodok_spearman_elite","rhodok_spearman_expert")

upgrade(troops,"rhodok_crossbowman_recruit","rhodok_crossbowman_regular")
upgrade(troops,"rhodok_crossbowman_regular","rhodok_crossbowman_elite")
upgrade(troops,"rhodok_crossbowman_elite","rhodok_crossbowman_expert")

upgrade(troops,"rhodok_stormtroop_recruit","rhodok_stormtroop_regular")
upgrade(troops,"rhodok_stormtroop_regular","rhodok_stormtroop_elite")
upgrade(troops,"rhodok_stormtroop_elite","rhodok_stormtroop_expert")

upgrade(troops,"rhodok_pistolero_regular","rhodok_pistolero_elite")

upgrade(troops,"rhodok_condottieri_elite","rhodok_condottieri_expert")


#New Sarranid Tree!
upgrade(troops,"sarranid_irregular_levy","sarranid_irregular_recruit")
upgrade(troops,"sarranid_irregular_recruit","sarranid_irregular_regular")
upgrade(troops,"sarranid_irregular_regular","sarranid_irregular_elite")
upgrade(troops,"sarranid_irregular_elite","sarranid_irregular_expert")

upgrade(troops,"sarranid_archer_recruit","sarranid_archer_regular")
upgrade(troops,"sarranid_archer_regular","sarranid_archer_elite")
upgrade(troops,"sarranid_archer_elite","sarranid_archer_expert")

upgrade(troops,"sarranid_swordsman_regular","sarranid_swordsman_elite")
upgrade(troops,"sarranid_swordsman_elite","sarranid_swordsman_expert")

upgrade(troops,"sarranid_afrizar_regular","sarranid_afrizar_elite")
upgrade(troops,"sarranid_afrizar_elite","sarranid_afrizar_expert")

upgrade(troops,"sarranid_horseman_regular","sarranid_horseman_elite")
upgrade(troops,"sarranid_horseman_elite","sarranid_horseman_expert")

#New Bandit trees!
upgrade(troops,"looter","looter_normal")
upgrade(troops,"looter_normal","looter_hard")

upgrade(troops,"mountain_bandit","mountain_bandit_normal")
upgrade(troops,"mountain_bandit_normal","mountain_bandit_hard")

upgrade(troops,"forest_bandit","forest_bandit_normal")
upgrade(troops,"forest_bandit_normal","forest_bandit_hard")
upgrade(troops,"forest_bandit_hard","swadian_royal_huntsman")

upgrade(troops,"steppe_bandit","steppe_bandit_normal")
upgrade(troops,"steppe_bandit_normal","steppe_bandit_hard")
upgrade(troops,"steppe_bandit_hard","khergit_chu_ko_nu")

upgrade(troops,"ruski_soldat","ruski_soldat_normal")
upgrade(troops,"ruski_soldat_normal","ruski_soldat_hard")
upgrade(troops,"ruski_soldat_hard","rhulg_destroyer")

upgrade(troops,"sea_raider","sea_raider_normal")
upgrade(troops,"sea_raider_normal","sea_raider_hard")
upgrade(troops,"sea_raider_hard","nord_felhammir")

upgrade(troops,"desert_bandit","desert_bandit_normal")
upgrade(troops,"desert_bandit_normal","desert_bandit_hard")
upgrade(troops,"desert_bandit_hard","sarranid_camel_jezzailis")

upgrade(troops,"bandit","bandit_normal")
upgrade(troops,"bandit_normal","bandit_hard")

upgrade(troops,"brigand","brigand_normal")
upgrade(troops,"brigand_normal","brigand_hard")

upgrade(troops,"manhunter","slave_driver")

#upgrade(troops,"forest_bandit","mercenary_crossbowman")

upgrade(troops,"slave_driver","slave_hunter")
upgrade(troops,"slave_hunter","slave_crusher")
upgrade(troops,"slave_crusher","slaver_chief")

upgrade(troops,"peasant_woman","mercenary_woman")
